/*
	sagan_data.c

	data i/o and manipulation functions

      int     SaEvalEquation()
      int     SaReadCore()
      void    SaCullTable()
      void    SaCull()
      void    SaGaussianLog()
      void    SaGaussianHole()
      void    SaDecimate()
      int     SaSmoothLog()
      int     SaSmoothCore()
      int     SaInputStrat()
      int     SaInputLog()
      int     SaInputData()
      int     SaFreeData()
*/
#include <stdio.h>
#include <search.h>
#include <malloc.h>
#include <math.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <X11/cursorfont.h>
#include "sagan-creation-c.h"
#include "sagan_vars.h"

#define	ODPKEYLEN	(20)
HashInfo	**hashinfo;
extern	int	hash_num;
char    buffer[MAXBUF];
void    SaCleanUp();

int     SaEvalEquation(data_val,cull_equ)
float           data_val;
Equation        cull_equ;
{
        int     qual1,qual2,qual;

        qual1=qual2=qual=GOOD;
        if(cull_equ.sign1 == LESS_THAN && data_val<cull_equ.value1)
                qual1 =BAD_CULL1;
        else if(cull_equ.sign1 == LESS_THAN_OR_EQUAL_TO &&
                                data_val <= cull_equ.value1)
                qual1 =BAD_CULL1;
        else if(cull_equ.sign1 == GREATER_THAN && data_val>cull_equ.value1)
                qual1 =BAD_CULL1;
        else if(cull_equ.sign1 == GREATER_THAN_OR_EQUAL_TO &&
                        data_val>=cull_equ.value1)
                qual1 =BAD_CULL1;
        else if(cull_equ.sign1 == EQUAL_TO && data_val == cull_equ.value1)
                qual1 =BAD_CULL1;

        if(cull_equ.join == ONLY)
                qual=qual1;
        else
        {
                if(cull_equ.sign2 == LESS_THAN && data_val<cull_equ.value2)
                        qual2 =BAD_CULL2;
                else if(cull_equ.sign2 == LESS_THAN_OR_EQUAL_TO &&
                                        data_val <= cull_equ.value2)
                        qual2 =BAD_CULL2;
                else if(cull_equ.sign2 == GREATER_THAN && data_val>cull_equ.value2)
                        qual2 =BAD_CULL2;
                else if(cull_equ.sign2 == GREATER_THAN_OR_EQUAL_TO &&
                                        data_val>=cull_equ.value2)
                        qual2 =BAD_CULL2;
                else if(cull_equ.sign2 == EQUAL_TO && data_val == cull_equ.value2)
                        qual2 =BAD_CULL2;
 
                if(cull_equ.join == OR)
                        qual=qual1|qual2;
        }
 
        return(qual);
}

int     SaEvalEquationAlt(data_val,cull_equ)
float           data_val;
Equation        cull_equ;
{
        int     qual;

        qual=NO;
        if(cull_equ.sign1 == LESS_THAN && data_val<cull_equ.value1)
                qual = YES;
        else if(cull_equ.sign1 == LESS_THAN_OR_EQUAL_TO &&
                                data_val <= cull_equ.value1)
                qual = YES;
        else if(cull_equ.sign1 == GREATER_THAN && data_val>cull_equ.value1)
                qual = YES;
        else if(cull_equ.sign1 == GREATER_THAN_OR_EQUAL_TO &&
                        data_val>=cull_equ.value1)
                qual = YES;
        else if(cull_equ.sign1 == EQUAL_TO && data_val == cull_equ.value1)
                qual = YES;

        if(cull_equ.join == ONLY) {
        }
        else
        {
                if(cull_equ.sign2 == LESS_THAN && data_val<cull_equ.value2)
                        qual = YES;
                else if(cull_equ.sign2 == LESS_THAN_OR_EQUAL_TO &&
                                        data_val <= cull_equ.value2)
                        qual = YES;
                else if(cull_equ.sign2 == GREATER_THAN && data_val>cull_equ.value2)
                        qual = YES;
                else if(cull_equ.sign2 == GREATER_THAN_OR_EQUAL_TO &&
                                        data_val>=cull_equ.value2)
                        qual = YES;
                else if(cull_equ.sign2 == EQUAL_TO && data_val == cull_equ.value2)
                        qual = YES;
        }

        if(qual == NO) {
          return(0);
        }
        else {
          return(LOG_VALUE_CULLED);
        }
}

int	SaReadCore(fptr,dataptr)
FILE	*fptr;
DataFile	*dataptr;
{
	int	value;
	switch (dataptr->in_format)
	{
		case MST95REPORT:
			value=SaReadMST95report(fptr,dataptr);
			break;
		case TKREPORT:
			value=SaReadTKreport(fptr,dataptr);
			break;
		case ODPOTHER:
                case ODPOTHER1:
                case ODPOTHER2:
                case ODPOTHER3:
                case ODPOTHER4:
                case ODPOTHER5:
			value=SaReadOtherOdp(fptr,dataptr);
			break;
                case JANUSCLEAN:
                case JANUSORIG:
                        value=SaReadJanus(fptr,dataptr);
                        break;
		default:
			value=True;
	}
	return(value);
}

void SaCullTable(ds, ptr,name,flag)
int     ds;
Hole	*ptr;
char	*name;
int	flag;
{
	int         i,j,k;
        XmString    xmstring;
        XmFontList  fontlist;

        XtVaGetValues(messageBoxWorking,XmNlabelFontList,&fontlist,NULL);

/* if no hash table, read stored file */
	if(flag==CREATE_HASH)
	{

		if((hashinfo=SaReadCullTable(name,&hash_num))== NULL)
		{
			SaCleanUp(COREDATA);
			info.error=True;
			if((info.message=(char*)calloc((int)strlen(name)+100,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaCullTable 1");
		 	(void)sprintf(info.message,"Error reading Cull Table %s", name);
			SaManageMessage(info);
			return;
		}
		else
		{
			if((charptr=(char*)calloc((int)strlen(name)+100,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaCullTable 1b");
			(void)sprintf(charptr,"\nCull Table %s read",name);
			SaUpdateReport(charptr);
			free(charptr);
		}
	}
	/* search hash table for values */
	for(i=0;i<ptr->numcores;i++)
	{
                if((charptr=(char*)calloc(500,CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("SaCullTable");
                (void)sprintf(charptr,"\nData : Using Cull Table to cull data hole %c, core %d.", ptr->name, ptr->core[i]->num);
                xmstring=XmStringCreateLocalized(&charptr[1]);
                XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring,
                              XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                XtResizeWindow(messageBoxWorking);
                XmUpdateDisplay(messageBoxWorking);
                XmStringFree(xmstring);
                free(charptr);

		for(j=0;j<ptr->core[i]->numvalues;j++)
		{
                	for(k=0; k<hash_num; ++k)
                        {
                            if(data[ds]->leg == hashinfo[k]->leg &&
                               data[ds]->site == hashinfo[k]->site &&
                               ptr->name == hashinfo[k]->hole &&
                               ptr->core[i]->num == hashinfo[k]->core &&
                               strcmp(ptr->core[i]->value[j]->section, hashinfo[k]->section) == 0)
                            {
                              if(fabs(ptr->core[i]->value[j]->top_int - hashinfo[k]->top_int) <= CULLTABLEDEPTHCHECK)
                              {
                                ptr->core[i]->value[j]->quality=ptr->core[i]->value[j]->quality|BAD_CULLTABLE;
                              }
                            }
                        }
		}
	}
	if(flag==DESTROY_HASH)
	{
		for(i=0;i<hash_num;i++)
		{
			if(hashinfo[i])
			{
				free(hashinfo[i]);
				hashinfo[i]=NULL;
			}
		}
		hash_num=0;
		if(hashinfo)
		{
			free(hashinfo);
			hashinfo=NULL;
		}
	}
}

void	SaCull(ds, ptr,type,cullinfo,flag)
int     ds;
Hole	*ptr;
int	type;
Cull	cullinfo;
int	flag;
{
	int	i,j;
	Core	*coreptr;
	Value	*valueptr;

	if(cullinfo.top > 0)	/* remove top portion of core */
	{
		for(i=0; i<ptr->numcores; ++i)
		{
                    if(ptr->core[i]->numvalues > 0) 
                    {    
			coreptr=ptr->core[i];
                        j = 0;
			while(strcmp(coreptr->value[j]->section,"1")==0 && coreptr->value[j]->top_int < cullinfo.top)
                        {
			      coreptr->value[j]->quality=coreptr->value[j]->quality|BAD_TOP;
                              ++j;
                              if(j >= coreptr->numvalues) {
                                break;
                              }
			}
                    }
		}
	}
	if(cullinfo.by_core_num)	/* remove cores > given number */
	{
		for(i=0;i<ptr->numcores;i++)
		{
			if(ptr->core[i]->num > cullinfo.max_core)
			{
				coreptr=ptr->core[i];
				for(j=0;j<coreptr->numvalues;j++)
					coreptr->value[j]->quality=coreptr->value[j]->quality|BAD_CORE_NUM;
			}
		}
	}
	if(cullinfo.method==CULL_PARAMETERS || cullinfo.method == CULL_PARAMETER_AND_TABLE)
	{
		for(i=0;i<ptr->numcores;i++)
		{
			coreptr=ptr->core[i];
			for(j=0;j<coreptr->numvalues;j++)	/* loop through data */
			{
				valueptr=coreptr->value[j];
				if(type==GRAPE)
					valueptr->quality= valueptr->quality|SaEvalEquation(*valueptr->data,cullinfo.grape);
				else if(type==PWAVE)
				{
					valueptr->quality= valueptr->quality|SaEvalEquation(*valueptr->data,cullinfo.pwave);
                                        if(openfileinfo.in_data_format == MST95REPORT) {
					  if(valueptr->pwave->sig_strength < cullinfo.pwave_signal_max)
							valueptr->quality=valueptr->quality|BAD_PWAVE_SIGNAL;
                                        }
				}
				else if(type==SUSCEPTIBILITY)
					valueptr->quality=valueptr->quality| SaEvalEquation(*valueptr->data,cullinfo.susceptibility);
				else if(type==NATURALGAMMA)
					valueptr->quality=valueptr->quality|SaEvalEquation(*valueptr->data,cullinfo.naturalgamma);
				else if(type==REFLECTANCE)
					valueptr->quality= valueptr->quality|SaEvalEquation(*valueptr->data,cullinfo.reflectance);
				else if(type == OTHERTYPE)
					valueptr->quality=valueptr->quality|SaEvalEquation(*valueptr->data,cullinfo.otherodp);
                                else
                                        valueptr->quality=valueptr->quality|SaEvalEquation(*valueptr->data,cullinfo.otherodp);
                                if(valueptr->quality != GOOD) {
                                  ++cull.cnt;
                                }
			}
		}
	}
	if(cullinfo.method==CULL_TABLE || cullinfo.method == CULL_PARAMETER_AND_TABLE)
		SaCullTable(ds, ptr,cullinfo.filename,flag);
}

void	SaGaussianLog(logptr)
LogData	*logptr;
{
  int       i,j,k,inc,l,m,num_interp,datptr_cnt,end_cnt,num_depths,highfound,lowfound;
  float     halfwidth,*weight,*datptr,start_depth,end_depth,*sm_datptr,target;
  LogValue  *valueptr;
  void      gauss_weights();
	
  if(logptr->info.smooth.width_units != POINTS && logptr->info.smooth.width_units != DEPTH) {
    logptr->info.smooth.method = NONE;
    logptr->info.smooth.plot = UNSMOOTH;
    return;
  }

  /* must use an odd number of weights 
  */
  if(logptr->info.smooth.width%2==0) {
     logptr->info.smooth.width+=1;
  }
  halfwidth=(int)((logptr->info.smooth.width-1)/2);
  /* get filter weights 
  */
  if((weight=(float*)calloc(logptr->info.smooth.width+1,sizeof(float)))==NULL)
		SaBadMemAlloc("SaGaussianLog 1");
  gauss_weights(logptr->info.smooth.width,weight);

  /* loop through log data 
  */
  if(logptr->info.smooth.width_units==POINTS)
  {
    /* allocate storage  
    */
    if((datptr=(float*)calloc(logptr->info.smooth.width+1,sizeof(float)))==NULL)
                SaBadMemAlloc("SaGaussianLog 2");
    /* loop through data (first and last "halfwidth" values are same as raw data 
    */
    for(j=0;j<halfwidth && j<logptr->data_num;j++) {
      if(logptr->value[j]->quality == GOOD) {
        logptr->value[j]->sm_data = logptr->value[j]->data;
      }
    }
    /* loop through data until at last half width 
    */
    for(;j<logptr->data_num-halfwidth;j++) {
      /* first value in smoothing */
      inc=j-halfwidth;
      /* raw value at this point must be good */
      valueptr=logptr->value[j];
      if(valueptr->quality!=GOOD) {
      }
      /* fill data arrays */
      for(k=inc,num_interp=0,datptr_cnt=0;datptr_cnt<logptr->info.smooth.width && k<logptr->data_num && 
           !(valueptr->quality);k++,datptr_cnt++) {
        if(logptr->value[k]->quality==GOOD) {
          datptr[datptr_cnt]=logptr->value[k]->data;
        }
        else {     /* interpolate data point */
          /* find next lower point in width range */
          for(l=k-1;l>=inc && logptr->value[l]->quality!=GOOD;l--);
          if(l<inc) {    
            /* no lower point in range 
            valueptr->quality=valueptr->quality|BAD_SMOOTH;
            */
          }
          else {
            /* find next higher point in width range */
            for(m=k+1;m<=j+halfwidth && logptr->value[m]->quality!=GOOD;m++);
            if(m>j+halfwidth) {   
              /* no higher point in range 
              valueptr->quality=valueptr->quality|BAD_SMOOTH;
              */
            }
            else {
              datptr[datptr_cnt]=logptr->value[l]->data+(logptr->value[m]->data - logptr->value[l]->data)*
                                 (logptr->value[m]->depth-logptr->value[l]->depth)/logptr->value[k]->depth;
              num_interp++;
            }
          }
        }
      }
      if(num_interp>halfwidth) {
        valueptr->quality=valueptr->quality|BAD_SMOOTH;
      }

      /* compute smoothed value 
      */
      if(valueptr->quality == GOOD) {
        valueptr->sm_data=0.0;
        for(k=0;k<logptr->info.smooth.width;k++) {
          valueptr->sm_data+=datptr[k]*weight[k+1];
        }
      }
      else {
        valueptr->sm_data=valueptr->data;
      }
    }
    /* loop through last half width values 
    */
    for(;j<logptr->data_num;j++) {
      valueptr->sm_data=valueptr->data;
    }
  }

  else if(logptr->info.smooth.width_units==DEPTH) {

	/* find first & last good points */
		for(j=0;j<logptr->data_num&& logptr->value[j]->quality!=GOOD ;j++);
		start_depth=logptr->value[j]->depth * 100.0;
		for(k=logptr->data_num-1;k>=0 && logptr->value[k]->quality!=GOOD;k--);
		end_cnt=k;
		if(end_cnt<0 || j>=logptr->data_num)
		{
                        free(weight);
                        weight = NULL;
			info.error=True;
			info.caller=SMOOTHLOG;
			if((info.message=calloc(200,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaGaussianLog 1");
			(void)sprintf(info.message,"Error fewer points than smoothing width!");
			SaManageMessage(info);
			return;
		}
		end_depth=logptr->value[end_cnt]->depth * 100.0;
	/* allocate data */
		num_depths=1+end_depth-start_depth;
		if(num_depths<=halfwidth)
		{
                        free(weight);
                        weight = NULL;
			info.error=True;
			info.caller=SMOOTHLOG;
			if((info.message=calloc(200,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaGaussianLog 1");
			(void)sprintf(info.message,"Error fewer points than smoothing width!");
			SaManageMessage(info);
			return;
		}
		if((datptr=(float*)calloc(num_depths,sizeof(float)))==NULL)
			SaBadMemAlloc("SaGaussianLog 2");
		if((sm_datptr=(float*)calloc(num_depths,sizeof(float)))==NULL)
			SaBadMemAlloc("SaGaussianLog 3");
	/* interpolate 1 cm array of entire core */
		valueptr=logptr->value[j];	
		datptr[l=0]=valueptr->data;
		for(m=j,l++,target=start_depth+1;target<end_depth && l<num_depths;target++,l++)
		{
		/* find value >= to target value */
			highfound=lowfound=False;
			while(m<logptr->data_num && k>=0 && !(lowfound&highfound))
			{
				if(!highfound && !(target<=rint((double)logptr->value[m]->depth*100.0)
						&& logptr->value[m]->quality==GOOD))
					m++;
				else if(!highfound)
				{
					highfound=True;
					k=m;
				}
				else 
				{
					if(!(target>rint((double)logptr->value[k]->depth*100.0)&&
							logptr->value[k]->quality==GOOD))
						k--;
					else
						lowfound=True;
				}
			}
			if(m>=logptr->data_num|| k<0)
			{
                                free(datptr);
                                datptr = NULL;
                                free(sm_datptr);
                                sm_datptr = NULL;
                                free(weight);
                                weight = NULL;
				info.error=True;
				if((info.message=calloc(200,CHAR_SIZE))==NULL)
					SaBadMemAlloc("SaGaussianLog 4");
				(void)sprintf(info.message,"Error trying to create smooth by cm array!");
				SaManageMessage(info);
				return;
			}
			if(target==rint((double)logptr->value[m]->depth*100.0))	/* exact match */
				datptr[l]=logptr->value[m]->data;
			else /* interpolate */
				datptr[l]=logptr->value[k]->data+(logptr->value[m]->data -logptr->value[k]->data)*
						 ((target/100.0)-logptr->value[k]->depth)/
						(logptr->value[m]->depth-logptr->value[k]->depth);
		}
                if(l<num_depths) {
                  datptr[l]=logptr->value[end_cnt]->data;
                }
	/* smooth data array */
	/* loop through data (first and last "halfwidth" values are same as raw data */
		for(j=0;j<halfwidth;j++)
			sm_datptr[j]=datptr[j];
		/* compute smoothed value */
		for(;j<num_depths-halfwidth;j++)
		{
			sm_datptr[j]=0.0;
			for(k=0,l=j-halfwidth+1;k<logptr->info.smooth.width;k++,l++)
				sm_datptr[j]+=datptr[l]*weight[k+1];
		}
	/* loop through last half width values */
		for(;j<num_depths;j++)
			sm_datptr[j]=datptr[j];
	/* fill in real smoothed array */
		for(j=0;j<logptr->data_num;j++)
		{
			valueptr=logptr->value[j];
                        if(valueptr->quality != GOOD)
                        {
                          valueptr->quality=valueptr->quality|BAD_SMOOTH;
                        }
                        else
                        {
			  if(rint((double)valueptr->depth*100.0)<(start_depth+halfwidth))
			  {
				valueptr->quality=valueptr->quality|BAD_SMOOTH;
				valueptr->sm_data=valueptr->data;
			  }
			  else if(rint((double)valueptr->depth*100.0)>(end_depth-halfwidth))
			  {
				valueptr->quality=valueptr->quality|BAD_SMOOTH;
				valueptr->sm_data=valueptr->data;
			  }
			  else
			  {
				k=(int)(rint((double)valueptr->depth*100.0)-start_depth);
				valueptr->sm_data=sm_datptr[k];
			  }
                        }
		}
                free(sm_datptr);
                sm_datptr = NULL;
	}
	
	if(weight)
	{
		free(weight);
		weight=NULL;
	}
	if(datptr)
	{
		free(datptr);
		datptr=NULL;
	}
}
	

void	SaGaussianHole(ptr)
Hole	*ptr;
{
  int     i,j,k,l,m,n,num_interp=0,inc,datptr_cnt,max_depths=0,num_depths=0,this_num=0,end_cnt,highfound,lowfound;
  float   halfwidth,*weight,target,end_depth;
  float   *datptr,*sm_datptr,start_depth;
  double  rint();
  Core    *coreptr;
  void    gauss_weights();

  /* use gaussian filter */
  if(smooth.method==GAUSSIAN)
  {
    /* must use an odd number of weights */
    if(smooth.width%2==0)
      smooth.width+=1;
    halfwidth=(int)((smooth.width-1)/2);
    /* get filter weights */
    if((weight=(float*)calloc(smooth.width+1,sizeof(float)))==NULL)
      SaBadMemAlloc("Sa");
    gauss_weights(smooth.width,weight);
    /* allocate storage arrays */
    if(smooth.width_units==POINTS)
    {
      sm_datptr=NULL;
      if((datptr=(float*)calloc(smooth.width+1,sizeof(float)))==NULL)
                                SaBadMemAlloc("Sa");
    }
    else
    {
      max_depths=0;
      for(i=0;i<ptr->numcores;i++)
      {
        coreptr=ptr->core[i];
        this_num=(coreptr->value[coreptr->numvalues-1]->mbsf-coreptr->value[0]->mbsf)*100;
        if(this_num> max_depths) max_depths=this_num;
      }
      max_depths+=2;
      if((sm_datptr=(float*)calloc(max_depths,sizeof(float)))==NULL)
                                SaBadMemAlloc("Sa");
      if((datptr=(float*)calloc(max_depths,sizeof(float)))==NULL)
                                SaBadMemAlloc("Sa");
    }
    /* loop through cores */
    for(i=0;i<ptr->numcores;i++)
    {
      coreptr=ptr->core[i];
      coreptr->smooth_status = NONE;
      if(smooth.width_units==POINTS)
      {
        /* loop through data (first and last "halfwidth" values are same as raw data if good! */
        for(j=0;j<halfwidth && j<coreptr->numvalues;j++)
        {
          if(coreptr->value[j]->quality == GOOD)
          {
            coreptr->value[j]->sm_data=*coreptr->value[j]->data;
            coreptr->value[j]->smooth_ok = SMTAIL;
          }
          else
          {
            coreptr->value[j]->smooth_ok = SMBADDATA;
          }
        }

        if(j>= coreptr->numvalues)
        {
          coreptr->smooth_status = SMTOOFEWPTS;
        }
        else
        {
          /* loop through data until at last half width */
          for(;j<coreptr->numvalues-halfwidth;j++)
          {
            /* let smooth be ok until proven otherwise */
            coreptr->value[j]->smooth_ok = SMOK;
            inc=j-halfwidth;    /* first value in smoothing */
            /* raw value at this point must be good */
            if(coreptr->value[j]->quality!=GOOD)
            {
              coreptr->value[j]->smooth_ok = SMBADDATA;
            }
            /* fill data arrays */
            for(k=inc,num_interp=0,datptr_cnt=0;datptr_cnt<smooth.width && k<coreptr->numvalues &&
                                                        !(coreptr->value[j]->quality);k++,datptr_cnt++)
            {
              if(coreptr->value[k]->quality==GOOD)
              {
                datptr[datptr_cnt]=*coreptr->value[k]->data;
              }
              else      /* interpolate data point */
              {
                /* find next lower point in width range */
                for(l=k-1;l>=inc && coreptr->value[l]->quality!=GOOD;l--);
                if(l<inc)       /* no lower point in range */
                {
                  coreptr->value[j]->smooth_ok = SMTOOFEWPTS;
                }
                else
                {
                  /* find next higher point in width range */
                  for(m=k+1;m<=j+halfwidth && coreptr->value[m]->quality!=GOOD;m++);
                  if(m>j+halfwidth)     /* no higher point in range */
                  {
                    coreptr->value[j]->smooth_ok = SMTOOFEWPTS;
                  }
                  else
                  {
                    datptr[datptr_cnt]=*coreptr->value[l]->data+(*coreptr->value[m]->data -
                                                                *coreptr->value[l]->data)*
                                (coreptr->value[m]->mbsf-coreptr->value[l]->mbsf)/
                                                        coreptr->value[k]->mbsf;
                    num_interp++;
                  }
                }
              }
            }
            if(num_interp>halfwidth)
            {
              coreptr->value[j]->smooth_ok = SMBADCALC;
            }
            /* compute smoothed value */
            if(coreptr->value[j]->smooth_ok < SMOK)
            {
            }
            else if(!coreptr->value[j]->quality)
            {
              coreptr->value[j]->sm_data=0.0;
              for(k=0;k<smooth.width;k++)
                coreptr->value[j]->sm_data+=datptr[k]*weight[k+1];
            }
            else
            {
              coreptr->value[j]->smooth_ok = SMBADCALC;
            }
          }
          if(j<coreptr->numvalues)
          {
            /* loop through last half width values */
            for(;j<coreptr->numvalues;j++)
            {
              if(coreptr->value[j]->quality == GOOD)
              {
                coreptr->value[j]->sm_data=*coreptr->value[j]->data;
                coreptr->value[j]->smooth_ok = SMTAIL;
              }
              else
              {
                coreptr->value[j]->smooth_ok = SMBADDATA;
              }
            }
          }

          /* check if have some smoothed points
          */
          n = 0;
          for(j=0; j<coreptr->numvalues; ++j)
          {
            if(coreptr->value[j]->smooth_ok == SMOK || coreptr->value[j]->smooth_ok == SMTAIL)
            {
              ++n;
            }
          }
          if(n < TOOFEWSMOOTHED)
          {
            coreptr->smooth_status = SMTOOFEWPTS;
          }
          if(coreptr->smooth_status == NONE) {
            coreptr->smooth_status = SMOK;
          }
        }
      }
      else if(smooth.width_units==DEPTH)
      {
        /* find first & last good points */
        for(j=0;j<coreptr->numvalues && coreptr->value[j]->quality!=GOOD ;j++);
        for(k=coreptr->numvalues-1;k>=0 && coreptr->value[k]->quality!=GOOD;k--);
        end_cnt=k;
        if(end_cnt<0 || j>=coreptr->numvalues)
        {
          coreptr->smooth_status = SMTOOFEWPTS;
        }
        else
        {
          start_depth=coreptr->value[j]->mbsf*100.0;
          end_depth=coreptr->value[end_cnt]->mbsf*100.0;
          num_depths=1+end_depth-start_depth;
          if(num_depths<=halfwidth)
          {
            coreptr->smooth_status = SMTOOFEWPTS;
          }
          else if(num_depths>max_depths)
          {
            coreptr->smooth_status = SMTOOFEWPTS;
          }
          else
          {
            /* interpolate 1 cm array of entire core */
            datptr[l=0]=*coreptr->value[j]->data;
            for(m=j,l++,target=start_depth+1;target<end_depth && l<max_depths;target++,l++)
            {
              /* find value >= to target value */
              highfound=lowfound=False;
              while(m<coreptr->numvalues && k>=0 && !(lowfound&highfound))
              {
                if(!highfound && !(target<=rint((double)(coreptr->value[m]->mbsf*100))
                                                        && coreptr->value[m]->quality==GOOD))
                  m++;
                else if(!highfound)
                {
                  highfound=True;
                  k=m;
                }
                else
                {
                  if(!(target>rint((double)(coreptr->value[k]->mbsf*100))&&
                                        coreptr->value[k]->quality==GOOD))
                    k--;
                  else
                    lowfound=True;
                }
              }
              if(m>=coreptr->numvalues || k<0)
              {
                coreptr->smooth_status = SMTOOFEWPTS;
              }
              else {
                if(target==rint((double)(coreptr->value[m]->mbsf*100)))       /* exact match */
                                        datptr[l]=*coreptr->value[m]->data;
                else /* interpolate */
                  datptr[l]=*coreptr->value[k]->data+(*coreptr->value[m]->data -*coreptr->value[k]->data)*
                                                        (target/100.0-coreptr->value[k]->mbsf)/
                                        (coreptr->value[m]->mbsf-coreptr->value[k]->mbsf);
              }
            }
            if(l>max_depths)
            {
              coreptr->smooth_status = SMTOOFEWPTS;
            }
            if(coreptr->smooth_status == NONE)
            {
              datptr[l]=*coreptr->value[end_cnt]->data;
              /* smooth data array */
              /* loop through data (first and last "halfwidth" values are same as raw data */
              for(j=0;j<halfwidth;j++)
                sm_datptr[j]=datptr[j];
              /* compute smoothed value */
              for(;j<num_depths-halfwidth;j++)
              {
                sm_datptr[j]=0.0;
                for(k=0,l=j-halfwidth+1;k<smooth.width;k++,l++)
                  sm_datptr[j]+=datptr[l]*weight[k+1];
              }
              /* loop through last half width values */
              for(;j<num_depths;j++)
                sm_datptr[j]=datptr[j];
              /* fill in real smoothed array */
              for(j=0;j<coreptr->numvalues;j++)
              {
                if(coreptr->value[j]->quality != GOOD)
                {
                  coreptr->value[j]->smooth_ok = SMBADDATA;
                }
                else
                {
                  if(rint((double)(coreptr->value[j]->mbsf*100.0))<(start_depth+halfwidth))
                  {
                    coreptr->value[j]->sm_data=*coreptr->value[j]->data;
                                               coreptr->value[j]->smooth_ok = SMTAIL;
                  }
                  else if(rint((double)(coreptr->value[j]->mbsf*100.0))>(end_depth-halfwidth))
                  {
                    coreptr->value[j]->sm_data=*coreptr->value[j]->data;
                                              coreptr->value[j]->smooth_ok = SMTAIL;
                  }
                  else
                  {
                    k=(int)(rint((double)(coreptr->value[j]->mbsf*100.0))-start_depth);
                    coreptr->value[j]->sm_data=sm_datptr[k];
                    coreptr->value[j]->smooth_ok = SMOK;
                  }
                }
              }
              coreptr->smooth_status = SMOK;
            }
          }
        }
      }
    }
    if(sm_datptr)
    {
      free(sm_datptr);
      sm_datptr=NULL;
    }
    if(datptr)
    {
      free(datptr);
      datptr=NULL;
    }
    if(weight)
    {
      free(weight);
      weight=NULL;
    }
  }
}
	
void	SaDecimate(ptr,type,deci_num)
Hole	*ptr;
int	type;
int	deci_num;
{
	int	i,j;
	Core	*coreptr;
        Cursor cursor;

  cursor = XCreateFontCursor(XtDisplay(formMain), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formMain), XtWindow(formMain), cursor);
  }

	for(i=0;i<ptr->numcores; i++)
	{
		coreptr=ptr->core[i];
		for(j=0;j<coreptr->numvalues;j++)
		{
                        if(coreptr->value[j]->quality & DECIMATED) {
                          coreptr->value[j]->quality = coreptr->value[j]->quality ^ DECIMATED;
                        }
			if(j%deci_num!=0)
				coreptr->value[j]->quality=coreptr->value[j]->quality|DECIMATED;
		}
	}

  if(cursor) {
    XUndefineCursor(XtDisplay(formMain), XtWindow(formMain));
    XFreeCursor(XtDisplay(formMain), cursor);
  }
}

int	current_file=0,lineno=0,current_hole_num=0;
char	*nameptr;
FILE	*fptr;

int	SaSmoothLog(client_data)
XtPointer client_data;
{
	int	which_axis;
	LogData	*datptr;
        void DrawDrawingAreaLog();

/* find out which log type is being read */
	XtVaGetValues(formLogInfo,XmNuserData,&which_axis,NULL);
	datptr=logdata[lset];
/* smooth data */
	if(datptr->info.smooth.method==GAUSSIAN)
		SaGaussianLog(datptr);
/* set values smoothed true */
	XtUnmanageChild(messageBoxWorking);
	DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
	return(True);
}

int	SaSmoothCore(client_data)
XtPointer	client_data;
{
	XmString	xmstring;
        void            DrawDrawingAreaCores();

	if(current_hole_num==0)
		have_smooth_cores=DONT;

	if(smooth.method==GAUSSIAN && current_hole_num<data[dset]->numholes)
	{
		SaGaussianHole(data[dset]->holes[current_hole_num]);
		current_hole_num++;
	}

	if(current_hole_num>=data[dset]->numholes)
	{
		have_smooth_cores=DO;
		current_hole_num=0;
		XtUnmanageChild(messageBoxWorking);
		DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
	}
	else
	{
		if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
               		SaBadMemAlloc("SaSmoothCore 1");
                 (void)sprintf(charptr,"\nSmoothed Data hole %c.",data[dset]->holes[current_hole_num]->name);
                 SaUpdateReport(charptr);
                 xmstring=XmStringCreateLocalized(&charptr[1]);
                 XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                 XtResizeWindow(messageBoxWorking);
                 XmUpdateDisplay(messageBoxWorking);
                 XmStringFree(xmstring);
                 free(charptr);
	}

	return(have_smooth_cores);
}

int	stratfileopen=False;

int SaInputStrat(client_data)
XtPointer client_data;
{
	XmString	xmstring;
	StratFile	*stratptr;
	int	        i,j,found=0;
	Core	        *coreptr;
        void            MapStratToAffine(), 
                        MapStratToeld(),
                        MapStratToData(),
                        FindStratDepthOffset(),
                        CalcStratELD(),
                        DrawDrawingAreaCores();

	stratptr=strat[strat_file_cnt-1];
	if(!stratfileopen)
	{
		if ((fptr=fopen(stratptr->filename,"r"))==NULL)
		{
			info.error=True;
			info.fatal=False;
			info.caller=OPENSTRAT;
                        if((info.message=(char*)calloc((int)strlen(stratptr->filename)+100,CHAR_SIZE))==NULL)
                               	SaBadMemAlloc("SaInputstrat 1");
			else
				(void)sprintf(info.message,"Error opening input file %s\n",stratptr->filename);
			SaManageMessage(info);
			return(True);
		}
		else
		{
                        if((charptr=(char*)calloc((int)strlen(stratptr->filename)+100,CHAR_SIZE))==NULL)
                               	SaBadMemAlloc("SaInputstrat 2");
			(void)sprintf(buffer,"\nStrat file %s read.(%s)",stratptr->filename,
				strat_type[stratptr->type]);
			SaUpdateReport(buffer);
			free(charptr);
			stratfileopen=True;
			if((stratptr->data= (StratData*)calloc((stratptr->alloc_cnt=ALLOC_INT),sizeof(StratData)))==NULL)
				SaBadMemAlloc("BAD MEM");
			stratptr->data_cnt=0;
		}
	}
	if(stratptr->data_cnt+1 >= stratptr->alloc_cnt)
	{
                        if((charptr= (char*)realloc(stratptr->data,(stratptr->alloc_cnt+=ALLOC_INT)*sizeof(StratData)))==NULL)
                                SaBadMemAlloc("BAD MEM");
                        else
                                stratptr->data=(StratData*)charptr;
	}
	stratfileopen=SaReadStrat(fptr,&stratptr->data[stratptr->data_cnt]);
	if(stratfileopen)
	{
		if(stratptr->data_cnt % 500 == 0)
		{
                        if((charptr=(char*)calloc((int)strlen(stratptr->filename)+100,CHAR_SIZE))==NULL)
                               	SaBadMemAlloc("SaInputstrat 2");
			(void)sprintf(charptr,"Strat : Reading Strat file %s (line number %d)",stratptr->filename,
					stratptr->data_cnt);
			xmstring=XmStringCreateLocalized(charptr);
			XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                        XtResizeWindow(messageBoxWorking);
                        XmUpdateDisplay(messageBoxWorking);
			XmStringFree(xmstring);
			free(charptr);
		}
		stratptr->data_cnt++;
		return(False);
	}
	else
	{
		if(fptr) fclose(fptr);
		XtUnmanageChild(messageBoxWorking);
		have_strat=DO;
                XtSetSensitive(pushButtonClearStrat, True);
                if(have_log == DO || have_ancill == DO || have_cores == DO) {
                  XtSetSensitive(pushButtonClearAll, True);
                }
                if(have_cores == DO) {
                  if(have_affine == DO) {
                    MapStratToAffine();
                  }
                  MapStratToeld();
                  for(i=0; i<dataset_cnt; ++i) {
                    MapStratToData(i);
                  }
                  FindStratDepthOffset();
                  CalcStratELD();
                  DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
                }
		return(True);
	}
}

int start = YES;

int SaInputLog(client_data)
XtPointer client_data;
{
	int	        i, j, alloc_int,thislog, outoforder, scaleval, err;
        char            *string;
        String          str;
	XmString	xmstring;
	LogValue	*valueptr, **holdlog;
        void            SaCullLog(), SaDecimateLog(), DrawDrawingAreaCores(), DrawDrawingAreaLog(), 
                        ChangeDataSet(), GetTieArrayNum(), SetLogSetOptionMenu(), FindCurrentTieArray(),
                        AutoPair();
        int             CountTies(), CorrelationSetup();

	/* find out which log type is being read */
		XtVaGetValues(formLogInfo,XmNuserData,&thislog,NULL);
	if(!done_reading)
	{
	/* allocate block of data */
		alloc_int=LOG_DATA_ALLOC;
		if(start == YES)
		{
                        start = NO;
                        ++logset_cnt;
                        lset = logset_cnt - 1;
                        if(logset_cnt > 1) {
                           if((charptr= (char*)realloc((char*)logdata,(logset_cnt*sizeof(LogData*))))==NULL) {
                                SaBadMemAlloc("SaInputLog");
                           }
                           else {
                                logdata=(LogData**)charptr;
                           }
                        }
                        else if(logset_cnt == 1) {
                          if((logdata = (LogData**)calloc(logset_cnt, sizeof(LogData*)))==NULL) {
                            SaBadMemAlloc("SaInputLog");
                          }
                        }
                        else {
                          are_reading_log = NO;
                          return(True);
                        }
                        if((logdata[lset] = (LogData*)calloc(1, sizeof(LogData)))==NULL) {
                            SaBadMemAlloc("SaInputLog");
                        }
                        logdata[lset]->data_num=0;
			if((logdata[lset]->value=(LogValue**)calloc((logdata[lset]->data_alloc=alloc_int),sizeof(LogValue*)))==NULL)
				SaBadMemAlloc("SaInputLog 1");
			for(i=0;i<logdata[lset]->data_alloc;i++)
			{
				if((logdata[lset]->value[i]=(LogValue*)malloc(sizeof(LogValue)))==NULL)
					SaBadMemAlloc("SaInputLog 1a");
			}

                        /* assign info to this log set */
                        if((logdata[lset]->info.filename=(char*)calloc((int)strlen(loginfo.filename)+10, CHAR_SIZE)) == NULL) {
                                 SaBadMemAlloc("FileSelectionOK1");
                        }
                        (void)sprintf(logdata[lset]->info.filename, "%s", loginfo.filename);
                        if((logdata[lset]->info.dirname=(char*)calloc((int)strlen(loginfo.dirname)+10, CHAR_SIZE)) == NULL) {
                                 SaBadMemAlloc("FileSelectionOK2");
                        }
                        (void)sprintf(logdata[lset]->info.dirname, "%s", loginfo.dirname);
                        if((logdata[lset]->info.id=(char**)calloc(1, sizeof(char*))) == NULL) {
                                 SaBadMemAlloc("FileSelectionOK2");
                        }
                        if((logdata[lset]->info.id[0]=(char*)calloc(25, sizeof(char))) == NULL) {
                                 SaBadMemAlloc("FileSelectionOK2");
                        }
                        (void)sprintf(logdata[lset]->info.id[0], "%s", loginfo.id[loginfo.data_col]);
                        logdata[lset]->info.format = loginfo.format;
                        logdata[lset]->info.leg = loginfo.leg;
                        logdata[lset]->info.core_data_type = loginfo.core_data_type;
                        logdata[lset]->info.decimate_value = loginfo.decimate_value;
                        logdata[lset]->info.missing_value = loginfo.missing_value;
                        logdata[lset]->info.depth_min = loginfo.depth_min;
                        logdata[lset]->info.depth_max = loginfo.depth_max;
                        logdata[lset]->info.smooth = loginfo.smooth;
                        logdata[lset]->info.cull = loginfo.cull;
                        (void)sprintf(logdata[lset]->info.hole, "%s", loginfo.hole);
                        logdata[lset]->info.fptr = loginfo.fptr;
		}
		else
		{
			if((charptr=(char*)realloc(logdata[lset]->value,(logdata[lset]->data_alloc+alloc_int)*sizeof(LogValue)))==NULL)
				SaBadMemAlloc("SaInputLog 2");
			else
			{
				logdata[lset]->value=(LogValue**)charptr;
				for(i=logdata[lset]->data_alloc,logdata[lset]->data_alloc+=alloc_int;
							i<logdata[lset]->data_alloc;i++)
				{
					if((logdata[lset]->value[i]=(LogValue*)malloc(sizeof(LogValue)))==NULL)
						SaBadMemAlloc("SaInputLog 2a");
				}
			}
		}
	/* read a block of data */
		for(i=0;i<alloc_int;i++)
		{
			if(fgets(buffer,1000,logdata[lset]->info.fptr)==NULL)
			{
				if(thislog==LOG)
				{
					XtSetSensitive(pushButtonOpenStrat,True);
				}
				else
				{
					ancill.data_cnt++;
					if(ancill.data_cnt==1)
						have_anc1=DO;
					else if(ancill.data_cnt==2)
						have_anc2=DO;
					else if(ancill.data_cnt==3)
					{
						have_anc3=DO;
						XtSetSensitive(pushButtonOpenAnc,False);
					}
				}
				done_reading=True;
				fclose(loginfo.fptr);
			}
                        else if((strncmp(buffer,"\n",1) == 0) ||
                                (strcmp(buffer,"") == 0) ||
                                (strncmp(buffer,"                    ",20) == 0)) 
                        {
                        }
			else
			{
				valueptr=logdata[lset]->value[logdata[lset]->data_num];
				valueptr->data=getfloat(buffer,loginfo.data_col);
				valueptr->sm_data=valueptr->data;
				valueptr->depth=getfloat(buffer,0);
                                if(have_cores == DO) {
                                  if(eqlogdepth.did_log_mudline_adj == YES) {
                                    valueptr->depth = valueptr->depth + eqlogdepth.autocorr_log_mudline_offset;
                                  }
                                }
                                if(valueptr->data == logdata[lset]->info.missing_value) {
		  		  valueptr->quality=BADLOG;
                                }
                                else {
                                  valueptr->quality=GOOD;
                                }
				logdata[lset]->data_num++;
			}
		}
	}

			
	if(done_reading)
	{
                start = YES;

                /* is log depth in descending order? or reverse order?
                   or out of order
                */
                outoforder = 0;
                if(logdata[lset]->value[0]->depth > logdata[lset]->value[logdata[lset]->data_num-1]->depth) {
                  for(i=0; i<logdata[lset]->data_num-1; ++i) {
                    if(logdata[lset]->value[i]->depth < logdata[lset]->value[i+1]->depth) {
                      ++outoforder;
                      logdata[lset]->value[i]->quality = BADLOG;
                    }
                  }
                }
                else if(logdata[lset]->value[0]->depth < logdata[lset]->value[logdata[lset]->data_num-1]->depth) {
                  for(i=0; i<logdata[lset]->data_num-1; ++i) {
                    if(logdata[lset]->value[i]->depth > logdata[lset]->value[i+1]->depth) {
                      ++outoforder;
                      logdata[lset]->value[i]->quality = BADLOG;
                    }
                  }
                }
                /* too many out of order
                */
                if(outoforder > logdata[lset]->data_num/2) {
                  (void)sprintf(buffer, "Error; found too many log depths out of order.");
                  SaManageWarningBox(buffer);
                  SaCleanUp(LOGDATA);
                  --logset_cnt;
                  lset = logset_cnt - 1;
                  are_reading_log = NO;
                  return(True);
                }
                /* depth in reverse order; change it to descending order
                */
                else if(logdata[lset]->value[0]->depth > logdata[lset]->value[logdata[lset]->data_num-1]->depth) {
                  if((holdlog=(LogValue**)calloc(logdata[lset]->data_num, sizeof(LogValue*)))==NULL)
                  {
                                SaBadMemAlloc("SaInputLog");
                  }
                  else
                  {
                    for(i=0; i<logdata[lset]->data_num; ++i) 
                    {
                      if((holdlog[i]=(LogValue*)malloc(sizeof(LogValue)))==NULL)
                      {
                                           SaBadMemAlloc("SaInputLog 2a");
                      }
                      else 
                      {
                        holdlog[i]->depth = logdata[lset]->value[i]->depth;
                        holdlog[i]->data = logdata[lset]->value[i]->data;
                        holdlog[i]->sm_data = logdata[lset]->value[i]->sm_data;
                        holdlog[i]->quality = logdata[lset]->value[i]->quality;
                      }
                    }
                  }
                  j = logdata[lset]->data_num - 1;
                  for(i=0; i<logdata[lset]->data_num; ++i) {
                    logdata[lset]->value[i]->depth = holdlog[j]->depth;
                    logdata[lset]->value[i]->data = holdlog[j]->data;
                    logdata[lset]->value[i]->sm_data = holdlog[j]->sm_data;
                    logdata[lset]->value[i]->quality = holdlog[j]->quality;
                    --j;
                  }
                  for(i=0; i<logdata[lset]->data_num; ++i) {
                    free(holdlog[i]);
                    holdlog[i] = NULL;
                  }
                  free(holdlog);
                  holdlog = NULL;
                }
                 
                have_log=DO;
                logdata[lset]->flip = NO;
                XmToggleButtonSetState(toggleButtonLogAxisFlipNo, True, False);
                XmToggleButtonSetState(toggleButtonLogAxisFlipYes, False, False);
                XtSetSensitive(pushButtonClearLog, True);
                XtSetSensitive(cascadeButtonProcessLog, True);
                if(logset_cnt >= NDATASET) {
                  XtSetSensitive(pushButtonOpenLog, False);
                }
                if(have_cores == DO || have_ancill == DO || have_strat == DO) {
                  XtSetSensitive(pushButtonClearAll, True);
                }

                if(logdata[lset]->info.cull.method == CULL_PARAMETERS) {
                  if((charptr=(char*)calloc((int)strlen(logtype[thislog])+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaInputLog Cull");
                  (void)sprintf(charptr,"\nCulling Log %s data.",logtype[thislog]);
                  SaUpdateReport(charptr);
                  xmstring=XmStringCreateLocalized(&charptr[1]);
                  XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                  XtResizeWindow(messageBoxWorking);
                  XmStringFree(xmstring);
                  free(charptr);
                  XmUpdateDisplay(messageBoxWorking);
                  SaCullLog(logdata[lset]);
                }
                if(logdata[lset]->info.smooth.method == GAUSSIAN) {
                  if((charptr=(char*)calloc((int)strlen(logtype[thislog])+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaInputLog Cull");
                  (void)sprintf(charptr,"\nSmoothing Log %s data.",logtype[thislog]);
                  SaUpdateReport(charptr);
                  xmstring=XmStringCreateLocalized(&charptr[1]);
                  XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                  XtResizeWindow(messageBoxWorking);
                  XmStringFree(xmstring);
                  free(charptr);
                  XmUpdateDisplay(messageBoxWorking);
                  SaGaussianLog(logdata[lset]);
                }
                if(logdata[lset]->info.decimate_value > 1) {
                  if((charptr=(char*)calloc((int)strlen(logtype[thislog])+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaInputLog Decimate");
                  (void)sprintf(charptr,"\nDecimating Log %s data.",logtype[thislog]);
                  SaUpdateReport(charptr);
                  xmstring=XmStringCreateLocalized(&charptr[1]);
                  XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                  XtResizeWindow(messageBoxWorking);
                  XmStringFree(xmstring);
                  free(charptr);
                  XmUpdateDisplay(messageBoxWorking);
                  SaDecimateLog(logdata[lset]);
                }
                if((logdata[lset]->good=(int*)calloc(logdata[lset]->data_num,sizeof(int))) == NULL)
                                                SaBadMemAlloc("SaInputDatA");
                logdata[lset]->numgood = 0;
                for(i=0; i<logdata[lset]->data_num; ++i) {
                  if(logdata[lset]->value[i]->quality == GOOD) {
                    logdata[lset]->good[logdata[lset]->numgood] = i;
                    ++logdata[lset]->numgood;
                  }
                }

                FindLogMinMax(logdata[lset]);
                SetLogMinMax(logdata[lset]);
                if(logset_cnt == 1) {
                  scaleval = logdata[lset]->info.depth_min;
                  XtVaSetValues(scaleDrawingAreas, XmNvalue, scaleval, NULL);
                }
 
                if(have_coretotie == DO && ties->nties > 0) {
                  for(i=0; i<ties->nties; ++i) {
                    GetTieArrayNum(LOGDATA, ties->tie[i], lset, 0, 0, &err);
                  }
                  XtSetSensitive(pushButtonAutoCorrelate, False);
                }
                else if(have_splice == DO && ties->nties > 1) {
                  ties->tie[0]->log = -1;
                  ties->tie[0]->array = -1;
                  for(i=1; i<ties->nties; ++i) {
                    GetTieArrayNum(LOGDATA, ties->tie[i], lset, 0, 0, &err);
                  }
                  XtSetSensitive(pushButtonAutoCorrelate, False);
                }
                else {
                  XtSetSensitive(pushButtonAutoCorrelate, True);
                }
                if(have_tie2 == DO) {
                  FindCurrentTieArray(LOGDATA, lset);
                }
                if(have_tie1 == DO && have_tie2 == DO && ties->coretie != MISSING && ties->logtie != MISSING) {
                  if(err = CorrelationSetup(dset) != 0) {
                    have_correlate = DONT;
                    if(err == 2) {
                      (void)sprintf(buffer, "Error; correlation, new data set is missing tie pt, could not interpolate.\n");
                    }
                    else {
                      (void)sprintf(buffer, "Error doing correlation.\n");
                    }
                    SaManageWarningBox(buffer);
                  }
                  else {
                    have_correlate = DO;
                  }
                }

                if(have_tie1 == DONT && have_tie2 == DONT && ties->nties < 1 && have_cores == DO) {
                  if(err = CountTies() > 0) {
                    XtSetSensitive(pushButtonAutoCorrelate, False);
                  }
                  else {
                    XtSetSensitive(pushButtonAutoCorrelate, True);
                  }
                }
                else if(have_cores == DONT) {
                  XtSetSensitive(pushButtonAutoCorrelate, False);
                }

                if(have_coretotie == DO || (have_splice == DO && sset >= 0 && sset < splice_cnt)) {

                }
                else if(have_splice == DO && (sset < 0 || sset >= splice_cnt)) {
                  XtSetSensitive(formEquivLogDepthTool, False);
                }
                else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
                  XtSetSensitive(formEquivLogDepthTool, True);
                  XtSetSensitive(textFieldSelectHole, False);
                  XtSetSensitive(textFieldSelectCore, False);
                  XtSetSensitive(pushButtonSelectCore, False);
                  XtSetSensitive(pushButtonCancelCore, False);
                  XtSetSensitive(pushButtonClearTie, False);
                  XtSetSensitive(pushButtonClearLastTieSaved, False);
                  XtSetSensitive(pushButtonShowHideTieList, True);
                  XtSetSensitive(pushButtonAddTieToList, False);
                  XtSetSensitive(toggleButtonAddTieToCore, False);
                  XtSetSensitive(toggleButtonAddTieToAll, False);
                  XtSetSensitive(pushButtonOptimizeTie, True);
                  XtSetSensitive(pushButtonAdjustToTiesParameters, False);
                  XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
                  XtSetSensitive(pushButtonSaveChanges, False);
                  XtSetSensitive(pushButtonUndoSaveChange, False);
                  XtSetSensitive(pushButtonEvalCoreLogData, True);
                }
                else if(have_cores == DO) {
                  XtSetSensitive(formEquivLogDepthTool, True);
                  XtSetSensitive(textFieldSelectHole, True);
                  XtSetSensitive(textFieldSelectCore, True);
                  XtSetSensitive(pushButtonSelectCore, True);
                  XtSetSensitive(pushButtonCancelCore, False);
                  XtSetSensitive(pushButtonClearTie, False);
                  XtSetSensitive(pushButtonClearLastTieSaved, False);
                  XtSetSensitive(pushButtonShowHideTieList, False);
                  XtSetSensitive(pushButtonAddTieToList, False);
                  XtSetSensitive(toggleButtonAddTieToCore, False);
                  XtSetSensitive(toggleButtonAddTieToAll, False);
                  XtSetSensitive(pushButtonOptimizeTie, False);
                  XtSetSensitive(pushButtonAdjustToTiesParameters, False);
                  XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
                  XtSetSensitive(pushButtonSaveChanges, False);
                  XtSetSensitive(pushButtonUndoSaveChange, False);
                  XtSetSensitive(pushButtonEvalCoreLogData, True);
                }

                if(logdata[lset]->info.smooth.method == NONE) {
                  XtSetSensitive(toggleButtonDisplayLogUnsmoothed, True);
                  XtSetSensitive(toggleButtonDisplayLogSmooth, False);
                  XtSetSensitive(toggleButtonDisplayLogBoth, False);
                }
                else {
                  XtSetSensitive(toggleButtonDisplayLogUnsmoothed, True);
                  XtSetSensitive(toggleButtonDisplayLogSmooth, True);
                  XtSetSensitive(toggleButtonDisplayLogBoth, True);
                }

                SetLogSetOptionMenu(lset);

                /* map log type to data type
                */
                logdata[lset]->info.maptodata = -1;
                if(have_cores == DO) {
                  for(i=0; i<dataset_cnt; ++i) {
                    if(logdata[lset]->info.core_data_type == data[i]->type) {
                      logdata[lset]->info.maptodata = i;
                      data[i]->maptolog = lset;
                      break;
                    }
                  }
                  err = 0;
                  for(i=0; i<logset_cnt; ++i) {
                    if(logdata[i]->info.maptodata == -1) {
                      ++err;
                    }
                  }
                  if(err < logset_cnt) {
                    XtSetSensitive(pushButtonPairCoreLog, True);
                    XtSetSensitive(labelAutoPair, True);
                    if(autopair == YES) {
                    }
                    else {
                      autopair = NO;
                    }
                  }
                  else {
                    autopair = NO;
                    XtSetSensitive(pushButtonPairCoreLog, False);
                    XtSetSensitive(labelAutoPair, False);
                  }
                }
                else {
                  autopair = NO;
                  XtSetSensitive(pushButtonPairCoreLog, False);
                  XtSetSensitive(labelAutoPair, False);
                }
                AutoPair(autopair);

                are_reading_log = NO;
                DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
                DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);

		done_reading=False;
		XtUnmanageChild(messageBoxWorking);
		return(True);
	}
	else
	{
		if((charptr=(char*)calloc((int)strlen(logdata[lset]->info.filename)+100,CHAR_SIZE))==NULL)
			SaBadMemAlloc("SaInputLog 5");
		(void)sprintf(charptr,"%s: (line %d)",logdata[lset]->info.filename,logdata[lset]->data_num);
		xmstring=XmStringCreateLocalized(&charptr[1]);
		XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                XtResizeWindow(messageBoxWorking);
                XmUpdateDisplay(messageBoxWorking);
		XmStringFree(xmstring);
		free(charptr);
		return(False);
	}
}
	
int SaInputData(client_data)
XtPointer client_data;
{
	int	        i,j,k, n, hashflag, len, col, skip, err, numhole, foundit;
        char            *string;
        String          data_col, str;
	Boolean	        switched=False;
	Hole	        *tmphole;
	XmString	xmstring;
	XmFontList	fontlist;
        XEvent          event;
        void            MapStratToData(), 
                        MapStratToeld(),
                        MakeNewELDTable(),
                        CheckELDTableMissingHole(),
                        MapHoleToELDTable(),
                        CheckELDTableMissingCore(),
                        MapCoreToELDTable(),
                        SetupCalcEquivLogDepths(),
                        MapHoleToAffineTable(),
                        MapCoreToAffineTable(),
                        MapStratToAffine(),
                        MakeNewAffineTable(),
                        CheckAffineTableMissingCore(),
                        CheckAffineTableMissingHole(),
                        SetupMakingSpliceSeries(),
                        FindCurrentTieArray(),
                        GetTieArrayNum(),
                        CancelCore(),
                        AdjustToTies(),
                        ChangeLogSet(),
                        AutoPair(),
                        SetDataSetOptionMenu(),
                        DrawDrawingAreaCores(), DrawDrawingAreaLog();
        int             SaReadAffineTable(),
                        CorrelationSetup(),
                        CheckDepthInDescendingOrder();
        float           CalcMbsfMcdRatio();

        XtVaGetValues(messageBoxWorking,XmNlabelFontList,&fontlist,NULL);

	if(current_file==0 && !done_reading)
	{
                are_reading_dataset = YES;
		done_reading=False;
		if(decimate_value>1)
			done_decimate=False;
		else
			done_decimate=True;
		if(cull.method!=NONE || cull.top>0 || cull.by_core_num)
			done_cull=False;
		else
			done_cull=True;
                if(smooth.method == GAUSSIAN) {
                  done_smooth = False;
                }
                else {
                  done_smooth = True;
                }
		current_hole_num=0;

                /* if data format is ODP Other get the column number for data value
                */
                if(openfileinfo.in_data_format == ODPOTHER || openfileinfo.in_data_format == ODPOTHER1 ||
                   openfileinfo.in_data_format == ODPOTHER2 || openfileinfo.in_data_format == ODPOTHER3 ||
                   openfileinfo.in_data_format == ODPOTHER4 || openfileinfo.in_data_format == ODPOTHER5)
                {
                  data_col = XmTextGetString(textFieldODPOtherFormatValueCol);
                  openfileinfo.other_format_value_col = atoi(data_col);
                  XtFree(data_col);

                  /* make the format. note that it is dimensioned in sagan_defs.h to [100]
                  */
                  (void)sprintf(openfileinfo.other_format, "");
                  switch (openfileinfo.in_data_format)
                  {
                       case    ODPOTHER:
                       case    ODPOTHER1:
                               (void)strcpy(openfileinfo.other_format, "%d %d %c %d %c %s %f %f %f");
                               col = 9;
                               break;
                       case    ODPOTHER2:
                               (void)strcpy(openfileinfo.other_format, "%d %c %d %c %s %f %f %f");
                               col = 8;
                               break;
                       case    ODPOTHER3:
                               (void)strcpy(openfileinfo.other_format, "%d %c %d %s %f %f %f");
                               col = 7;
                               break;
                       case    ODPOTHER4:
                               (void)strcpy(openfileinfo.other_format, "%d %c %d %s %f %f");
                               col = 6;
                               break;
                       case    ODPOTHER5:
                               (void)strcpy(openfileinfo.other_format, "%d %d %c %d %c %s %f %f");
                               col = 8;
                               break;
                       default:
                               info.error=True;
                               info.fatal=False;
                               if((info.message=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData");
                               (void)sprintf(info.message,"Could not make the format for ODP other; no match for format type.\n");
                               info.type=XmDIALOG_ERROR;
                               info.caller=NONE;
                               info.default_button=XmDIALOG_OK_BUTTON;
                               current_file=lineno=current_hole_num=0;
                               done_decimate=done_cull=done_reading=False;
                               XtUnmanageChild(messageBoxWorking);
                               SaManageMessage(info);
                               return(True);
                  }
                  if((skip = openfileinfo.other_format_value_col - col - 1) <= 0) {
                  }
                  else {
                    for(i=1; i<=skip; ++i) {
                      (void)strcat(openfileinfo.other_format, " %*s");
                    }
                  }
                  (void)strcat(openfileinfo.other_format, " %f");
                }

                /* allocate space for data set
                */
                ++dataset_cnt;
                dset = dataset_cnt - 1;
                if(dataset_cnt > 1) {
                   if((charptr= (char*)realloc((char*)data,(dataset_cnt*sizeof(DataFile*))))==NULL) {
                                SaBadMemAlloc("SaInputData");
                   }
                   else {
                                data=(DataFile**)charptr;
                   }
                }
                else if(dataset_cnt == 1) {
                  if((data = (DataFile**)calloc(dataset_cnt, sizeof(DataFile*)))==NULL) {
                    SaBadMemAlloc("SaInputData");
                  }
                }
                if((data[dset] = (DataFile*)calloc(1, sizeof(DataFile)))==NULL) {
                  SaBadMemAlloc("SaInputData");
                }

                data[dset]->in_format = openfileinfo.in_data_format;
                data[dset]->type = openfileinfo.in_data_type;
                data[dset]->numfiles = openfileinfo.num_in_files;
                data[dset]->readerrors=0;
                data[dset]->decimate_value = decimate_value;
                if((data[dset]->name=(char**)calloc(openfileinfo.num_in_files,sizeof(char*)))==NULL) {
                  SaBadMemAlloc("SaInputData");
                }
                for(i=0; i<openfileinfo.num_in_files; ++i) {
                  if((data[dset]->name[i]=(char*)calloc(strlen(openfileinfo.in_file_names[i])+5,CHAR_SIZE))==NULL) {
                    SaBadMemAlloc("SaInputData");
                  }
                  (void)sprintf(data[dset]->name[i], "%s", openfileinfo.in_file_names[i]);
                  free(openfileinfo.in_file_names[i]);
                  openfileinfo.in_file_names[i] = NULL;
                }

                current_hole_num=0;
                data[dset]->holes=NULL;
	}
/* if done reading, process data */
	else if(done_reading && !info.error)
	{
		if(current_hole_num<data[dset]->numholes)
		{
 
                        /* check that depth is in descending order
                        */
                        err = CheckDepthInDescendingOrder(dset);
                        if(err == YES) {
                          if(dataset_cnt == 1) {
                            SaCleanUp(COREDATA);
                          }
                          else if(dataset_cnt > 1) {
                            SaFreeData(data[dset]);
                            if(data[dset])
                            {
                              free(data[dset]);
                              data[dset]=NULL;
                            }
                            --dataset_cnt;
                            dset = dataset_cnt - 1;
                          }
                          current_file=lineno=current_hole_num=0;
                          done_decimate=done_cull=done_reading=False;
                          XtUnmanageChild(messageBoxWorking);
                          return(True);
                        }

			i=current_hole_num;
			if(!done_cull)
			{
				if(cull.method==CULL_TABLE || cull.method == CULL_PARAMETER_AND_TABLE)
				{
					if(i==0 && hashflag!=CREATE_HASH)
						hashflag=CREATE_HASH;
					else if(i+1==data[dset]->numholes)
					 	hashflag=DESTROY_HASH;
					else
						hashflag=0;
				}
                                if((charptr=(char*)calloc(500,CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("SaInputData 1");
                                (void)sprintf(charptr,"\nData : Culling data hole %c.",data[dset]->holes[i]->name);
                                xmstring=XmStringCreateLocalized(&charptr[1]);
                                XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring,
                                              XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                                XtResizeWindow(messageBoxWorking);
                                XmUpdateDisplay(messageBoxWorking);
                                XmStringFree(xmstring);
                                free(charptr);
                                cull.cnt = 0;
				SaCull(dset, data[dset]->holes[i],data[dset]->type,cull,hashflag);
                                n = 0;
                                for(j=0; j<data[dset]->holes[i]->numcores; ++j) {
                                  n = n + data[dset]->holes[i]->core[j]->numvalues; 
                                }
                                if(cull.cnt > (err = n * TOOMANYCULLED)) {
                                  if(dataset_cnt == 1) {
                                    SaCleanUp(COREDATA);
                                  }
                                  else if(dataset_cnt > 1) {
                                    SaFreeData(data[dset]);
                                    if(data[dset])
                                    {
                                      free(data[dset]);
                                      data[dset]=NULL;
                                    }
                                    --dataset_cnt;
                                    dset = dataset_cnt - 1;
                                  }
                                  (void)sprintf(buffer, "Error; Too many values culled. Check cull parameters and data type and reread.");
                                  SaManageWarningBox(buffer);
                                  current_file=lineno=current_hole_num=0;
                                  done_decimate=done_cull=done_reading=False;
                                  XtUnmanageChild(messageBoxWorking);
                                  return(True);
                                }
				if(i+1==data[dset]->numholes)
				{
					done_cull=True;
					current_hole_num=-1;
					if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
						SaBadMemAlloc("SaInputData 1");
					(void)sprintf(charptr,"\nData : Done culling data hole %c.",data[dset]->holes[i]->name);
				}
				else
				{
					if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
						SaBadMemAlloc("SaInputData 1");
					(void)sprintf(charptr,"\nData : Done culling data hole %c.",data[dset]->holes[i]->name);
				}
				SaUpdateReport(charptr);
				xmstring=XmStringCreateLocalized(&charptr[1]);
				XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, 
                                              XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                                XtResizeWindow(messageBoxWorking);
                                XmUpdateDisplay(messageBoxWorking);
				XmStringFree(xmstring);
				free(charptr);
			}
                        else if(!done_smooth)
                        {
                                while(XCheckMaskEvent(XtDisplay(messageBoxWorking), ButtonPressMask, &event)) {
                                  if(event.xany.window == XtWindow(messageBoxWorking)) {
                                    XtDispatchEvent(&event);
                                    smooth.method = NONE;
                                    smooth.plot = UNSMOOTH;
                                  }
                                }
                                if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData");
                                (void)sprintf(charptr,"\nData : Smoothing data hole %c.",data[dset]->holes[i]->name);
                                SaUpdateReport(charptr);
                                xmstring=XmStringCreateLocalized(&charptr[1]);
                                XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, 
                                              XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                                XmStringFree(xmstring);
                                free(charptr);
                                XtResizeWindow(messageBoxWorking);
                                XmUpdateDisplay(messageBoxWorking);
                                if(smooth.method == GAUSSIAN) {
                                  SaGaussianHole(data[dset]->holes[i]);
                                }
                                else {
                                }
                                if(i+1==data[dset]->numholes || smooth.method == NONE)
                                {
                                        current_hole_num=-1;
                                        done_smooth=True;
                                }
                        }
			else if(!done_decimate)
			{
                                if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData 3");
				(void)sprintf(charptr,"\nData : Decimating data hole %c.",data[dset]->holes[i]->name);
				SaUpdateReport(charptr);
				xmstring=XmStringCreateLocalized(&charptr[1]);
				XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, 
                                              XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                                XtResizeWindow(messageBoxWorking);
                                XmUpdateDisplay(messageBoxWorking);
				XmStringFree(xmstring);
				free(charptr);
				if(decimate_value>1)
					SaDecimate(data[dset]->holes[i],data[dset]->type,decimate_value);
				if(i+1==data[dset]->numholes || decimate_value<=1)
				{
					current_hole_num=-1;
					done_decimate=True;
				}
			}
			
		current_hole_num++;
		}
/* an error occurred somewhere... Close working box and clean up */
		if(info.error)
		{
                        if(dataset_cnt == 1) {
			  SaCleanUp(COREDATA);
                        }
                        else if(dataset_cnt > 1) {
                          SaFreeData(data[dset]);
                          if(data[dset])
                          {
                            free(data[dset]);
                            data[dset]=NULL;
                          }
                          --dataset_cnt;
                          dset = dataset_cnt - 1;
                        }
			current_file=lineno=current_hole_num=0;
                        done_decimate=done_cull=done_reading=False;
			XtUnmanageChild(messageBoxWorking);
			info.error=False;
                        if((info.message=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData");
                        (void)sprintf(info.message,"Unknown read error has occured. Can't recover.\n");
                        info.type=XmDIALOG_ERROR;
                        info.caller=NONE;
                        info.default_button=XmDIALOG_OK_BUTTON;
                        SaManageMessage(info);
			return(True);
		}
		/* done reading and processing data */
		if(done_decimate&&done_smooth&&done_cull&&done_reading)
		{
                        if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData 3");
                        (void)sprintf(charptr,"\nData: Done processing. Setting up.");
                        SaUpdateReport(charptr);
                        xmstring=XmStringCreateLocalized(&charptr[1]);
                        XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, 
                                      XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                        XtResizeWindow(messageBoxWorking);
                        XmUpdateDisplay(messageBoxWorking);
                        XmStringFree(xmstring);
                        free(charptr);

			done_decimate=done_cull=done_reading=False;
			XtSetSensitive(pushButtonAccessDatabase,FALSE);

                        have_cores = DO;
                        XmListDeleteAllItems(listOpenSelected);
			current_hole_num=0;
                        if(cull.method!=NONE) {
                          data[dset]->culled = YES;
                        }
                        else {
                          data[dset]->culled = YES;
                        }

                        PutHoleNamesInList(dset);
                        /* set some initial values in core and tielist structs dealing with ties
                        */
                        data[dset]->flip = NO;
                        XmToggleButtonSetState(toggleButtonVarAxisFlipNo, True, False);
                        XmToggleButtonSetState(toggleButtonVarAxisFlipYes, False, False);
                        XmToggleButtonSetState(toggleButtonAutoCorrelateFlipYes, False, False);
                        XmToggleButtonSetState(toggleButtonAutoCorrelateFlipNo, True, False);
                        for(i=0; i<data[dset]->numholes; ++i) {
                          data[dset]->holes[i]->map_hole_to_eqlogdepth = -1;
                          if(data[dset]->holes[i]->core[data[dset]->holes[i]->numcores-1]->num > data[dset]->holes[i]->numcores) {
                            if(data[dset]->holes[i]->core[data[dset]->holes[i]->numcores-1]->num > max_num_cores) {
                              max_num_cores = data[dset]->holes[i]->core[data[dset]->holes[i]->numcores-1]->num;
                            }
                          }
                          else {
                            if(data[dset]->holes[i]->numcores > max_num_cores) {
                              max_num_cores = data[dset]->holes[i]->numcores;
                            }
                          }
                          for(j=0; j<data[dset]->holes[i]->numcores; ++j) {
                            data[dset]->holes[i]->core[j]->have_equil_log_depths = DONT;
                            data[dset]->holes[i]->core[j]->map_core_to_eqlogdepth = -1;
                            data[dset]->holes[i]->core[j]->have_tie_list = DONT;
                            data[dset]->holes[i]->core[j]->affine_offset = 0.0;
                       
                            if((data[dset]->holes[i]->core[j]->good=(int*)calloc(data[dset]->holes[i]->core[j]->numvalues,sizeof(int))) == NULL)
                                                SaBadMemAlloc("SaInputDatA");
                            data[dset]->holes[i]->core[j]->numgood = 0;
                            for(k=0; k<data[dset]->holes[i]->core[j]->numvalues; ++k) {
                              if(data[dset]->holes[i]->core[j]->value[k]->quality == GOOD) {
                                data[dset]->holes[i]->core[j]->good[data[dset]->holes[i]->core[j]->numgood] = k;
                                ++data[dset]->holes[i]->core[j]->numgood;
                              }
                            }
                          }
                        }

                        /* read the affine table
                        */
                        if(affinetable.read_affine == NO && dataset_cnt == 1) {
                        }
                        else if(dataset_cnt == 1 && affinetable.read_affine == YES) {

                          if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData 3");
                          (void)sprintf(charptr,"\nData: Reading Affine table.");
                          SaUpdateReport(charptr);
                          xmstring=XmStringCreateLocalized(&charptr[1]);
                          XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, 
                                        XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                          XtResizeWindow(messageBoxWorking);
                          XmUpdateDisplay(messageBoxWorking);
                          XmStringFree(xmstring);
                          free(charptr);

                          err = SaReadAffineTable(affinetable.filename, dset);
                          if(err == False) {
                            have_affine = DO;
                            data[dset]->mbsf_mcd_ratio = CalcMbsfMcdRatio(dset);
                            if(have_strat == DO) {
                              MapStratToAffine();
                            }
                          }
                          else {
                            have_affine = DONT;
                          }
                        }
                        else if(dataset_cnt > 1 && have_affine == DO) {
                          CheckAffineTableMissingHole(&affinetable, dset);
                          MapHoleToAffineTable(&affinetable, dset);
                          CheckAffineTableMissingCore(&affinetable, dset);
                          MapCoreToAffineTable(&affinetable, dset);
                          for(i=0; i<dset; ++i) {
                            MapHoleToAffineTable(&affinetable, i);
                            MapCoreToAffineTable(&affinetable, i);
                            data[i]->mbsf_mcd_ratio = CalcMbsfMcdRatio(i);
                          }
                        }
                        else {
                        }
                        if(have_affine == DO) {
                          XmToggleButtonSetState(toggleButtonAddTieToCore, False, False);
                          XmToggleButtonSetState(toggleButtonAddTieToAll, True, False);
                        }
                        else {
                          XmToggleButtonSetState(toggleButtonAddTieToCore, True, False);
                          XmToggleButtonSetState(toggleButtonAddTieToAll, False, False);
                        }
                        /* disallow affine read after first dataset has been read
                        */
                        if(dataset_cnt == 1) {
                          XtSetSensitive(pushButtonOpenAffine, False);
                        }

                        /* if are reading a eqlog table read it and map data to it
                        */
                        if(dataset_cnt == 1 && eqlogdepth.read_equil_log == YES) {
                          
                          if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData 3");
                          (void)sprintf(charptr,"\nData: Reading ELD table.");
                          SaUpdateReport(charptr);
                          xmstring=XmStringCreateLocalized(&charptr[1]);
                          XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, 
                                        XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                          XtResizeWindow(messageBoxWorking);
                          XmUpdateDisplay(messageBoxWorking);
                          XmStringFree(xmstring);
                          free(charptr);

                          err = ReadEquivLogDepthTable();
                          /* check eqlogdepth table for missing holes, cores
                             then map the data to the table
                          */
                          if(err == 0) {
                            CheckELDTableMissingHole(&eqlogdepth, dset);
                            MapHoleToELDTable(&eqlogdepth, dset);
                            CheckELDTableMissingCore(&eqlogdepth, dset);
                            MapCoreToELDTable(&eqlogdepth, dset);
                            if(have_strat == DO) {
                              MapStratToeld();
                            }

                            /* calculate the equiv log depths
                            */
                            SetupCalcEquivLogDepths(dset);

                            /* if have log data and got a mud line adj. from eld table
                               add the adjustment to log(ancill) depth
                            */
                            if(have_log == DO && eqlogdepth.did_log_mudline_adj == YES) {
                              for(j=0; j<logset_cnt; ++j) {
                                for(i=0; i<logdata[lset]->data_num; ++i) {
                                  logdata[lset]->value[i]->depth = logdata[lset]->value[i]->depth + eqlogdepth.autocorr_log_mudline_offset;
                                }
                              }
                            }
                            if(have_ancill == DO && eqlogdepth.did_log_mudline_adj == YES) {
                              for(i=0; i<ancill.data_num; ++i) {
                                ancill.value[i]->depth = ancill.value[i]->depth + eqlogdepth.autocorr_log_mudline_offset;
                              }
                            }
                            XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
                            XtSetSensitive(pushButtonSaveCoreData, True);
                          }
                          else if(err > 0) {
                            eqlogdepth.read_equil_log = NO;
                            XtSetSensitive(pushButtonSaveCoreData, False);
                          }
                          else {
                            (void)printf("\n");
                          }
                        }
                        /* if aren't reading a eqlog table and this is the first data set
                           make a new table. if have an affine table base the ELD Table on it.
                           otherwise use this first data set.
                        */
                        else if(dataset_cnt == 1 && eqlogdepth.read_equil_log == NO) {
                          if(have_affine == DO) {
                            numhole = affinetable.numhole;
                            eqlogdep_core_alloc = 0;
                            for(i=0; i<affinetable.numhole; ++i) {
                              if(affinetable.hole[i]->numcore > eqlogdep_core_alloc) {
                                eqlogdep_core_alloc = affinetable.hole[i]->numcore;
                              }
                            }
                            if(numhole < 1 || eqlogdep_core_alloc < 1) {
                              SaManageWarningBox("Something is very wrong with affine table. Please Exit Sagan.");
                              return;
                            } 
                          }
                          else {
                            numhole = data[dset]->numholes;
                            eqlogdep_core_alloc = max_num_cores;
                          }
                          if((eqlogdepth.hole=(EqLogDepthHole**)calloc(numhole,sizeof(EqLogDepthHole*)))==NULL) {
                                        SaBadMemAlloc("Sa");
                          }
                          for(i=0; i<numhole; ++i) {
                            if((eqlogdepth.hole[i]=(EqLogDepthHole*)calloc(1,sizeof(EqLogDepthHole)))==NULL) {
                                        SaBadMemAlloc("Sa");
                            }
                          }
                          for(i=0; i<numhole; ++i) {
                            if((eqlogdepth.hole[i]->core=(EqLogDepthCore**)calloc(eqlogdep_core_alloc,sizeof(EqLogDepthCore*)))==NULL) {
                                        SaBadMemAlloc("Sa"); 
                            }
                            for(j=0; j<eqlogdep_core_alloc; ++j) {
                              if((eqlogdepth.hole[i]->core[j]=(EqLogDepthCore*)calloc(1,sizeof(EqLogDepthCore)))==NULL) {
                                        SaBadMemAlloc("Sa");
                              }
                              eqlogdepth.hole[i]->core[j]->nties = 0;
                            }
                          }
                          for(i=0;i<data[dset]->numholes; ++i) {
                            for(j=0; j<data[dset]->holes[i]->numcores; ++j) {
                              for(k=0; k< data[dset]->holes[i]->core[j]->numvalues; ++k) {
                                data[dset]->holes[i]->core[j]->value[k]->eld = data[dset]->holes[i]->core[j]->value[k]->mbsf;
                                if(have_affine == DO) {
                                  data[dset]->holes[i]->core[j]->value[k]->eld = data[dset]->holes[i]->core[j]->value[k]->eld +
                                                                                             data[dset]->holes[i]->core[j]->affine_offset;
                                }
                              }
                            }
                          }
                          MakeNewELDTable(&eqlogdepth, dset);
                          MapHoleToELDTable(&eqlogdepth, dset);
                          MapCoreToELDTable(&eqlogdepth, dset);
                          eld.nholes = eqlogdepth.numhole;
                          if((eld.hole_eqt = (HoleELDEqt**)calloc(eld.nholes, sizeof(HoleELDEqt*)))==NULL) {
                                           SaBadMemAlloc("SaInputData");
                          }
                          for(i=0; i<eld.nholes; ++i) {
                            /* allocate mem for hole equation
                            */
                            if((eld.hole_eqt[i] = (HoleELDEqt*)calloc(1, sizeof(HoleELDEqt)))==NULL) {
                                    SaBadMemAlloc("MakeHoleEqt");
                            }
                            eld.hole_eqt[i]->nties = 0;
                          }
                          if(have_strat == DO) {
                            MapStratToeld();
                          }
                          if(have_affine == DO) {
                            eqlogdepth.affine_status = YESAFFINE;
                            if((eqlogdepth.affine_filename=(char*)calloc((int)strlen(affinetable.filename)+10, CHAR_SIZE)) == NULL) {
                                             SaBadMemAlloc("Sa");
                            }
                            (void)sprintf(eqlogdepth.affine_filename, "%s", affinetable.filename);
                          }
                          else {
                            eqlogdepth.affine_status = NOAFFINE;
                          }
                          eqlogdepth.did_log_mudline_adj = NO;
                          eqlogdepth.autocorr_log_mudline_offset = 0.0;
                        }
                        /* if this is not the first data set map it to the eqlog table checking for missing holes and cores
                        */
                        else if(dataset_cnt > 1) {
                          CheckELDTableMissingHole(&eqlogdepth, dset);
                          MapHoleToELDTable(&eqlogdepth, dset);
                          CheckELDTableMissingCore(&eqlogdepth, dset);
                          for(i=0; i<dataset_cnt; ++i) {
                            MapHoleToELDTable(&eqlogdepth, i);
                            MapCoreToELDTable(&eqlogdepth, i);
                          }
                          if(have_strat == DO) {
                            MapStratToeld();
                          }
                          SetupCalcEquivLogDepths(dset);
                        }
                        if(dataset_cnt == 1) {
                          XtSetSensitive(pushButtonOpenEquivLogDepth, False);
                        }

                        /* set the default open data directory */
                        XtFree(open_data_directory);
                        len = (int)strlen(current_path);
                        if((open_data_directory=(char*)calloc(len+10,CHAR_SIZE))==NULL)
                                       SaBadMemAlloc("SaInputData 4a");
                        if(len == 0) {
                        }
                        else if(strcmp(&current_path[len-1], "/") != 0) {
                          (void)sprintf(open_data_directory, "%s/", current_path);
                        }
                        else {
                          (void)sprintf(open_data_directory, "%s", current_path);
                        }

                        SetDataSetOptionMenu(dset);

                        if(have_strat == DO) {
                          MapStratToData(dset);
                        }

                        if(smooth.method == NONE) {
                          XtSetSensitive(toggleButtonDisplayCoreUnsmoothed, True);
                          XtSetSensitive(toggleButtonDisplayCoreSmooth, False);
                          XtSetSensitive(toggleButtonDisplayCoreBoth, False);
                        }
                        else {
                          XtSetSensitive(toggleButtonDisplayCoreUnsmoothed, True);
                          XtSetSensitive(toggleButtonDisplayCoreSmooth, True);
                          XtSetSensitive(toggleButtonDisplayCoreBoth, True);
                        }

                        if((err = FindMinMax(dset)) > 0) {
                        }
                        else {
                        }
                        SetMinMax(dset);

                        if(have_splice_table == DO) {
                          SetupMakingSpliceSeries();
                          if(eqlogdepth.read_equil_log == YES) {
                            XtSetSensitive(pushButtonSaveSplice, True);
                          }
                        }
                        else if(have_splice == DO) {
                          /* see if this new core data set is of same type as 
                             one of the splices. if so map splice to it
                          */
                          for(i=0; i<splice_cnt; ++i) {
                            if(data[dset]->type == splice[i]->data_type) {
                              splice[i]->maptodata = dset;
                              sset = i;
                            }
                          }
                        }
                        if(have_coretotie == DO || have_splice == DO) {
                          /* find if core to tie is in this data set
                          */
                          foundit = NO;
                          if(have_coretotie == DO) {
                            for(i=0; i<data[dset]->numholes; ++i) {
                              if(data[dset]->holes[i]->name == ties->tiecoreholename) {
                                for(j=0; j<data[dset]->holes[i]->numcores; ++j) {
                                  if(data[dset]->holes[i]->core[j]->num == ties->tiecorename) {
                                    foundit = YES;
                                    ties->hole = i;
                                    ties->core = j;
                                    break;
                                  }
                                }
                              }
                            }
                          }
                          else if(have_splice == DO) {
                            foundit = YES;
                            ties->hole = 0;
                            ties->core = 0;
                          }
                          if(foundit == YES) {
                            if(ties->temp_equivlogdep) {
                              free(ties->temp_equivlogdep);
                              ties->temp_equivlogdep = NULL;
                            }
                            if(ties->coredepth) {
                              free(ties->coredepth);
                              ties->coredepth = NULL;
                            }
                            if(have_coretotie == DO) {
                              ties->core_alloc = data[dset]->holes[ties->hole]->core[ties->core]->numvalues;
                            }
                            else if(have_splice == DO) {
                              ties->core_alloc = splice[sset]->numvalues;
                            }
                            if((ties->temp_equivlogdep=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                                                SaBadMemAlloc("Sa");
                            if((ties->coredepth=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                                                SaBadMemAlloc("Sa");
                            if(have_coretotie == DO) {
                              for(j=0; j<data[dset]->holes[ties->hole]->core[ties->core]->numvalues; ++j) {
                                if(data[dset]->holes[ties->hole]->core[ties->core]->value[j]->quality  & BAD_SB_DEPTH) {
                                  ties->temp_equivlogdep[j] = BADPT;
                                  ties->coredepth[j] = BADPT;
                                }
                                else {
                                  ties->coredepth[j] = data[dset]->holes[ties->hole]->core[ties->core]->value[j]->mbsf +
                                  data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
                                  if(eqlogdepth.autocorr_status == YES) {
                                    ties->coredepth[j] = ties->coredepth[j] * (eqlogdepth.autocorr_squish/100.0);
                                  }
                                  ties->temp_equivlogdep[j]= ties->coredepth[j];
                                }
                              }
                              if(ties->nties > 0) {
                                for(k=0; k<ties->nties; ++k) {
                                  GetTieArrayNum(COREDATA, ties->tie[k], dset, ties->hole, ties->core, &err);
                                }
                                AdjustToTies(dset);
                              }
                              else if(data[dset]->holes[ties->hole]->core[ties->core]->have_equil_log_depths == DO) {
                                for(j=0; j<data[dset]->holes[ties->hole]->core[ties->core]->numvalues; ++j) {
                                  if(data[dset]->holes[ties->hole]->core[ties->core]->value[j]->quality  & BAD_SB_DEPTH) {
                                    ties->temp_equivlogdep[j] = BADPT;
                                  }
                                  else {
                                    ties->temp_equivlogdep[j] = data[dset]->holes[ties->hole]->core[ties->core]->value[j]->eld;
                                  }
                                }
                              }
                              if(have_tie1 == DO) {
                                FindCurrentTieArray(COREDATA, dset);
                              }
                            }
                            else if(have_splice == DO) {
                              if(ties->nties > 1) {
                                ties->tie[0]->log = -1;
                                ties->tie[0]->array = -1;
                                for(i=1; i<ties->nties; ++i) {
                                  GetTieArrayNum(SPLICEDATA, ties->tie[i], sset, 0, 0, &err);
                                }
                                AdjustToTies(sset);
                              }
                              else {
                                for(i=0; i<splice[sset]->numvalues; ++i) {
                                  if(splice[sset]->value[i]->quality  & BAD_SB_DEPTH) {
                                    ties->temp_equivlogdep[i] = BADPT;
                                    ties->coredepth[i] = BADPT;
                                  }
                                  else {
                                    ties->coredepth[i] = splice[sset]->value[i]->mcd;
                                    if(eqlogdepth.autocorr_status == YES) {
                                      ties->coredepth[i] = ties->coredepth[i] * (eqlogdepth.autocorr_squish/100.0);
                                    }
                                    ties->temp_equivlogdep[i]= ties->coredepth[i];
                                  }
                                }
                              }
                              for(i=0; i<splice[sset]->numvalues; ++i) {
                                splice[sset]->value[i]->eld = ties->temp_equivlogdep[i];
                              }           
                              if(have_tie1 == DO) {
                                FindCurrentTieArray(SPLICEDATA, sset);
                              }
                            }
                            FindAveDepth(dset, ties->hole, ties->core, &ties->avedepthstep);
                            if((string=(char*)calloc(100,CHAR_SIZE))==NULL)
                                              SaBadMemAlloc("SaInputData");
                            if(ties->avedepthstep >= logdata[lset]->avedepthstep) {
                              (void)sprintf(string, "%.2f", ties->avedepthstep);
                            }
                            else {
                              (void)sprintf(string, "%.2f", logdata[lset]->avedepthstep);
                            }
                            XtVaSetValues(textFieldDepthStep, XmNvalue, string, NULL);
                            free(string);
                            string = NULL;
                            if(have_tie1 == DO && have_tie2 == DO && ties->coretie != MISSING && ties->logtie != MISSING) {
                              if(err = CorrelationSetup(dset) != 0) {
                                have_correlate = DONT;
                                if(err == 2) {
                                  (void)sprintf(buffer, "Error; correlation, new data set is missing tie pt, could not interpolate.\n");
                                }
                                else {
                                  (void)sprintf(buffer, "Error doing correlation.\n");
                                }
                                SaManageWarningBox(buffer);
                              }
                              else {
                                have_correlate = DO;
                              }
                            }
                          }
                          else {
                            have_coretotie = DONT;
                            CancelCore((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
                          }
                        }

                        XtSetSensitive(pushButtonOpenStrat, True);
                        if(have_affine == DO && have_splice == DONT) {
/*
                          XtSetSensitive(pushButtonOpenSplicer, True);
*/
                          XtSetSensitive(pushButtonOpenSpliceTable, True);
                        }
                        XtSetSensitive(pushButtonClearAll, True);
                        XtSetSensitive(pushButtonClearCores, True);
                        XtSetSensitive(cascadeButtonProcessData, True);
                        if(have_log == DO || have_ancill == DO || have_strat == DO) {
                          XtSetSensitive(pushButtonClearAll, True);
                        }
                        if(have_splice == DO && (sset < 0 || sset >= splice_cnt)) {
                          XtSetSensitive(formEquivLogDepthTool, False);
                        }
                        else if(have_log == DO) {
                          if(have_splice == DO && ties->nties > 1) {
                            XtSetSensitive(pushButtonAutoCorrelate, False);
                          }
                          else if((n = CountTies()) > 0 || have_coretotie == DO) {
                            XtSetSensitive(pushButtonAutoCorrelate, False);
                          }
                          else {
                            XtSetSensitive(pushButtonAutoCorrelate, True);
                          }
                          if(have_coretotie == DO) {
                          }
                          else {
                            XtSetSensitive(formEquivLogDepthTool, True);
                            XtSetSensitive(pushButtonSelectCore, True);
                            XtSetSensitive(pushButtonCancelCore, False);
                            XtSetSensitive(pushButtonClearTie, False);
                            XtSetSensitive(pushButtonClearLastTieSaved, False);
                            XtSetSensitive(pushButtonShowHideTieList, False);
                            XtSetSensitive(pushButtonAddTieToList, False);
                            XtSetSensitive(toggleButtonAddTieToCore, False);
                            XtSetSensitive(toggleButtonAddTieToAll, False);
                            XtSetSensitive(pushButtonOptimizeTie, False);
                            XtSetSensitive(pushButtonAdjustToTiesParameters, False);
                            XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
                            XtSetSensitive(pushButtonSaveChanges, False);
                            XtSetSensitive(pushButtonUndoSaveChange, False);
                          }
                        }

                        /* map data to log
                        */
                        data[dset]->maptolog = -1;
                        if(have_log == DO) {
                          XtSetSensitive(pushButtonEvalCoreLogData, True);
                          for(i=0; i<logset_cnt; ++i) {
                            if(logdata[i]->info.core_data_type == data[dset]->type) {
                              data[dset]->maptolog = i;
                              logdata[i]->info.maptodata = dset;
                              break;
                            }
                          }
                          err = 0;
                          for(i=0; i<dataset_cnt; ++i) {
                            if(data[i]->maptolog == -1) {
                              ++err;
                            }
                          }
                          if(err < dataset_cnt) {
                            XtSetSensitive(pushButtonPairCoreLog, True);
                            XtSetSensitive(labelAutoPair, True);
                            if(autopair == YES) {
                            }
                            else {
                              autopair = NO;
                            }
                          }
                          else {
                            autopair = NO;
                            XtSetSensitive(pushButtonPairCoreLog, False);
                            XtSetSensitive(labelAutoPair, False);
                          }
                        }
                        else {
                          autopair = NO;
                          XtSetSensitive(pushButtonPairCoreLog, False);
                          XtSetSensitive(labelAutoPair, False);
                        }
                        AutoPair(autopair);

                        are_reading_dataset = NO;
			DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
                        if(have_log == DO && logdata[lset]->info.depth_max < maxdep) {
                          DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
                        }
                        XtUnmanageChild(messageBoxWorking);
			return(True);
		}
		return(False);

	}
/* if all data read, set holes and start processing next time */
	if(current_file >= data[dset]->numfiles && !datafileopen)	/* all data read */
	{
		current_file=0;
		/* sort holes alpha/numerically */
		for(i=0;i<data[dset]->numholes-1;i++)
		{
			switched=False;
			for(j=i+1;!switched && j<data[dset]->numholes;j++)
			{
				if(data[dset]->holes[j]->name < data[dset]->holes[i]->name)
				{
					tmphole=data[dset]->holes[i];
					data[dset]->holes[i]=data[dset]->holes[j];
					data[dset]->holes[j]=tmphole;
					switched=True;
					i--;
				}
			}
		}
/* an error occurred somewhere... Close working box and clean up */
                if(info.error)
                {
                        if(dataset_cnt == 1) {
		  	  SaCleanUp(COREDATA);
                        }
                        else if(dataset_cnt > 1) {
                          SaFreeData(data[dset]);
                          if(data[dset])
                          {
                            free(data[dset]);
                            data[dset]=NULL;
                          }
                          --dataset_cnt;
                          dset = dataset_cnt - 1;
                        }
			current_file=lineno=current_hole_num=0;
                        done_decimate=done_cull=done_reading=False;
                        XtUnmanageChild(messageBoxWorking);
                        info.error=True;
                        info.fatal=False;
                        info.default_button=XmDIALOG_OK_BUTTON;
                        info.type=XmDIALOG_ERROR;
                        info.caller=READCORE;
                        if((info.message=(char*)calloc(200,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData 8");
                        (void)sprintf(info.message,"Unknown read error occured. Can't recover.");
                        SaManageMessage(info);
                        return(True);
                }
		else
		{
			done_reading=True;
			if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaInputData 7");
			(void)sprintf(charptr,"\nData : Done reading files. Preparing to process...");
			SaUpdateReport(charptr);
			xmstring=XmStringCreateLocalized(&charptr[1]);
			XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring,
                                      XmNwidth,XmStringWidth(fontlist,xmstring)+150, NULL);
                        XtResizeWindow(messageBoxWorking);
                        XmUpdateDisplay(messageBoxWorking);
			XmStringFree(xmstring);
			free(charptr);
			return(False);
		}
	}
/* find current filename */
	else if(!datafileopen)
	{
		nameptr=data[dset]->name[current_file++];
		if((fptr=fopen(nameptr,"r"))==NULL)
		{
                        if(dataset_cnt == 1) {
			  SaCleanUp(COREDATA);
                        }
                        else if(dataset_cnt > 1) {
                          SaFreeData(data[dset]);
                          if(data[dset])
                          {
                            free(data[dset]);
                            data[dset]=NULL;
                          }
                          --dataset_cnt;
                          dset = dataset_cnt - 1;
                        }
                        XtUnmanageChild(messageBoxWorking);
                        current_file=lineno=current_hole_num=0;
                        done_decimate=done_cull=done_reading=False;
			info.error=True;
			info.fatal=False;
			info.default_button=XmDIALOG_OK_BUTTON;
			info.type=XmDIALOG_ERROR;
			info.caller=READCORE;
			if((info.message=(char*)calloc(50+(int)strlen(nameptr),CHAR_SIZE))==NULL)
					SaBadMemAlloc("SaInputData 8");
			(void)sprintf(info.message,"Error opening input file %s\n",nameptr);
			SaManageMessage(info);
			return(True);
		}
		else
		{
			lineno=0;
			datafileopen=True;
			if((charptr=(char*)calloc(100+(int)strlen(nameptr),CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaInputData 9");
			(void)sprintf(charptr,"Data : Reading file %s (line=%d, total errors=%d)     ",nameptr,lineno,data[dset]->readerrors);
			xmstring=XmStringCreateLocalized(charptr);
			XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring,
                                      XmNwidth,XmStringWidth(fontlist,xmstring)+150,NULL);
                        XtResizeWindow(messageBoxWorking);
                        XmUpdateDisplay(messageBoxWorking);
			XmStringFree(xmstring);
			free(charptr);

                        /* if have header get num of lines and read that many from current file
                        */
                        if(data[dset]->in_format != JANUSORIG && XmToggleButtonGetState(toggleButtonHeaderYes) == True) {
                          str = XmTextGetString(textFieldHeaderLines);
                          skip = atoi(str);
                          XtFree(str);
                          for(i=1; i<=skip; ++i) {
                            if(fgets(buffer,999,fptr)==NULL) {
                              /* error
                              */

                              if(dataset_cnt == 1) {
                                SaCleanUp(COREDATA);
                              }
                              else if(dataset_cnt > 1) {
                                SaFreeData(data[dset]);
                                if(data[dset])
                                {
                                  free(data[dset]);
                                  data[dset]=NULL;
                                }
                                --dataset_cnt;
                                dset = dataset_cnt - 1;
                              }
                              current_file=lineno=current_hole_num=0;
                              done_decimate=done_cull=done_reading=False;
                              XtUnmanageChild(messageBoxWorking);
                              info.error=False;
                              if((info.message=(char*)calloc(100,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData");
                              (void)sprintf(info.message,"Error: the number of header lines exceeded the length for the data file.\n");
                              info.type=XmDIALOG_ERROR;
                              info.caller=NONE;
                              info.default_button=XmDIALOG_OK_BUTTON;
                              SaManageMessage(info);
                              return(True);
                            }
                          }
                        }

                        /* if format type is Janus Orig then read current file until record
                           reads <PRE><!--START OF OUTPUT--> then read one more to skip header
                           if don't reach this record then error
                           note that if there was an error getting data then the nest line will
                           have some message stating this. fine out what that is.
                        */
                        if(data[dset]->in_format == JANUSORIG) {
                          while(fgets(buffer,999,fptr)!=NULL) {
                            if(strcmp(buffer, "<PRE><!--START OF OUTPUT-->\n") == 0) {
                              if(fgets(buffer,999,fptr)==NULL) {
                                printf("err\n");
                                /* error
                                */    
                              }
                              break;
                            }
                          }
                        }
		}
	}
/* read some data */
	if(datafileopen)
	{
		if(SaReadCore(fptr,data[dset]))
		{
		/* end of file found */
			fclose(fptr);
			datafileopen=False;
			if((charptr=(char*)calloc(200+(int)strlen(nameptr),CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaInputData 10");
			(void)sprintf(charptr,"\nRead file %s (total lines =%d, total errors=%d)",nameptr,lineno,data[dset]->readerrors);
			SaUpdateReport(charptr);
			free(charptr);
		}
		else
		/* not end of file */
		{
			if(++lineno % 500 == 0)
			{
                        	if((charptr=(char*)calloc(200+(int)strlen(nameptr),CHAR_SIZE))==NULL) 
                               		SaBadMemAlloc("SaInputData 11");
				(void)sprintf(charptr,"Data : Reading file %s (line=%d, total errors=%d)",nameptr,lineno,data[dset]->readerrors);
				xmstring=XmStringCreateLocalized(charptr);
				XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring,
                                              XmNwidth,XmStringWidth(fontlist,xmstring)+150,NULL);
                                XtResizeWindow(messageBoxWorking);
                                XmUpdateDisplay(messageBoxWorking);
				XmStringFree(xmstring);
				free(charptr);
			}
		}
/* an error occurred somewhere... Close working box and clean up */
                if(info.error)
                {
                        if(dataset_cnt == 1) {
			  SaCleanUp(COREDATA);
                        }
                        else if(dataset_cnt > 1) {
                          SaFreeData(data[dset]);
                          if(data[dset])
                          {
                            free(data[dset]);
                            data[dset]=NULL;
                          }
                          --dataset_cnt;
                          dset = dataset_cnt - 1;
                        }
                        fclose(fptr);
                        datafileopen=False;
			current_file=lineno=current_hole_num=0;
                        done_decimate=done_cull=done_reading=False;
                        XtUnmanageChild(messageBoxWorking);
                        info.error=True;
                        info.fatal=False;
                        info.default_button=XmDIALOG_OK_BUTTON;
                        info.type=XmDIALOG_ERROR;
                        info.caller=READCORE;
                        if((info.message=(char*)calloc(200,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInputData 8");
                        (void)sprintf(info.message,"Unknown read error occured. Can't recover.",nameptr);
                        SaManageMessage(info);
                        return(True);
                }
		else
			return(False);
	}
}

int SaFreeData(dataptr)
DataFile	*dataptr;
{
	int	i,j,k;

	for(i=0;i<dataptr->numfiles;i++)
	{
		if(dataptr->name[i] && (int)strlen(dataptr->name[i])>0)
		{
			XtFree(dataptr->name[i]);
			dataptr->name[i]=NULL;
		}
	}
	for(i=0;dataptr->holes && dataptr->holes[i] && i<dataptr->numholes;i++)
	{
		for(j=0;dataptr->holes[i]->core && j<dataptr->holes[i]->numcores;j++)
		{
			for(k=0;dataptr->holes[i]->core[j]->value && k< dataptr->holes[i]->core[j]->allocvalues;k++)
			{
				if(dataptr->type==GRAPE && dataptr->holes[i]->core[j]->value[k]->grape)
				{
					free(dataptr->holes[i]->core[j]->value[k]->grape);
					dataptr->holes[i]->core[j]->value[k]->grape=NULL;
				}
				else if(dataptr->type==PWAVE && dataptr->holes[i]->core[j]->value[k]->pwave)
				{
					free(dataptr->holes[i]->core[j]->value[k]->pwave);
					dataptr->holes[i]->core[j]->value[k]->pwave=NULL;
				}
				else if(dataptr->type==NATURALGAMMA && dataptr->holes[i]->core[j]->value[k]->naturalgamma)
				{
					free(dataptr->holes[i]->core[j]->value[k]->naturalgamma);
					dataptr->holes[i]->core[j]->value[k]->naturalgamma=NULL;
				}
				else if(dataptr->type==SUSCEPTIBILITY && dataptr->holes[i]->core[j]->value[k]->susceptibility)
				{
					free(dataptr->holes[i]->core[j]->value[k]->susceptibility);
					dataptr->holes[i]->core[j]->value[k]->susceptibility=NULL;
				}
				else if(dataptr->type==REFLECTANCE && dataptr->holes[i]->core[j]->value[k]->reflectance)
				{
					free(dataptr->holes[i]->core[j]->value[k]->reflectance);
					dataptr->holes[i]->core[j]->value[k]->reflectance=NULL;
				}
				free(dataptr->holes[i]->core[j]->value[k]);
				dataptr->holes[i]->core[j]->value[k]=NULL;
			}
			free(dataptr->holes[i]->core[j]->value);
			dataptr->holes[i]->core[j]->value=NULL;
                        if(dataptr->holes[i]->core[j]->good) {
                          free(dataptr->holes[i]->core[j]->good);
                          dataptr->holes[i]->core[j]->good=NULL;
                        }
			free(dataptr->holes[i]->core[j]);
			dataptr->holes[i]->core[j]=NULL;
		}
		free(dataptr->holes[i]);
		dataptr->holes[i]=NULL;
	}
        if(dataptr->holes)
        {
                free(dataptr->holes);
                dataptr->holes=NULL;
        }
        if(dataptr->name && (int)strlen((char*)dataptr->name)>0)
        {
                free(dataptr->name);
                dataptr->name=NULL;
        }
        if(dataptr->holes_used && (int)strlen(dataptr->holes_used)>0)
        {
                free(dataptr->holes_used);
                dataptr->holes_used=NULL;
        }
        dataptr->readerrors=0;
        dataptr->numholes=0;
        dataptr->numfiles=0;
        return;
}

int SaInputAncillary(client_data)
XtPointer client_data;
{
        int             i, j, n, alloc_int, outoforder;
        char            *string;
        XmString        xmstring;
        AncillValue        *valueptr, **holdanc;
        void            DrawDrawingAreaCores(), DrawDrawingAreaLog();

        if(!done_reading)
        {
        /* allocate block of data */
                alloc_int=LOG_DATA_ALLOC;
                if(ancill.data_alloc<=0)
                {
                        ancill.data_num=0;
                        if((ancill.value=(AncillValue**)calloc((ancill.data_alloc=alloc_int),sizeof(AncillValue*)))==NULL)
                                SaBadMemAlloc("SaInputAncill 1");
                        for(i=0;i<ancill.data_alloc;i++)
                        {
                                if((ancill.value[i]=(AncillValue*)malloc(sizeof(AncillValue)))==NULL)
                                        SaBadMemAlloc("SaInputAncill 1a");
                                if((ancill.value[i]->data=(float*)calloc(ancill.data_cnt, sizeof(float)))==NULL)
                                                SaBadMemAlloc("SaInputAncill 1b");
                        }
                }
                else
                {
                        if((charptr=(char*)realloc(ancill.value,(ancill.data_alloc+alloc_int)*sizeof(AncillValue)))==NULL)
                                SaBadMemAlloc("SaInputAncill 2");
                        else
                        {
                                ancill.value=(AncillValue**)charptr;
                                for(i=ancill.data_alloc,ancill.data_alloc+=alloc_int;
                                                        i<ancill.data_alloc;i++)
                                {
                                        if((ancill.value[i]=(AncillValue*)malloc(sizeof(AncillValue)))==NULL)
                                                SaBadMemAlloc("SaInputAncill 2a");
                                        if((ancill.value[i]->data=(float*)calloc(ancill.data_cnt, sizeof(float)))==NULL)
                                                SaBadMemAlloc("SaInputAncill 2a");
                                }
                        }
                }
                /* read a block of data */
                for(i=0;i<alloc_int;i++)
                {
                        if(fgets(buffer,1000,ancill.fptr)==NULL)
                        {
                                done_reading=True;
                                done_decimate = True;
                                if(ancill.decimate_value<=1)
                                        done_decimate=True;
                                else
                                        done_decimate=False;
                                fclose(ancill.fptr);
                        }
                        else if((strncmp(buffer,"\n",1) == 0) ||
                                (strcmp(buffer,"") == 0) ||
                                (strncmp(buffer,"                    ",20) == 0))
                        {
                        }
                        else
                        {
                                valueptr=ancill.value[ancill.data_num];
                                for(j=0; j<ancill.data_cnt; ++j) {
                                  valueptr->data[j]=getfloat(buffer,ancill.data_col[j]);
                                }
                                valueptr->depth=getfloat(buffer,0);
                                if(have_cores == YES) {
                                  if(eqlogdepth.did_log_mudline_adj == YES) {
                                    valueptr->depth = valueptr->depth + eqlogdepth.autocorr_log_mudline_offset;
                                  }
                                }
                                if(valueptr->data[0] == ancill.missing_value) {
                                  valueptr->quality=BADLOG;
                                }
                                else {
                                  valueptr->quality=GOOD;
                                }
                                ancill.data_num++;
                        }
                }
        }
        else if(!done_decimate)
        {
                if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaInputAncill Decimate");
                (void)sprintf(charptr,"\nDecimated Ancillary data.");
                SaUpdateReport(charptr);
                xmstring=XmStringCreateLocalized(&charptr[1]);
                XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                XtResizeWindow(messageBoxWorking);
                XmUpdateDisplay(messageBoxWorking);
                XmStringFree(xmstring);
                free(charptr);
                for(i=0;i<ancill.data_num;i++)
                {
                        if(i%ancill.decimate_value!=0)
                                ancill.value[i]->quality=ancill.value[i]->quality|DECIMATED;
                }
                done_decimate=True;
        }

        if(done_reading & done_decimate)
        {
                /* is ancillary depth in descending order? or reverse order?
                   or out of order
                */
                outoforder = 0;
                if(ancill.value[0]->depth > ancill.value[ancill.data_num-1]->depth) {
                  for(i=0; i<ancill.data_num-1; ++i) {
                    if(ancill.value[i]->depth < ancill.value[i+1]->depth) {
                      ++outoforder;
                      ancill.value[i]->quality = BADLOG;
                    }
                  }
                }
                else if(ancill.value[0]->depth < ancill.value[ancill.data_num-1]->depth) {
                  for(i=0; i<ancill.data_num-1; ++i) {
                    if(ancill.value[i]->depth > ancill.value[i+1]->depth) {
                      ++outoforder;
                      ancill.value[i]->quality = BADLOG;
                    }
                  }
                }
                /* too many out of order
                */
               if(outoforder > ancill.data_num/2) {
                  (void)sprintf(buffer, "Error; found too many ancillary depths out of order.");
                  SaManageWarningBox(buffer);
                }
                /* depth in reverse order; change it to descending order
                */
                else if(ancill.value[0]->depth > ancill.value[ancill.data_num-1]->depth) {
                  if((holdanc=(AncillValue**)calloc(ancill.data_num, sizeof(AncillValue*)))==NULL)
                  {
                                SaBadMemAlloc("SaInputAncill");
                  }
                  else
                  {
                    for(i=0; i<ancill.data_num; ++i)
                    {
                      if((holdanc[i]=(AncillValue*)malloc(sizeof(AncillValue)))==NULL)
                      {
                                           SaBadMemAlloc("SaInputAncill 2a");
                      }
                      else
                      {
                        if((holdanc[i]->data=(float*)calloc(ancill.data_cnt, sizeof(float)))==NULL)
                                                SaBadMemAlloc("SaInputAncill 2a");
                        holdanc[i]->depth = ancill.value[i]->depth;
                        for(j=0; j<ancill.data_cnt; ++j) {
                          holdanc[i]->data[j] = ancill.value[i]->data[j];
                        }
                        holdanc[i]->quality = ancill.value[i]->quality;
                      }
                    }
                  }
                  n = ancill.data_num - 1;
                  for(i=0; i<ancill.data_num; ++i) {
                    ancill.value[i]->depth = holdanc[n]->depth;
                    for(j=0; j<ancill.data_cnt; ++j) {
                      ancill.value[i]->data[j] = holdanc[n]->data[j];
                    }
                    ancill.value[i]->quality = holdanc[n]->quality;
                    --n;
                  }
                  for(i=0; i<ancill.data_num; ++i) {
                    free(holdanc[i]);
                    holdanc[i] = NULL;
                  }
                  free(holdanc);
                  holdanc = NULL;
                }
                have_ancill=DO;
                XtSetSensitive(pushButtonClearAncillary, True);
                XtSetSensitive(pushButtonOpenAnc, False);
                FindAncMinMax(&ancill);
                SetAncMinMax(ancill);
                if(have_log == DO || have_cores == DO || have_strat == DO) {
                  XtSetSensitive(pushButtonClearAll, True);
                }

                /* change the custom color labels to real names(s)
                */
                if((string=(char*)calloc(100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaInputAncill");
                (void)sprintf(string, "%s", ancill.id[1]);
                xmstring = XmStringCreateSimple(string);
                XtVaSetValues(labelColorAncillaryLog1, XmNlabelString, xmstring, NULL);
                XmStringFree(xmstring);
                if(ancill.data_cnt > 1) {
                  (void)sprintf(string, "%s", ancill.id[2]);
                  xmstring = XmStringCreateSimple(string);
                  XtVaSetValues(labelColorAncillaryLog2, XmNlabelString, xmstring, NULL);
                  XmStringFree(xmstring);
                }
                if(ancill.data_cnt > 2) {
                  (void)sprintf(string, "%s", ancill.id[3]);
                  xmstring = XmStringCreateSimple(string);
                  XtVaSetValues(labelColorAncillaryLog3, XmNlabelString, xmstring, NULL);
                  XmStringFree(xmstring);
                }
                free(string);

                DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
                DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);

                done_reading=done_decimate=False;
                XtUnmanageChild(messageBoxWorking);
                return(True);
        }
        else
        {
                if((charptr=(char*)calloc((int)strlen(ancill.filename)+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaInputAncill 5");
                (void)sprintf(charptr,"%s: (line %d)",ancill.filename,ancill.data_num);
                xmstring=XmStringCreateLocalized(&charptr[1]);
                XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                XtResizeWindow(messageBoxWorking);
                XmUpdateDisplay(messageBoxWorking);
                XmStringFree(xmstring);
                free(charptr);
                return(False);
        }
}

int SaInputSplice(file)
char *file;
{
  int i, err, s;
  char *string, *logstring;
  XmString xmstring;
  Cursor cursor;
  void SetSpliceMinMax(), DrawDrawingAreaCores(), DrawDrawingAreaLog(), 
       SearchELDTableForSpliceTies(), SaGaussianSplice(), SaCullSplice(), SaDecimateSplice();
  int FindSpliceMinMax();

  cursor = XCreateFontCursor(XtDisplay(formMain), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formMain), XtWindow(formMain), cursor);
  }
  XtManageChild(messageBoxWorking);
  if((charptr=(char*)calloc(150,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaSmoothCore 1");
  (void)sprintf(charptr,"\nReading Splice %s", file);
  SaUpdateReport(charptr);
  xmstring=XmStringCreateLocalized(&charptr[1]);
  XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
  XtResizeWindow(messageBoxWorking);
  XmUpdateDisplay(messageBoxWorking);
  XmStringFree(xmstring);
  free(charptr);

  info.error = False;

  ++splice_cnt;
  s = splice_cnt - 1;
  if(splice_cnt == 1) {
    if((splice = (Splice**)calloc(1, sizeof(Splice*)))==NULL) {
                    SaBadMemAlloc("Sa");
    }
    if((splice[0] = (Splice*)calloc(1, sizeof(Splice)))==NULL) {
                    SaBadMemAlloc("Sa");
    }
  }
  else if(splice_cnt > 1) {
    if((charptr= (char*)realloc((char*)splice,(splice_cnt*sizeof(Splice*))))==NULL) {
                                SaBadMemAlloc("SaInputData");
    }
    else {
      splice=(Splice**)charptr;
    }
    if((splice[s] = (Splice*)calloc(1, sizeof(Splice)))==NULL) {
                    SaBadMemAlloc("Sa");
    }
  }
  else {
    return;
  }
  splice[s]->alloc_cnt = SPLICE_ALLOC;

  if((splice[s]->value=(SpliceValue**)calloc((splice[s]->alloc_cnt),sizeof(SpliceValue*)))==NULL)
                                SaBadMemAlloc("SaInputSplice");
  for(i=0; i<splice[s]->alloc_cnt; ++i) {
    if((splice[s]->value[i]=(SpliceValue*)calloc(1, sizeof(SpliceValue)))==NULL)
                               SaBadMemAlloc("SaInputSplice");
  }

  SaReadSplice(file, s);
  if(info.error == True) {
    have_splice = DONT;
    SaCleanUp(SPLICEDATA);
    return;
  }
  else {
    have_splice = DO;
    XtSetSensitive(pushButtonOpenSplicer, True);
    XtSetSensitive(pushButtonClearSplice, True);
    XtSetSensitive(pushButtonClearAll, True);
    XtSetSensitive(pushButtonOpenSpliceTable, False);
    err = FindSpliceMinMax(s);
    SetSpliceMinMax(splice[s]);
  }

  /* assign the data type
  */
  if(splice_data_type == GRAPE) {
    splice[s]->data_type = GRAPE;
  }
  else if(splice_data_type == PWAVE) {
    splice[s]->data_type = PWAVE;
  }
  else if(splice_data_type == SUSCEPTIBILITY) {
    splice[s]->data_type = SUSCEPTIBILITY;
  }
  else if(splice_data_type == NATURALGAMMA) {
    splice[s]->data_type = NATURALGAMMA;
  }
  else if(splice_data_type == REFLECTANCE) {
    splice[s]->data_type = REFLECTANCE;
  }
  else if(splice_data_type == OTHERTYPE) {
    splice[s]->data_type = OTHERTYPE;
  }
  else {
    splice[s]->data_type = OTHERTYPE;
  }

  /*map splice to correct core data
  */
  splice[s]->maptodata = -1;
  for(i=0; i<dataset_cnt; ++i) {
    if(splice[s]->data_type == data[i]->type) {
      splice[s]->maptodata = i;
      break;
    }
  }
  sset = -1;
  for(i=0; i<splice_cnt; ++i) {
    if(dset == splice[s]->maptodata) {
      sset = i;
    }
  }
  if(splice[s]->maptodata == -1) {
    (void)sprintf(buffer, "Warning: This Splice series does not have a corresponding core data set.");
    SaManageWarningBox(buffer);
  }
  else if(data[splice[s]->maptodata]->culled == YES) {
    SaCullSplice(s, splice[s]->data_type);
  }
  if(decimate_value > 1) {
    SaDecimateSplice(s, decimate_value);
  }

  if(smooth.method == NONE) {
    splice[s]->smooth.method = NONE;
  }
  else if(smooth.method == GAUSSIAN) {
    splice[s]->smooth.method = GAUSSIAN;
    splice[s]->smooth.method = GAUSSIAN;
    splice[s]->smooth.width = smooth.width;
    splice[s]->smooth.width_units = smooth.width_units;
    SaGaussianSplice(s);
  }
  else {
    splice[s]->smooth.method = NONE;
  }
  if(smooth.plot == UNSMOOTH) {
    splice[s]->smooth.plot = UNSMOOTH;
  }
  else if(smooth.plot == SMOOTH) {
    splice[s]->smooth.plot = SMOOTH;
  }
  else if(smooth.plot == SMOOTHBOTH) {
    splice[s]->smooth.plot = SMOOTHBOTH;
  }
  else {
    splice[s]->smooth.plot = UNSMOOTH;
  }

  /* if this is the first splice series read alloc some space for tie list
  */
  if(splice_cnt == 1) {
    ties->avedepthstep = splice[s]->avedepstep;
    ties->alloc_cnt = MAXTIES;
    if((ties->tie=(TieList**)calloc(ties->alloc_cnt,sizeof(TieList*)))==NULL)
                                SaBadMemAlloc("Sa");
    for(i=0; i<ties->alloc_cnt; ++i) {
      if((ties->tie[i]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                                SaBadMemAlloc("Sa");
    }
    ties->core_alloc = splice[s]->numvalues;
    if((ties->temp_equivlogdep=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
    if((ties->coredepth=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
    /* give the tie list a tie that fixes 0mcd eq 0 log
    */
    if((string=(char*)calloc(200,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("Sa");
    if((logstring=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("Sa");
    ties->nties = 1;
    ties->tie[0]->holename = splice[s]->value[0]->hole;
    ties->tie[0]->corename = splice[s]->value[0]->corename;
    ties->tie[0]->array = 0;
    ties->tie[0]->log = 0;
    ties->tie[0]->logdep = 0.0;
    ties->tie[0]->corembsf = 0.0;
    ties->tie[0]->coremcd = 0.0;
    ties->tie[0]->coreautoeld = 0.0;
    ties->tie[0]->equivlogdep = 0.0;
    ties->tie[0]->temp_eld = 0.0;
    ties->tie[0]->rate = 1.0;
    ties->tie[0]->b = 0.0;
    ties->tie[0]->shared = NO;
    ties->tie[0]->topsect = 0.0;
    ties->tie[0]->botsect = 0.0;
    (void)sprintf(ties->tie[0]->coresection, "1");
    (void)sprintf(string,"Core:  %4d %4d %c  %3d %c %2s %5.1f %5.1f  %8.2fmbsf  %8.2fmcd",
                splice[s]->leg,
                splice[s]->site,
                splice[s]->value[0]->hole,
                splice[s]->value[0]->corename,
                splice[s]->value[0]->type,
                ties->tie[0]->coresection,
                ties->tie[0]->topsect,
                ties->tie[0]->botsect,
                ties->tie[0]->corembsf,
                ties->tie[0]->coremcd);
    if(eqlogdepth.autocorr_status == YES) {
      (void)sprintf(logstring, "  %8.2feld(auto)", ties->tie[0]->coreautoeld);
      (void)strncat(string, logstring, (int)strlen(logstring));
    }
    (void)sprintf(logstring, "  Tied to    Log:  %8.3fm");
    (void)strncat(string, logstring, (int)strlen(logstring));

    xmstring = XmStringCreateSimple(string);
    XmListAddItemUnselected(listTieList, xmstring, 0);
    XmStringFree(xmstring);
    free(string);
    string = NULL;
    free(logstring);
    logstring = NULL;

    /* if have read eld table get any ties that are in the splice
    */
    if(eqlogdepth.read_equil_log == YES) {
      XtSetSensitive(pushButtonSaveSplice, True);
    }
    else {
      XtSetSensitive(pushButtonSaveSplice, False);
    }
  }
  else if(ties->core_alloc < splice[s]->numvalues) {
    ties->core_alloc = splice[s]->numvalues;
    if((charptr=(char*)realloc(ties->temp_equivlogdep, ties->core_alloc*sizeof(float))) == NULL) {
                                SaBadMemAlloc("Sa");
    }
    else {
      ties->temp_equivlogdep=(float*)charptr;
    }
    if((charptr=(char*)realloc(ties->coredepth, ties->core_alloc*sizeof(float))) == NULL) {
                             SaBadMemAlloc("Sa");
    }
    else {
      ties->coredepth=(float*)charptr;
    }
  }

  /* assign temp_equivlogdep 
  */
  for(i=0; i<splice[s]->numvalues; ++i) {
    if(splice[s]->value[i]->quality  & BAD_SB_DEPTH) {
      ties->temp_equivlogdep[i] = BADPT;
      ties->coredepth[i] = BADPT;
    }
    else {
      ties->coredepth[i] = splice[s]->value[i]->mcd;
      if(eqlogdepth.autocorr_status == YES) {
        ties->coredepth[i] = ties->coredepth[i] * (eqlogdepth.autocorr_squish/100.0);
      }
      ties->temp_equivlogdep[i]= ties->coredepth[i];
    }
  }

  /*if eld table has ties find the ones that belong in splice
     this will be for current data type not necessarily
     the splice just read in
  */
  if(sset >= 0 && sset < splice_cnt) {
    SearchELDTableForSpliceTies(sset);
  }

  if(sset < 0 || sset >= splice_cnt) {
    XtSetSensitive(formEquivLogDepthTool, False);
  }
  else if(have_log == DO) {
    if(ties->nties > 1) {
      XtSetSensitive(pushButtonAutoCorrelate, False);
    }
    else {
      XtSetSensitive(pushButtonAutoCorrelate, True);
    }
    XtSetSensitive(formEquivLogDepthTool, True);
    XtSetSensitive(pushButtonSelectCore, False);
    XtSetSensitive(pushButtonCancelCore, False);
    XtSetSensitive(pushButtonClearTie, False);
    XtSetSensitive(pushButtonClearLastTieSaved, False);
    XtSetSensitive(pushButtonShowHideTieList, True);
    XtSetSensitive(pushButtonAddTieToList, False);
    XtSetSensitive(toggleButtonAddTieToCore, False);
    XtSetSensitive(toggleButtonAddTieToAll, False);
    XtSetSensitive(pushButtonOptimizeTie, True);
    XtSetSensitive(pushButtonAdjustToTiesParameters, False);
    XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
    XtSetSensitive(pushButtonSaveChanges, False);
    XtSetSensitive(pushButtonUndoSaveChange, False);
  }

  DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
  DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);

  XtUnmanageChild(messageBoxWorking);
  if(cursor) {
    XUndefineCursor(XtDisplay(formMain), XtWindow(formMain));
    XFreeCursor(XtDisplay(formMain), cursor);
  }
}

void SaCullLog(logptr)
LogData *logptr;
{
  int i, culldep;

  if(logptr->info.cull.depmax <= logptr->value[0]->depth) {
    culldep = NO;
  }
  else if(logptr->info.cull.depmin <= logptr->value[0]->depth && logptr->info.cull.depmax >= logptr->value[logptr->data_num-1]->depth) { 
    culldep = NO;
  }
  else {
    culldep = YES;
  }

  for(i=0; i<logptr->data_num; ++i) {
    if(logptr->value[i]->quality & LOG_DECIMATED) {
      logptr->value[i]->quality = LOG_DECIMATED;
    }
    else {
      logptr->value[i]->quality = GOOD;
    }
    if(logptr->value[i]->data == logptr->info.missing_value) {
      logptr->value[i]->quality = logptr->value[i]->quality|BADLOG;
    }
    else {
      if(culldep == YES && (logptr->value[i]->depth < logptr->info.cull.depmin ||
         logptr->value[i]->depth > logptr->info.cull.depmax)) {
        logptr->value[i]->quality = logptr->value[i]->quality|LOG_VALUE_CULLED;
      }
      else {
        logptr->value[i]->quality = logptr->value[i]->quality|SaEvalEquationAlt(logptr->value[i]->data, logptr->info.cull.equat);
      }
    }
  }
}

void SaDecimateLog(logptr)
LogData *logptr;
{
  int i;
  Cursor cursor;

  cursor = XCreateFontCursor(XtDisplay(formMain), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formMain), XtWindow(formMain), cursor);
  }

  for(i=0;i<logdata[lset]->data_num;i++) {
    if(logptr->value[i]->quality & LOG_DECIMATED) {
      logptr->value[i]->quality = logptr->value[i]->quality ^ LOG_DECIMATED;
    }
    if(i%logptr->info.decimate_value!=0) {
      logptr->value[i]->quality = logptr->value[i]->quality|LOG_DECIMATED;
    }
  }

  if(cursor) {
    XUndefineCursor(XtDisplay(formMain), XtWindow(formMain));
    XFreeCursor(XtDisplay(formMain), cursor);
  }
}

void SaCullProcess(ds)
int ds;
{
  int i, j, k, n, hashflag, dum;
  XmString xmstring;
  Cursor cursor;
  void ApplyCoreAdjustToOneCore(), AdjustToTies(), DrawDrawingAreaCores(), DrawDrawingAreaLog(), 
       SaGaussianSplice(), SaCullSplice();
  int  MakeSpliceSeries();

  cursor = XCreateFontCursor(XtDisplay(formMain), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formMain), XtWindow(formMain), cursor);
  }
  XmUpdateDisplay(formMain);

  n = 0;
  cull.cnt = 0;
  for(i=0; i<data[ds]->numholes; ++i) {
    if(cull.method==CULL_TABLE || cull.method == CULL_PARAMETER_AND_TABLE) {
                                        
      if(i==0 && hashflag!=CREATE_HASH) {
        hashflag=CREATE_HASH;
      }
      else if(i+1==data[dset]->numholes) {
        hashflag=DESTROY_HASH;
      }
      else {
        hashflag=0;
      }
    }
    else {
      hashflag=0;
    }

    if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("SaCullProcess");
    (void)sprintf(charptr,"\nProcess : Culling data hole %c.",data[ds]->holes[i]->name);
    SaUpdateReport(charptr);
    free(charptr);

    /* take away all quality bitwise values for culling
       but if the pt was decimated leave that in quality
    */
    for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
      for(k=0; k<data[ds]->holes[i]->core[j]->numvalues; ++k) {
        if(data[ds]->holes[i]->core[j]->value[k]->quality & DECIMATED) {
          data[ds]->holes[i]->core[j]->value[k]->quality = DECIMATED;
        }
        else {
          data[ds]->holes[i]->core[j]->value[k]->quality = GOOD;
        }
        if(data[ds]->holes[i]->core[j]->value[k]->mbsf == BAD_CODE) {
          data[ds]->holes[i]->core[j]->value[k]->quality = data[ds]->holes[i]->core[j]->value[k]->quality|BAD_SB_DEPTH;
        }
      }
      n = n + data[ds]->holes[i]->core[j]->numvalues;
    } 
    /* cull-recull data set
    */
    SaCull(ds, data[ds]->holes[i],data[ds]->type,cull,hashflag);

    if(smooth.method == GAUSSIAN) {
      if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("SaCullProcess");
      (void)sprintf(charptr,"\nProcess : Resmoothing data hole %c.",data[ds]->holes[i]->name);
      SaUpdateReport(charptr);
      free(charptr);
      SaGaussianHole(data[ds]->holes[i]);
    }
  }
  if(cull.cnt < (n * TOOMANYCULLED)) {
    data[ds]->culled = YES;
  }
  else {
    (void)sprintf(buffer, "Error; Too many values culled. Check cull parameters and recull.");
    SaManageWarningBox(buffer);
    /* take away the culling
    */
    data[ds]->culled = NO;
    for(i=0; i<data[ds]->numholes; ++i) {
      for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
        for(k=0; k<data[ds]->holes[i]->core[j]->numvalues; ++k) {
          if(data[ds]->holes[i]->core[j]->value[k]->quality & DECIMATED) {
            data[ds]->holes[i]->core[j]->value[k]->quality = DECIMATED;
          }
          else {
            data[ds]->holes[i]->core[j]->value[k]->quality = GOOD;
          }
          if(data[ds]->holes[i]->core[j]->value[k]->mbsf == BAD_CODE) {
            data[ds]->holes[i]->core[j]->value[k]->quality = data[ds]->holes[i]->core[j]->value[k]->quality|BAD_SB_DEPTH;
          }
        }
      }
    }
  }

  FindMinMax(ds);
  SetMinMax(ds);

  if(data[ds]->culled == YES) {
    if(have_splice_table == DO) {
      MakeSpliceSeries(sset, &dum);
      SaGaussianSplice(sset);
    }
    else if(have_splice == DO) {
      for(i=0; i<splice_cnt; ++i) {
        if(splice[i]->data_type == data[dset]->type) {
          SaCullSplice(i, splice[i]->data_type);
          if(splice[i]->smooth.method == GAUSSIAN) {
            SaGaussianSplice(i);
          }
        }
      }
    }
  }

  if(XtIsManaged(XtParent(drawingAreaCores))) {
    core_clip_what = NONE;
    DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
  }
  if(have_splice == DO) {
    if(XtIsManaged(XtParent(drawingAreaLog))) {
      log_clip_what = NONE;
      DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
    }
  }

  if(cursor) {
    XUndefineCursor(XtDisplay(formMain), XtWindow(formMain));
    XFreeCursor(XtDisplay(formMain), cursor);
  }
}

void SaSmoothProcess()
{
  int i, j, k;
  XmString xmstring;
  Cursor cursor;
  void DrawDrawingAreaCores(), DrawDrawingAreaLog(), SaGaussianSplice();

  cursor = XCreateFontCursor(XtDisplay(formMain), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formMain), XtWindow(formMain), cursor);
  }
  XmUpdateDisplay(formMain);

  if(smooth_who == SMOOTHCORE) {

    if(smooth.method == GAUSSIAN) {
      for(i=0; i<dataset_cnt; ++i) {
        for(j=0; j<data[i]->numholes; ++j) {
          if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("SaSmoothProcess");
          (void)sprintf(charptr,"\nProcess : Smoothing data hole %c.",data[i]->holes[j]->name);
          SaUpdateReport(charptr);
          free(charptr);
          SaGaussianHole(data[i]->holes[j]);
        }
        FindMinMax(i);
        SetMinMax(i);
      }
    }
    else if(smooth.method == NONE) {
      if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("SaSmoothProcess");
      (void)sprintf(charptr,"\nProcess : No smoothing of data.");
      SaUpdateReport(charptr);
      free(charptr);
    }

    if(have_splice == DO) {
      for(i=0; i<splice_cnt; ++i) {
        if(smooth.method == NONE) {
          splice[i]->smooth.method = NONE;
        }
        else if(smooth.method == GAUSSIAN) {
          splice[i]->smooth.method = GAUSSIAN;
          splice[i]->smooth.width = smooth.width;
          splice[i]->smooth.width_units = smooth.width_units;
          SaGaussianSplice(i);
        }
        else {
          splice[i]->smooth.method = NONE;
        }
        if(smooth.plot == UNSMOOTH) {
          splice[i]->smooth.plot = UNSMOOTH;
        }
        else if(smooth.plot == SMOOTH) {
          splice[i]->smooth.plot = SMOOTH;
        }
        else if(smooth.plot == SMOOTHBOTH) {
          splice[i]->smooth.plot = SMOOTHBOTH;
        }
        else {
          splice[i]->smooth.plot = UNSMOOTH;
        }
      }
    }
  }

  else if(smooth_who == SMOOTHLOG) {

    if(logdata[lset]->info.smooth.method == GAUSSIAN) {
      if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("CullLOgOK 2");
      (void)sprintf(charptr,"\nProcess : Smoothing Log Data.");
      SaUpdateReport(charptr);
      free(charptr);
      SaGaussianLog(logdata[lset]);
      FindLogMinMax(logdata[lset]);
      SetLogMinMax(logdata[lset]);
    }
    else if(logdata[lset]->info.smooth.method == NONE) {
      if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("CullLOgOK 2");
      (void)sprintf(charptr,"\nProcess : No smoothing of Log Data.");
      SaUpdateReport(charptr);
      free(charptr);
    }
  }

  if(smooth_who == SMOOTHCORE) {
    if(XtIsManaged(XtParent(drawingAreaCores))) {
      core_clip_what = NONE;
      DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
    }
  }
  if(smooth_who == SMOOTHLOG || (smooth_who == SMOOTHCORE && have_splice == DO)) {
    if(XtIsManaged(XtParent(drawingAreaLog))) {
      log_clip_what = NONE;
      DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
    }
  }

  if(cursor) {
    XUndefineCursor(XtDisplay(formMain), XtWindow(formMain));
    XFreeCursor(XtDisplay(formMain), cursor);
  }
}

int     SaInputSplicerTable(client_data)
XtPointer       client_data;
{
  XmString        xmstring;
  int     i,j,k, err, status, sm, dum1, dum2, splicer_read_done, SaReadSplicerTable();
  Core    *coreptr;
  Value   *valueptr;
  char    *ptr, *string, *logstring;
  FILE    *fptr;
  void  DrawDrawingAreaLog(), CheckSpliceTableOffsets(), SetupMakingSpliceSeries(), GetTieArrayNum();

  if((fptr=fopen(splicertable.filename,"r"))==NULL)
  {
    info.error=True;
    info.fatal=False;
    info.caller=NONE;
    info.type=XmDIALOG_WARNING;
    info.default_button=XmDIALOG_OK_BUTTON;
    if((info.message=(char*)calloc((int)strlen(splicertable.filename)+50,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaInputSplicerTable 1");
    (void)sprintf(info.message,"Error opening input file %s\n",splicertable.filename);
    SaManageMessage(info);
    return(True);
  }
  else
  {
    if((charptr=(char*)calloc((int)strlen(splicertable.filename)+50,CHAR_SIZE))==NULL)
                               SaBadMemAlloc("SaInputSplicerTable 1");
    (void)sprintf(charptr,"\nSplicer table file %s used.\n",splicertable.filename);
    SaUpdateReport(charptr);
    free(charptr);
    XtResizeWindow(messageBoxWorking);
    XmUpdateDisplay(messageBoxWorking);
    /* set up splicer table */
    splicertable.alloc_cnt=ALLOC_INT;
    splicertable.data_cnt=1; /* first value isn't read from table */
    splicer_read_done = False;
    if((splicertable.data=(SplicerTableInfo*)calloc(splicertable.alloc_cnt,sizeof(SplicerTableInfo)))==NULL)
                                SaBadMemAlloc("SaInputSplicerTable");
    /* read splicer table */
    while(!splicer_read_done)
    {
      if(splicertable.data_cnt+1>=splicertable.alloc_cnt)
      {
        if((ptr=(char*)realloc((char*)splicertable.data,(splicertable.alloc_cnt+=ALLOC_INT)*sizeof(SplicerTableInfo)))==NULL)
                                                SaBadMemAlloc("SaInputSplicerTable");
        else
          splicertable.data=(SplicerTableInfo*)ptr;
      }
      if((status=SpReadSplicerTable(fptr,splicertable.data,&splicertable.data_cnt))==EOF)
      {
        splicer_read_done=True;
      }
      else if(!(status==9 || status==19))     /* error reading */
      {
        fclose(fptr);
        free(splicertable.data);
        XtUnmanageChild(messageBoxWorking);
        info.error=True;
        info.fatal=False;
        if((info.message=(char*)calloc((int)strlen(splicertable.filename)+100,CHAR_SIZE))==NULL)
                                              SaBadMemAlloc("SaInputSplicerTable");
        (void)sprintf(info.message,"Error reading splicer table %s.",splicertable.filename);
        info.caller=NONE;
        info.type=XmDIALOG_ERROR;
        info.default_button=XmDIALOG_OK_BUTTON;
        SaManageMessage(info);
        return(True);
      }
    }
    fclose(fptr);

    if((charptr=(char*)calloc((int)strlen(splicertable.filename)+200,CHAR_SIZE))==NULL)
                              SaBadMemAlloc("SaInputSplicerTable 3");
    (void)sprintf(charptr,"Splicer : Read splicer table file %s (total ties %d)",splicertable.filename,
                                      (int)splicertable.data_cnt/2);
    SaUpdateReport(charptr);
    xmstring=XmStringCreateLocalized(charptr);
    XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
    XtResizeWindow(messageBoxWorking);
    XmUpdateDisplay(messageBoxWorking);
    XmStringFree(xmstring);
    free(charptr);
  }

  if(splicertable.data_cnt > 1) {
    splicertable.data[0].site = splicertable.data[1].site;
    splicertable.data[0].hole=splicertable.data[1].hole;
    splicertable.data[0].core=splicertable.data[1].core;
    splicertable.data[0].core_type=splicertable.data[1].core_type;
    (void)strcpy(splicertable.data[0].section,"1");
    splicertable.data[0].top_int=0.0;
    splicertable.data[0].bot_int=0.0;
    splicertable.data[0].mbsf=0.0;
    splicertable.data[0].mcd=splicertable.data[0].mbsf +(splicertable.data[1].mcd-splicertable.data[1].mbsf);
  }

  /* check affine table for offset mismatch
     if everything ok make the splice series
  */

  if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                              SaBadMemAlloc("SaInputSplicerTable 3");
  (void)sprintf(charptr,"Splicer : Making the splice series.");
  SaUpdateReport(charptr);
  xmstring=XmStringCreateLocalized(charptr);
  XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
  XtResizeWindow(messageBoxWorking);
  XmUpdateDisplay(messageBoxWorking);
  XmStringFree(xmstring);
  free(charptr);

  if(have_affine == DO) {
    CheckSpliceTableOffsets();
  }
  else {
    (void)sprintf(buffer, "Warning: You do not have an affine table.");
    SaManageWarningBox(buffer);
  }

  have_splice_table = DO;

  if(have_cores == DO) {
    SetupMakingSpliceSeries();
  }
  if(eqlogdepth.read_equil_log == YES) {
    XtSetSensitive(pushButtonSaveSplice, True);
  }
  else {
    XtSetSensitive(pushButtonSaveSplice, False);
  }

  /* note; do not need to cull or decimate splice series; it gets its quality from core data
  */

  if(have_splice == DO && have_log == DO) {
    if(ties->nties > 1) {
      ties->tie[0]->log = -1;
      ties->tie[0]->array = -1;
      for(i=1; i<ties->nties; ++i) {
        GetTieArrayNum(SPLICEDATA, ties->tie[i], sset, 0, 0, &err);
        GetTieArrayNum(LOGDATA, ties->tie[i], lset, 0, 0, &err);
      }
      XtSetSensitive(pushButtonAutoCorrelate, False);
    }
    else {
      XtSetSensitive(pushButtonAutoCorrelate, True);
    }
    XtSetSensitive(textFieldSelectHole, False);
    XtSetSensitive(textFieldSelectCore, False);
    XtSetSensitive(formEquivLogDepthTool, True);
    XtSetSensitive(pushButtonSelectCore, False);
    XtSetSensitive(pushButtonCancelCore, False);
    XtSetSensitive(pushButtonClearTie, False);
    XtSetSensitive(pushButtonClearLastTieSaved, False);
    XtSetSensitive(pushButtonShowHideTieList, True);
    XtSetSensitive(pushButtonAddTieToList, False);
    XtSetSensitive(toggleButtonAddTieToCore, False);
    XtSetSensitive(toggleButtonAddTieToAll, False);
    XtSetSensitive(pushButtonOptimizeTie, True);
    XtSetSensitive(pushButtonAdjustToTiesParameters, False);
    XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
    XtSetSensitive(pushButtonSaveChanges, False);
    XtSetSensitive(pushButtonUndoSaveChange, False);
  }
  else if(have_splice == DO && ties->nties > 1) {
    ties->tie[0]->log = -1;
    ties->tie[0]->array = -1;
    for(i=1; i<ties->nties; ++i) {
      GetTieArrayNum(SPLICEDATA, ties->tie[i], sset, 0, 0, &err);
    }
  }

  XtSetSensitive(pushButtonOpenSplicer, False);
  XtSetSensitive(pushButtonOpenSpliceTable, False);
  XtSetSensitive(pushButtonClearSplice, True);
  XtSetSensitive(pushButtonClearAll, True);

  XtUnmanageChild(messageBoxWorking);

  if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
    if(XtIsManaged(XtParent(drawingAreaLog))) {
      DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
    }
  }

  return(True);
}

void SaCullSplice(s, type)
int s;
int type;
{
  int j, err;
  char *cstring;

  /*get the cull values for this data type
  */
  switch(type)
  {
    case GRAPE:
      err = SaGetEquationValues(&cull.grape);
      break;
    case PWAVE:
      err = SaGetEquationValues(&cull.pwave);
      if(openfileinfo.in_data_format == MST95REPORT) {
         cstring=XmTextGetString(textFieldCullSignal);
         cull.pwave_signal_max=(float)atof(cstring);
         XtFree(cstring);
      }
      break;
    case SUSCEPTIBILITY:
      err = SaGetEquationValues(&cull.susceptibility);
      break;
    case NATURALGAMMA:
      err = SaGetEquationValues(&cull.naturalgamma);
      break;
    case REFLECTANCE:
      err = SaGetEquationValues(&cull.reflectance);
      break;
    case OTHERTYPE:
      err = SaGetEquationValues(&cull.otherodp);
      break;
    default:
      err = 1;
      break;
  }
  if(err != 0) {
    return;
  }

  /* take away all quality bitwise values for culling
     but if the pt was decimated leave that in quality
  */
  for(j=0; j<splice[s]->numvalues; ++j) {
    if(splice[s]->value[j]->quality & DECIMATED) {
      splice[s]->value[j]->quality = DECIMATED;
    }
    else {
      splice[s]->value[j]->quality = GOOD;
    }
  }

  for(j=0;j<splice[s]->numvalues;j++) {
    if(type==GRAPE) {
      splice[s]->value[j]->quality= splice[s]->value[j]->quality|SaEvalEquation(splice[s]->value[j]->data,cull.grape);
    }
    else if(type==PWAVE) {
      splice[s]->value[j]->quality= splice[s]->value[j]->quality|SaEvalEquation(splice[s]->value[j]->data,cull.pwave);
      if(openfileinfo.in_data_format == MST95REPORT) {
        /* splice series does not have sig strength. what to do?
        */
      }
    }
    else if(type==SUSCEPTIBILITY) {
      splice[s]->value[j]->quality= splice[s]->value[j]->quality| SaEvalEquation(splice[s]->value[j]->data,cull.susceptibility);
    }
    else if(type==NATURALGAMMA) {
      splice[s]->value[j]->quality= splice[s]->value[j]->quality|SaEvalEquation(splice[s]->value[j]->data,cull.naturalgamma);
    }
    else if(type==REFLECTANCE) {
      splice[s]->value[j]->quality= splice[s]->value[j]->quality|SaEvalEquation(splice[s]->value[j]->data,cull.reflectance);
    }
    else if(type == OTHERTYPE) {
      splice[s]->value[j]->quality=splice[s]->value[j]->quality|SaEvalEquation(splice[s]->value[j]->data,cull.otherodp);
    }
    else {
      splice[s]->value[j]->quality=splice[s]->value[j]->quality|SaEvalEquation(splice[s]->value[j]->data,cull.otherodp);
    } 
  }
}

void    SaDecimateSplice(s ,deci_num)
int     s;
int     deci_num;
{
  int    j;
  Cursor cursor;

  cursor = XCreateFontCursor(XtDisplay(formMain), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formMain), XtWindow(formMain), cursor);
  }

  for(j=0;j<splice[s]->numvalues;j++) {
    if(splice[s]->value[j]->quality & DECIMATED) {
      splice[s]->value[j]->quality = splice[s]->value[j]->quality ^ DECIMATED;
    }
    if(j%deci_num!=0) {
      splice[s]->value[j]->quality=splice[s]->value[j]->quality|DECIMATED;
    }
  }

  if(cursor) {
    XUndefineCursor(XtDisplay(formMain), XtWindow(formMain));
    XFreeCursor(XtDisplay(formMain), cursor);
  }
}

void SaGaussianSplice(s)
int  s;
{
  int     i,j,k,l,m,n,num_interp=0,inc,datptr_cnt,max_depths=0,num_depths=0,this_num=0,end_cnt,highfound,lowfound;
  float   halfwidth,*weight,target,end_depth;
  float   *datptr,*sm_datptr,start_depth;
  double  rint();
  void    gauss_weights();

  /* use gaussian filter */
  if(splice[s]->smooth.method==GAUSSIAN)
  {
    /* must use an odd number of weights */
    if(splice[s]->smooth.width%2==0)
      splice[s]->smooth.width+=1;
    halfwidth=(int)((splice[s]->smooth.width-1)/2);
    /* get filter weights */
    if((weight=(float*)calloc(splice[s]->smooth.width+1,sizeof(float)))==NULL)
      SaBadMemAlloc("Sa");
    gauss_weights(splice[s]->smooth.width,weight);
    /* allocate storage arrays */
    if(splice[s]->smooth.width_units==POINTS)
    {
      sm_datptr=NULL;
      if((datptr=(float*)calloc(splice[s]->smooth.width+1,sizeof(float)))==NULL)
                                SaBadMemAlloc("Sa");
    }
    else if(splice[s]->smooth.width_units==DEPTH) 
    {
      max_depths = (splice[s]->value[splice[s]->numvalues-1]->mcd-splice[s]->value[0]->mcd)*100 + 1;
      if((sm_datptr=(float*)calloc(max_depths+100,sizeof(float)))==NULL)
                                SaBadMemAlloc("Sa");
      if((datptr=(float*)calloc(max_depths+100,sizeof(float)))==NULL)
                                SaBadMemAlloc("Sa");
    }

    if(splice[s]->smooth.width_units==POINTS)
    {
      /* loop through data (first and last "halfwidth" values are same as raw data if good! */
      for(j=0;j<halfwidth && j<splice[s]->numvalues;j++)
      {
        if(splice[s]->value[j]->quality == GOOD)
        {
          splice[s]->value[j]->sm_data=splice[s]->value[j]->data;
          splice[s]->value[j]->smooth_ok = SMTAIL;
        }
        else
        {
          splice[s]->value[j]->smooth_ok = SMBADDATA;
        }
      }

      if(j>= splice[s]->numvalues)
      {
      }
      else
      {
        /* loop through data until at last half width */
        for(;j<splice[s]->numvalues-halfwidth;j++)
        {
          /* let smooth be ok until proven otherwise */
          splice[s]->value[j]->smooth_ok = SMOK;
          inc=j-halfwidth;    /* first value in smoothing */
          /* raw value at this point must be good */
          if(splice[s]->value[j]->quality!=GOOD)
          {
            splice[s]->value[j]->smooth_ok = SMBADDATA;
          }
          /* fill data arrays */
          for(k=inc,num_interp=0,datptr_cnt=0;datptr_cnt<splice[s]->smooth.width && k<splice[s]->numvalues &&
                                                      !(splice[s]->value[j]->quality);k++,datptr_cnt++)
          {
            if(splice[s]->value[k]->quality==GOOD)
            {
              datptr[datptr_cnt]=splice[s]->value[k]->data;
            }
            else      /* interpolate data point */
            {
              /* find next lower point in width range */
              for(l=k-1;l>=inc && splice[s]->value[l]->quality!=GOOD;l--);
              if(l<inc)       /* no lower point in range */
              {
                splice[s]->value[j]->smooth_ok = SMTOOFEWPTS;
              }
              else
              {
                /* find next higher point in width range */
                for(m=k+1;m<=j+halfwidth && splice[s]->value[m]->quality!=GOOD;m++);
                if(m>j+halfwidth)     /* no higher point in range */
                {
                  splice[s]->value[j]->smooth_ok = SMTOOFEWPTS;
                }
                else
                {
                  datptr[datptr_cnt]=splice[s]->value[l]->data+(splice[s]->value[m]->data -
                                                              splice[s]->value[l]->data)*
                              (splice[s]->value[m]->mbsf-splice[s]->value[l]->mbsf)/
                                                      splice[s]->value[k]->mbsf;
                  num_interp++;
                }
              }
            }
          }
          if(num_interp>halfwidth)
          {
            splice[s]->value[j]->smooth_ok = SMBADCALC;
          }
          /* compute smoothed value */
          if(splice[s]->value[j]->smooth_ok < SMOK)
          {
          }
          else if(!splice[s]->value[j]->quality)
          {
            splice[s]->value[j]->sm_data=0.0;
            for(k=0;k<splice[s]->smooth.width;k++)
              splice[s]->value[j]->sm_data+=datptr[k]*weight[k+1];
          }
          else
          {
            splice[s]->value[j]->smooth_ok = SMBADCALC;
          }
        }
        if(j<splice[s]->numvalues)
        {
          /* loop through last half width values */
          for(;j<splice[s]->numvalues;j++)
          {
            if(splice[s]->value[j]->quality == GOOD)
            {
              splice[s]->value[j]->sm_data=splice[s]->value[j]->data;
              splice[s]->value[j]->smooth_ok = SMTAIL;
            }
            else
            {
              splice[s]->value[j]->smooth_ok = SMBADDATA;
            }
          }
        }

        /* check if have some smoothed points
        */
        n = 0;
        for(j=0; j<splice[s]->numvalues; ++j)
        {
          if(splice[s]->value[j]->smooth_ok == SMOK || splice[s]->value[j]->smooth_ok == SMTAIL)
          {
            ++n;
          }
        }
        if(n < TOOFEWSMOOTHED)
        {
        }
      }
    }
    else if(splice[s]->smooth.width_units==DEPTH)
    {
      /* find first & last good points */
      for(j=0;j<splice[s]->numvalues && splice[s]->value[j]->quality!=GOOD ;j++);
      for(k=splice[s]->numvalues-1;k>=0 && splice[s]->value[k]->quality!=GOOD;k--);
      end_cnt=k;
      if(end_cnt<0 || j>=splice[s]->numvalues)
      {
      }
      else
      {
        start_depth=splice[s]->value[j]->mcd*100.0;
        end_depth=splice[s]->value[end_cnt]->mcd*100.0;
        num_depths=1+end_depth-start_depth;
        if(num_depths<=halfwidth)
        {
        }
        else if(num_depths>max_depths)
        {
        }
        else
        {
          /* interpolate 1 cm array of entire core */
          datptr[l=0]=splice[s]->value[j]->data;
          for(m=j,l++,target=start_depth+1;target<end_depth && l<max_depths;target++,l++)
          {
            /* find value >= to target value */
            highfound=lowfound=False;
            while(m<splice[s]->numvalues && k>=0 && !(lowfound&highfound))
            {
              if(!highfound && !(target<=rint((double)(splice[s]->value[m]->mcd*100))
                                                      && splice[s]->value[m]->quality==GOOD))
                m++;
              else if(!highfound)
              {
                highfound=True;
                k=m;
              }
              else
              {
                if(!(target>rint((double)(splice[s]->value[k]->mcd*100))&&
                                      splice[s]->value[k]->quality==GOOD))
                  k--;
                else
                  lowfound=True;
              }
            }
            if(m>=splice[s]->numvalues || k<0)
            {
              free(datptr);
              datptr = NULL;
              free(sm_datptr);
              sm_datptr = NULL;
              free(weight);
              weight = NULL;
              info.error=True;
              if((info.message=calloc(200,CHAR_SIZE))==NULL)
                                    SaBadMemAlloc("SaGaussianSplice");
              (void)sprintf(info.message,"Error trying to create smooth by cm array!");
              SaManageMessage(info);
              return;
            }
            else {
              if(target==rint((double)(splice[s]->value[m]->mcd*100)))       /* exact match */
                                      datptr[l]=splice[s]->value[m]->data;
              else /* interpolate */
                datptr[l]=splice[s]->value[k]->data+(splice[s]->value[m]->data -splice[s]->value[k]->data)*
                                                      (target/100.0-splice[s]->value[k]->mcd)/
                                      (splice[s]->value[m]->mcd-splice[s]->value[k]->mcd);
            }
          }
          if(l>max_depths)
          {
          }
          else
          {
            datptr[l]=splice[s]->value[end_cnt]->data;
            /* smooth data array */
            /* loop through data (first and last "halfwidth" values are same as raw data */
            for(j=0;j<halfwidth;j++)
              sm_datptr[j]=datptr[j];
            /* compute smoothed value */
            for(;j<num_depths-halfwidth;j++)
            {
              sm_datptr[j]=0.0;
              for(k=0,l=j-halfwidth+1;k<splice[s]->smooth.width;k++,l++)
                sm_datptr[j]+=datptr[l]*weight[k+1];
            }
            /* loop through last half width values */
            /* fill in real smoothed array */
            for(j=0;j<splice[s]->numvalues;j++)
            {
              if(splice[s]->value[j]->quality != GOOD)
              {
                splice[s]->value[j]->smooth_ok = SMBADDATA;
              }
              else
              {
                if(rint((double)(splice[s]->value[j]->mcd*100.0))<(start_depth+halfwidth))
                {
                  splice[s]->value[j]->sm_data=splice[s]->value[j]->data;
                  splice[s]->value[j]->smooth_ok = SMTAIL;
                }
                else if(rint((double)(splice[s]->value[j]->mcd*100.0))>(end_depth-halfwidth))
                {
                  splice[s]->value[j]->sm_data=splice[s]->value[j]->data;
                  splice[s]->value[j]->smooth_ok = SMTAIL;
                }
                else
                {
                  k=(int)(rint((double)(splice[s]->value[j]->mcd*100.0))-start_depth);
                  splice[s]->value[j]->sm_data=sm_datptr[k];
                  splice[s]->value[j]->smooth_ok = SMOK;
                }
              }
            }
          }
        }
      }
    }
    if(sm_datptr)
    {
      free(sm_datptr);
      sm_datptr=NULL;
    }
    if(datptr)
    {
      free(datptr);
      datptr=NULL;
    }
    if(weight)
    {
      free(weight);
      weight=NULL;
    }
  }

  else {

  }
}

int SaOutputCoreData()
{
  int i, err, len;
  char *file;
  String pref, suff;
  XmString xmstring;
  int WriteCoreData();

  /* get the file(s) prefix and suffix
  */
  pref = XmTextGetString(textFieldSaveCoreDataFilePref);
  suff = XmTextGetString(textFieldSaveCoreDataFileSuff);

  if((file=(char*)calloc((int)1000,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaOutputCoreData");
  len = (int)strlen(save_coredata_dirname);
  if(strcmp(&save_coredata_dirname[len-1], "/") == 0) {
    *strrchr(save_coredata_dirname,'/')= '\0';
  }

  for(i=0; i<data[dset]->numholes; ++i) {
    (void)sprintf(file, "%s/%s%d%c.%s", save_coredata_dirname, pref, data[dset]->site, data[dset]->holes[i]->name, suff); 

    if((charptr=(char*)calloc(100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaSmoothCore 1");
    (void)sprintf(charptr,"\nWriting file %s",file);
    SaUpdateReport(charptr);
    xmstring=XmStringCreateLocalized(&charptr[1]);
    XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
    XtResizeWindow(messageBoxWorking);
    XmUpdateDisplay(messageBoxWorking);
    XmStringFree(xmstring);
    free(charptr);
    
    err = WriteCoreData(data[dset]->holes[i], file, dset);
    if(err > 0) {
      (void)sprintf(buffer, "Error; Writing file %s.", file);
                  SaManageWarningBox(buffer);     
      XtUnmanageChild(messageBoxWorking);
      return 1;
    }
  }
  XtFree(pref);
  XtFree(suff);
  free(file);
  file = NULL;
  XtUnmanageChild(messageBoxWorking);

  return 0;
}

void SaOutputSpliceData()
{
  int err;
  int WriteSpliceData();

  WriteSpliceData(sset, dset); 
}

void SaOutputLogCoreDataPairs(file)
char *file;
{
  int WriteLogCoreDataPairs();

  WriteLogCoreDataPairs(file);
}

int     currentstrat_file=0;
int     strat_out_open=False;
int SaOutputStrat()
{
        int             i,SaWriteStrat();
        XmString        xmstring;
        if(!strat_out_open)
        {
                if((fptr=fopen(outstrat_filename,"w"))==NULL)
                {
                        info.error=True;
                        if((info.message=(char*)calloc((int)strlen(outstrat_filename)+100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaOutputStrat 1");
                        (void)sprintf(info.message,"Error Opening output strat file %s.",outstrat_filename);
                        info.fatal=False;
                        info.caller=NONE;
                        info.type=XmDIALOG_WARNING;
                        info.default_button=XmDIALOG_OK_BUTTON;
                        SaManageMessage(info);
                        return(True);
                }
                else
                {
                        strat_out_open=True;
                        if((charptr=(char*)calloc((int)strlen(outstrat_filename)+100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaOutputStrat 2");
                        (void)sprintf(charptr,"\nOutput strat file %s.",outstrat_filename);
                        SaUpdateReport(charptr);
                        free(charptr);
                }
        }
        if(currentstrat_file<strat_file_cnt)
        {
                for(i=0;i<strat[currentstrat_file]->data_cnt;i++)
                        (void)SaWriteStrat(fptr,&strat[currentstrat_file]->data[i]);
                if((charptr=(char*)calloc((int)strlen(strat[currentstrat_file]->filename)+100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaOutputStrat 3");
                (void)sprintf(charptr,"Strat: Output data from input file %s.",strat[currentstrat_file]->filename);
                xmstring=XmStringCreateLocalized(&charptr[1]);
                XtVaSetValues(messageBoxWorking,XmNmessageString,xmstring, NULL);
                XmStringFree(xmstring);
                free(charptr);
                currentstrat_file++;
                return(False);
        }
        else
        {
                fclose(fptr);
                strat_out_open=False;
                currentstrat_file=0;
                XtUnmanageChild(messageBoxWorking);
                return(True);
        }
}
