/*
	file: sagan_defaults.c

	These functions are used for reading and writing the
	default files for sagan programs.

  void Sacheck_defaults()
  int Saread_defaults()
  void Sawrite_defaults()

*/
#include <stdio.h>
#include <Xm/Xm.h>
#include <malloc.h>
#include <math.h>
#include <string.h>
#include "sagan-creation-c.h"
#include "sagan_vars.h"

#define	MAXLINE	(1000)

void Sacheck_defaults()
{
	/* this function will eventually check for valid defaults */

  if(smooth.method == NONE) {
    smooth.plot = UNSMOOTH;
  }
}

/*      Function Name: Saread_defaults
 *
 *      Description:   	This function reads the set defaults from the
 *		       	supplied input file.
 *			
 *
 *      Arguments:      char	*filename:	name of defaults file
 */
int Saread_defaults(filename)
char	*filename;
{
	FILE	*fptr;
	char	line[MAXLINE],flag[MAXLINE],equal[2],value[MAXLINE];
	int	value_int,i;

/* set automatic defaults */

        job = MCD_VS_LOG;

        ancill.data_cnt = 0;
	ancill.filename=ancill.dirname=NULL;
	ancill.format=ancill.id_num=ancill.id_alloc=0;
	ancill.missing_value=-999.25;
	ancill.axis_min[0]=0.0;
	ancill.axis_max[0]=0.0;
        ancill.axis_min[1]=0.0;
        ancill.axis_max[1]=0.0;
        ancill.axis_min[2]=0.0;
        ancill.axis_max[2]=0.0;
	ancill.axislen=LOG_AXIS_LEN;
	ancill.decimate_value=1;
        loginfo.missing_value = -999.25;
/* smooth values */
	smooth.method=NONE;
	smooth.width=9;
	smooth.width_units=POINTS;
        smooth.plot = UNSMOOTH;
        loginfo.smooth.method=NONE;
        loginfo.smooth.width=9;
        loginfo.smooth.width_units=POINTS;
        loginfo.smooth.plot = UNSMOOTH;
/* decimate_value */
	decimate_value=1;
        loginfo.decimate_value = 1;
/* cull values */
	cull.method=CULL_PARAMETERS;
	cull.top=5.0;
	cull.max_core=999;
	cull.by_core_num=False;
	if((cull.filename= (char*)calloc(25,CHAR_SIZE))==NULL)
		SaBadMemAlloc("SaInitialize 3");
	(void)strcpy(cull.filename,"culltable.dat");
	if((cull.dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
		SaBadMemAlloc("SaInitialize 4");
	(void)strcpy(cull.dirname,current_path);
	cull.grape.sign1=LESS_THAN_OR_EQUAL_TO;
	cull.grape.sign2=GREATER_THAN_OR_EQUAL_TO;
	cull.grape.join=OR;
	cull.grape.value1=1.0;
	cull.grape.value2=2.8;
	cull.pwave.sign1=LESS_THAN;
	cull.pwave.sign2=GREATER_THAN_OR_EQUAL_TO;
	cull.pwave.join=ONLY;
	cull.pwave.value1=0.0;
	cull.pwave.value2=255;
	cull.pwave_signal_max=167.0;
	cull.susceptibility.sign1=LESS_THAN;
	cull.susceptibility.sign2=GREATER_THAN;
	cull.susceptibility.join=ONLY;
	cull.susceptibility.value1=-99999.0;
	cull.susceptibility.value2=99999.0;
	cull.naturalgamma.sign1=LESS_THAN;
	cull.naturalgamma.sign2=GREATER_THAN_OR_EQUAL_TO;
	cull.naturalgamma.join=ONLY;
	cull.naturalgamma.value1=0.0;
	cull.naturalgamma.value2=99999.0;
	cull.reflectance.sign1=LESS_THAN_OR_EQUAL_TO;
	cull.reflectance.sign2=GREATER_THAN_OR_EQUAL_TO;
	cull.reflectance.join=OR;
	cull.reflectance.value1=0.0;
	cull.reflectance.value2=100.0;
	cull.otherodp.sign1=LESS_THAN;
	cull.otherodp.sign2=GREATER_THAN;
	cull.otherodp.join=OR;
	cull.otherodp.value1=-99999.0;
	cull.otherodp.value2=99999.0;
        loginfo.cull.method=NONE;
        loginfo.cull.equat.sign1=LESS_THAN_OR_EQUAL_TO;
        loginfo.cull.equat.sign2=GREATER_THAN_OR_EQUAL_TO;
        loginfo.cull.equat.join=OR;
        loginfo.cull.equat.value1=0.0;
        loginfo.cull.equat.value2=5000.0;
        loginfo.cull.depmin=0.0;
        loginfo.cull.depmax=0.0;
        /* data type */
        openfileinfo.in_data_type=GRAPE;
/* data format */
        openfileinfo.in_data_format=MST95REPORT;
        openfileinfo.other_format_value_col = 10;
/* io paths */
        affinetable.read_affine = NO;
        if((open_data_directory= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(open_data_directory,current_path);
        if((affinetable.filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(affinetable.filename,"affinetable.table");
        if((affinetable.dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(affinetable.dirname,current_path);
        if((splice_filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(splice_filename,"splice.dat");
        if((splice_dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(splice_dirname,current_path);
        if((splicertable.filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(splicertable.filename,"splice.table");
        if((splicertable.dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(splicertable.dirname,current_path);
        if((loginfo.filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(loginfo.filename,"log.dat");
        if((loginfo.dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(loginfo.dirname,current_path);
        if((ancill.filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(ancill.filename,"ancill.dat");
        if((ancill.dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(ancill.dirname,current_path);
        if((strat_dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(strat_dirname,current_path);
        if((outstrat_filename= (char*)calloc((int)strlen(current_path)+50,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(outstrat_filename,current_path);
        (void)strcat(outstrat_filename,"strat.out");
        eqlogdepth.read_equil_log = NO;
        if((eqlogdepth.filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(eqlogdepth.filename,"eqlogdepth.table");
        if((eqlogdepth.dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(eqlogdepth.dirname,current_path);
        if((save_coredata_filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(save_coredata_filename,"eqlogdepth.table");
        if((save_coredata_dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(save_coredata_dirname,current_path);
        if((save_splice_filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(save_splice_filename,"eqlogdepth.table");
        if((save_splice_dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(save_splice_dirname,current_path);
        if((datapairs_filename= (char*)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(datapairs_filename,"logcorepairs.dat");
        if((datapairs_dirname= (char*)calloc((int)strlen(current_path)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sacheck_defaults");
        (void)strcpy(datapairs_dirname,current_path);
/* text report*/
	if((report.name = (char*)calloc(25,CHAR_SIZE))==NULL)
		SaBadMemAlloc("SaInitialize 6b");
	(void)strcpy(report.name,"sagan.rpt");
/* color stuff */
        colorset = 6;
	(void)strcpy(cust_color[BG_COLOR],"black");
	(void)strcpy(cust_color[CLINE_LABEL],"white");
	(void)strcpy(cust_color[CLINE_COREODPSBD],"yellow");
        (void)strcpy(cust_color[CLINE_COREMCD],"blue");
	(void)strcpy(cust_color[CLINE_CORESMOOTH],"orange");
	(void)strcpy(cust_color[CLINE_COREEQUIVLOGDEPTH],"turquoise");
	(void)strcpy(cust_color[CLINE_LOG],"turquoise");
	(void)strcpy(cust_color[CLINE_LOGADJDEPTH],"green");
	(void)strcpy(cust_color[CLINE_LOGSMOOTH],"orange");
	(void)strcpy(cust_color[CLINE_TIEPT1],"green");
	(void)strcpy(cust_color[CLINE_TIEPT2],"red");
        (void)strcpy(cust_color[CLINE_TIELINESADJUSTDONE],"green");
	(void)strcpy(cust_color[CLINE_TIELINELAST],"pink");
        (void)strcpy(cust_color[CLINE_TIESELECT],"red");
        (void)strcpy(cust_color[CLINE_COREOVERLAYINCORR],"red");
        (void)strcpy(cust_color[CLINE_COREOVERLAYOUTCORR],"pink");
	(void)strcpy(cust_color[CLINE_PALEOMAG],"red");
	(void)strcpy(cust_color[CLINE_DIATOM],"limegreen");
	(void)strcpy(cust_color[CLINE_RAD],"white");
	(void)strcpy(cust_color[CLINE_FORAM],"pink");
	(void)strcpy(cust_color[CLINE_NANNO],"lightblue");
	(void)strcpy(cust_color[CLINE_CORRELATION],"red");
	(void)strcpy(cust_color[CLINE_CORRELATIONBEST],"yellow");
	(void)strcpy(cust_color[CLINE_ADJUSTTOTIESSTATS],"green");
        (void)strcpy(cust_color[CLINE_ANCILLARY1],"red");
        (void)strcpy(cust_color[CLINE_ANCILLARY2],"green");
        (void)strcpy(cust_color[CLINE_ANCILLARY3],"blue");
	
/* open defaults file */
        if((fptr=fopen(filename,"r"))==NULL)
		return(FALSE);

/* loop through defaults file */
	while(fgets(line,MAXLINE,fptr)!=NULL)
	{
	/* check if valid defaults file */
		if(line[0]!= '#' && (int)strlen(line)>0)
		{
			if((int)sscanf(line,"%s %s %s",flag,equal,value)==3 && !(int)strcmp(equal,"="))
			{
			/* set default values */
                                if(!(int)strcmp(flag,"JOB"))
                                        job = atoi(value);
			/* smoothing defaults */
				else if(!(int)strcmp(flag,"SMOOTH_METHOD"))
					smooth.method= atoi(value);
				else if(!(int)strcmp(flag,"SMOOTH_WIDTH"))
					smooth.width=atoi(value);
				else if(!(int)strcmp(flag,"SMOOTH_WIDTH_UNITS"))
					smooth.width_units=atoi(value);
                                else if(!(int)strcmp(flag,"SMOOTH_PLOT"))
                                        smooth.plot=atoi(value);
			/* decimate_values */
				else if(!(int)strcmp(flag,"DECIMATE_VALUE"))
					decimate_value=atoi(value);
			/* cull values */
				else if(!(int)strcmp(flag,"CULL_METHOD"))
					cull.method=atoi(value);
				else if(!(int)strcmp(flag,"CULL_TOP"))
					(void)sscanf(value,"%f",&cull.top);
				else if(!(int)strcmp(flag,"CULL_BY_CORE_NUM"))
					(void)sscanf(value,"%d",&cull.by_core_num);
				else if(!(int)strcmp(flag,"CULL_CORE_MAX"))
					(void)sscanf(value,"%d",&cull.max_core);
				else if(!(int)strcmp(flag,"CULL_TABLE"))
				{
					if((charptr=(char*)realloc((char*)cull.filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
						SaBadMemAlloc("SaInitialize 9");
					else
					{
						cull.filename=charptr;
						(void)strcpy(cull.filename,value);
					}
				}
				else if(!(int)strcmp(flag,"CULL_DIRECTORY"))
				{
					if((charptr=(char*)realloc((char*)cull.dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
						SaBadMemAlloc("SaInitialize 10");
					else
					{
						cull.dirname=charptr;
						(void)strcpy(cull.dirname,value);
					}
				}
				else if(!(int)strcmp(flag,"PWAVE_SIGNAL_MAX"))
					cull.pwave_signal_max=(float)atof(value);
				else if(!(int)strcmp(flag,"GRAPE_EQUATION_1"))
					cull.grape.sign1=atoi(value);
				else if(!(int)strcmp(flag,"GRAPE_EQUATION_2"))
					cull.grape.sign2=atoi(value);
				else if(!(int)strcmp(flag,"GRAPE_JOIN"))
					cull.grape.join=atoi(value);
				else if(!(int)strcmp(flag,"GRAPE_VALUE_1"))
					cull.grape.value1=(float)atof(value);
				else if(!(int)strcmp(flag,"GRAPE_VALUE_2"))
					cull.grape.value2=(float)atof(value);
				else if(!(int)strcmp(flag,"PWAVE_EQUATION_1"))
					cull.pwave.sign1=atoi(value);
				else if(!(int)strcmp(flag,"PWAVE_EQUATION_2"))
					cull.pwave.sign2=atoi(value);
				else if(!(int)strcmp(flag,"PWAVE_JOIN"))
					cull.pwave.join=atoi(value);
				else if(!(int)strcmp(flag,"PWAVE_VALUE_1"))
					cull.pwave.value1=(float)atof(value);
				else if(!(int)strcmp(flag,"PWAVE_VALUE_2"))
					cull.pwave.value2=(float)atof(value);
				else if(!(int)strcmp(flag,"SUSCEPTIBILITY_EQUATION_1"))
					cull.susceptibility.sign1=atoi(value);
				else if(!(int)strcmp(flag,"SUSCEPTIBILITY_EQUATION_2"))
					cull.susceptibility.sign2=atoi(value);
				else if(!(int)strcmp(flag,"SUSCEPTIBILITY_JOIN"))
					cull.susceptibility.join=atoi(value);
				else if(!(int)strcmp(flag,"SUSCEPTIBILITY_VALUE_1"))
					cull.susceptibility.value1=(float)atof(value);
				else if(!(int)strcmp(flag,"SUSCEPTIBILITY_VALUE_2"))
					cull.susceptibility.value2=(float)atof(value);
				else if(!(int)strcmp(flag,"NATURALGAMMA_EQUATION_1"))
					cull.naturalgamma.sign1=atoi(value);
				else if(!(int)strcmp(flag,"NATURALGAMMA_EQUATION_2"))
					cull.naturalgamma.sign2=atoi(value);
				else if(!(int)strcmp(flag,"NATURALGAMMA_JOIN"))
					cull.naturalgamma.join=atoi(value);
				else if(!(int)strcmp(flag,"NATURALGAMMA_VALUE_1"))
					cull.naturalgamma.value1=(float)atof(value);
				else if(!(int)strcmp(flag,"NATURALGAMMA_VALUE_2"))
					cull.naturalgamma.value2=(float)atof(value);
				else if(!(int)strcmp(flag,"REFLECTANCE_EQUATION_1"))
					cull.reflectance.sign1=atoi(value);
				else if(!(int)strcmp(flag,"REFLECTANCE_EQUATION_2"))
					cull.reflectance.sign2=atoi(value);
				else if(!(int)strcmp(flag,"REFLECTANCE_JOIN"))
					cull.reflectance.join=atoi(value);
				else if(!(int)strcmp(flag,"REFLECTANCE_VALUE_1"))
					cull.reflectance.value1=(float)atof(value);
				else if(!(int)strcmp(flag,"REFLECTANCE_VALUE_2"))
					cull.reflectance.value2=(float)atof(value);
				else if(!(int)strcmp(flag,"OTHERODP_EQUATION_1"))
					cull.otherodp.sign1=atoi(value);
				else if(!(int)strcmp(flag,"OTHERODP_EQUATION_2"))
					cull.otherodp.sign2=atoi(value);
				else if(!(int)strcmp(flag,"OTHERODP_JOIN"))
					cull.otherodp.join=atoi(value);
				else if(!(int)strcmp(flag,"OTHERODP_VALUE_1"))
					cull.otherodp.value1=(float)atof(value);
				else if(!(int)strcmp(flag,"OTHERODP_VALUE_2"))
					cull.otherodp.value2=(float)atof(value);
			/* data type */
				else if(!(int)strcmp(flag,"DATA_TYPE"))
                                        openfileinfo.in_data_type=atoi(value);
			/* data format */
				else if(!(int)strcmp(flag,"IN_DATA_FORMAT"))
                                        openfileinfo.in_data_format=atoi(value);
                                else if(!(int)strcmp(flag,"OTHER_FORMAT_COL"))
                                        openfileinfo.other_format_value_col=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_MISSING_VALUE"))
                                     loginfo.missing_value=(float)atof(value);
                                else if(!(int)strcmp(flag,"LOG_DECIMATE_VALUE"))
                                        loginfo.decimate_value=atoi(value);
                        /* smoothing defaults */
                                else if(!(int)strcmp(flag,"LOG_SMOOTH_METHOD"))
                                        loginfo.smooth.method= atoi(value);
                                else if(!(int)strcmp(flag,"LOG_SMOOTH_WIDTH"))
                                        loginfo.smooth.width=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_SMOOTH_WIDTH_UNITS"))
                                        loginfo.smooth.width_units=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_SMOOTH_PLOT"))
                                        loginfo.smooth.plot=atoi(value);
                       /* cull values */
                                else if(!(int)strcmp(flag,"LOG_CULL_METHOD"))
                                        loginfo.cull.method=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_EQUATION_1"))
                                        loginfo.cull.equat.sign1=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_EQUATION_2"))
                                        loginfo.cull.equat.sign2=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_JOIN"))
                                        loginfo.cull.equat.join=atoi(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_VALUE_1"))
                                        loginfo.cull.equat.value1=(float)atof(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_VALUE_2"))
                                        loginfo.cull.equat.value2=(float)atof(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_TOPDEP"))
                                        loginfo.cull.depmin=(float)atof(value);
                                else if(!(int)strcmp(flag,"LOG_CULL_BOTDEP"))
                                        loginfo.cull.depmax=(float)atof(value);
                                else if(!(int)strcmp(flag,"CORE_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)open_data_directory,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                open_data_directory=charptr;
                                                (void)strcpy(open_data_directory,value);
                                        }
                                }
                                /* affine table values */
                                else if(!strcmp(flag,"USE_AFFINE_TABLE"))
                                        affinetable.read_affine=atoi(value);
                                else if(!(int)strcmp(flag,"AFFINE_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)affinetable.filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                affinetable.filename=charptr;
                                                (void)strcpy(affinetable.filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"AFFINE_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)affinetable.dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                affinetable.dirname=charptr;
                                                (void)strcpy(affinetable.dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SPLICE_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)splice_filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                splice_filename=charptr;
                                                (void)strcpy(splice_filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SPLICE_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)splice_dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                splice_dirname=charptr;
                                                (void)strcpy(splice_dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SPLICE_TABLE_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)splicertable.filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                splicertable.filename=charptr;
                                                (void)strcpy(splicertable.filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SPLICE_TABLE_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)splicertable.dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                splicertable.dirname=charptr;
                                                (void)strcpy(splicertable.dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"LOG_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)loginfo.filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                loginfo.filename=charptr;
                                                (void)strcpy(loginfo.filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"LOG_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)loginfo.dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                loginfo.dirname=charptr;
                                                (void)strcpy(loginfo.dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"ANCILLARY_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)ancill.dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                ancill.dirname=charptr;
                                                (void)strcpy(ancill.dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"STRAT_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)strat_dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                strat_dirname=charptr;
                                                (void)strcpy(strat_dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SAVE_STRAT_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)outstrat_filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                outstrat_filename=charptr;
                                                (void)strcpy(outstrat_filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"READ_EQLOGDEPTH_TABLE"))
                                        eqlogdepth.read_equil_log = atoi(value);
                                else if(!(int)strcmp(flag,"EQLOGDEPTH_TABLE"))
                                {
                                        if((charptr=(char*)realloc((char*)eqlogdepth.filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                eqlogdepth.filename=charptr;
                                                (void)strcpy(eqlogdepth.filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"EQLOGDEPTH_TABLE_DIRECTORY"))
                                {
                                        if((charptr=(char*)realloc((char*)eqlogdepth.dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                eqlogdepth.dirname=charptr;
                                                (void)strcpy(eqlogdepth.dirname,value);
                                        }
                                }

                                else if(!(int)strcmp(flag,"SAVE_COREDATA_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)save_coredata_filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                save_coredata_filename=charptr;
                                                (void)strcpy(save_coredata_filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SAVE_COREDATA_DIRNAME"))
                                {
                                        if((charptr=(char*)realloc((char*)save_coredata_dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                save_coredata_dirname=charptr;
                                                (void)strcpy(save_coredata_dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SAVE_SPLICE_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)save_splice_filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                save_splice_filename=charptr;
                                                (void)strcpy(save_splice_filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SAVE_SPLICE_DIRNAME"))
                                {
                                        if((charptr=(char*)realloc((char*)save_splice_dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                save_splice_dirname=charptr;
                                                (void)strcpy(save_splice_dirname,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SAVE_LOGCOREDATAPAIRS_FILENAME"))
                                {
                                        if((charptr=(char*)realloc((char*)datapairs_filename,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                datapairs_filename=charptr;
                                                (void)strcpy(datapairs_filename,value);
                                        }
                                }
                                else if(!(int)strcmp(flag,"SAVE_LOGCOREDATAPAIRS_DIRNAME"))
                                {
                                        if((charptr=(char*)realloc((char*)datapairs_dirname,((int)strlen(value)+2)*CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("Sacheck_defaults");
                                        else
                                        {
                                                datapairs_dirname=charptr;
                                                (void)strcpy(datapairs_dirname,value);
                                        }
                                }
			/* color defaults */
                                else if(!(int)strcmp(flag,"COLORSET"))
                                        colorset = atoi(value);
				else if(!(int)strcmp(flag,"BG_COLOR"))
					(void)strcpy(cust_color[BG_COLOR],value);
				else if(!(int)strcmp(flag,"CLINE_LABEL"))
					(void)strcpy(cust_color[CLINE_LABEL],value);
				else if(!(int)strcmp(flag,"CLINE_COREODPSBD"))
					(void)strcpy(cust_color[CLINE_COREODPSBD],value);
                                else if(!(int)strcmp(flag,"CLINE_COREMCD"))
                                        (void)strcpy(cust_color[CLINE_COREMCD],value);
				else if(!(int)strcmp(flag,"CLINE_CORESMOOTH"))
					 (void)strcpy(cust_color[CLINE_CORESMOOTH],value);
				else if(!(int)strcmp(flag,"CLINE_COREEQUIVLOGDEPTH"))
					(void)strcpy(cust_color[CLINE_COREEQUIVLOGDEPTH],value);
				else if(!(int)strcmp(flag,"CLINE_LOG"))
					(void)strcpy(cust_color[CLINE_LOG],value);
				else if(!(int)strcmp(flag,"CLINE_LOGADJDEPTH"))
					(void)strcpy(cust_color[CLINE_LOGADJDEPTH],value);
				else if(!(int)strcmp(flag,"CLINE_LOGSMOOTH"))
					(void)strcpy(cust_color[CLINE_LOGSMOOTH],value);
				else if(!(int)strcmp(flag,"CLINE_TIEPT1"))
					(void)strcpy(cust_color[CLINE_TIEPT1],value);
				else if(!(int)strcmp(flag,"CLINE_TIEPT2"))
					(void)strcpy(cust_color[CLINE_TIEPT2],value);
                                else if(!(int)strcmp(flag,"CLINE_TIELINESADJUSTDONE"))
                                        (void)strcpy(cust_color[CLINE_TIELINESADJUSTDONE],value);
				else if(!(int)strcmp(flag,"CLINE_TIELINELAST"))
					(void)strcpy(cust_color[CLINE_TIELINELAST],value);
                                else if(!(int)strcmp(flag,"CLINE_TIESELECT"))
                                        (void)strcpy(cust_color[CLINE_TIESELECT],value);
                                else if(!(int)strcmp(flag,"CLINE_COREOVERLAYINCORR"))
                                        (void)strcpy(cust_color[CLINE_COREOVERLAYINCORR],value);
                                else if(!(int)strcmp(flag,"CLINE_COREOVERLAYOUTCORR"))
                                        (void)strcpy(cust_color[CLINE_COREOVERLAYOUTCORR],value);
                                else if(!(int)strcmp(flag,"CLINE_PALEOMAG"))
                                        (void)strcpy(cust_color[CLINE_PALEOMAG],value);
				else if(!(int)strcmp(flag,"CLINE_DIATOM"))
					(void)strcpy(cust_color[CLINE_DIATOM],value);
				else if(!(int)strcmp(flag,"CLINE_RAD"))
					(void)strcpy(cust_color[CLINE_RAD],value);
				else if(!(int)strcmp(flag,"CLINE_FORAM"))
					(void)strcpy(cust_color[CLINE_FORAM],value);
				else if(!(int)strcmp(flag,"CLINE_NANNO"))
					(void)strcpy(cust_color[CLINE_NANNO],value);
				else if(!(int)strcmp(flag,"CLINE_CORRELATION"))
					(void)strcpy(cust_color[CLINE_CORRELATION],value);
				else if(!(int)strcmp(flag,"CLINE_CORRELATIONBEST"))
					(void)strcpy(cust_color[CLINE_CORRELATIONBEST],value);
				else if(!(int)strcmp(flag,"CLINE_ADJUSTTOTIESSTATS"))
					(void)strcpy(cust_color[CLINE_ADJUSTTOTIESSTATS],value);
                                else if(!(int)strcmp(flag,"CLINE_ANCILLARY1"))
                                        (void)strcpy(cust_color[CLINE_ANCILLARY1],value);
                                else if(!(int)strcmp(flag,"CLINE_ANCILLARY2"))
                                        (void)strcpy(cust_color[CLINE_ANCILLARY2],value);
                                else if(!(int)strcmp(flag,"CLINE_ANCILLARY3"))
                                        (void)strcpy(cust_color[CLINE_ANCILLARY3],value);
                                
			/* unknown flag */
				else
					(void)fprintf(stderr,"sagan: ERROR: Unknown defaults flag %s.\n",flag);
			}
			else
			{
				(void)fprintf(stderr,"sagan: ERROR: Unknown defaults file line type: ");
				(void)fputs(line,stderr);
			}
		}
	}

	if(fptr!=NULL) fclose(fptr);
	Sacheck_defaults();
	return(TRUE);
}


/*      Function Name:	Sawrite_defaults
 *
 *      Description:   	This function writes the defaults values to the
 *		       	specified file.
 *			
 *
 *      Arguments:      FILE		*fptr:	defaults file pointer
 */

void Sawrite_defaults(fptr)
FILE	*fptr;
{
/* information header */
	(void)fprintf(fptr,"#\n# %s\n#\n",SAGAN_VERSION);

        (void)fprintf(fptr,"JOB\t\t\t\t=\t%d\n",job);
/* smoothing information */
	(void)fprintf(fptr,"SMOOTH_METHOD\t\t\t=\t%d\n",smooth.method);
	(void)fprintf(fptr,"SMOOTH_WIDTH\t\t\t=\t%d\n",smooth.width);
	(void)fprintf(fptr,"SMOOTH_WIDTH_UNITS\t\t=\t%d\n",smooth.width_units);
        (void)fprintf(fptr,"SMOOTH_PLOT\t\t\t=\t%d\n",smooth.plot);
/* decimation information */
	(void)fprintf(fptr,"DECIMATE_VALUE\t\t\t=\t%d\n",decimate_value);
/* cull information */
	(void)fprintf(fptr,"CULL_METHOD\t\t\t=\t%d\n",cull.method);
	(void)fprintf(fptr,"CULL_TOP\t\t\t=\t%.1f\n",cull.top);
	(void)fprintf(fptr,"CULL_BY_CORE_NUM\t\t=\t%d\n",cull.by_core_num);
	(void)fprintf(fptr,"CULL_CORE_MAX\t\t\t=\t%d\n",cull.max_core);
	(void)fprintf(fptr,"CULL_TABLE\t\t\t=\t%s\n",cull.filename);
	(void)fprintf(fptr,"CULL_DIRECTORY\t\t\t=\t%s\n",cull.dirname);
	(void)fprintf(fptr,"PWAVE_SIGNAL_MAX\t\t=\t%.1f\n",cull.pwave_signal_max);
	(void)fprintf(fptr,"GRAPE_EQUATION_1\t\t=\t%d\n",cull.grape.sign1);
	(void)fprintf(fptr,"GRAPE_VALUE_1\t\t\t=\t%.1f\n",cull.grape.value1);
	(void)fprintf(fptr,"GRAPE_JOIN\t\t\t=\t%d\n",cull.grape.join);
	(void)fprintf(fptr,"GRAPE_EQUATION_2\t\t=\t%d\n",cull.grape.sign2);
	(void)fprintf(fptr,"GRAPE_VALUE_2\t\t\t=\t%.1f\n",cull.grape.value2);
	(void)fprintf(fptr,"PWAVE_EQUATION_1\t\t=\t%d\n",cull.pwave.sign1);
	(void)fprintf(fptr,"PWAVE_VALUE_1\t\t\t=\t%.1f\n",cull.pwave.value1);
	(void)fprintf(fptr,"PWAVE_JOIN\t\t\t=\t%d\n",cull.pwave.join);
	(void)fprintf(fptr,"PWAVE_EQUATION_2\t\t=\t%d\n",cull.pwave.sign2);
	(void)fprintf(fptr,"PWAVE_VALUE_2\t\t\t=\t%.1f\n",cull.pwave.value2);
	(void)fprintf(fptr,"SUSCEPTIBILITY_EQUATION_1\t=\t%d\n",cull.susceptibility.sign1);
	(void)fprintf(fptr,"SUSCEPTIBILITY_VALUE_1\t\t=\t%.1f\n",cull.susceptibility.value1);
	(void)fprintf(fptr,"SUSCEPTIBILITY_JOIN\t\t=\t%d\n",cull.susceptibility.join);
	(void)fprintf(fptr,"SUSCEPTIBILITY_EQUATION_2\t=\t%d\n",cull.susceptibility.sign2);
	(void)fprintf(fptr,"SUSCEPTIBILITY_VALUE_2\t\t=\t%.1f\n",cull.susceptibility.value2);
	(void)fprintf(fptr,"NATURALGAMMA_EQUATION_1\t\t=\t%d\n",cull.naturalgamma.sign1);
	(void)fprintf(fptr,"NATURALGAMMA_VALUE_1\t\t=\t%.1f\n",cull.naturalgamma.value1);
	(void)fprintf(fptr,"NATURALGAMMA_JOIN\t\t=\t%d\n",cull.naturalgamma.join);
	(void)fprintf(fptr,"NATURALGAMMA_EQUATION_2\t\t=\t%d\n",cull.naturalgamma.sign2);
	(void)fprintf(fptr,"NATURALGAMMA_VALUE_2\t\t=\t%.1f\n",cull.naturalgamma.value2);
	(void)fprintf(fptr,"REFLECTANCE_EQUATION_1\t\t=\t%d\n",cull.reflectance.sign1);
	(void)fprintf(fptr,"REFLECTANCE_VALUE_1\t\t=\t%.1f\n",cull.reflectance.value1);
	(void)fprintf(fptr,"REFLECTANCE_JOIN\t\t=\t%d\n",cull.reflectance.join);
	(void)fprintf(fptr,"REFLECTANCE_EQUATION_2\t\t=\t%d\n",cull.reflectance.sign2);
	(void)fprintf(fptr,"REFLECTANCE_VALUE_2\t\t=\t%.1f\n",cull.reflectance.value2);
	(void)fprintf(fptr,"OTHERODP_EQUATION_1\t\t=\t%d\n",cull.otherodp.sign1);
	(void)fprintf(fptr,"OTHERODP_VALUE_1\t\t=\t%.1f\n",cull.otherodp.value1);
	(void)fprintf(fptr,"OTHERODP_JOIN\t\t\t=\t%d\n",cull.otherodp.join);
	(void)fprintf(fptr,"OTHERODP_EQUATION_2\t\t=\t%d\n",cull.otherodp.sign2);
	(void)fprintf(fptr,"OTHERODP_VALUE_2\t\t=\t%.1f\n",cull.otherodp.value2);
/* data type */
	(void)fprintf(fptr,"DATA_TYPE\t\t\t=\t%d\n",openfileinfo.in_data_type);
/* data format*/
	(void)fprintf(fptr,"IN_DATA_FORMAT\t\t\t=\t%d\n",openfileinfo.in_data_format);
        (void)fprintf(fptr,"OTHER_FORMAT_COL\t\t=\t%d\n",openfileinfo.other_format_value_col);
        (void)fprintf(fptr,"LOG_MISSING_VALUE\t\t=\t%f\n",loginfo.missing_value);
        (void)fprintf(fptr,"LOG_DECIMATE_VALUE\t\t=\t%d\n",loginfo.decimate_value);
/* smoothing information */
        (void)fprintf(fptr,"LOG_SMOOTH_METHOD\t\t=\t%d\n",loginfo.smooth.method);
        (void)fprintf(fptr,"LOG_SMOOTH_WIDTH\t\t=\t%d\n",loginfo.smooth.width);
        (void)fprintf(fptr,"LOG_SMOOTH_WIDTH_UNITS\t\t=\t%d\n",loginfo.smooth.width_units);
        (void)fprintf(fptr,"LOG_SMOOTH_PLOT\t\t\t=\t%d\n",loginfo.smooth.plot);
/* culling */
        (void)fprintf(fptr,"LOG_CULL_METHOD\t\t\t=\t%d\n",loginfo.cull.method);
        (void)fprintf(fptr,"LOG_CULL_EQUATION_1\t\t=\t%d\n",loginfo.cull.equat.sign1);
        (void)fprintf(fptr,"LOG_CULL_VALUE_1\t\t=\t%.1f\n",loginfo.cull.equat.value1);
        (void)fprintf(fptr,"LOG_CULL_JOIN\t\t\t=\t%d\n",loginfo.cull.equat.join);
        (void)fprintf(fptr,"LOG_CULL_EQUATION_2\t\t=\t%d\n",loginfo.cull.equat.sign2);
        (void)fprintf(fptr,"LOG_CULL_VALUE_2\t\t=\t%.1f\n",loginfo.cull.equat.value2);
        (void)fprintf(fptr,"LOG_CULL_TOPDEP\t\t\t=\t%.3f\n",loginfo.cull.depmin);
        (void)fprintf(fptr,"LOG_CULL_BOTDEP\t\t\t=\t%.3f\n",loginfo.cull.depmax);
/* io paths */
        (void)fprintf(fptr,"CORE_DIRECTORY\t\t\t=\t%s\n",open_data_directory);
        (void)fprintf(fptr,"USE_AFFINE_TABLE\t\t=\t%d\n",affinetable.read_affine);
        (void)fprintf(fptr,"AFFINE_FILENAME\t\t\t=\t%s\n",affinetable.filename);
        (void)fprintf(fptr,"AFFINE_DIRECTORY\t\t=\t%s\n",affinetable.dirname);
        (void)fprintf(fptr,"SPLICE_FILENAME\t\t\t=\t%s\n",splice_filename);
        (void)fprintf(fptr,"SPLICE_DIRECTORY\t\t=\t%s\n",splice_dirname);
        (void)fprintf(fptr,"SPLICE_TABLE_FILENAME\t\t=\t%s\n",splicertable.filename);
        (void)fprintf(fptr,"SPLICE_TABLE_DIRECTORY\t\t=\t%s\n",splicertable.dirname);
        (void)fprintf(fptr,"LOG_FILENAME\t\t\t=\t%s\n",loginfo.filename);
        (void)fprintf(fptr,"LOG_DIRECTORY\t\t\t=\t%s\n",loginfo.dirname);
        (void)fprintf(fptr,"ANCILLARY_DIRECTORY\t\t=\t%s\n",ancill.dirname);
        (void)fprintf(fptr,"STRAT_DIRECTORY\t\t\t=\t%s\n",strat_dirname);
        (void)fprintf(fptr,"SAVE_STRAT_FILENAME\t\t=\t%s\n",outstrat_filename);
        (void)fprintf(fptr,"READ_EQLOGDEPTH_TABLE\t\t=\t%d\n",eqlogdepth.read_equil_log);
        (void)fprintf(fptr,"EQLOGDEPTH_TABLE\t\t=\t%s\n",eqlogdepth.filename);
        (void)fprintf(fptr,"EQLOGDEPTH_TABLE_DIRECTORY\t=\t%s\n",eqlogdepth.dirname);
        (void)fprintf(fptr,"SAVE_COREDATA_FILENAME\t\t=\t%s\n",save_coredata_filename);
        (void)fprintf(fptr,"SAVE_COREDATA_DIRNAME\t\t=\t%s\n",save_coredata_dirname);
        (void)fprintf(fptr,"SAVE_SPLICE_FILENAME\t\t=\t%s\n",save_splice_filename);
        (void)fprintf(fptr,"SAVE_SPLICE_DIRNAME\t\t=\t%s\n",save_splice_dirname);
        (void)fprintf(fptr,"SAVE_LOGCOREDATAPAIRS_FILENAME\t=\t%s\n",datapairs_filename);
        (void)fprintf(fptr,"SAVE_LOGCOREDATAPAIRS_DIRNAME\t=\t%s\n",datapairs_dirname);
/* color defaults */
        (void)fprintf(fptr,"COLORSET\t\t\t=\t%d\n",colorset);
	(void)fprintf(fptr,"BG_COLOR\t\t\t=\t%s\n",cust_color[BG_COLOR]);
	(void)fprintf(fptr,"CLINE_LABEL\t\t\t=\t%s\n",cust_color[CLINE_LABEL]);
	(void)fprintf(fptr,"CLINE_COREODPSBD\t\t=\t%s\n",cust_color[CLINE_COREODPSBD]);
        (void)fprintf(fptr,"CLINE_COREMCD\t\t\t=\t%s\n",cust_color[CLINE_COREMCD]);
	(void)fprintf(fptr,"CLINE_CORESMOOTH\t\t=\t%s\n",cust_color[CLINE_CORESMOOTH]);
	(void)fprintf(fptr,"CLINE_COREEQUIVLOGDEPTH\t\t=\t%s\n",cust_color[CLINE_COREEQUIVLOGDEPTH]);
	(void)fprintf(fptr,"CLINE_LOG\t\t\t=\t%s\n",cust_color[CLINE_LOG]);
	(void)fprintf(fptr,"CLINE_LOGADJDEPTH\t\t=\t%s\n",cust_color[CLINE_LOGADJDEPTH]);
	(void)fprintf(fptr,"CLINE_LOGSMOOTH\t\t\t=\t%s\n",cust_color[CLINE_LOGSMOOTH]);
	(void)fprintf(fptr,"CLINE_TIEPT1\t\t\t=\t%s\n",cust_color[CLINE_TIEPT1]);
	(void)fprintf(fptr,"CLINE_TIEPT2\t\t\t=\t%s\n",cust_color[CLINE_TIEPT2]);
        (void)fprintf(fptr,"CLINE_TIELINESADJUSTDONE\t=\t%s\n",cust_color[CLINE_TIELINESADJUSTDONE]);
	(void)fprintf(fptr,"CLINE_TIELINELAST\t\t=\t%s\n",cust_color[CLINE_TIELINELAST]);
        (void)fprintf(fptr,"CLINE_TIESELECT\t\t\t=\t%s\n",cust_color[CLINE_TIESELECT]);
        (void)fprintf(fptr,"CLINE_COREOVERLAYINCORR\t\t=\t%s\n",cust_color[CLINE_COREOVERLAYINCORR]);
        (void)fprintf(fptr,"CLINE_COREOVERLAYOUTCORR\t=\t%s\n",cust_color[CLINE_COREOVERLAYOUTCORR]);
	(void)fprintf(fptr,"CLINE_PALEOMAG\t\t\t=\t%s\n",cust_color[CLINE_PALEOMAG]);
	(void)fprintf(fptr,"CLINE_DIATOM\t\t\t=\t%s\n",cust_color[CLINE_DIATOM]);
	(void)fprintf(fptr,"CLINE_RAD\t\t\t=\t%s\n",cust_color[CLINE_RAD]);
	(void)fprintf(fptr,"CLINE_FORAM\t\t\t=\t%s\n",cust_color[CLINE_FORAM]);
	(void)fprintf(fptr,"CLINE_NANNO\t\t\t=\t%s\n",cust_color[CLINE_NANNO]);
	(void)fprintf(fptr,"CLINE_CORRELATION\t\t=\t%s\n",cust_color[CLINE_CORRELATION]);
	(void)fprintf(fptr,"CLINE_CORRELATIONBEST\t\t=\t%s\n",cust_color[CLINE_CORRELATIONBEST]);
	(void)fprintf(fptr,"CLINE_ADJUSTTOTIESSTATS\t\t=\t%s\n",cust_color[CLINE_ADJUSTTOTIESSTATS]);
        (void)fprintf(fptr,"CLINE_ANCILLARY1\t\t=\t%s\n",cust_color[CLINE_ANCILLARY1]);
        (void)fprintf(fptr,"CLINE_ANCILLARY2\t\t=\t%s\n",cust_color[CLINE_ANCILLARY2]);
        (void)fprintf(fptr,"CLINE_ANCILLARY3\t\t=\t%s\n",cust_color[CLINE_ANCILLARY3]);
}
