/* sagan_defs.h
*/
#include <stdio.h>
#include "myDebug.h"
#define	SAGAN_VERSION	"Sagan version Motif-demo"
#define	MAXBUF		(1000)
#define	ALLOC_INT	(100)
#define SPLICE_ALLOC    (1000)
#define CHAR_SIZE	sizeof(char)
#define NDATASET        9

#define SHIP   1
#define SHORE  2
int whereami;

typedef	struct	_Information
{
	char	*message,
		*helpfile;
	int	fatal,
		error,
		caller;
	unsigned char	type,
			default_button;
} Information;

/* clean up
*/
#define ALL         (1)
#define COREDATA    (2)
#define LOGDATA     (3)
#define AFFINE      (4)
#define STRATDATA   (5)
#define SPLICEDATA  (6)
#define ANCDATA     (7)
#define ELDTABLE    (8)
#define SPLICETABLE (9)

/* caller functions */
#define	INCOMPLETE	(0)
#define	BADMEMALLOC	(1)
#define	MANAGEWARNINGBOX	(2)
#define	OPENCULL	(3)
#define	OPENCULLTABLE	(4)
#define	SMOOTHCORE	(5)
#define	OUTPUTDATA	(6)
#define	READTKREPORT	(7)
#define	READOTHERODP	(8)
#define	READMST95REPORT	(9)
#define	READCORE	(10)
#define	EXIT		(11)
#define	OPENLOG		(12)
#define	LOGINFO		(13)
#define	READLOG		(14)
#define	OPENANC		(15)
#define	SMOOTHLOG	(16)
#define	OPENSTRAT	(17)
#define	OPENSPLICE	(18)
#define	DECIMATECORE	(19)
#define	DECIMATELOG	(20)
#define	DECIMATEANC	(21)
#define READJANUS       (22)

typedef struct _OpenFileInfo
{
    char  **in_file_names;
    int   in_data_type;
    int   in_data_format;
    int   num_in_files;
    int   other_format_value_col;
    char  other_format[100];
} OpenFileInfo;

typedef struct _Smooth
{
	int	method,
                plot,
		width,
		width_units;
	float	*weight;
} Smooth;

typedef struct  _Equation
{
        float   value1,
                value2;
        int     sign1,
                sign2,
                join;
} Equation;

typedef struct  _Cull
{
        int     method,
                cnt;
        char    *filename,
                *dirname;
        float   top,
                pwave_signal_max;
        int     by_core_num,
                max_core;
        Equation        grape,
                        pwave,
                        susceptibility,
                        naturalgamma,
                        reflectance,
                        otherodp;
} Cull;

typedef struct _CullLog
{
        int       method,
                  cnt;
        float     depmin,
                  depmax;
        Equation  equat;
} CullLog;

typedef struct _LogInfo
{
	      FILE    *fptr;
        int     format,
                leg,
                data_col,
                core_data_type,
                maptodata,
                id_num,
                id_alloc;
        char    *filename,
                *dirname,
                **id,
                hole[10];
        Smooth  smooth;
        int     decimate_value;
        float   missing_value,
                depth_min,
                depth_max;
        CullLog cull;
} LogInfo;
 
typedef	struct	_LogValue
{
	float	depth,
		data,
		sm_data;
	int	quality;
} LogValue;
	
/* Log data structure */
typedef	struct	_LogData
{
	int	data_num,
                flip,
                axislen,
                data_alloc,
                *good,
                numgood;
        float   avedepthstep;
	LogValue	**value;
        LogInfo info;
	double	axis_min,
                axis_max;
} LogData;

typedef struct  _AncillValue
{
        float   depth,
                *data;
        int     quality;
} AncillValue;

/* Ancillary data structure */
typedef struct  _AncillData
{
        FILE    *fptr;
        int     format,
                leg,
                *data_col,
                id_num,
                id_alloc,
                data_num,
                data_cnt,
                data_alloc,
                decimate_value;
        char    *filename,
                *dirname,
                *varname,
                sitehole[10],
                **id;
        float   missing_value;
        AncillValue        **value;
        double  axis_min[3],axis_max[3],
                depth_min,depth_max;
        int     axislen;
} AncillData;

#define BADLOG              (1)
#define LOG_VALUE_CULLED    (2)
#define LOG_DECIMATED       (4)

/* Log data types */
#define	LOG	(0)
#define	ANC1	(1)
#define	ANC2	(2)
#define	ANC3	(3)
static	char	*logtype[]={"Log", "Ancillary 1","Ancillary 2","Ancillary 3"};

/* Log data formats */
#define	NUMLOGFORMATS	(3)
#define	ODPCDROM	(0)
#define	ODPSHIP		(1)
#define FMSSTACK        (2)
static	char	*logformat[]={"ODP CD Rom","ODP Shipboard", "FMS Stack"};

#define LOG_ID_ALLOC    (20)
#define LOG_DATA_ALLOC  (100)
#define MAXLOGFILES     (4)

typedef struct  _ReportFile
{
        char    *name;
        int     saveflag,       /* true if report file is to be saved */
                savecurrent;    /* true if report save from current position
                                        rather than start of report */
        FILE    *ptr;
        int     lineno;
} ReportFile;

typedef	struct	_ODPKey
{
	int	leg,
                subleg,
                site,
                core,
                maptohole[NDATASET],
                maptocore[NDATASET],
                in_affine,
                aff_map_hole,
                aff_map_core,
                eld_map_hole,
                eld_map_core,
                did_eld;
	char	section[3];
	char	hole,
                coretype;
	float	interval,
		mbsf,
                aff_offset;
	float	eld;
} ODPKey;

typedef struct  _StratData
{
        char    name[51],
                code[11];
        int     type,
                x_pix_loc,
                y_pix_loc,
                hole_is_in_dataset;
        float   top_age,
                bot_age,
                aveage,
                avembsf,
                avemcd,
                sedrate;
        ODPKey  top,
                bot;
} StratData;

typedef struct  _StratFile
{
        char    *filename,
                *dirname;
        int     type,
                alloc_cnt,
                data_cnt,
                autocorr_status;
        StratData       *data;
} StratFile;

/* strat types */
static  char    *strat_type[]={"Diatoms","Radiolaria","Foraminifera","Nannofossils","Paleomag"};
#define DIATOMS         (0)
#define RADIOLARIA      (1)
#define FORAMINIFERA    (2)
#define NANNOFOSSILS    (3)
#define PALEOMAG        (4)


typedef	struct	_HashInfo
{
	int	leg;
	char	subleg;
	int	site;
	int	value2;
	char	hole;
	int	core;
	char	coretype;
	char	section[3];
	float	top_int;
	int	value;
} HashInfo;

typedef	struct	_Grapedata
{
	float	density,
		b_corr_density;
	int	gamma_counts;
} Grapedata;

typedef	struct	_Pwavedata
{
	float	velocity;
	int	sig_strength,
		travel_time,
		displacement;
} Pwavedata;

typedef	struct	_Natgammadata
{
	float	total_cnt;
	int	sample_time,
		seg1_cnt,
		seg2_cnt,
		seg3_cnt,
		seg4_cnt,
		seg5_cnt;
} Natgammadata;

typedef struct	_Susceptibility
{
	float	observed,
		corrected;
} Susceptibility;

typedef	struct	_Reflectance
{
	float	L,
		a,
		b;
	char	munsell_hue[7],
		munsell_value[10];
	float	nm_value[31];
} Reflectance;

typedef	struct	_Value
{
	char	type;
	char	section[3];
	float	top_int,
		bot_int,
		mbsf,
                eld,
		sm_data,
		*data;
	Grapedata	*grape;
	Pwavedata	*pwave;
	Natgammadata	*naturalgamma;
	Susceptibility	*susceptibility;
	Reflectance	*reflectance;
	float	otherodp;
	int	run_num,
		quality,
                smooth_ok;
} Value;

/* structure for the ties per core
*/
#define MAXTIES 100
typedef struct  _TieList
{
  char  coresection[3],
        holename,
        coretype;
  int   array,             /* within core; array numbers of the ties */
        log,               /* within log; array numbers of the ties */
        selected,
        shared,
        corename;
  float topsect,
        botsect,
        rate,              /* rate of depth change between ties; a of linear equation */
        b,                 /* b of linear equation; Y intercept */
        corembsf,          /* core depth at ties;  mbsf*/
        coremcd,
        coreautoeld,
        equivlogdep,       /* core depth at ties; equil log depth if done previouly or read in; else, sbd */
        temp_eld,          /* eq log depth for current ties */
        coreval,
        coresmval,
        logdep,
        logval,
        logsmval;            /* log depth at ties */
} TieList;

typedef struct  _EqLogDepthCore
{
  int      name,
           nties,
           did_eld,
           affine_status;
  char     type;
  int      map_core_to_data[NDATASET];
  float    affine_offset,
           mindep,
           maxdep;
  TieList  **tie;
} EqLogDepthCore;

/* values for affine status
*/
#define     NOAFFINE        (0)    /* do not have an affine table */
#define     YESAFFINE       (1)
#define     YESAFFINE_DID   (2)    /* have affine and core had did_affine_offset = YES */
#define     YESAFFINE_DIDNT (3)    /* have affine and core had did_affine_offset = NO */

typedef struct  _EqLogDepthHole
{
  int             numcore;
  char            name;
  int             map_hole_to_data[NDATASET];
  EqLogDepthCore  **core;
} EqLogDepthHole;

typedef struct  _EqLogDepthTable
{
  char           *filename,
                 *dirname,
                 *affine_filename;
  int             site,
                  leg,
                  numhole,
                  read_equil_log,
                  affine_status,
                  autocorr_status,
                  did_log_mudline_adj;
  float           autocorr_log_mudline_offset,
                  autocorr_squish;
  EqLogDepthHole  **hole;
} EqLogDepthTable;

typedef struct  _AffineCore
{
  int    name;
  char   type;
  float  mbsf;
  int    did_offset;
  int    data_map_to_core[NDATASET];
  float  cum_depth_offset;
} AffineCore;

typedef struct  _AffineHole
{
  int         numcore;
  char        name;
  int         data_map_to_hole[NDATASET];
  AffineCore  **core;
} AffineHole;

typedef struct  _AffineTable
{
  char        *filename,
              *dirname;
  int         leg,
              site,
              numhole,
              read_affine,
              core_alloc_cnt,
              core_alloc_inc;
  AffineHole  **hole;
} AffineTable;

typedef	struct	_Core
{
	int	num,
		numvalues,
		allocvalues,
                have_equil_log_depths,
                have_tie_list,
                smooth_status,
                affine_map_to_core,
                did_affine_offset,
                map_core_to_eqlogdepth,
                *good,
                numgood;
        float   affine_offset,
                avedepstep;
	Value	**value;
} Core;

typedef struct	_Hole
{
	char	name;
	int	numcores,
                affine_map_to_hole,
                map_hole_to_eqlogdepth;
	Core	**core;
} Hole;

typedef struct _DataFile
{
	char	**name,
                subleg,
                *holes_used;
	int	type,
		in_format,
		out_format,
		numfiles,
		readerrors,
		leg,
                decimate_value,
                flip,
		site,
		numholes, 
                culled,
                maptolog;
        float   min,
                max,
                mbsf_mcd_ratio,
                avedepstep;
	Hole	**holes;
} DataFile;

/* Core Data types */
#define NUM_DATA_TYPES	(6)
#define	GRAPE		(0)
#define	PWAVE		(1)
#define	SUSCEPTIBILITY	(2)
#define	NATURALGAMMA	(3)
#define	REFLECTANCE	(4)
#define OTHERTYPE       (5)

/* default mins and maxs */
#define GRAPEMIN        (1.0)
#define GRAPEMAX        (3.0)
#define PWAVEMIN        (1500.0)
#define PWAVEMAX        (5000.0)
#define SUSCMIN         (-10.0)
#define SUSCMAX         (1000.0)
#define NGAMMAMIN       (0.0)
#define NGAMMAMAX       (1000.0)
#define REFLECTMIN      (0.0)
#define REFLECTMAX      (100.0)
#define OTHERMIN        (0.0)
#define OTHERMAX        (100.0)

static  char    *type[]={"GRAPE","PWAVE","SUSCEPTIBILITY",
        "NATURAL GAMMA","% REFLECTANCE","OTHER"};
 
static  char    *output_type[]={"grfix","pwfix","susfix","ngfix","refl","other"};

/* Core Data Formats */
#define NUM_FORMATS	(9)
#define	MST95REPORT	(0)
#define	TKREPORT	(1)
#define OSUSCAT         (2)
#define JANUS           (3)
#define JANUSCLEAN      (4)
#define JANUSORIG       (5)
#define ODPOTHER        (6)
#define ODPOTHER1       (7)
#define ODPOTHER2       (8)
#define ODPOTHER3       (9)
#define ODPOTHER4       (10)
#define ODPOTHER5       (11)
static	char	*format[]={"MST95 REPORT","TK REPORT","OSU/SCAT","JANUS","JANUSCLEAN","JANUSORIG","ODP OTHER","ODP OTHER1","ODP OTHER2","ODP OTHER3","ODP OTHER4","ODP OTHER5"};

/* open */
#define	FILE_FILTER	(0)
#define	DIRECTORIES	(1)

/* smoothing*/
#define	NONE		(0)
#define	GAUSSIAN	(1)
#define	LOWPASS		(2)
static	char	*smoothtypes[]={"None","Gaussian","Low Pass"};
static  char    *plot_display[]={"Unsmoothed Only","Smoothed Only",
                "Both Smoothed and Unsmoothed"};

#define	DEPTH	(0)
#define	POINTS	(1)
static	char	*smooth_units[]={"Depth (cm)","Points"};
int     smooth_who;
 
#define UNSMOOTH    (0)
#define SMOOTH      (1)
#define SMOOTHBOTH  (2)
#define SMOK            (1)
#define SMTAIL          (2)
#define SMBADDATA       (-1)
#define SMTOOFEWPTS     (-2)
#define SMBADCALC       (-3)
#define TOOFEWSMOOTHED  (5)    /* don't use smooth if have less than this number in a core */
#define BADPT           (-999.0)

/* culling */
#define	LESS_THAN			(0)
#define	LESS_THAN_OR_EQUAL_TO		(1)
#define	GREATER_THAN			(2)
#define	GREATER_THAN_OR_EQUAL_TO	(3)
#define	EQUAL_TO			(4)

#define	OR	(1)
#define	ONLY	(2)

#define	CULL_PARAMETERS	         (1)
#define	CULL_TABLE	         (2)
#define CULL_PARAMETER_AND_TABLE (3)
#define CULLTABLEDEPTHCHECK      (1.0)   /* section cm depth to use for whether to cull a value using cull table*/
                                         /* is the sample this close to cull table */

/* data quality flags */
#define	GOOD			(0)
#define	BAD_TOP			(1)
#define	BAD_CULL1		(2)
#define	BAD_CULL2		(4)
#define	DECIMATED		(8)
#define BAD_PWAVE_SIGNAL	(16)
#define	BAD_SB_DEPTH		(32)
#define	BAD_CORE_NUM		(64)
#define BAD_CULLTABLE           (128)
#define	BAD_SMOOTH		(256)

#define BAD_CODE        -999.99

/* hash values */
#define	CREATE_HASH	(1)
#define	DESTROY_HASH	(2)

/* help  */
#define ABOUTHELP               "SaAbout.help"
#define FILEMENUHELP            "SaMenuFile.help"
#define DISPLAYMENUHELP         "SaMenuDisplay.help"
#define PROCESSMENUHELP         "SaMenuProcess.help"
#define DATAFORMATINPUTHELP     "SaFormatInput.help"
#define DATAFORMATOUTPUTHELP    "SaFormatOutput.help"
#define DATAFILESGENERALHELP    "SaDataInfo.help"
#define DATAFILESOPENHELP       "SaDataOpen.help"
#define DATAFILESSAVEHELP       "SaDataSave.help"
#define AFFINETABLEGENERALHELP  "SaAffineInfo.help"
#define AFFINETABLEOPENHELP     "SaAffineOpen.help"
#define SPLICEGENERALHELP       "SaSplicerTableInfo.help"
#define SPLICETABLEOPENHELP     "SaSplicerTableOpen.help"
#define SPLICESERIESOPENHELP    "SaSplicerSeriesOpen.help"
#define SPLICESERIESSAVEHELP    "SaSplicerSeriesSave.help"
#define STRATGENERALHELP        "SaStratInfo.help"
#define STRATOPENHELP           "SaStratOpen.help"
#define STRATSAVEHELP           "SaStratSave.help"
#define LOGGENERALHELP          "SaLogInfo.help"
#define LOGOPENHELP             "SaLogOpen.help"
#define ANCILOPENHELP           "SaLogAncilOpen.help"
#define LOGSAVEHELP             "SaLogSave.help"
#define ELDTABLEGENERALHELP     "SaELDTableInfo.help"
#define ELDTABLEOPENHELP        "SaELDTableOpen.help"
#define ELDTABLESAVEHELP        "SaELDTableSave.help"
#define PROCESSDATAGENERALHELP  "SaProcessDataInfo.help"
#define CULLHELP                "SaProcessCull.help"
#define SMOOTHHELP              "SaProcessSmooth.help"
#define DECIMATEHELP            "SaProcessDecimate.help"
#define DETRENDHELP             "SaProcessDetrend.help"
#define DISPLAYHELP             "SaDisplay.help"
#define COLORHELP               "SaCustomColor.help"
#define CULLTABLEOPEN           "SaCullOpen.help"
#define ELDTOOLHELP             "SaELDTool.help"
#define AUTOCORRHELP            "SaAutoCorrelate.help"
#define CORELOGPAIRHELP         "SaCoreLogDataPair.help"
#define AUTOPAIRHELP            "SaAutoPair.help"
#define REPORTFILEHELP          "SaReportSave.help"
#define DEFAULTFILEHELP         "SaSaganDefaults.help"
#define PREVIEWINPUTHELP        "SaPreviewInput.help"
#define CLEARWORKSPACEHELP      "SaClearWorkspace.help"
#define EXITHELP                "SaExit.help"
#define FILESELECTHELP          "SaFileSelection.help"
#define MESSAGEBOXHELP          "SaMessageBox.help"
#define WORKINGDIALOGHELP       "SaWorkingDialog.help"
#define JANUSHELP               "SaJanus.help"


/* i/o file types
*/
int fileselection;
#define PREVIEWINPUT            (1)
#define OPENCOREDATA            (2)
#define OPENLOGDATA             (3) 
#define OPENANCILDATA           (4)
#define OPENSPLICEDATA          (5)
#define OPENCULLTABLEDATA       (6)
#define OPENEQUIVLOGDEPTHTABLE  (7)
#define OPENSTRATDATA           (8)
#define OPENSPLICETABLE         (9)
#define OPENAFFINE              (10)
#define SAVEEQUIVLOGDEPTHTABLE  (11)
#define SAVECORETOELD           (12)
#define SAVESPLICETOELD         (13)
#define SAVEREPORT              (14)
#define SAVESTRAT               (15)
#define SAVELOGCOREDATAPAIRS    (16)
 
#define OPEN       (1)
#define SAVE       (2)

#define OK        0
#define CANTOPEN  1
#define DIR       2
#define DOOR      3
#define EXEC      4
#define FIFO      5
#define LINK      6
#define SOCK      7
#define FILEXISTS 8

GC        gc_drawingAreaCores;
GC        gc_drawingAreaLog;
GC        gc_drawingAreaCorrelate;
GC        gc_drawingAreaAdjustToTiesInfo;
GC        gc_drawingAreaOptimizeTieCorrelate;
GC        gc_drawingAreaSuggestLogDepthAdjustStats;
GC        gc_colorListSelect;
GC        gc_drawingAreaAutoCorrelate;
GC        gc_drawingAreaEvalCoreLogData;

/*max array size for 'dummy' cleaned up core data
  used for plotting and correlation and max array
  size for the correlation values.
*/
#define MAXCORE        200
#define MAXPERCORE    1000
#define MAXCORR       1000

/* drawing constraints
*/
#define OFF                     50    /*pixels*/
#define YAXIS_ANNOT_OFF         25    /*pixels*/
#define SPACE                   20    /*pixels*/
#define VAR_AXIS_LEN           120    /*pixels*/
#define LOG_AXIS_LEN           120    /*pixels*/
#define SCALE_VALUE              0    /*meters*/
#define SCALE_MULT               5    /*meters*/
#define MINDEP                 0.0    /*meter*/
#define MAXDEP               100.0    /*meter*/
#define DEPTIC                 5.0    /*meters*/
#define DEPMETERDISPLAYED     50.0    /*meter interval displayed*/
#define DEPPERPIX             0.01    /*meters per pixel*/
#define GAP                    0.5    /*meter*/
#define CHECKSPLICEOFFSET     0.05    /*meter*/
#define CLOSE                    5    /*pixel check for cursor position close enough to real data*/
#define TOOFEWCORRPAIRS          5    /*discard correlatiions with fewer than this n*/
#define TOOMANYCULLED          0.5    /* fraction if exceed; too many culled */
#define TOOMANYOUTOFORDER      100    /* if this many are out of depth order stop checking */
#define CHECKVALUEDIFF         0.1    /*check for value difference of good tie; fraction of std*/
#define ROUNDCHECK           0.001    /*for equating two values same or not.*/
#define NCORESSTD                5    /*number of cores(from top, every hole) to find std in; to set var axis min & max*/
#define PLUSMINUSSTD           3.0    /*number of std awy from mean to set variable axis min & max*/
#define DEPTHCHANGE           10.0    /*percent depth change of composited cores*/

/* on,off's for whether have stuff or have done something
*/
#define DO          1
#define DONT        0
#define YES         1
#define NO          0
#define TEMP        2
#define AUTO        3
#define ELD         4
#define OVERLAY     5
#define TIE1        1
#define TIE2        2
#define CORESAVED   3
#define CORECANCEL  4
#define INTERP     -1
#define MISSING    -2

int dataset_cnt;
int logset_cnt;
int splice_cnt;
int splice_did_warn;
int splice_data_type;
int dset;
int sset;
int lset;
int total_num_ties;
int have_cores;
int have_splice;
int have_splice_table;
int have_smooth_cores;
int have_smooth_log;
int have_smooth_anc1, have_smooth_anc2,have_smooth_anc3;
int have_log;
int have_ancill,have_anc1,have_anc2,have_anc3;
int have_coretotie;
int have_splicetotie;
int missing_coretotie;
int have_tie1;
int have_tie2;
int last_tie;
int have_ties;
int have_correlate;
int have_strat;
int have_equivlogdep;
int have_affine;
int have_autocorr;
int are_reading_dataset;
int are_reading_log;
int autopair;

int eqlog;              /* indicates type of core depth using in log window 
                           NO= orig depth, YES= equivelent log depth(saved current run or read in)
                           TEMP= new, temp equiv. log depth (not yet saved) 
                           OVERLAY = temp equiv. plus result of current tie*/
int eqlogdep_core_alloc;      /* used to allocate space for the eqlogdepth table core values
                                 will make the table sym. so all holes have same num cores
                              */
int max_num_cores;

int open_dataform_is_up;

/*frequently used drawing variables like scales and such
*/
int    top_off;
int    scale_val;
int    scale_mult;
int    varaxislen;
int    logaxislen;
int    core_clip_what;
int    log_clip_what;
 
float  mindep;
float  maxdep;
float  mincoredep;
float  maxcoredep;
float  minlogdep;
float  maxlogdep;
float  minancdep;
float  maxancdep;
float  top_meter;
float  varperpix;
float  depperpix;
float  depperpixsp;
float  lagperpix;
float  corrperpix;
float  offsetdepperpix;
float  depthoffsetperpix;
 
double minvar;
double maxvar;
double minlog;
double maxlog;
double realminvar;
double realmaxvar;
double realminlog;
double realmaxlog;
double realminanc[3];
double realmaxanc[3];
double deptic;
double depmeterdisplayed;

/*default lead,lag, depth step and window length
*/
#define LEADLAG    1.0     /* meters*/
#define WINLEN     2.0     /* meters*/
#define DEPSTEP    0.05     /* meters*/
#define LOGDEPSTEP 0.15     /*m */
 
/* things will need in many functions regarding 
   'ties' and correlation info
*/
int    lead_lag;
int    tie_corr_num;
int    best_corr_num;

/* handles to the different types of lines and symbols
   that can be displayed in selected colors
*/
#define NCOLOR                        27
#define BG_COLOR                       0
#define CLINE_LABEL                    1 
#define CLINE_COREODPSBD               2
#define CLINE_COREMCD                  3
#define CLINE_CORESMOOTH               4
#define CLINE_COREEQUIVLOGDEPTH        5
#define CLINE_LOG                      6
#define CLINE_LOGADJDEPTH              7
#define CLINE_LOGSMOOTH                8
#define CLINE_TIEPT1                   9
#define CLINE_TIEPT2                  10
#define CLINE_TIELINESADJUSTDONE      11
#define CLINE_TIELINELAST             12
#define CLINE_TIESELECT               13
#define CLINE_COREOVERLAYINCORR       14
#define CLINE_COREOVERLAYOUTCORR      15
#define CLINE_PALEOMAG                16
#define CLINE_FORAM                   17
#define CLINE_NANNO                   18
#define CLINE_RAD                     19
#define CLINE_DIATOM                  20
#define CLINE_CORRELATION             21
#define CLINE_CORRELATIONBEST         22
#define CLINE_ADJUSTTOTIESSTATS       23
#define CLINE_ANCILLARY1              24
#define CLINE_ANCILLARY2              25
#define CLINE_ANCILLARY3              26

#define COLOR_LIST_FILE

/* Colormap and Pixel array to hold the pixel values
   for foreground colors
*/
Colormap   cmap;
Colormap   auto_corr_cmap;
Pixel      fgPixel[NCOLOR];
Pixel      tempfgPixel[NCOLOR];
Pixel      bgPixel;
Pixel      tempbgPixel;

#define NCUSTOMECOLORBUTT     35

/* the color sets
*/
int      colorset;

#define  EARTH_COLORSET      1
#define  CORPORATE_COLORSET  2
#define  SANTAFE_COLORSET    3
#define  ODP_COLORSET        4
#define  MARITIME_COLORSET   5
#define  CUSTOME_COLORSET    6

/* the foreground and background color for sagan's logo
*/
#define LOGO_BG     "gainsboro"
#define LOGO_FG     "red"

/* color values for auto corr coef boxes
*/
Colormap cmap_autocorr;
#define  NCOLOR_AUTOCORR     100
Pixel    pixel_autocorr[NCOLOR_AUTOCORR];
#define  MAXRGB              65535

int      color_choice_autocorr;
#define  NAUTOCORRCOLOR      9
#define  REDTOYELLOW         1
#define  PURPLETOWHITE       2
#define  AQUATOWHITE         3
#define  BLUETOAQUA          4
#define  BLUETOVIOLET        5
#define  GREENTOYELLOW       6
#define  GREENTOAQUA         7
#define  YELLOWTOWHITE       8
#define  REDTOVIOLET         9

char prgfgcolor[50];
char prgbgcolor[50];
int  prgfg;             /* was foreground color read on command line args */
int  prgbg;             /* was background color read on command line args */

/* structure for ties
*/
typedef struct  _Ties
{
  char     tiecoreholename,
           tiecoretype,
           tiecoresection[3];
  int      nties,                /* number of ties in list */
           hole,                 /* array indicy of hole of core being tied  */
           core,                 /* array indicy of core being tied */
           logtie,               /* array indicy of the current tie in log */
           coretie,              /* array indicy of the current tie in core being tied */
           tiecorename,
           alloc_cnt ,           /* amount allocated for TieList in Ties */
           core_alloc,           /* amount allocated for selected core to tie */
           have_new_ties,
           lasttiesaved;
  float    avedepthstep,         /* for one core being worked on; average depth step for correlation */
           winlen,
           leadlag,
           affine_offset,
           tiecoretopsect,
           tiecorebotsect,
           tiecorembsf,
           tiecoremcd,
           tiecoreautoeld,
           tiecoreeqlogdep,
           tiecoretempeqlogdep,
           tiecoreval,
           tiecoresmval,
           tielogdep,
           tielogval,
           tielogsmval,
           *coredepth,           /* the core depth to apply the ties to to create the temp_equivlogdep */ 
                                 /* it may be mbsf, mcd, mbsf*squish, mcd*squish */
           *temp_equivlogdep;    /* for one core being worked on; temporary equil log depths */
  TieList  **tie;
} Ties;

/* data struct for the correlation values
*/
typedef struct _CorrelationTable
{
  int    lag,
         lead,
         num_lead_lag,   
         n[MAXCORR],  
         leadlag[MAXCORR];
  float  coef[MAXCORR],
         leadlag_in_meter[MAXCORR],
         depoffset[MAXCORR];
} CorrelationTable;

typedef struct _SpliceValue
{
  char    hole,
          type,
          section[3];
  int     corename,
          quality,
          smooth_ok;
  float   sect_top,
          sect_bot,
          mbsf,
          mcd,
          eld,
          sm_data,
          data,
          affine_offset;
} SpliceValue;

typedef struct _Splice
{
  int           numvalues,
                alloc_cnt,
                leg,
                site,
                data_type,
                maptodata,
                *good,
                numgood;
  float         min,
                max,
                minreal, 
                maxreal,
                depth_min,
                depth_max, 
                avedepstep;
  SpliceValue   **value;
  Smooth        smooth;
} Splice;

typedef struct  _SplicerTableInfo
{
        int     site;
        char    hole;
        int     core;
        char    core_type;
        char    section[3];
        char    type_of_splice[10];
        float   top_int,
                bot_int,
                mbsf,
                mcd;
} SplicerTableInfo;


typedef struct  _SplicerTableFile
{
        char                *filename;
        char                *dirname;
        int                 alloc_cnt,
                            data_cnt;
        SplicerTableInfo    *data;
} SplicerTableFile;

typedef struct _Box
{
  int   row,
        col,
        ncore,
        rank,
        xpix_loc,
        ypix_loc;
  float sumr,
        squish,
        offset,
        stdev,
        *r,
        sum_r;
} Box;

typedef struct _AutoCoreAdjust
{
  int     nhole,
          nsquish,
          noffset,
          nboxes,
          boxes_so_far,
          box_size,
          alloc_boxes,
          is_allocated,
          in_progress,
          override_aff,
          draw_results;
  float   range_offset[2],
          range_squish[2],
          offset_inc,
          squish_inc,
          best_offset,
          best_squish;       
  Box     **box;
} AutoCoreAdjust;

#define STRETCH      (95.0)
#define SQUISH       (85.0)
#define INC_SQUISH   (0.5)     /* percent */
#define NEG_OFFSET   (-5.0)
#define POS_OFFSET   (5.0)
#define INC_OFFSET   (2.0)       /* this times depth step = inc */
#define MIN_SQUISH   (70.0)      /*percent*/
#define MAX_STRETCH  (120.0)     /*percent*/
#define MIN_OFFSET   (-20.0)     /*meter*/
#define MAX_OFFSET   (20.0) 

int   job;

/* job types */
#define MCD_VS_LOG            (1)
#define SPLICE_VS_LOG         (2)
#define MCD_TO_SPLICE_VS_LOG  (3)
#define MBSF_VS_LOG           (4)

typedef struct _Corr_Rank
{
  int    n,
         use_thisone,
         previous_one;
  int    *box;
  float  *r;
} Corr_Rank;

typedef struct _HoleTie
{
  float  mbsf,
         mcd,
         autoeld,
         log,
         a,
         b;
} HoleTie;

typedef struct _HoleELDEqt
{
  int     nties;
  HoleTie **tie;
} HoleELDEqt;

typedef struct _ELDEqt
{
  int         nholes;
  HoleELDEqt  **hole_eqt; 
} ELDEqt;

typedef struct _LogCoreDataPairs
{
  char   holename;
  int    n,
         hole,
         *corename;
  float  *logdat,
         *coredat,
         *mbsf,
         *mcd,
         *eld;

} LogCoreDataPairs;

int    datapairs_cnt,
       datapairs_how,
       datapairs_sm;
float  datapairs_dt,
       datapairs_topdep,
       datapairs_botdep,
       datapairs_a,
       datapairs_b;

#define TOLOGDEPTH   (1)
#define TOCOREDEPTH  (2)
#define CONSTANTDT   (3)
