/* 
      sagan_eventproc_functions.c

      sagan's drawing area event procedures

    void EventProcDrawingAreaCores()
    void EventProcDrawingAreaLog()
    void EventProcDrawingAreaAdjustToTiesInfo()
    void EventProcDrawingAreaCorrelate()
    void EventProcDrawingAreaAutoCorrelate()
*/
#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/Text.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "sagan-creation-c.h"
#include "sagan_vars.h"

char    buffer[MAXBUF];

void
EventProcDrawingAreaCores(w, event, args, num_args)
Widget w;
XButtonEvent *event;
String *args;
int *num_args;
{
  int x, y, hole, core, array, sm, starting_hole, found_strat, i, j;
  char *string, *catstring;
  float depth, var, *pvar;
  Dimension win_height, win_width;
  Boolean mapped;
  XmString xmstring;
  void DrawCores();

  x = event->x;
  y = event->y;
  XtVaGetValues(w, XmNheight, &win_height, XmNwidth, &win_width, NULL);

  XtVaGetValues(formEquivLogDepthTool, XmNmappedWhenManaged, &mapped, NULL);

  /* calculate the cursor location in depth(m)
     note that top_off is the pixel offset from top of drawingAreaEdit
          and top_meter is the value of the top of the depth axis
  */
  depth = (((y - top_off) * depperpix) + top_meter);
  
  if((y < top_off || y > win_height || x < OFF || x > win_width) || have_cores == DONT) {
    xmstring = XmStringCreateSimple("");
    XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }
  else if(have_cores == DO) {
    /* get the cores scale value. it will be the array num of
       the first hole to search. in other words; where to 'skip to'
    */
    XmScaleGetValue(scaleCores, &starting_hole);
    if(starting_hole >= data[dset]->numholes || starting_hole < 0) {
      return;
    }

    if(smooth.method != NONE) {
      sm = YES;
    }
    else {
      sm = NO;
    }

    if((string = (char*)calloc(500, CHAR_SIZE)) == NULL)
                       SaBadMemAlloc("Sa");
    if((catstring = (char*)calloc(500, CHAR_SIZE)) == NULL)
                       SaBadMemAlloc("Sa");

    if((int)strcmp(args[0], "but_1_down") == 0 || (int)strcmp(args[0], "but_1_move") == 0 ||
       (int)strcmp(args[0], "but_2_down") == 0 || (int)strcmp(args[0], "but_2_move") == 0 ||
       (int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0) {

      FindNearestSample(dset, starting_hole, &hole, &core, &array, x, y, sm, depth, varaxislen);
      if(hole == -1 || core == -1 || array == -1) {
        (void)sprintf(string, "");
        found_strat = NO;
        if(have_strat == DO) {
          for(i=0; i<strat_file_cnt; ++i){
            if(found_strat == YES) {
              break;
            }
            for(j=0; j<strat[i]->data_cnt; ++j){
              if(strat[i]->data[j].x_pix_loc < 0 || strat[i]->data[j].y_pix_loc < 0) {

              }
              else if(abs(strat[i]->data[j].x_pix_loc + 5 - x) < CLOSE &&
                 abs(strat[i]->data[j].y_pix_loc - 5 - y) < CLOSE) {
                found_strat = YES;
                if(fabs(strat[i]->data[j].top_age - strat[i]->data[j].bot_age) < ROUNDCHECK) {
                  (void)sprintf(string,"%d%c  %s  %6.3fMa  %6.2f-%6.2fmbsf",
                   strat[i]->data[j].top.site, strat[i]->data[j].top.hole, strat[i]->data[j].name, strat[i]->data[j].top_age,
                   strat[i]->data[j].top.mbsf, strat[i]->data[j].bot.mbsf);
                }
                else {
                  (void)sprintf(string,"%d%c  %s  %6.3f-%6.3fMa  %6.2f-%6.2fmbsf", strat[i]->data[j].top.site,
                   strat[i]->data[j].top.hole, strat[i]->data[j].name, strat[i]->data[j].top_age, strat[i]->data[j].bot_age,
                   strat[i]->data[j].top.mbsf, strat[i]->data[j].bot.mbsf);
                }
                if(strat[i]->data[j].top.in_affine == YES || strat[i]->data[j].bot.in_affine == YES) {
                  (void)sprintf(catstring,"  %6.2f-%6.2fmcd", strat[i]->data[j].top.mbsf + strat[i]->data[j].top.aff_offset,
                             strat[i]->data[j].bot.mbsf + strat[i]->data[j].bot.aff_offset);
                  (void)strncat(string, catstring, (int)strlen(catstring));
                }
                if(strat[i]->autocorr_status == YES || (strat[i]->data[j].top.did_eld == YES && strat[i]->data[j].bot.did_eld == YES)) {
                  (void)sprintf(catstring,"  %6.2f-%6.2feld", strat[i]->data[j].top.eld, strat[i]->data[j].bot.eld);
                  (void)strncat(string, catstring, (int)strlen(catstring));
                }
                else if(strat[i]->data[j].top.did_eld == YES) {
                  (void)sprintf(catstring,"  %6.2feld", strat[i]->data[j].top.eld);
                  (void)strncat(string, catstring, (int)strlen(catstring));
                }
                else if(strat[i]->data[j].bot.did_eld == YES) {
                  (void)sprintf(catstring,"  %6.2feld", strat[i]->data[j].bot.eld);
                  (void)strncat(string, catstring, (int)strlen(catstring));
                }
                xmstring = XmStringCreateSimple(string);
                XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
                XmStringFree(xmstring);
              }
            }
          }
        }
      }
      else {
        found_strat = NO;
        (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       data[dset]->leg,
                       data[dset]->site,
                       data[dset]->holes[hole]->name,
                       data[dset]->holes[hole]->core[core]->num,
                       data[dset]->holes[hole]->core[core]->value[array]->type,
                       data[dset]->holes[hole]->core[core]->value[array]->section,
                       data[dset]->holes[hole]->core[core]->value[array]->top_int,
                       data[dset]->holes[hole]->core[core]->value[array]->bot_int,
                       data[dset]->holes[hole]->core[core]->value[array]->mbsf,
                       *data[dset]->holes[hole]->core[core]->value[array]->data);
        if(smooth.method != NONE) {
          (void)sprintf(catstring, " %.2f", data[dset]->holes[hole]->core[core]->value[array]->sm_data);
          (void)strncat(string, catstring, (int)strlen(catstring));
        }
        if(have_affine == DO && data[dset]->holes[hole]->core[core]->did_affine_offset ==  YES) {
          (void)sprintf(catstring, "    %.2f(mcd, Y)", data[dset]->holes[hole]->core[core]->value[array]->mbsf +
                                               data[dset]->holes[hole]->core[core]->affine_offset);
          (void)strncat(string, catstring, (int)strlen(catstring));
        }
        else if(have_affine == DO && data[dset]->holes[hole]->core[core]->did_affine_offset ==  NO) {
          (void)sprintf(catstring, "    %.2f(mcd, N)", data[dset]->holes[hole]->core[core]->value[array]->mbsf +
                                               data[dset]->holes[hole]->core[core]->affine_offset);
          (void)strncat(string, catstring, (int)strlen(catstring));
        }
        if(data[dset]->holes[hole]->core[core]->have_equil_log_depths == DO) {
          (void)sprintf(catstring, "    %.2f(eld)", data[dset]->holes[hole]->core[core]->value[array]->eld);
          (void)strncat(string, catstring, (int)strlen(catstring));
        }
      }
      xmstring = XmStringCreateSimple(string);
      XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
      XmStringFree(xmstring);
    }
    else if(((int)strcmp(args[0], "but_1_up") == 0 || (int)strcmp(args[0], "but_2_up") == 0 ||
            (int)strcmp(args[0], "but_3_up") == 0 || (int)strcmp(args[0], "but_1_click") == 0) && mapped == True) {
      FindNearestSample(dset, starting_hole, &hole, &core, &array, x, y, sm, depth, varaxislen);
      if(hole == -1 || core == -1) {
        (void)sprintf(string, "");
        XtVaSetValues(textFieldSelectHole, XmNvalue, string, NULL);
        XtVaSetValues(textFieldSelectCore, XmNvalue, string, NULL);
      }
      else if(have_splice == DONT) {
        (void)sprintf(string, "%c", data[dset]->holes[hole]->name);
        XtVaSetValues(textFieldSelectHole, XmNvalue, string, NULL);
        (void)sprintf(string, "%d", data[dset]->holes[hole]->core[core]->num);
        XtVaSetValues(textFieldSelectCore, XmNvalue, string, NULL);
      }
      xmstring = XmStringCreateSimple("");
      XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
      XmStringFree(xmstring);

      if((int)strcmp(args[0], "but_1_click") == 0 && have_log == DO) {
        if((have_coretotie == DO && ties->have_new_ties == DONT) || have_coretotie == DONT) {
          SelectCore(NULL, NULL, NULL);
        }
      }
    }
    else if((int)strcmp(args[0], "return_down") == 0 && have_log == DO && mapped == True && have_splice == DONT) {
      if((have_coretotie == DO && ties->nties < 1) || have_coretotie == DONT) {
          SelectCore(NULL, NULL, NULL);
      }
    }
    else {
    }
   
    if(data[dset]->numholes > 1 && ((int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0)) {   
      core_clip_what = NONE;
      DrawCores(drawingAreaCores, XtDisplay(drawingAreaCores), XtWindow(drawingAreaCores), 
                gc_drawingAreaCores, core_clip_what, YES, y);
    }
    else if((int)strcmp(args[0], "but_3_up") == 0) {
      DrawCores(drawingAreaCores, XtDisplay(drawingAreaCores), XtWindow(drawingAreaCores),
                gc_drawingAreaCores, core_clip_what, NO, 0);
    }

    free(string);
    string = NULL;
    free(catstring);
    catstring = NULL;

  }
}

void
EventProcDrawingAreaLog(w, event, args, num_args)
Widget w;
XButtonEvent *event;
String *args;
int *num_args;
{
  int i, x, y, k, s, e, array, current, pos, err, derr, report, scale_val, dum, tie, place;
  char string[500], varstring[200];
  float depth, chlow, chhigh, diff, val, valsm, autooff;
  double checkdep;
  Boolean mapped;
  Dimension win_height, win_width;
  String str;
  XmString xmstring;
  extern void DrawDrawingAreaAdjustToTiesInfo();
  extern void  AdjustToTies();
  extern int   FindWhereInCore(), FindWhereInSplice(), FindWhereInLog();

  x = event->x;
  y = event->y;
  XtVaGetValues(w, XmNheight, &win_height, XmNwidth, &win_width, NULL);

  XtVaGetValues(formEquivLogDepthTool, XmNmappedWhenManaged, &mapped, NULL);

  /* calculate the cursor location in depth(m)
     note that top_off is the pixel offset from top of drawingAreaEdit
          and top_meter is the value of the top of the depth axis
  */
  depth = (((y - top_off) * depperpix) + top_meter);
  /* get the log scale value.
  */
  XmScaleGetValue(scaleLog, &scale_val);

  if(y < top_off || y > win_height || x < 0 || x > win_width) {
    xmstring = XmStringCreateSimple("");
    XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }
  else if(have_log == DO || have_ancill == DO || have_splice == DO) {
    
    /* if are auto corring and there is a test overlay get the current
         log offset
    */
    if(autocoreadj.draw_results == YES) {
      str = XmTextGetString(textFieldAutoCorrResultOffset);
      autooff = atof(str);
      XtFree(str);
    }
    else {
      autooff = 0.0;
    }

    /* button event in the area of window where log is plotted
    */
    if(have_log == DO &&
       (scale_val == 0 && x >= 2 * logaxislen && x < 3 * logaxislen && strncmp(args[0], "but", 3) == 0) ||
       (scale_val == 1 && x >= 0 && x < logaxislen && strncmp(args[0], "but", 3) == 0)) {

      /* if are in the depth range of the log data set
         find the nearest sample
      */
      diff = 99999.0;
      array = -1;
      if(depth >= logdata[lset]->value[0]->depth + autooff &&
                   depth <= logdata[lset]->value[logdata[lset]->data_num-1]->depth + autooff) {
        for(i=0; i<logdata[lset]->data_num; ++i) {
          checkdep = depth - (logdata[lset]->value[i]->depth + autooff);
          if(logdata[lset]->value[i]->data == logdata[lset]->info.missing_value) {
          }
          else if(logdata[lset]->value[i]->quality != GOOD) {
          }
          else if(fabs(checkdep) < diff){
            diff = fabs(checkdep);
            array = i;
          }
          else if(fabs(checkdep) > diff){
            break;
          }
        }
      }

      if(array != -1) {
        if(eqlogdepth.did_log_mudline_adj == YES) {
          (void)sprintf(string,"%.3f   %.3f(mud line adj.)   %.3f",
            (logdata[lset]->value[array]->depth - eqlogdepth.autocorr_log_mudline_offset), 
             logdata[lset]->value[array]->depth, logdata[lset]->value[array]->data);
        }
        else if(autocoreadj.draw_results == YES) {
          (void)sprintf(string,"%.3f   %.3f(test mud line adj.)   %.3f",
            logdata[lset]->value[array]->depth, logdata[lset]->value[array]->depth + autooff, logdata[lset]->value[array]->data);
        }
        else {
          (void)sprintf(string,"%.3f   %.3f",
            logdata[lset]->value[array]->depth, logdata[lset]->value[array]->data);
        }
        if(logdata[lset]->info.smooth.method != NONE) {
          (void)sprintf(varstring, " %.2f", logdata[lset]->value[array]->sm_data);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }

        if((int)strcmp(args[0], "but_1_down") == 0 || (int)strcmp(args[0], "but_1_move") == 0 ||
             (int)strcmp(args[0], "but_2_down") == 0 || (int)strcmp(args[0], "but_2_move") == 0 ||
             (int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0) {
          xmstring = XmStringCreateSimple(string);
          XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
        }

        if(XtIsManaged(formAutoCorrelate)) {
          return;
        }
          
        if(((int)strcmp(args[0], "but_3_up") == 0 || (int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0)
             && (have_coretotie == DO || have_splice == DO) && mapped == True && scale_val == 0) {
          if(have_tie1 == DO && ties->nties > 0) {
            if(ties->tiecoremcd < ties->tie[0]->coremcd) {
              if(logdata[lset]->value[array]->depth >= ties->tie[0]->logdep) {
                return;
              }
            }
            else if(ties->tiecoremcd > ties->tie[ties->nties-1]->coremcd) {
              if(logdata[lset]->value[array]->depth <= ties->tie[ties->nties-1]->logdep) {
                return;
              }
            }
            else if(ties->nties > 1) {
              for(i=0; i<ties->nties-1; ++i) {
                if(ties->tiecoremcd > ties->tie[i]->coremcd && ties->tiecoremcd < ties->tie[i+1]->coremcd) {
                  if(logdata[lset]->value[array]->depth <= ties->tie[i]->logdep || logdata[lset]->value[array]->depth >= ties->tie[i+1]->logdep) {
                    return;
                  }
                }
              }
            }
          }
          (void)strcat(string, "  tie");
          xmstring = XmStringCreateSimple(string);
          XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          ties->logtie = array;
          last_tie = TIE2;
          have_tie2 = YES;
          ties->tielogdep = logdata[lset]->value[ties->logtie]->depth;
          ties->tielogval = logdata[lset]->value[ties->logtie]->data;
          ties->tielogsmval = logdata[lset]->value[ties->logtie]->sm_data;
          if(XtIsManaged(XtParent(drawingAreaLog))) {
            log_clip_what = NONE;
            DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
          }
        }

        if((int)strcmp(args[0], "but_1_up") == 0 || (int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_3_up") == 0) {
          if(have_tie1 == DO) {
            if(x >= logaxislen) {
              eqlog = TEMP;
            }
            else {
              eqlog = NO;
            }
            if(have_cores == DO && have_splice == DONT) {
              if(ties->coretie != -1) {
                val = *data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->data;
                valsm = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->sm_data;
                err = 0;
              }
              else {
                err = InterpolateCoreTiePtValue(dset, ties->hole, ties->core, NO, ties->tiecorembsf, &val, &dum);
                if(err == 0 && smooth.method != NONE) {
                  derr = InterpolateCoreTiePtValue(dset, ties->hole, ties->core, YES, ties->tiecorembsf, &valsm, &dum);
                }
              }
              if(err == 0) {
                (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       data[dset]->leg,
                       data[dset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
                if(smooth.method != NONE) {
                  (void)sprintf(varstring, " %.2f", valsm);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset ==  YES) {
                  (void)sprintf(varstring, "    %.2f(mcd, Y)", ties->tiecoremcd);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                else if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == NO) {
                  (void)sprintf(varstring, "    %.2f(mcd, N)", ties->tiecoremcd);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                if(eqlog == TEMP) {
                  (void)sprintf(varstring, "   %.2f(eld)", ties->tiecoretempeqlogdep);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                if(ties->coretie == -1) {
                  (void)strcat(string, "  IntTie");
                }
                else {
                  (void)strcat(string, "  Tie");
                }
                xmstring = XmStringCreateSimple(string);
              }
              else {
                xmstring = XmStringCreateSimple("The tie is not within the range of this core");
              }
            }
            else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
              if(ties->coretie != -1) {
                val = splice[sset]->value[ties->coretie]->data;
                valsm = splice[sset]->value[ties->coretie]->sm_data;
                err = 0;
              }
              else {
                err = InterpolateSpliceTiePtValue(sset, NO, ties->tiecoremcd, &val, &dum);
                if(err == 0 && splice[sset]->smooth.method != NONE) {
                  derr = InterpolateSpliceTiePtValue(sset, YES, ties->tiecoremcd, &valsm, &dum);
                }
              }
              if(err == 0) {
                (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       splice[sset]->leg,
                       splice[sset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
                if(splice[sset]->smooth.method != NONE) {
                   (void)sprintf(varstring, "   %.2f", valsm);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                if(eqlog == TEMP) {
                  (void)sprintf(varstring, "   %.2f(mcd)  %.2f(eld)", ties->tiecoremcd, 
                                                    ties->tiecoretempeqlogdep);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                else {
                  (void)sprintf(varstring, "   %.2f(mcd)", ties->tiecoremcd);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                if(ties->coretie == -1) {
                  (void)strcat(string, "  IntTie");
                }
                else {
                  (void)strcat(string, "  Tie");
                }
                xmstring = XmStringCreateSimple(string);
              }
              else {
                xmstring = XmStringCreateSimple("The tie is not within the range of this splice");
              }
            }
            else {
              xmstring = XmStringCreateSimple("");
            }
          }
          else {
            xmstring = XmStringCreateSimple("");
          }
          XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          if(have_tie2 == DO) {
            if(ties->logtie == MISSING) {
              xmstring = XmStringCreateSimple("Tie pt. missing in this log");
            }
            else {
              if(eqlogdepth.did_log_mudline_adj == YES) {
                (void)sprintf(string,"%.3f   %.3f(mud line adj.)   %.3f", ties->tielogdep - eqlogdepth.autocorr_log_mudline_offset,
                              ties->tielogdep, ties->tielogval);
              }
              else if(autocoreadj.draw_results == YES) {
                (void)sprintf(string,"%.3f   %.3f(test mud line adj.)   %.3f", ties->tielogdep, ties->tielogdep + autooff, ties->tielogval);
              }
              else {
                (void)sprintf(string,"%.3f   %.3f", ties->tielogdep, ties->tielogval);
              }
              if(logdata[lset]->info.smooth.method != NONE) {
                (void)sprintf(varstring, " %.2f", ties->tielogsmval);
                (void)strncat(string, varstring, (int)strlen(varstring));
              }
              if(ties->logtie == -1) {
                (void)strcat(string, "  IntTie");
              }
              else {
                (void)strcat(string, "  Tie");
              }
              xmstring = XmStringCreateSimple(string);
            }
          }
          else {
            xmstring = XmStringCreateSimple("");
          }
          XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
        }
      }

      else {
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
      }
    }

    else if(have_ancill == DO && 
            ((scale_val == 1 && x >= logaxislen && x < 2 * logaxislen && strncmp(args[0], "but", 3) == 0) ||
             (have_log == DONT && scale_val == 0 && x >= 0 && x < logaxislen))) {

      if((int)strcmp(args[0], "but_1_up") == 0 || (int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_3_up") == 0) {
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
      }	
      else {

        /* if are in the depth range of the ancillary data set
           find the nearest sample
        */
        diff = 99999.0;
        array = -1;
        if(depth >= ancill.value[0]->depth &&
                 depth <= ancill.value[ancill.data_num-1]->depth) {
          for(i=0; i<ancill.data_num; ++i) {
            checkdep = depth - ancill.value[i]->depth;
            if(ancill.value[i]->data[0] == ancill.missing_value) {
            }
            else if(ancill.value[i]->quality != GOOD) {
            }
            else if(fabs(checkdep) < diff){
              diff = fabs(checkdep);
              array = i;
            }
            else if(fabs(checkdep) > diff){
              break;
            }
          }
        }

        if(array != -1) {
          if(eqlogdepth.did_log_mudline_adj == YES) {
            (void)sprintf(string,"%.3f   %.3f(adj.)",
              (ancill.value[array]->depth - eqlogdepth.autocorr_log_mudline_offset), ancill.value[array]->depth);
          }
          else {
            (void)sprintf(string,"%.3f",
              ancill.value[array]->depth);
          }
          for(i=0; i<ancill.data_cnt; ++i) {
            (void)sprintf(varstring, "   %.2f(%s)", ancill.value[array]->data[i], ancill.id[i+1]);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }

          if((int)strcmp(args[0], "but_1_down") == 0 || (int)strcmp(args[0], "but_1_move") == 0 ||
             (int)strcmp(args[0], "but_2_down") == 0 || (int)strcmp(args[0], "but_2_move") == 0 ||
             (int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0) {
            xmstring = XmStringCreateSimple(string);
            XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
          }
        }
      }
    }

    /* button event in the area of window where core to tie is plotted
       and have a core to tie
    */
    else if(have_log == DO && have_coretotie == YES && x >= 0 && x <= 2 * logaxislen && strncmp(args[0], "but", 3) == 0) {

      if(x >= logaxislen) {
        s = 0;
        while(ties->temp_equivlogdep[s] == BADPT && s < data[dset]->holes[ties->hole]->core[ties->core]->numvalues) {
          ++s;
        }
        if(s >= data[dset]->holes[ties->hole]->core[ties->core]->numvalues) {
          return;
        }
        e = data[dset]->holes[ties->hole]->core[ties->core]->numvalues -1;
        while(ties->temp_equivlogdep[e] == BADPT && e > 0) {
          --e;
        }
        if(e < 1) {
          return;
        }
        eqlog = TEMP;
        chlow = ties->temp_equivlogdep[s];
        chhigh = ties->temp_equivlogdep[e];
      }
      else {
        s = 0;
        while(data[dset]->holes[ties->hole]->core[ties->core]->value[s]->quality != GOOD && 
              s < data[dset]->holes[ties->hole]->core[ties->core]->numvalues) {
          ++s;
        }
        if(s >= data[dset]->holes[ties->hole]->core[ties->core]->numvalues) {
          return;
        }
        e = data[dset]->holes[ties->hole]->core[ties->core]->numvalues -1;
        while(data[dset]->holes[ties->hole]->core[ties->core]->value[e]->quality != GOOD && e > 0) {
          --e;
        }
        if(e < 1) {
          return;
        }
        eqlog = NO;
        chlow = data[dset]->holes[ties->hole]->core[ties->core]->value[s]->mbsf + 
                data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
        chhigh = data[dset]->holes[ties->hole]->core[ties->core]->value[e]->mbsf +
                 data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
      }

      if(depth >= chlow && depth <= chhigh) {

        /* find the nearest sample
        */
        diff = 99999.0;
        array = -1;
        for(k=0; k<data[dset]->holes[ties->hole]->core[ties->core]->numvalues; ++k){
          if(eqlog == TEMP) {
            chlow = depth - ties->temp_equivlogdep[k];
          }
          else if(eqlog == YES) {
            chlow = depth - data[dset]->holes[ties->hole]->core[ties->core]->value[k]->eld;
          }
          else {
            chlow = depth - (data[dset]->holes[ties->hole]->core[ties->core]->value[k]->mbsf +
                    data[dset]->holes[ties->hole]->core[ties->core]->affine_offset);
          }
          checkdep = chlow;
          if(smooth.method == NO && (data[dset]->holes[ties->hole]->core[ties->core]->value[k]->quality & ~BAD_SMOOTH) != GOOD) {
          }
          else if(smooth.method == YES && data[dset]->holes[ties->hole]->core[ties->core]->value[k]->quality != GOOD){
          }
          else if(fabs(checkdep) < diff){
            diff = fabs(checkdep);
            array = k;
          }
          else if(fabs(checkdep) > diff){
            break;
          }
        }

        if(array != -1) {
          (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       data[dset]->leg,
                       data[dset]->site,
                       data[dset]->holes[ties->hole]->name,
                       data[dset]->holes[ties->hole]->core[ties->core]->num,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[array]->type,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[array]->section,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[array]->top_int,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[array]->bot_int,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf,
                       *data[dset]->holes[ties->hole]->core[ties->core]->value[array]->data);
          if(smooth.method != NONE) {
            (void)sprintf(varstring, " %.2f", data[dset]->holes[ties->hole]->core[ties->core]->value[array]->sm_data);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset ==  YES) {
            (void)sprintf(varstring, "    %.2f(mcd, Y)", data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf +
                                                 data[dset]->holes[ties->hole]->core[ties->core]->affine_offset);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          else if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == NO) {
            (void)sprintf(varstring, "    %.2f(mcd, N)", data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf +
                                                 data[dset]->holes[ties->hole]->core[ties->core]->affine_offset);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(eqlog == TEMP) {
            (void)sprintf(varstring, "   %.2f(eld)", ties->temp_equivlogdep[array]);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }

          if((int)strcmp(args[0], "but_1_down") == 0 || (int)strcmp(args[0], "but_1_move") == 0 ||
             (int)strcmp(args[0], "but_2_down") == 0 || (int)strcmp(args[0], "but_2_move") == 0 ||
             (int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0) {
            xmstring = XmStringCreateSimple(string);
            XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
          }

          /* check that tie pt does not cause crossing of tie lines
          */
          if(((int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_2_down") == 0 ||
             (int)strcmp(args[0], "but_2_move") == 0) && mapped == True) {
            if(have_tie2 == DO && ties->nties > 0) {
              if(ties->tielogdep < ties->tie[0]->logdep) {
                if(data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf >= ties->tie[0]->corembsf) {
                  return;
                }
              }
              else if(ties->tielogdep > ties->tie[ties->nties-1]->logdep) {
                if(data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf <= ties->tie[ties->nties-1]->corembsf) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tielogdep > ties->tie[i]->logdep && ties->tielogdep < ties->tie[i+1]->logdep) {
                    if(data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf <= ties->tie[i]->corembsf || 
                       data[dset]->holes[ties->hole]->core[ties->core]->value[array]->mbsf >= ties->tie[i+1]->corembsf) {
                      return;
                    }
                  }
                }
              }
            }
            (void)strcat(string, "  tie");
            xmstring = XmStringCreateSimple(string);
            XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
            ties->coretie = array;
            last_tie = TIE1;
            have_tie1 = YES;
            ties->tiecoreholename = data[dset]->holes[ties->hole]->name;
            ties->tiecorename = data[dset]->holes[ties->hole]->core[ties->core]->num;
            ties->tiecoretype = data[dset]->holes[ties->hole]->core[ties->core]->value[0]->type;
            strcpy(ties->tiecoresection ,data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->section);
            ties->tiecoretopsect = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->top_int;
            ties->tiecorebotsect = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->bot_int;
            ties->tiecorembsf = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->mbsf;
            ties->tiecoremcd = ties->tiecorembsf + data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
            ties->tiecoretempeqlogdep = ties->temp_equivlogdep[ties->coretie];
            if(eqlogdepth.autocorr_status == YES) {
              ties->tiecoreautoeld = ties->tiecoremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              ties->tiecoreautoeld = ties->tiecoremcd;
            }
            if(XtIsManaged(XtParent(drawingAreaLog))) {
              log_clip_what = NONE;
              DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
            }
          }

          if((int)strcmp(args[0], "but_1_up") == 0 || (int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_3_up") == 0) {
            if(have_tie1 == DO) {
              if(x >= logaxislen) {
                eqlog = TEMP;
              }
              else {
                eqlog = NO;
              }
              if(have_cores == DO && have_splice == DONT) {
                if(ties->coretie != -1) {
                  val = *data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->data;
                  valsm = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->sm_data;
                  err = 0;
                }
                else {
                  err = InterpolateCoreTiePtValue(dset, ties->hole, ties->core, NO, ties->tiecorembsf, &val, &dum);
                  if(err == 0 && smooth.method != NONE) {
                    derr = InterpolateCoreTiePtValue(dset, ties->hole, ties->core, YES, ties->tiecorembsf, &valsm, &dum);
                  }
                }
                if(err == 0) {
                  (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       data[dset]->leg,
                       data[dset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
                  if(smooth.method != NONE) {
                    (void)sprintf(varstring, " %.2f", valsm);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset ==  YES) {
                    (void)sprintf(varstring, "    %.2f(mcd, Y)", ties->tiecoremcd);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  else if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == NO) {
                    (void)sprintf(varstring, "    %.2f(mcd, N)", ties->tiecoremcd);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  if(eqlog == TEMP) {
                    (void)sprintf(varstring, "   %.2f(eld)", ties->tiecoretempeqlogdep);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  if(ties->coretie == -1) {
                    (void)strcat(string, "  IntTie");
                  }
                  else {
                    (void)strcat(string, "  Tie");
                  }
                  xmstring = XmStringCreateSimple(string);
                }
                else {
                  xmstring = XmStringCreateSimple("The tie is not within the range of this core");
                }
              }
              else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
                if(ties->coretie != -1) {
                  val = splice[sset]->value[ties->coretie]->data;
                  valsm = splice[sset]->value[ties->coretie]->sm_data;
                  err = 0;
                }
                else {
                  err = InterpolateSpliceTiePtValue(sset, NO, ties->tiecoremcd, &val, &dum);
                  if(err == 0 && splice[sset]->smooth.method != NONE) {
                    derr = InterpolateSpliceTiePtValue(sset, YES, ties->tiecoremcd, &valsm, &dum);
                  }
                }
                if(err == 0) {
                  (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       splice[sset]->leg,
                       splice[sset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
                  if(splice[sset]->smooth.method != NONE) {
                     (void)sprintf(varstring, "   %.2f", valsm);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  if(eqlog == TEMP) {
                    (void)sprintf(varstring, "   %.2f(mcd)  %.2f(eld)", ties->tiecoremcd,
                                                    ties->tiecoretempeqlogdep);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  else {
                    (void)sprintf(varstring, "   %.2f(mcd)", ties->tiecoremcd);
                    (void)strncat(string, varstring, (int)strlen(varstring));
                  }
                  if(ties->coretie == -1) {
                    (void)strcat(string, "  IntTie");
                  }
                  else {
                    (void)strcat(string, "  Tie");
                  }
                  xmstring = XmStringCreateSimple(string);
                }
                else {
                  xmstring = XmStringCreateSimple("The tie is not within the range of this splice");
                }
              }
              else {
                xmstring = XmStringCreateSimple("");
              }
            }
            else {
              xmstring = XmStringCreateSimple("");
            }
            XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
            if(have_tie2 == DO) {
              if(ties->logtie == MISSING) {
                xmstring = XmStringCreateSimple("Tie pt. missing in this log");
              }
              else {
                if(eqlogdepth.did_log_mudline_adj == YES) {
                  (void)sprintf(string,"%.3f   %.3f(mud line adj.)   %.3f", ties->tielogdep - eqlogdepth.autocorr_log_mudline_offset,
                                ties->tielogdep, ties->tielogval);
                }
                else if(autocoreadj.draw_results == YES) {
                  (void)sprintf(string,"%.3f   %.3f(test mud line adj.)   %.3f", ties->tielogdep, ties->tielogdep + autooff, ties->tielogval);
                }
                else {
                  (void)sprintf(string,"%.3f   %.3f", ties->tielogdep, ties->tielogval);
                }
                if(logdata[lset]->info.smooth.method != NONE) {
                  (void)sprintf(varstring, " %.2f", ties->tielogsmval);
                  (void)strncat(string, varstring, (int)strlen(varstring));
                }
                if(ties->logtie == -1) {
                  (void)strcat(string, "  IntTie");
                }
                else {
                  (void)strcat(string, "  Tie");
                }
                xmstring = XmStringCreateSimple(string);
              }
            }
            else {
              xmstring = XmStringCreateSimple("");
            }
            XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
          }
        }
        else {
          xmstring = XmStringCreateSimple("");
          XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
        }
      }
      else {
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
      }
    }

    else if(have_splice == DO && sset >= 0 && sset < splice_cnt &&
            (scale_val == 0 && x >= 0 && x <= 2 * logaxislen && strncmp(args[0], "but", 3) == 0)) {
      if(x >= logaxislen) {
        eqlog = TEMP;
        chlow = ties->temp_equivlogdep[0];
        chhigh = ties->temp_equivlogdep[splice[sset]->numvalues-1];
      }
      else {
        eqlog = NO;
        chlow = splice[sset]->value[0]->mcd;
        chhigh = splice[sset]->value[splice[sset]->numvalues-1]->mcd;
      }
      if(depth >= chlow && depth <= chhigh) {

        /* find the nearest sample
        */
        diff = 99999.0;
        array = -1;
        for(k=0; k<splice[sset]->numvalues; ++k){
          if(eqlog == TEMP) {
            chlow = depth - ties->temp_equivlogdep[k];
          }
          else {
            chlow = depth - splice[sset]->value[k]->mcd;
          }
          checkdep = chlow;
          if(splice[sset]->smooth.method == NO && (splice[sset]->value[k]->quality & ~BAD_SMOOTH) != GOOD) {
          }
          else if(splice[sset]->smooth.method == YES && splice[sset]->value[k]->quality != GOOD){
          }
          else if(fabs(checkdep) < diff){
            diff = fabs(checkdep);
            array = k;
          }
          else if(fabs(checkdep) > diff){
            break;
          }
        }
  
        if(array != -1) {
          (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       splice[sset]->leg,
                       splice[sset]->site,
                       splice[sset]->value[array]->hole,
                       splice[sset]->value[array]->corename,
                       splice[sset]->value[array]->type,
                       splice[sset]->value[array]->section,
                       splice[sset]->value[array]->sect_top,
                       splice[sset]->value[array]->sect_bot,
                       splice[sset]->value[array]->mbsf,
                       splice[sset]->value[array]->data);
          if(splice[sset]->smooth.method != NONE) {
            (void)sprintf(varstring, "   %.2f", splice[sset]->value[array]->sm_data);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(eqlog == TEMP) {
            (void)sprintf(varstring, "   %.2f(mcd)  %.2f(eld)", splice[sset]->value[array]->mcd, 
                                                    ties->temp_equivlogdep[array]);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          else {
            (void)sprintf(varstring, "   %.2f(mcd)", splice[sset]->value[array]->mcd);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          xmstring = XmStringCreateSimple(string);
          XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          if(XtIsManaged(formAutoCorrelate)) {
            return;
          }
          if(have_log == DONT) {
            return;
          }

          /* check that tie pt does not cause crossing of tie lines
          */
          if(((int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_2_down") == 0 ||
             (int)strcmp(args[0], "but_2_move") == 0) && mapped == True) {
            if(have_tie2 == DO && ties->nties > 0) {
              if(ties->tielogdep < ties->tie[0]->logdep) {
                if(splice[sset]->value[array]->mbsf >= ties->tie[0]->corembsf) {
                  return;
                }
              }
              else if(ties->tielogdep > ties->tie[ties->nties-1]->logdep) {
                if(splice[sset]->value[array]->mbsf <= ties->tie[ties->nties-1]->corembsf) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tielogdep > ties->tie[i]->logdep && ties->tielogdep < ties->tie[i+1]->logdep) {
                    if(splice[sset]->value[array]->mbsf <= ties->tie[i]->corembsf ||
                       splice[sset]->value[array]->mbsf >= ties->tie[i+1]->corembsf) {
                      return;
                    }
                  }
                }
              }
            }
            (void)strcat(string, "  tie");
            xmstring = XmStringCreateSimple(string);
            XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
            ties->coretie = array;
            last_tie = TIE1;
            have_tie1 = YES;
            ties->tiecoreholename = splice[sset]->value[ties->coretie]->hole;
            ties->tiecorename = splice[sset]->value[ties->coretie]->corename;
            ties->tiecoretype = splice[sset]->value[ties->coretie]->type;
            strcpy(ties->tiecoresection , splice[sset]->value[ties->coretie]->section);
            ties->tiecoretopsect = splice[sset]->value[ties->coretie]->sect_top;
            ties->tiecorebotsect = splice[sset]->value[ties->coretie]->sect_bot;
            ties->tiecorembsf = splice[sset]->value[ties->coretie]->mbsf;
            ties->tiecoremcd = splice[sset]->value[ties->coretie]->mcd;
            ties->tiecoretempeqlogdep = ties->temp_equivlogdep[ties->coretie];
            if(eqlogdepth.autocorr_status == YES) {
              ties->tiecoreautoeld = ties->tiecoremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              ties->tiecoreautoeld = ties->tiecoremcd;
            }
            if(XtIsManaged(XtParent(drawingAreaLog))) {
              log_clip_what = NONE;
              DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
            }
          }
        }
        else {
          xmstring = XmStringCreateSimple("");
          XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
        }
      }
    } 


    /* if event was an arrow
    */
    else if(((int)strcmp(args[0], "arrow_up") == 0 || (int)strcmp(args[0], "arrow_down") == 0) &&
            ((have_log == DO && have_coretotie == DO) || 
             (have_splice == DO && sset >= 0 && sset < splice_cnt && have_log == DO)) &&
                      last_tie != NONE && mapped == True){

      report = NO;

      if((int)strcmp(args[0], "arrow_up") == 0) {
        if(last_tie == TIE1 && have_coretotie == DO){
          if(ties->coretie == -1) {
            err = FindWhereInCore(data[dset]->holes[ties->hole]->core[ties->core], ties->tiecorembsf, &place);
            if(err == 0) { 
              ties->coretie = place + 1;
            }
          }
          if(ties->coretie - 1 >= 0 && ties->coretie < data[dset]->holes[ties->hole]->core[ties->core]->numvalues){
            if(have_tie2 == DO && ties->nties > 0) {
              if(ties->tielogdep < ties->tie[0]->logdep) {
                if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie-1]->mbsf >= ties->tie[0]->corembsf) {
                  return;
                }
              }
              else if(ties->tielogdep > ties->tie[ties->nties-1]->logdep) {
                if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie-1]->mbsf <= ties->tie[ties->nties-1]->corembsf) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tielogdep > ties->tie[i]->logdep && ties->tielogdep < ties->tie[i+1]->logdep) {
                    if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie-1]->mbsf <= ties->tie[i]->corembsf ||
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie-1]->mbsf >= ties->tie[i+1]->corembsf) {
                      return;
                    }
                  }
                }
              }
            }
            if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie-1]->quality != GOOD) {
              i = ties->coretie - 1;
              while(data[dset]->holes[ties->hole]->core[ties->core]->value[i]->quality != GOOD && i>0) {
                --i;
              }
              if(i<0) {
                return;
              }
              else if(i == 0 && data[dset]->holes[ties->hole]->core[ties->core]->value[i]->quality != GOOD) {
                return;
              }
              else {
                ties->coretie = i;
              }
            }
            else {
              --ties->coretie;
            }
            report = TIE1;
            ties->tiecoreholename = data[dset]->holes[ties->hole]->name;
            ties->tiecorename = data[dset]->holes[ties->hole]->core[ties->core]->num;
            ties->tiecoretype = data[dset]->holes[ties->hole]->core[ties->core]->value[0]->type;
            strcpy(ties->tiecoresection ,data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->section);
            ties->tiecoretopsect = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->top_int;
            ties->tiecorebotsect = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->bot_int;
            ties->tiecorembsf = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->mbsf;
            ties->tiecoremcd = ties->tiecorembsf + data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
            ties->tiecoretempeqlogdep = ties->temp_equivlogdep[ties->coretie];
            if(eqlogdepth.autocorr_status == YES) {
              ties->tiecoreautoeld = ties->tiecoremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              ties->tiecoreautoeld = ties->tiecoremcd;
            }
          }
          else {
            return;
          }
        }
        else if(last_tie ==  TIE1 && have_splice == DO){
          if(ties->coretie == -1) {
            err = FindWhereInSplice(sset, ties->tiecoremcd, &place);
            if(err == 0) {
              ties->coretie = place + 1;
            }
          }
          if(ties->coretie - 1 >= 0){
            if(have_tie2 == DO && ties->nties > 0) {
              if(ties->tielogdep < ties->tie[0]->logdep) {
                if(splice[sset]->value[ties->coretie-1]->mcd >= ties->tie[0]->coremcd) {
                  return;
                }
              }
              else if(ties->tielogdep > ties->tie[ties->nties-1]->logdep) {
                if(splice[sset]->value[ties->coretie-1]->mcd <= ties->tie[ties->nties-1]->coremcd) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tielogdep > ties->tie[i]->logdep && ties->tielogdep < ties->tie[i+1]->logdep) {
                    if(splice[sset]->value[ties->coretie-1]->mcd <= ties->tie[i]->coremcd ||
                       splice[sset]->value[ties->coretie-1]->mcd >= ties->tie[i+1]->coremcd) {
                      return;
                    }
                  }
                }
              }
            }
            if(splice[sset]->value[ties->coretie-1]->quality != GOOD) {
              i = ties->coretie - 1;
              while(splice[sset]->value[i]->quality != GOOD && i>0) {
                --i;
              }
              if(i<0) {
                return;
              }
              else if(i == 0 && splice[sset]->value[i]->quality != GOOD) {
                return;
              }
              else {
                ties->coretie = i;
              }
            }
            else {
              --ties->coretie;
            }
            report = TIE1;
            ties->tiecoreholename = splice[sset]->value[ties->coretie]->hole;
            ties->tiecorename = splice[sset]->value[ties->coretie]->corename;
            ties->tiecoretype = splice[sset]->value[ties->coretie]->type;
            strcpy(ties->tiecoresection , splice[sset]->value[ties->coretie]->section);
            ties->tiecoretopsect = splice[sset]->value[ties->coretie]->sect_top;
            ties->tiecorebotsect = splice[sset]->value[ties->coretie]->sect_bot;
            ties->tiecorembsf = splice[sset]->value[ties->coretie]->mbsf;
            ties->tiecoremcd = splice[sset]->value[ties->coretie]->mcd;
            ties->tiecoretempeqlogdep = ties->temp_equivlogdep[ties->coretie];
            if(eqlogdepth.autocorr_status == YES) {
              ties->tiecoreautoeld = ties->tiecoremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              ties->tiecoreautoeld = ties->tiecoremcd;
            }
          }
          else {
            return;
          }
        }
        else if(last_tie ==  TIE2 && have_log == DO){
          if(ties->logtie == -1) {
            err = FindWhereInLog(logdata[lset], ties->tielogdep, &place);
            if(err == 0) {
              ties->logtie = place + 1;
            }
          }
          if(ties->logtie - 1 >= 0 && ties->logtie - 1 < logdata[lset]->data_num){
            if(have_tie1 == DO && ties->nties > 0) {
              if(ties->tiecorembsf < ties->tie[0]->corembsf) {
                if(logdata[lset]->value[ties->logtie-1]->depth >= ties->tie[0]->logdep) {
                  return;
                }
              }
              else if(ties->tiecorembsf > ties->tie[ties->nties-1]->corembsf) {
                if(logdata[lset]->value[ties->logtie-1]->depth <= ties->tie[ties->nties-1]->logdep) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tiecorembsf > ties->tie[i]->corembsf && ties->tiecorembsf < ties->tie[i+1]->corembsf) {
                    if(logdata[lset]->value[ties->logtie-1]->depth <= ties->tie[i]->logdep || 
                       logdata[lset]->value[ties->logtie-1]->depth >= ties->tie[i+1]->logdep) {
                      return;
                    }
                  }
                }
              }
            }
            if(logdata[lset]->value[ties->logtie-1]->data == logdata[lset]->info.missing_value || logdata[lset]->value[ties->logtie-1]->quality != GOOD) {
              i = ties->logtie - 1;
              while((logdata[lset]->value[i]->data == logdata[lset]->info.missing_value || logdata[lset]->value[i]->quality != GOOD) &&
                     i>0) {
                --i;
              }
              if(i<0) {
                return;
              }
              else if(i == 0 && (logdata[lset]->value[i]->data == logdata[lset]->info.missing_value || logdata[lset]->value[i]->quality != GOOD)) {
                return;
              }
              else {
                ties->logtie = i;
              }
            }
            else {
              --ties->logtie;
            }
            report = TIE2;
            ties->tielogdep = logdata[lset]->value[ties->logtie]->depth;
            ties->tielogval = logdata[lset]->value[ties->logtie]->data;
            ties->tielogsmval = logdata[lset]->value[ties->logtie]->sm_data;
          }
          else {
            return;
          }
        }
      }

      /* if event was an arrow key down, increment the array number
      */
      else if((int)strcmp(args[0], "arrow_down") == 0 && last_tie != NONE && 
              ((have_coretotie == DO && have_log == DO) || (have_splice == DO && have_log == DO))){

        if(last_tie == TIE1 && have_coretotie == DO){
          if(ties->coretie == -1) {
            err = FindWhereInCore(data[dset]->holes[ties->hole]->core[ties->core], ties->tiecorembsf, &place);
            if(err == 0) {
              ties->coretie = place;
            }
          }
          if(ties->coretie != -1 && ties->coretie + 1 < data[dset]->holes[ties->hole]->core[ties->core]->numvalues){
            if(have_tie2 == DO && ties->nties > 0) {
              if(ties->tielogdep < ties->tie[0]->logdep) {
                if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie+1]->mbsf >= ties->tie[0]->corembsf) {
                  return;
                }
              }
              else if(ties->tielogdep > ties->tie[ties->nties-1]->logdep) {
                if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie+1]->mbsf <= ties->tie[ties->nties-1]->corembsf) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tielogdep > ties->tie[i]->logdep && ties->tielogdep < ties->tie[i+1]->logdep) {
                    if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie+1]->mbsf <= ties->tie[i]->corembsf ||
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie+1]->mbsf >= ties->tie[i+1]->corembsf) {
                      return;
                    }
                  }
                }
              }
            }
            if(data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie+1]->quality != GOOD) {
              i = ties->coretie + 1;
              while(data[dset]->holes[ties->hole]->core[ties->core]->value[i]->quality != GOOD && 
                    i<data[dset]->holes[ties->hole]->core[ties->core]->numvalues-1) {
                ++i;
              }
              if(i>=data[dset]->holes[ties->hole]->core[ties->core]->numvalues) {
                return;
              }
              else if(i == data[dset]->holes[ties->hole]->core[ties->core]->numvalues - 1 &&
                      data[dset]->holes[ties->hole]->core[ties->core]->value[i]->quality != GOOD) {
                return;
              }
              else {
                ties->coretie = i;
              }
            }
            else {
              ++ties->coretie;
            }
            report = TIE1;
            ties->tiecoreholename = data[dset]->holes[ties->hole]->name;
            ties->tiecorename = data[dset]->holes[ties->hole]->core[ties->core]->num;
            ties->tiecoretype = data[dset]->holes[ties->hole]->core[ties->core]->value[0]->type;
            strcpy(ties->tiecoresection ,data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->section);
            ties->tiecoretopsect = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->top_int;
            ties->tiecorebotsect = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->bot_int;
            ties->tiecorembsf = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->mbsf;
            ties->tiecoremcd = ties->tiecorembsf + data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
            ties->tiecoretempeqlogdep = ties->temp_equivlogdep[ties->coretie];
            if(eqlogdepth.autocorr_status == YES) {
              ties->tiecoreautoeld = ties->tiecoremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              ties->tiecoreautoeld = ties->tiecoremcd;
            }
          }
          else {
            return;
          }   
        }
        else if(last_tie ==  TIE1 && have_splice == DO){
          if(ties->coretie == -1) {
            err = FindWhereInSplice(sset, ties->tiecoremcd, &place);
            if(err == 0) {
              ties->coretie = place;
            }
          }
          if(ties->coretie != -1 && ties->coretie + 1 < splice[sset]->numvalues){
            if(have_tie2 == DO && ties->nties > 0) {
              if(ties->tielogdep < ties->tie[0]->logdep) {
                if(splice[sset]->value[ties->coretie+1]->mcd >= ties->tie[0]->coremcd) {
                  return;
                }
              }
              else if(ties->tielogdep > ties->tie[ties->nties-1]->logdep) {
                if(splice[sset]->value[ties->coretie+1]->mcd <= ties->tie[ties->nties-1]->coremcd) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tielogdep > ties->tie[i]->logdep && ties->tielogdep < ties->tie[i+1]->logdep) {
                    if(splice[sset]->value[ties->coretie+1]->mcd <= ties->tie[i]->coremcd ||
                       splice[sset]->value[ties->coretie+1]->mcd >= ties->tie[i+1]->coremcd) {
                      return;
                    }
                  }
                }
              }
            }
            if(splice[sset]->value[ties->coretie+1]->quality != GOOD) {
              i = ties->coretie + 1;
              while(splice[sset]->value[i]->quality != GOOD && i<splice[sset]->numvalues-1) {
                ++i;
              }
              if(i>=splice[sset]->numvalues) {
                return;
              }
              else if(i == splice[sset]->numvalues - 1 && splice[sset]->value[i]->quality != GOOD) {
                return;
              }
              else {
                ties->coretie = i;
              }
            }
            else {
              ++ties->coretie;
            }
            report = TIE1;
            ties->tiecoreholename = splice[sset]->value[ties->coretie]->hole;
            ties->tiecorename = splice[sset]->value[ties->coretie]->corename;
            ties->tiecoretype = splice[sset]->value[ties->coretie]->type;
            strcpy(ties->tiecoresection , splice[sset]->value[ties->coretie]->section);
            ties->tiecoretopsect = splice[sset]->value[ties->coretie]->sect_top;
            ties->tiecorebotsect = splice[sset]->value[ties->coretie]->sect_bot;
            ties->tiecorembsf = splice[sset]->value[ties->coretie]->mbsf;
            ties->tiecoremcd = splice[sset]->value[ties->coretie]->mcd;
            ties->tiecoretempeqlogdep = ties->temp_equivlogdep[ties->coretie];
            if(eqlogdepth.autocorr_status == YES) {
              ties->tiecoreautoeld = ties->tiecoremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              ties->tiecoreautoeld = ties->tiecoremcd;
            }
          }
          else {
            return;
          }
        }
        else if(last_tie == TIE2 && have_log == DO){
          if(ties->logtie == -1) {
            err = FindWhereInLog(logdata[lset], ties->tielogdep, &place);
            if(err == 0) {
              ties->logtie = place;
            }
          }
          if(ties->logtie + 1 < logdata[lset]->data_num && ties->logtie >= 0){
            if(have_tie1 == DO && ties->nties > 0) {
              if(ties->tiecorembsf < ties->tie[0]->corembsf) {
                if(logdata[lset]->value[ties->logtie+1]->depth >= ties->tie[0]->logdep) {
                  return;
                }
              }
              else if(ties->tiecorembsf > ties->tie[ties->nties-1]->corembsf) {
                if(logdata[lset]->value[ties->logtie+1]->depth <= ties->tie[ties->nties-1]->logdep) {
                  return;
                }
              }
              else if(ties->nties > 1) {
                for(i=0; i<ties->nties-1; ++i) {
                  if(ties->tiecorembsf > ties->tie[i]->corembsf && ties->tiecorembsf < ties->tie[i+1]->corembsf) {
                    if(logdata[lset]->value[ties->logtie+1]->depth <= ties->tie[i]->logdep ||
                       logdata[lset]->value[ties->logtie+1]->depth >= ties->tie[i+1]->logdep) {
                      return;
                    }
                  }
                }
              }
            }
            if(logdata[lset]->value[ties->logtie+1]->data == logdata[lset]->info.missing_value || logdata[lset]->value[ties->logtie+1]->quality != GOOD) {
              i = ties->logtie + 1;
              while((logdata[lset]->value[i]->data == logdata[lset]->info.missing_value || logdata[lset]->value[i]->quality != GOOD) &&
                     i<logdata[lset]->data_num-1) {
                ++i;
              }
              if(i>=logdata[lset]->data_num) {
                return;
              }
              else if(i == logdata[lset]->data_num-1 && 
                      (logdata[lset]->value[i]->data == logdata[lset]->info.missing_value || logdata[lset]->value[i]->quality != GOOD)) {
                return;
              }
              else {
                ties->logtie = i;
              }
            }
            else {
              ++ties->logtie;
            }
            report = TIE2;
            ties->tielogdep = logdata[lset]->value[ties->logtie]->depth;
          }
          else {  
            return;
          }
        }
      }

      if((report == TIE1 || report == TIE2) && XtIsManaged(XtParent(drawingAreaLog)) == True) {
        log_clip_what = NONE;
        DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
      }

      if(have_coretotie == DO && report == TIE1) {
        (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       data[dset]->leg,
                       data[dset]->site,
                       data[dset]->holes[ties->hole]->name,
                       data[dset]->holes[ties->hole]->core[ties->core]->num,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->type,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->section,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->top_int,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->bot_int,
                       data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->mbsf,
                       *data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->data);
        if(smooth.method != NONE) {
          (void)sprintf(varstring, " %.2f", data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->sm_data);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }
        if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == YES) {
          (void)sprintf(varstring, "    %.2f(mcd, Y)", data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->mbsf +
                                               data[dset]->holes[ties->hole]->core[ties->core]->affine_offset);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }
        else if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == NO) {
          (void)sprintf(varstring, "    %.2f(mcd, N)", data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->mbsf +
                                                 data[dset]->holes[ties->hole]->core[ties->core]->affine_offset);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }
        (void)sprintf(varstring, "   %.2f(eld)", ties->temp_equivlogdep[ties->coretie]);
        (void)strncat(string, varstring, (int)strlen(varstring));
        (void)strcat(string, "  tie");
        xmstring = XmStringCreateSimple(string);
        XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
      }
      if(have_splice == DO && report == TIE1) {
        (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                     splice[sset]->leg,
                     splice[sset]->site,
                     splice[sset]->value[ties->coretie]->hole,
                     splice[sset]->value[ties->coretie]->corename,
                     splice[sset]->value[ties->coretie]->type,
                     splice[sset]->value[ties->coretie]->section,
                     splice[sset]->value[ties->coretie]->sect_top,
                     splice[sset]->value[ties->coretie]->sect_bot,
                     splice[sset]->value[ties->coretie]->mbsf,
                     splice[sset]->value[ties->coretie]->data);
        if(splice[sset]->smooth.method != NONE) {
          (void)sprintf(varstring, "   %.2f", splice[sset]->value[ties->coretie]->sm_data);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }
        (void)sprintf(varstring, "   %.2f(mcd)  %.2f(eld)", splice[sset]->value[ties->coretie]->mcd, 
                                                    ties->temp_equivlogdep[ties->coretie]);
        (void)strncat(string, varstring, (int)strlen(varstring));
        (void)strcat(string, "  tie");
        xmstring = XmStringCreateSimple(string);
        XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
        if(XtIsManaged(formAutoCorrelate)) {
          return;
        }
        if(have_log == DONT) {
          return;
        }
      }
      else if(report == TIE2) {
        (void)sprintf(string,"%.3f   %.3f",
          logdata[lset]->value[ties->logtie]->depth, logdata[lset]->value[ties->logtie]->data);
        if(logdata[lset]->info.smooth.method != NONE) {
          (void)sprintf(varstring, " %.2f", logdata[lset]->value[ties->logtie]->sm_data);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }
        (void)strcat(string, "  tie");
        xmstring = XmStringCreateSimple(string);
        XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
      }
    }

    else {
      xmstring = XmStringCreateSimple("");
      XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
      XmStringFree(xmstring);
    }

    /* if have either tie allow clear tie
    */
    if(have_tie1 == DO || have_tie2 == DO) {
      XtSetSensitive(pushButtonClearTie, True);
    }
    /* if have both tie points
    */
    if(have_tie1 == DO && have_tie2 == DO && 
       ((have_coretotie == DO && have_log == DO) || (have_splice == DO && have_log == DO)) && mapped == True) {
      XtSetSensitive(pushButtonAddTieToList, True);
      XtSetSensitive(pushButtonClearLastTieSaved, False);

      /* if this core not affine adjust disable share tie
      */
      if(have_coretotie == DO && have_affine == DO) {
        if(data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == YES) {
          XmToggleButtonSetState(toggleButtonAddTieToCore, False, False);
          XmToggleButtonSetState(toggleButtonAddTieToAll, True, False);
          XtSetSensitive(toggleButtonAddTieToAll, True);
          XtSetSensitive(toggleButtonAddTieToCore, True);
        }
        else {
          XmToggleButtonSetState(toggleButtonAddTieToCore, True, False);
          XmToggleButtonSetState(toggleButtonAddTieToAll, False, False);
          XtSetSensitive(toggleButtonAddTieToAll, False);
          XtSetSensitive(toggleButtonAddTieToCore, True);
        }
      }
      else if(have_coretotie == DO && have_affine == DONT) {
        XmToggleButtonSetState(toggleButtonAddTieToCore, True, False);
        XmToggleButtonSetState(toggleButtonAddTieToAll, False, False);
        XtSetSensitive(toggleButtonAddTieToAll, False);
        XtSetSensitive(toggleButtonAddTieToCore, True);
      }
      else if(have_splice == DO) {
        XmToggleButtonSetState(toggleButtonAddTieToCore, False, False);
        XmToggleButtonSetState(toggleButtonAddTieToAll, True, False);
        XtSetSensitive(toggleButtonAddTieToAll, True);
        XtSetSensitive(toggleButtonAddTieToCore, True);
      }
      else {
      }

      if((int)strcmp(args[0], "but_2_down") == 0 || (int)strcmp(args[0], "but_2_move") == 0 ||
         (int)strcmp(args[0], "but_3_down") == 0 || (int)strcmp(args[0], "but_3_move") == 0 ||
         (int)strcmp(args[0], "arrow_up") == 0 || (int)strcmp(args[0], "arrow_down") == 0) {
        have_ties = DO;
        have_correlate = DONT;
        err = 0;
        if(have_coretotie == DO) {
          err = CorrelationSetup(dset);
        }
        else if(have_splice == DO) {
          err = CorrelationSetup(0);
        }
        if(err != 0) {
          (void)sprintf(buffer, "error doing correlation.\n");
          have_correlate = DONT;
          SaManageWarningBox(buffer);
        }
        else {
          have_correlate = DO;
          XtSetSensitive(pushButtonReCorrelate, True);
        }
      }

      /* if event was return key press put the current tie in tie list
         check first that it is not allready in list and that it does
         not conflict with any existing ties (overlap, etc.)
      */
      else if((int)strcmp(args[0], "return_down") == 0){

        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
        XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);

        have_tie1 = DONT;
        have_tie2 = DONT;
        last_tie = NONE;
        have_ties = DONT;
        have_correlate = DONT;
        ties->have_new_ties = DO;

        if(ties->nties >= ties->alloc_cnt) {
          if((charptr=(char*)realloc((char*)ties->tie,(ties->alloc_cnt+MAXTIES)*sizeof(TieList*)))==NULL) {
                                SaBadMemAlloc("");
          }
          else {
            ties->tie = (TieList**)charptr;
          }
          for(i=ties->alloc_cnt; i<ties->alloc_cnt+MAXTIES; ++i) {
            if((ties->tie[i]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                                SaBadMemAlloc("Sa");
          }
          ties->alloc_cnt = ties->alloc_cnt + MAXTIES;
        }
        err = GetTieListPosition(NO, dset, &pos, ties);
        ties->lasttiesaved = pos - 1;

        if(err == 0) {
          if(have_coretotie == DO) {
            (void)sprintf(string,"Core:  %4d %4d %c  %3d %c %2s %5.1f %5.1f  %8.2fmbsf",
                       data[dset]->leg,
                       data[dset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf);
            if(have_affine == DO) {
              (void)sprintf(varstring, "  %8.2fmcd", ties->tiecoremcd);
              (void)strncat(string, varstring, (int)strlen(varstring));
            }
          }
          else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
            (void)sprintf(string,"Core:  %4d %4d %c  %3d %c %2s %5.1f %5.1f  %8.2fmbsf %8.2fmcd",
                       splice[sset]->leg,
                       splice[sset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       ties->tiecoremcd);
          }
          if(eqlogdepth.autocorr_status == YES) {
            (void)sprintf(varstring, "  %8.2feld(auto)", ties->tiecoreautoeld);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }

          (void)sprintf(varstring, " Tied to  Log:  %8.3fm", ties->tielogdep);
          (void)strncat(string, varstring, (int)strlen(varstring));
          if(ties->tie[ties->lasttiesaved]->shared == YES) {
            (void)strcat(string, "  S");
          }
          else {
            (void)strcat(string, "  NS");
          }

          xmstring = XmStringCreateSimple(string);
          XmListAddItemUnselected(listTieList, xmstring, pos);
          XmStringFree(xmstring);
          AdjustToTies(dset);
        }
        else {
        }

        XtSetSensitive(pushButtonClearTie, False);
        XtSetSensitive(pushButtonClearLastTieSaved, True);
        XtSetSensitive(pushButtonAddTieToList, False);
        XtSetSensitive(toggleButtonAddTieToCore, False);
        XtSetSensitive(toggleButtonAddTieToAll, False);
        XtSetSensitive(pushButtonSaveChanges, True);
        XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);

        if(XtIsManaged(XtParent(drawingAreaOptimizeTieCorrelate))) {
          XClearWindow(XtDisplay(drawingAreaOptimizeTieCorrelate), XtWindow(drawingAreaOptimizeTieCorrelate));
        }
        if(XtIsManaged(XtParent(drawingAreaCorrelate))) {
          XClearWindow(XtDisplay(drawingAreaCorrelate), XtWindow(drawingAreaCorrelate));
        }
        if(XtIsManaged(XtParent(drawingAreaLog))) {
          log_clip_what = NONE;
          DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
        }
        if(XtIsManaged(XtParent(drawingAreaAdjustToTiesInfo))) {
          DrawDrawingAreaAdjustToTiesInfo(drawingAreaAdjustToTiesInfo, (XtPointer)NULL, (XtPointer)NULL);
        }
      }
    }
  }
  if((int)strcmp(args[0], "but_1_up") == 0 || (int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_3_up") == 0) {
    if(have_tie1 == DO) {
      if(x >= logaxislen) {
        eqlog = TEMP;
      }
      else {
        eqlog = NO;
      }
      if(have_cores == DO && have_splice == DONT) {
        if(ties->coretie != -1) {
          val = *data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->data;
          valsm = data[dset]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->sm_data;
          err = 0;
        }
        else {
          err = InterpolateCoreTiePtValue(dset, ties->hole, ties->core, NO, ties->tiecorembsf, &val, &dum);
          if(err == 0 && smooth.method != NONE) {
            derr = InterpolateCoreTiePtValue(dset, ties->hole, ties->core, YES, ties->tiecorembsf, &valsm, &dum);
          }
        }
        if(err == 0) {
          (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                      data[dset]->leg,
                      data[dset]->site,
                      ties->tiecoreholename,
                      ties->tiecorename,
                      ties->tiecoretype,
                      ties->tiecoresection,
                      ties->tiecoretopsect,
                      ties->tiecorebotsect,
                      ties->tiecorembsf,
                      val);
          if(smooth.method != NONE) {
            (void)sprintf(varstring, " %.2f", valsm);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset ==  YES) {
            (void)sprintf(varstring, "    %.2f(mcd, Y)", ties->tiecoremcd);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          else if(have_affine == DO && data[dset]->holes[ties->hole]->core[ties->core]->did_affine_offset == NO) {
            (void)sprintf(varstring, "    %.2f(mcd, N)", ties->tiecoremcd);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(eqlog == TEMP) {
            (void)sprintf(varstring, "   %.2f(eld)", ties->tiecoretempeqlogdep);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(ties->coretie == -1) {
            (void)strcat(string, "  IntTie");
          }
          else {
            (void)strcat(string, "  Tie");
          }
          xmstring = XmStringCreateSimple(string);
        }
        else {
          xmstring = XmStringCreateSimple("The tie is not within the range of this core");
        }
      }
      else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
        if(ties->coretie != -1) {
          val = splice[sset]->value[ties->coretie]->data;
          valsm = splice[sset]->value[ties->coretie]->sm_data;
          err = 0;
        }
        else {
          err = InterpolateSpliceTiePtValue(sset, NO, ties->tiecoremcd, &val, &dum);
          if(err == 0 && splice[sset]->smooth.method != NONE) {
            derr = InterpolateSpliceTiePtValue(sset, YES, ties->tiecoremcd, &valsm, &dum);
          }
        }
        if(err == 0) {
          (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       splice[sset]->leg,
                       splice[sset]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
          if(splice[sset]->smooth.method != NONE) {
            (void)sprintf(varstring, "   %.2f", valsm);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(eqlog == TEMP) {
            (void)sprintf(varstring, "   %.2f(mcd)  %.2f(eld)", ties->tiecoremcd,
                                                   ties->tiecoretempeqlogdep);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          else {
            (void)sprintf(varstring, "   %.2f(mcd)", ties->tiecoremcd);
            (void)strncat(string, varstring, (int)strlen(varstring));
          }
          if(ties->coretie == -1) {
            (void)strcat(string, "  IntTie");
          }
          else {
            (void)strcat(string, "  Tie");
          }
          xmstring = XmStringCreateSimple(string);
        }
        else {
          xmstring = XmStringCreateSimple("The tie is not within the range of this splice");
        }
      } 
      else {
        xmstring = XmStringCreateSimple("");
      }
    }
    else {
      xmstring = XmStringCreateSimple("");
    }
    XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
    if(have_tie2 == DO) {
      if(ties->logtie == MISSING) {
        xmstring = XmStringCreateSimple("Tie pt. missing in this log");
      }
      else {
        if(eqlogdepth.did_log_mudline_adj == YES) {
          (void)sprintf(string,"%.3f   %.3f(mud line adj.)   %.3f", ties->tielogdep - eqlogdepth.autocorr_log_mudline_offset,
                        ties->tielogdep, ties->tielogval);
        }
        else if(autocoreadj.draw_results == YES) {
          (void)sprintf(string,"%.3f   %.3f(test mud line adj.)   %.3f", ties->tielogdep, ties->tielogdep + autooff, ties->tielogval);
        }
        else {
          (void)sprintf(string,"%.3f   %.3f", ties->tielogdep, ties->tielogval);
        }
        if(logdata[lset]->info.smooth.method != NONE) {
          (void)sprintf(varstring, " %.2f", ties->tielogsmval);
          (void)strncat(string, varstring, (int)strlen(varstring));
        }
        if(ties->logtie == -1) {
          (void)strcat(string, "  IntTie");
        }
        else {
          (void)strcat(string, "  Tie");
        }
        xmstring = XmStringCreateSimple(string);
      }
    }
    else {
      xmstring = XmStringCreateSimple("");
    }
    XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }

  if(have_tie1 == DO || have_tie2 == DO) {
    XtSetSensitive(pushButtonAutoCorrelate, False);
  }
  else if(have_coretotie == DO && ties->nties > 0) {
    XtSetSensitive(pushButtonAutoCorrelate, False);
  }
  else if(have_splice == DO && ties->nties > 1) {
    XtSetSensitive(pushButtonAutoCorrelate, False);
  }
}

void
EventProcDrawingAreaAdjustToTiesInfo(w, event, args, num_args)
Widget w;
XButtonEvent *event;
String *args;
int *num_args;
{
  int x, y, i;
  char string[200];
  float depth;
  Dimension win_height, win_width;

  x = event->x;
  y = event->y;
  XtVaGetValues(w, XmNheight, &win_height, XmNwidth, &win_width, NULL);

  if(y < OFF || y > win_height - SPACE || x < OFF || x > win_width - SPACE){
    (void)sprintf(string, "");
    XtVaSetValues(textFieldAdjustToTiesInfo, XmNvalue, string, NULL);
  }
  else {
    (void)sprintf(string, "%d  %d", y - OFF, x - OFF);
    if((int)strcmp(args[0], "but_1_down") == 0 ||
          (int)strcmp(args[0], "but_1_move") == 0) {
      XtVaSetValues(textFieldAdjustToTiesInfo, XmNvalue, string, NULL);
    }
    else {
      (void)sprintf(string, "");
      XtVaSetValues(textFieldAdjustToTiesInfo, XmNvalue, string, NULL);
    }
  }
}

void
EventProcDrawingAreaCorrelate(w, event, args, num_args)
Widget w;
XButtonEvent *event;
String *args;
int *num_args;
{
  int x, y, i, k, xaxispixlen;
  char *string;
  float check, diff;
  Dimension win_height, win_width;

  if(have_tie1 == DO && have_tie2 == DO) {

    x = event->x;
    y = event->y;
    XtVaGetValues(w, XmNheight, &win_height, XmNwidth, &win_width, NULL);

    if((string=(char*)calloc(500,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sa");

    if(y < SPACE || y > win_height - SPACE || x < SPACE || x > win_width - SPACE){
      (void)sprintf(string, "");
      XtVaSetValues(textFieldCorrelate, XmNvalue, string, NULL);
    }
    else {
      if((int)strcmp(args[0], "but_1_down") == 0 || (int)strcmp(args[0], "but_1_move") == 0) {
        xaxispixlen = ((int)win_width - (2 * SPACE))/2;
        check = corr.leadlag_in_meter[0]/lagperpix + SPACE + xaxispixlen;
        diff= fabs(x-check);
        k = 0;
        for(i=1; i<corr.num_lead_lag; ++i){
          check = corr.leadlag_in_meter[i]/lagperpix + SPACE + xaxispixlen;
          if(fabs(x-check) < diff){
            diff = fabs(x-check);
            k=i;
          }
        }

        (void)sprintf(string, "log lead/lag = %.2f(m)  corr. coef. =  %.3f  n = %d  core offset = %.2f(m)", 
                               corr.leadlag_in_meter[k], corr.coef[k], corr.n[k], corr.depoffset[k]);
      }
      else {
        (void)sprintf(string, "best: corr.coef. %.2f,  at a depth offset of %.2f",
                      corr.coef[best_corr_num], corr.depoffset[best_corr_num]);
      }
      XtVaSetValues(textFieldCorrelate, XmNvalue, string, NULL);
    }
    free(string);
    string = NULL;
  }
}

void
EventProcDrawingAreaAutoCorrelate(w, event, args, num_args)
Widget w;
XButtonEvent *event;
String *args;
int *num_args;
{
  char *str;
  int i, x, y, foundit, close;
  XmString xmstring;
  void     DrawLog(), AutoCorrDrawSelectedBox();

  if(have_autocorr == DONT) {
    return;
  }
  else if(have_autocorr == DO && autocoreadj.nboxes > 0) {

    
    x = event->x;
    y = event->y;

    close = autocoreadj.box_size/2.0 * 0.9;
    if(close < CLOSE) {
      close = CLOSE;
    }

    if((str=(char*)calloc(500,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sa");

    if((int)strcmp(args[0], "but_1_down") == 0 || (int)strcmp(args[0], "but_1_move") == 0 ||
       (int)strcmp(args[0], "but_2_down") == 0 || (int)strcmp(args[0], "but_2_up") == 0 || (int)strcmp(args[0], "but_2_move") == 0) {
      foundit = -1;
      for(i=0; i<autocoreadj.nboxes; ++i) {
        if(abs(autocoreadj.box[i]->xpix_loc - x)  < close && abs(autocoreadj.box[i]->ypix_loc - y)  < close) {
          foundit = i;
          break;
        }
      }
      if(foundit != -1) {
        if(autocoreadj.box[foundit]->sum_r == BADPT) {
          (void)sprintf(str, "offset %.2f(m)  stretch-compress %.2f    NA",
                          autocoreadj.box[foundit]->offset, autocoreadj.box[foundit]->squish);
          xmstring = XmStringCreateSimple(str);
        }
        else if((int)strcmp(args[0], "but_2_up") == 0) {
          (void)sprintf(str,"%.3f", autocoreadj.box[foundit]->squish);
          XtVaSetValues(textFieldAutoCorrResultSquish, XmNvalue, str, NULL);
          (void)sprintf(str,"%.2f", autocoreadj.box[foundit]->offset);
          XtVaSetValues(textFieldAutoCorrResultOffset, XmNvalue, str, NULL);
          (void)sprintf(str, "r = %.3f  n = %d   (%d)", autocoreadj.box[foundit]->sum_r, 
                          autocoreadj.box[foundit]->ncore, autocoreadj.box[foundit]->rank+1);
          xmstring = XmStringCreateSimple(str);
          XtVaSetValues(labelAutoCorrBestCorr, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          xmstring = XmStringCreateSimple("");
          log_clip_what = NONE;
          DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                log_clip_what, 0.0);
          if(corr_rank.n > 0) {
            corr_rank.previous_one = corr_rank.use_thisone;
            corr_rank.use_thisone = autocoreadj.box[foundit]->rank;
            AutoCorrDrawSelectedBox(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate), gc_drawingAreaAutoCorrelate);
          }
        }
        else {
          (void)sprintf(str, "offset %.2f(m)  stretch-compress %.2f   r %.3f  n %d   (%d)",  
                        autocoreadj.box[foundit]->offset, autocoreadj.box[foundit]->squish,
                        autocoreadj.box[foundit]->sum_r, autocoreadj.box[foundit]->ncore, autocoreadj.box[foundit]->rank+1);
          xmstring = XmStringCreateSimple(str);
        }
      }
      else {
        xmstring = XmStringCreateSimple("");
      }
    }
    else {
      xmstring = XmStringCreateSimple("");
    }
    free(str);
    str = NULL;
    XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }
}
