/*
	sagan_io.c

     These functions read and write files

     int SaReadLogHeader()
     int     SaReadStrat()
     int SaReadTKreport()
     int SaReadOtherOdp()
     int SaReadMST95report()
     int SaReadJanus()
     HashInfo **SaReadCullTable()
     int     SaWriteCullTable()
     void WriteEquivLogDepthTable()
     int ReadEquivLogDepthTable()
     void ReadPreviewInput()
     int SaReadAffineTable()
*/
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include "sagan-creation-c.h"
#include "sagan_vars.h"

#define	MAXLINE		(1000)
#define MAXPREVIEW      (1000)
char	line[MAXLINE];
char	error_string[MAXLINE];
char    buffer[MAXBUF];

int SaReadLogHeader(logptr)
LogInfo	*logptr;
{
	int	i,j;
	char	tmp_char[20];
	if(logptr->format==ODPCDROM)
	{
	/* read header lines */
		(void)fscanf(logptr->fptr,"%*s %s",logptr->hole);
		(void)fscanf(logptr->fptr,"%*s %d",&logptr->leg);
		(void)fscanf(logptr->fptr,"%*s %s",tmp_char);
		logptr->depth_min=atof(tmp_char);
		(void)fscanf(logptr->fptr,"%*s %s",tmp_char);
		logptr->depth_max=atof(tmp_char);
	}
	else
	{
		(void)sprintf(logptr->hole,"???");
		logptr->leg=-99;
		logptr->depth_min=0.0;
		logptr->depth_max=0.0;
	}
/* read column ids */
	while(fgets(buffer,1000,logptr->fptr)!=NULL && (int)strlen(buffer)<=2);
	if((logptr->id=(char**)calloc(logptr->id_alloc=LOG_ID_ALLOC,sizeof(char*)))==NULL)
		SaBadMemAlloc("SaReadLogHeader 1");
	else
	{
		for(i=0;i<logptr->id_alloc;i++)
		{
			if((logptr->id[i]=(char*)calloc(25,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaReadLogHeader 2");
		}
		for(i=0;getword(buffer,i,logptr->id[i]);i++)
		{
			if(i>=logptr->id_alloc)
			{
				logptr->id_alloc+=LOG_ID_ALLOC;		
				if((charptr=(char*)realloc(logptr->id,logptr->id_alloc*sizeof(char*)))==NULL)
					SaBadMemAlloc("SaReadLogHeader 3");
				else
				{
					logptr->id=(char**)charptr;
					for(j=i;j<logptr->id_alloc;j++)
					{
						if((logptr->id[j]=(char*)calloc(25,CHAR_SIZE))==NULL)
						SaBadMemAlloc("SaReadLogHeader 4");
					}
				}
		
			}
		}
		logptr->id_num=i-1;
		fgets(buffer,1000,logptr->fptr);
	}
}

int SaReadAncillHeader(ancillptr)
AncillData *ancillptr;
{
        int     i,j;
        char    tmp_char[20];
        if(ancillptr->format==ODPCDROM)
        {
        /* read header lines */
                (void)fscanf(ancillptr->fptr,"%*s %s",ancillptr->sitehole);
                (void)fscanf(ancillptr->fptr,"%*s %d",&ancillptr->leg);
                (void)fscanf(ancillptr->fptr,"%*s %s",tmp_char);
                ancillptr->depth_min=atof(tmp_char);
                (void)fscanf(ancillptr->fptr,"%*s %s",tmp_char);
                ancillptr->depth_max=atof(tmp_char);
        }
        else
        {
                sprintf(ancillptr->sitehole ,"?");
                ancillptr->leg=-99;
                ancillptr->depth_min=0.0;
                ancillptr->depth_max=0.0;
        }
/* read column ids */
        while(fgets(buffer,1000,ancillptr->fptr)!=NULL && (int)strlen(buffer)<=2);
        if((ancillptr->id=(char**)calloc(ancillptr->id_alloc=LOG_ID_ALLOC,sizeof(char*)))==NULL)
                SaBadMemAlloc("SaReadLogHeader 1");
        else
        {
                for(i=0;i<ancillptr->id_alloc;i++)
                {
                        if((ancillptr->id[i]=(char*)calloc(25,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaReadLogHeader 2");
                }
                for(i=0;getword(buffer,i,ancillptr->id[i]);i++)
                {
                        if(i>=ancillptr->id_alloc)
                        {
                                ancillptr->id_alloc+=LOG_ID_ALLOC;
                                if((charptr=(char*)realloc(ancillptr->id,ancillptr->id_alloc*sizeof(char*)))==NULL)
                                        SaBadMemAlloc("SaReadLogHeader 3");
                                else
                                {
                                        ancillptr->id=(char**)charptr;
                                        for(j=i;j<ancillptr->id_alloc;j++)
                                        {
                                                if((ancillptr->id[j]=(char*)calloc(25,CHAR_SIZE))==NULL)
                                                SaBadMemAlloc("SaReadLogHeader 4");
                                        }
                                }

                        }
                }
                ancillptr->id_num=i-1;
                fgets(buffer,1000,ancillptr->fptr);
        }
}


int     SaReadStrat(fptr,stratptr)
FILE    *fptr;
StratData       *stratptr;
{
        int     status,i;

        status=fscanf(fptr,"%s %s %f %f %d %d %c %d %c %s %f %f %d %d %c %d %c %s %f %f",
                stratptr->name,stratptr->code,&stratptr->top_age,&stratptr->bot_age,&stratptr->top.leg,
                &stratptr->top.site,&stratptr->top.hole,&stratptr->top.core,
                &stratptr->top.coretype,stratptr->top.section,&stratptr->top.interval,
                &stratptr->top.mbsf,&stratptr->bot.leg,&stratptr->bot.site,&stratptr->bot.hole,&stratptr->bot.core,
                &stratptr->bot.coretype,stratptr->bot.section,&stratptr->bot.interval,&stratptr->bot.mbsf);
        stratptr->top.hole=toupper(stratptr->top.hole);
        stratptr->top.coretype=toupper(stratptr->top.coretype);
        for(i=0;i<(int)strlen(stratptr->top.section);i++)
        {
                if(isalpha(stratptr->top.section[i]))
                        stratptr->top.section[i]=toupper(stratptr->top.section[i]);
        }
        stratptr->bot.hole=toupper(stratptr->bot.hole);
        stratptr->bot.coretype=toupper(stratptr->bot.coretype);
        for(i=0;i<(int)strlen(stratptr->bot.section);i++)
        {
                if(isalpha(stratptr->bot.section[i]))
                        stratptr->bot.section[i]=toupper(stratptr->bot.section[i]);
        }
        if(status==20)
                return(True);
        else if(status!=EOF)
        {
                info.error=True;
		info.fatal=False;
                SaManageWarningBox("Error reading stat file.");
                return(False);
        }
        else
                return(False);
 
}
 
int SaReadTKreport(fptr,dataptr)
FILE	*fptr;
DataFile	*dataptr;
{
	int	core_num,this_hole,this_core,max_alloc,cnt,i,num,status;
	Hole	*holeptr;
	Core	*coreptr;
	Value	*valueptr;
	char	hole_name,*filename;

	/*return False=more data, true=done */
	dataptr->subleg=' ';
	if((status=fscanf(fptr,"%d %d %c %d",&dataptr->leg,&dataptr->site,&hole_name,&core_num))!=4)
	{
		if(status==EOF)
			return(True);
		else
		{
			dataptr->readerrors++;
			return(False);
		}
	}
	else
	{
	/* check if new hole */
		hole_name=toupper(hole_name);
		if(dataptr->holes_used==NULL || strchr(dataptr->holes_used,hole_name)==NULL)
		{
			dataptr->numholes++;
			if(dataptr->numholes >1)	/* not first hole */
			{
				if((charptr=(char*)realloc(dataptr->holes_used,(dataptr->numholes+1)*CHAR_SIZE))==NULL)
					SaBadMemAlloc("SaReadTKreport 2");
				else
					dataptr->holes_used = charptr;
				if((charptr=(char*)realloc((char*)dataptr->holes,dataptr->numholes*sizeof(Hole*)))==NULL)
					SaBadMemAlloc("SaReadTKreport 3");
				else
					dataptr->holes=(Hole**)charptr;
			}
			else		/* first hole */
			{
				if((dataptr->holes_used=(char*)calloc(2,sizeof(char)))==NULL)
					SaBadMemAlloc("SaReadTKreport 4");
				dataptr->holes_used[0]='\0';
				if((dataptr->holes=(Hole**)calloc(dataptr->numholes,sizeof(Hole*)))==NULL)
					SaBadMemAlloc("SaReadTKreport 5");
			}
			/* set hole values */
			if((dataptr->holes[dataptr->numholes-1]=(Hole*)calloc(1,sizeof(Hole)))==NULL)
				SaBadMemAlloc("SaReadTKreport 6");
			this_hole=dataptr->numholes-1;
			holeptr=dataptr->holes[this_hole];
			holeptr->name=hole_name;
			(void)sprintf(dataptr->holes_used,"%s%c",dataptr->holes_used,hole_name);
			holeptr->numcores=0;
		}
		else	/* hole already exists */
			for(this_hole=0;this_hole<dataptr->numholes && 
					dataptr->holes[this_hole]->name!=hole_name;this_hole++);
		holeptr=dataptr->holes[this_hole];
		/* check if new core*/
		for(this_core=0;this_core<holeptr->numcores && core_num != holeptr->core[this_core]->num;this_core++);
		if(this_core >= holeptr->numcores)
		{
			if(holeptr->numcores++>0)	/* not first core*/
			{
				if((charptr=(char*)realloc((char*)holeptr->core,holeptr->numcores*sizeof(Core*)))==NULL)
					SaBadMemAlloc("SaReadTKreport 7");
				else
					data[dset]->holes[this_hole]->core=(Core**)charptr;
			}
			else		/* firstcore */
			{
				if((holeptr->core=(Core**)calloc(holeptr->numcores,sizeof(Core*)))==NULL)
					SaBadMemAlloc("SaReadTKreport 8");
			}
			/* set core values */
			if((holeptr->core[this_core]=(Core*)calloc(1,sizeof(Core)))==NULL)
				SaBadMemAlloc("SaReadTKreport 9");
			coreptr=holeptr->core[this_core];
			coreptr->num=core_num;
			coreptr->numvalues=0;
                        coreptr->good = NULL;
			if((coreptr->value=(Value**)calloc(ALLOC_INT,sizeof(Value*)))==NULL)
				SaBadMemAlloc("SaReadTKreport 10");
			for(coreptr->allocvalues=0;coreptr->allocvalues<ALLOC_INT;coreptr->allocvalues++)
			{
				if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
					SaBadMemAlloc("SaReadTKreport 11");
			}
		}
		coreptr=holeptr->core[this_core];
		coreptr->have_equil_log_depths=DONT;
		if(coreptr->numvalues++ >= coreptr->allocvalues)
		{
			max_alloc=coreptr->allocvalues+ALLOC_INT;
			if((charptr=(char*)realloc((char*)coreptr->value,max_alloc*sizeof(Value*)))==NULL)
				SaBadMemAlloc("SaReadTKreport 12");
			else
				coreptr->value=(Value**) charptr;
			for(;coreptr->allocvalues<max_alloc;coreptr->allocvalues++)
			{
				if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
					SaBadMemAlloc("SaReadTKreport 13");
			}
		}
		valueptr=coreptr->value[coreptr->numvalues-1];
	/* get standard core values */
		while((valueptr->type=fgetc(fptr))!=EOF && isspace(valueptr->type));
		if(valueptr->type==EOF)
			return(True);
		else
			valueptr->type=toupper(valueptr->type);
		while((valueptr->section[0]=fgetc(fptr))!=EOF && isspace(valueptr->section[0]));
		if(valueptr->section[0]==EOF)
			return(True);
		else
			valueptr->section[0]=toupper(valueptr->section[0]);
		valueptr->section[1]=fgetc(fptr);
		if(isspace(valueptr->section[1]))
			valueptr->section[1]='\0';
		else
			valueptr->section[1]=toupper(valueptr->section[1]);
		valueptr->section[1]='\0';
		if((status=fscanf(fptr,"%f %f %f",&valueptr->top_int,&valueptr->bot_int,&valueptr->mbsf))!=3)
		{
			if(status==EOF)
				return(True);
			else
			{
				dataptr->readerrors++;
				return(False);
			}
		}
		if(fabs(valueptr->mbsf - BAD_CODE) <= ROUNDCHECK)
			valueptr->quality=BAD_SB_DEPTH;
		else
			valueptr->quality=GOOD;
		valueptr->eld=valueptr->mbsf;
		valueptr->run_num=0;
	/* get type specific values */
		switch (dataptr->type)
		{
			case GRAPE:
				if((valueptr->grape=(Grapedata*)calloc(1,sizeof(Grapedata)))==NULL)
					SaBadMemAlloc("SaReadTKreport 15");
				if((status=fscanf(fptr,"%f %f",&valueptr->grape->density,&valueptr->grape->b_corr_density))
						!=2)
				{
					if(status==EOF)
						return(True);
					else
					{
						dataptr->readerrors++;
						info.error=True;
					}
				}
				else
				{
					valueptr->grape->gamma_counts=0;
					valueptr->data=&valueptr->grape->b_corr_density;
				}
				break;
			case SUSCEPTIBILITY:
				if((valueptr->susceptibility=(Susceptibility*)calloc(1,sizeof(Susceptibility)))==NULL)
					SaBadMemAlloc("SaReadTKreport 16");
				if((status=fscanf(fptr,"%f",&valueptr->susceptibility->corrected)) !=1)
				{
					if(status==EOF)
						return(True);
					else
					{
						dataptr->readerrors++;
						info.error=True;
					}
				}
				else
				{
					valueptr->susceptibility->observed=0.0;
					valueptr->data=&valueptr->susceptibility->corrected;
				}
				break;
			case PWAVE:
				if((valueptr->pwave=(Pwavedata*)calloc(1,sizeof(Pwavedata)))==NULL)
					SaBadMemAlloc("SaReadTKreport 17");
				if((status=fscanf(fptr,"%f",&valueptr->pwave->velocity)) !=1)
				{
					if(status==EOF)
						return(True);
					else
					{
						dataptr->readerrors++;
						info.error=True;
					}
				}
				else
				{
					valueptr->pwave->sig_strength=0;
					valueptr->pwave->travel_time=0;
					valueptr->pwave->displacement=0;
					valueptr->data=&valueptr->pwave->velocity;
				}
				break;
			case NATURALGAMMA:
                                if((valueptr->naturalgamma=(Natgammadata*)calloc(1,sizeof(Natgammadata)))==NULL)
                                        SaBadMemAlloc("SaReadTKreport 18");
                                if((status=fscanf(fptr,"%f %d %d %d %d %d %d %d",&valueptr->naturalgamma->total_cnt,
						&valueptr->naturalgamma->sample_time,
						&valueptr->naturalgamma->seg1_cnt,&valueptr->naturalgamma->seg2_cnt,
						&valueptr->naturalgamma->seg3_cnt,&valueptr->naturalgamma->seg4_cnt,
						&valueptr->naturalgamma->seg5_cnt,&valueptr->run_num))!=8)
                                {
					if(status==EOF)
						return(True);
					else
					{
                                        	dataptr->readerrors++;
                                        	info.error=True;
					}
                                }
                                else
                                {
                                        valueptr->data=&valueptr->naturalgamma->total_cnt;
                                }
                                break;
			case REFLECTANCE:
				if((valueptr->reflectance=(Reflectance*)calloc(1,sizeof(Reflectance)))==NULL)
					SaBadMemAlloc("SaReadTKreport 19");
				for(i=0;i<31;i++)
					valueptr->reflectance->nm_value[i]=0.0;
				if((status=fscanf(fptr,"%f",&valueptr->reflectance->nm_value[15]))!=1)
				{
					if(status==EOF)
						return(True);
					else
					{
						dataptr->readerrors++;
						info.error=True;
					}
				}
				else
				{
					valueptr->reflectance->L=0.0;
					valueptr->reflectance->a=0.0;
					valueptr->reflectance->b=0.0;
					valueptr->reflectance->munsell_hue[0]='\0';
					valueptr->reflectance->munsell_value[0]='\0';
					valueptr->data=&valueptr->reflectance->nm_value[15];
				}
				break;
                        case OTHERTYPE:
                                break;
			default:
				dataptr->readerrors++;
				info.error=True;
		}
		if(info.error==True)
		{
			info.fatal=False;
			info.default_button=XmDIALOG_OK_BUTTON;
			info.type=XmDIALOG_ERROR;
			info.caller=READTKREPORT;
			if((info.message=(char*)calloc(50+(int)strlen(format[dataptr->in_format])+(int)strlen(type[dataptr->type]),CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaReadTKreport 15");
			(void)sprintf(info.message,"Bad %s %s line",format[dataptr->in_format],type[dataptr->type]);
			SaManageMessage(info);
		}
		else
			valueptr->sm_data=*valueptr->data;
	}
	return(False);
}
int SaReadOtherOdp(fptr,dataptr)
FILE	*fptr;
DataFile	*dataptr;
{
	char	hole_name;
	int	core_num,status,this_hole,this_core,max_alloc;
	Value	tmp_value,*valueptr;
	Core	*coreptr;
	Hole	*holeptr;

	/*return False=more data, true=done */
/* read data line */
	dataptr->subleg=' ';
	if(fgets(buffer,1000,fptr)==NULL)
		return(True);
/* get variables */
	switch (dataptr->in_format)
	{
                case    ODPOTHER1:
			if((status=sscanf(buffer,openfileinfo.other_format,&dataptr->leg,
					&dataptr->site,&hole_name,&core_num,&tmp_value.type,
					tmp_value.section,&tmp_value.top_int,&tmp_value.bot_int,
					&tmp_value.mbsf,&tmp_value.otherodp))!=10)
			{
				if(status==EOF)
					return(True);
				else
					info.error=True;
			}
			break;
		case	ODPOTHER2:
			dataptr->leg=0;
			if((status=sscanf(buffer,openfileinfo.other_format,
					&dataptr->site,&hole_name,&core_num,&tmp_value.type,
					tmp_value.section,&tmp_value.top_int,&tmp_value.bot_int,
					&tmp_value.mbsf,&tmp_value.otherodp))!=9)
			{
				if(status==EOF)
					return(True);
				else
					info.error=True;
			}
			break;
		case	ODPOTHER3:
			dataptr->leg=0;
                        tmp_value.type = '?';
			if((status=sscanf(buffer,openfileinfo.other_format, &dataptr->site,&hole_name,&core_num,
					tmp_value.section,&tmp_value.top_int,&tmp_value.bot_int,
					&tmp_value.mbsf,&tmp_value.otherodp))!=8)
			{
				if(status==EOF)
					return(True);
				else
					info.error=True;
			}
			break;
		case	ODPOTHER4:
			dataptr->leg=0;
                        tmp_value.type = '?';
			if((status=sscanf(buffer,openfileinfo.other_format, &dataptr->site,&hole_name,&core_num,
					tmp_value.section,&tmp_value.top_int,
					&tmp_value.mbsf,&tmp_value.otherodp))!=7)
			{
				if(status==EOF)
					return(True);
				else
					info.error=True;
			}
			tmp_value.bot_int=tmp_value.top_int;
			break;
                case    ODPOTHER5:
                        if((status=sscanf(buffer,openfileinfo.other_format,&dataptr->leg,
                                        &dataptr->site,&hole_name,&core_num,&tmp_value.type,
                                        tmp_value.section,&tmp_value.top_int,
                                        &tmp_value.mbsf,&tmp_value.otherodp))!=9)
                        {
                                if(status==EOF)
                                        return(True);
                                else
                                        info.error=True;
                        }
                        tmp_value.bot_int=tmp_value.top_int;
                        break;
		default:
			info.fatal=False;
			info.default_button=XmDIALOG_OK_BUTTON;
			info.type=XmDIALOG_ERROR;
			info.caller=READOTHERODP;
			if((info.message=(char*)calloc(100,CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 1");
			(void)sprintf(info.message,"Invalid other odp data type %d",dataptr->type);
			SaManageMessage(info);
			return(True);
	}
	if(info.error)
	{
		info.fatal=False;
		info.default_button=XmDIALOG_OK_BUTTON;
		info.type=XmDIALOG_ERROR;
		info.caller=READOTHERODP;
		if((info.message=(char*)calloc(50+(int)strlen(type[dataptr->type])+
					(int)strlen(format[dataptr->in_format]),CHAR_SIZE))==NULL)
			SaBadMemAlloc("SaReadOtherOdp 2");
		(void)sprintf(info.message,"Bad %s %s line",format[dataptr->in_format],type[dataptr->type]);
		SaManageMessage(info);
		return(False);
	}
	/* check if new hole */
	hole_name=toupper(hole_name);
	if(dataptr->holes_used==NULL || strchr(dataptr->holes_used,hole_name)==NULL)
	{
		dataptr->numholes++;
		if(dataptr->numholes >1)	/* not first hole */
		{
			if((charptr=(char*)realloc(dataptr->holes_used,(dataptr->numholes+1)*CHAR_SIZE))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 3");
			else
				dataptr->holes_used = charptr;
			if((charptr=(char*)realloc((char*)dataptr->holes,dataptr->numholes*sizeof(Hole*)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 4");
			else
				dataptr->holes=(Hole**)charptr;
		}
		else		/* first hole */
		{
			if((dataptr->holes_used=(char*)calloc(2,sizeof(char)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 5");
			dataptr->holes_used[0]='\0';
			if((dataptr->holes=(Hole**)calloc(dataptr->numholes,sizeof(Hole*)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 6");
		}
		/* set hole values */
		if((dataptr->holes[dataptr->numholes-1]=(Hole*)calloc(1,sizeof(Hole)))==NULL)
			SaBadMemAlloc("SaReadOtherOdp 7");
		this_hole=dataptr->numholes-1;
		holeptr=dataptr->holes[this_hole];
		holeptr->name=hole_name;
		(void)sprintf(dataptr->holes_used,"%s%c",dataptr->holes_used,hole_name);
		holeptr->numcores=0;
	}
	else	/* hole already exists */
	{
		for(this_hole=0;this_hole<dataptr->numholes && dataptr->holes[this_hole]->name!=hole_name;this_hole++);
	}
	holeptr=dataptr->holes[this_hole];
	/* check if new core*/
	for(this_core=0;this_core<holeptr->numcores && core_num != holeptr->core[this_core]->num;this_core++);
	if(this_core >= holeptr->numcores)
	{
		if(holeptr->numcores++>0)	/* not first core*/
		{
			if((charptr=(char*)realloc((char*)holeptr->core,holeptr->numcores*sizeof(Core*)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 8");
			else
				data[dset]->holes[this_hole]->core=(Core**)charptr;
		}
		else		/* firstcore */
		{
			if((holeptr->core=(Core**)calloc(holeptr->numcores,sizeof(Core*)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 9");
		}
		/* set core values */
		if((holeptr->core[this_core]=(Core*)calloc(1,sizeof(Core)))==NULL)
			SaBadMemAlloc("SaReadOtherOdp 10");
		coreptr=holeptr->core[this_core];
		coreptr->num=core_num;
		coreptr->numvalues=0;
                coreptr->good = NULL;
		if((coreptr->value=(Value**)calloc(ALLOC_INT,sizeof(Value*)))==NULL)
			SaBadMemAlloc("SaReadOtherOdp 11");
		for(coreptr->allocvalues=0;coreptr->allocvalues<ALLOC_INT;coreptr->allocvalues++)
		{
			if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 12");
		}
	}
	coreptr=holeptr->core[this_core];
	coreptr->have_equil_log_depths=DONT;
	if(coreptr->numvalues++ >= coreptr->allocvalues)
	{
		max_alloc=coreptr->allocvalues+ALLOC_INT;
		if((charptr=(char*)realloc((char*)coreptr->value,max_alloc*sizeof(Value*)))==NULL)
			SaBadMemAlloc("SaReadOtherOdp 13");
		else
			coreptr->value=(Value**) charptr;
		for(;coreptr->allocvalues<max_alloc;coreptr->allocvalues++)
		{
			if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
				SaBadMemAlloc("SaReadOtherOdp 14");
		}
	}
	valueptr=coreptr->value[coreptr->numvalues-1];
/* get standard core values */
	valueptr->type=tmp_value.type;
	(void)strcpy(valueptr->section,tmp_value.section);
	valueptr->top_int=tmp_value.top_int;
	valueptr->bot_int=tmp_value.bot_int;
	valueptr->mbsf=tmp_value.mbsf;
	valueptr->eld=valueptr->mbsf;
	valueptr->otherodp=tmp_value.otherodp;
	if(fabs(valueptr->mbsf - BAD_CODE) <= ROUNDCHECK)
		valueptr->quality=BAD_SB_DEPTH;
	else
		valueptr->quality=GOOD;
	valueptr->run_num=0;
	valueptr->data=&valueptr->otherodp;
	valueptr->sm_data=*valueptr->data;
	return(False);

}

int SaReadMST95report(fptr,dataptr)
FILE	*fptr;
DataFile	*dataptr;
{
/* return False=more data, true=done */
	char	c_value[20];
	int	core,this_hole,this_core,max_alloc,nm_num,pos;
	Hole	*holeptr;
	Core	*coreptr;
	Value	*valueptr;
	int	i,len;
	float	fvalue;

/* read one data line */
	if(fgets(line,MAXLINE,fptr)!=NULL)
	{
	/* get rid of ending blanks */
		len=(int)strlen(line);
		for(i=len-1;i>0 && isspace(line[i]);i--)
			line[i]='\0';
		len=(int)strlen(line);
	/* get	standard header information */
		(void)strncpy(c_value,&line[1],3); c_value[3]='\0';
		dataptr->leg=atol(c_value);
		dataptr->subleg = line[5];
		(void)strncpy(c_value,&line[7],4); c_value[4]='\0';
		dataptr->site= atol(c_value);
		/* check if new hole */
		if(dataptr->holes_used==NULL || strchr(dataptr->holes_used,line[12])==NULL)
		{
			if(line[12]==' ') /* no hole given, ignore line */
			{
				dataptr->readerrors++;
				return(False);
			}
			dataptr->numholes++;
			if(dataptr->numholes >1)	/* not first hole */
			{
				if((charptr=(char*)realloc(dataptr->holes_used,(dataptr->numholes+1)*CHAR_SIZE))==NULL)
					SaBadMemAlloc("SaReadMST95report 1");
				else
					dataptr->holes_used = charptr;
				if((charptr=(char*)realloc((char*)dataptr->holes,dataptr->numholes*sizeof(Hole*)))==NULL)
					SaBadMemAlloc("SaReadMST95report 2");
				else
					dataptr->holes=(Hole**)charptr;
			}
			else		/* first hole */
			{
				if((dataptr->holes_used=(char*)calloc(2,sizeof(char)))==NULL)
					SaBadMemAlloc("SaReadMST95report 3");
				dataptr->holes_used[0]='\0';
				if((dataptr->holes=(Hole**)calloc(dataptr->numholes,sizeof(Hole*)))==NULL)
					SaBadMemAlloc("SaReadMST95report 4");
			}
			/* set hole values */
			if((dataptr->holes[dataptr->numholes-1]=(Hole*)calloc(1,sizeof(Hole)))==NULL)
				SaBadMemAlloc("SaReadMST95report 5");
			this_hole=dataptr->numholes-1;
			holeptr=dataptr->holes[this_hole];
			holeptr->name=toupper(line[12]);
			(void)sprintf(dataptr->holes_used,"%s%c",dataptr->holes_used,line[12]);
			holeptr->numcores=0;
		}
		else	/* hole already exists */
			for(this_hole=0;this_hole<dataptr->numholes && 
					dataptr->holes[this_hole]->name!=line[12];this_hole++);
		holeptr=dataptr->holes[this_hole];
		/* get core value */
		(void)strncpy(c_value,&line[14],3); c_value[3]='\0';
		core=atol(c_value);
		/* check if new core*/
		for(this_core=0;this_core<holeptr->numcores && core != holeptr->core[this_core]->num;this_core++);
		if(this_core >= holeptr->numcores)
		{
			if(holeptr->numcores++>0)	/* not first core*/
			{
				if((charptr=(char*)realloc((char*)holeptr->core,holeptr->numcores*sizeof(Core*)))==NULL)
					SaBadMemAlloc("SaReadMST95report 6");
				else
					data[dset]->holes[this_hole]->core=(Core**)charptr;
			}
			else		/* firstcore */
			{
				if((holeptr->core=(Core**)malloc(holeptr->numcores*sizeof(Core*)))==NULL)
					SaBadMemAlloc("SaReadMST95report 7");
			}
			/* set core values */
			if((holeptr->core[this_core]=(Core*)calloc(1,sizeof(Core)))==NULL)
				SaBadMemAlloc("SaReadMST95report 8");
			coreptr=holeptr->core[this_core];
			coreptr->num=core;
			coreptr->numvalues=0;
                        coreptr->good = NULL;
			if((coreptr->value=(Value**)calloc(ALLOC_INT,sizeof(Value*)))==NULL)
				SaBadMemAlloc("SaReadMST95report 9");
			for(coreptr->allocvalues=0;coreptr->allocvalues<ALLOC_INT;coreptr->allocvalues++)
			{
				if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
					SaBadMemAlloc("SaReadMST95report 10");
			}
		}
		coreptr=holeptr->core[this_core];
		coreptr->have_equil_log_depths=DONT;
		if(coreptr->numvalues++ >= coreptr->allocvalues)
		{
			max_alloc=coreptr->allocvalues+ALLOC_INT;
			if((charptr=(char*)realloc((char*)coreptr->value,max_alloc*sizeof(Value*)))==NULL)
				SaBadMemAlloc("SaReadMST95report 11");
			else
				coreptr->value=(Value**) charptr;
			for(;coreptr->allocvalues<max_alloc;coreptr->allocvalues++)
			{
				if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
					SaBadMemAlloc("SaReadMST95report 12");
			}
		}
		valueptr=coreptr->value[coreptr->numvalues-1];
	/* get standard core values */
		valueptr->type=line[18];
		valueptr->type=toupper(valueptr->type);
		if(isspace(line[20]))
		{
			(void)strncpy(valueptr->section,&line[21],1);
			valueptr->section[1]='\0';
		}
		else
			(void)strncpy(valueptr->section,&line[20],2);
		valueptr->section[2]='\0';
		for(i=0;i<(int)strlen(valueptr->section);i++)
		{
			if(isalpha(valueptr->section[i]))
				valueptr->section[i]=toupper(valueptr->section[i]);
		}
		(void)strncpy(c_value,&line[23],5); c_value[5]='\0';
		valueptr->top_int=(float)atof(c_value);
		(void)strncpy(c_value,&line[29],5); c_value[5]='\0';
		valueptr->bot_int=(float)atof(c_value);
		(void)strncpy(c_value,&line[35],7); c_value[7]='\0';
		valueptr->mbsf=(float)atof(c_value);
		if(fabs(valueptr->mbsf - BAD_CODE) <= ROUNDCHECK)
			valueptr->quality=BAD_SB_DEPTH;
		else
			valueptr->quality=GOOD;
		valueptr->eld=valueptr->mbsf;
	/* get data type specific values */
		if(dataptr->type==GRAPE)
		{
			if(len!=72)
			{
				info.error=True;
				info.fatal=False;
				info.caller=READMST95REPORT;
				info.type=XmDIALOG_WARNING;
				info.default_button=XmDIALOG_OK_BUTTON;
				if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL)
					SaBadMemAlloc("SaReadMST95report 13");
				(void)sprintf(info.message,"Invalid Grape input line.");
				SaManageMessage(info);
				return(True);
			}
			if((valueptr->grape=(Grapedata*)calloc(1,sizeof(Grapedata)))==NULL)
				SaBadMemAlloc("SaReadMST95report 14");
			(void)strncpy(c_value,&line[46],5);c_value[5]='\0';
			valueptr->grape->density=(float)atof(c_value);
			valueptr->data=&valueptr->grape->density;
			valueptr->sm_data=valueptr->grape->density;
			(void)strncpy(c_value,&line[54],5);c_value[5]='\0';
			valueptr->grape->b_corr_density=(float)atof(c_value);
			(void)strncpy(c_value,&line[62],5);c_value[5]='\0';
			valueptr->grape->gamma_counts=atol(c_value);
			(void)strncpy(c_value,&line[68],4);c_value[4]='\0';
			valueptr->run_num=atol(c_value);
		}	
		else if (dataptr->type==PWAVE)
		{
			if(len!=75)
			{
                                info.error=True;
                                info.fatal=False; 
                                info.caller=READMST95REPORT;
                                info.type=XmDIALOG_WARNING; 
                                info.default_button=XmDIALOG_OK_BUTTON; 
                                if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaReadMST95report 15");
				(void)sprintf(info.message,"Invalid Pwave input line.");
				SaManageMessage(info);
				return(True);
			}
			if((valueptr->pwave=(Pwavedata*)calloc(1,sizeof(Pwavedata)))==NULL)
				SaBadMemAlloc("SaReadMST95report 16");
			(void)strncpy(c_value,&line[44],7); c_value[7]='\0';
			valueptr->pwave->velocity=(float)atof(c_value);
			valueptr->data=&valueptr->pwave->velocity;
			valueptr->sm_data=valueptr->pwave->velocity;
			(void)strncpy(c_value,&line[51],4); c_value[4]='\0';
			valueptr->pwave->sig_strength=atol(c_value);
			(void)strncpy(c_value,&line[56],4); c_value[4]='\0';
			valueptr->pwave->travel_time=atol(c_value);
			(void)strncpy(c_value,&line[71],5); c_value[5]='\0';
			valueptr->pwave->displacement=atol(c_value);
			(void)strncpy(c_value,&line[78],4); c_value[4]='\0';
			valueptr->run_num=atol(c_value);
		}
		else if (dataptr->type==NATURALGAMMA)
		{
			if(len!=92)
			{
                                info.error=True; 
                                info.fatal=False;  
                                info.caller=READMST95REPORT; 
                                info.type=XmDIALOG_WARNING;  
                                info.default_button=XmDIALOG_OK_BUTTON;  
                                if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaReadMST95report 17");
				(void)sprintf(info.message,"Invalid Natural Gamma input line.");
                                SaManageMessage(info);
				return(True);
			}
			if((valueptr->naturalgamma=(Natgammadata*)calloc(1,sizeof(Natgammadata)))==NULL)
                                SaBadMemAlloc("SaReadMST95report 18");
			(void)strncpy(c_value,&line[44],8); c_value[8]='\0';
			valueptr->naturalgamma->total_cnt=(float)atol(c_value);
			valueptr->data=&valueptr->naturalgamma->total_cnt;
			valueptr->sm_data=valueptr->naturalgamma->total_cnt;
			(void)strncpy(c_value,&line[52],5); c_value[5]='\0';
			valueptr->naturalgamma->sample_time=atol(c_value);
			(void)strncpy(c_value,&line[58],5); c_value[5]='\0';
			valueptr->naturalgamma->seg1_cnt=atol(c_value);
			(void)strncpy(c_value,&line[64],5); c_value[5]='\0';
			valueptr->naturalgamma->seg2_cnt=atol(c_value);
			(void)strncpy(c_value,&line[70],5); c_value[5]='\0';
			valueptr->naturalgamma->seg3_cnt=atol(c_value);
			(void)strncpy(c_value,&line[76],5); c_value[5]='\0';
			valueptr->naturalgamma->seg4_cnt=atol(c_value);
			(void)strncpy(c_value,&line[82],5); c_value[5]='\0';
			valueptr->naturalgamma->seg5_cnt=atol(c_value);
			(void)strncpy(c_value,&line[88],4); c_value[4]='\0';
			valueptr->run_num=atol(c_value);
		}
		else if (dataptr->type==SUSCEPTIBILITY)
		{
			if(len!=63)
			{
                                info.error=True;  
                                info.fatal=False;   
                                info.caller=READMST95REPORT;  
                                info.type=XmDIALOG_WARNING;   
                                info.default_button=XmDIALOG_OK_BUTTON;   
                                if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL) 
                                        SaBadMemAlloc("SaReadMST95report 19");
				(void)sprintf(info.message,"Invalid Magnetic Susceptibility input line.");
                                SaManageMessage(info);
				return(True);
			}
			if((valueptr->susceptibility=(Susceptibility*)calloc(1,sizeof(Susceptibility)))==NULL)
                                SaBadMemAlloc("SaReadMST95report 20");
			(void)strncpy(c_value,&line[44],6); c_value[7]='\0';
			valueptr->susceptibility->observed=(float)atof(c_value);
			(void)strncpy(c_value,&line[50],8); c_value[7]='\0';
			valueptr->susceptibility->corrected=(float)atof(c_value);
			valueptr->data=&valueptr->susceptibility->corrected;
			valueptr->sm_data=valueptr->susceptibility->corrected;
			(void)strncpy(c_value,&line[59],4); c_value[4]='\0';
			valueptr->run_num=atol(c_value);
		}
		else if(dataptr->type==REFLECTANCE)
		{
			if(len!=267)
			{
                                info.error=True;   
                                info.fatal=False;    
                                info.caller=READMST95REPORT;   
                                info.type=XmDIALOG_WARNING;    
                                info.default_button=XmDIALOG_OK_BUTTON;    
                                if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL)  
                                        SaBadMemAlloc("SaReadMST95report 21");
				(void)sprintf(info.message,"Invalid Reflectance input line.");
                                SaManageMessage(info); 
				return(True);
			}
			if((valueptr->reflectance=(Reflectance*)calloc(1,sizeof(Reflectance)))==NULL)
				SaBadMemAlloc("SaReadMST95report 22");
			(void)strncpy(c_value,&line[45],6); c_value[7]='\0';
			valueptr->reflectance->L=(float)atof(c_value);
			(void)strncpy(c_value,&line[52],6); c_value[7]='\0';
			valueptr->reflectance->a=(float)atof(c_value);
			(void)strncpy(c_value,&line[59],6); c_value[7]='\0';
			valueptr->reflectance->b=(float)atof(c_value);
			(void)strncpy(valueptr->reflectance->munsell_hue,&line[66],6);
			(void)strncpy(valueptr->reflectance->munsell_value,&line[73],9);
			pos=81;
			for(nm_num=0;nm_num<31;nm_num++)
			{
				(void)strncpy(c_value,&line[pos],6); c_value[7]='\0';
				valueptr->reflectance->nm_value[nm_num]=(float)atof(c_value);
				pos+=6;
			}
			valueptr->data=&valueptr->reflectance->nm_value[15];
			valueptr->sm_data=valueptr->reflectance->nm_value[15];
			valueptr->run_num=0;
		}
                else if(dataptr->type==OTHERTYPE)
                {
                }
		else
		{       info.error=True;    
                        info.fatal=False;     
                        info.caller=READMST95REPORT;    
                        info.type=XmDIALOG_WARNING;     
                        info.default_button=XmDIALOG_OK_BUTTON;     
                        if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL)   
                        	SaBadMemAlloc("SaReadMST95report 21");
			(void)sprintf(info.message,"Invalid data type %d.\n",dataptr->type);
                        (void)sprintf(info.message,"Invalid Reflectance input line.");
                        SaManageMessage(info); 
			return(True);
		}
		/* return not end of file */
		return(False);
	}
	else	/* end of file found */
		return(True);
}

HashInfo **SaReadCullTable(filename,num)
char	*filename;
int	*num;
{
	int	fdcull,i,alloc_num,j,
		hashsize,status;
	HashInfo	**cullinfo;

/* open input cull table */
	if((fdcull=open(filename,O_RDONLY|O_CREAT,0))== -1)
		return((HashInfo**)NULL);
	else
	{
	/* allocate initial array */
		alloc_num=ALLOC_INT;
		if((cullinfo=(HashInfo**)calloc(alloc_num,sizeof(HashInfo*)))==NULL)
			SaBadMemAlloc("SaCullTable 2");
		hashsize=sizeof(HashInfo);
		for(i=0;i<alloc_num;i++)
		{
			if((cullinfo[i]=(HashInfo*)calloc(1,hashsize))==NULL)
				SaBadMemAlloc("SaCullTable 3");
		}
	/* read cull table */
		status=hashsize;
		for(j=0;status==hashsize;j++)
		{
		/* allocate more memory if necessary */
			if(j>=alloc_num)
			{
				alloc_num+=ALLOC_INT;
				if((charptr=(char*)realloc((char*)cullinfo,alloc_num*sizeof(HashInfo*)))==NULL)
					SaBadMemAlloc("SaCullTable 4");
				else
					cullinfo=(HashInfo**)charptr;
				for(i=j;i<alloc_num;i++)
				{
					if((cullinfo[i]=(HashInfo*)calloc(1,hashsize))==NULL)
						SaBadMemAlloc("SaCullTable 5");
				}
			}
			status=read(fdcull,(char *) cullinfo[j],hashsize);
		}
		*num=j-1;
		close(fdcull);
	}
	if(status!=0)
		return((HashInfo**)NULL);
	else
		return(cullinfo);
}

int	SaWriteCullTable(dataptr,filename)
DataFile	*dataptr;
char		*filename;
{
	int	fdcull,flag;
	int	i,j,k;
	HashInfo	hashinfo;
	Core	*coreptr;


/* open output cull table */
	if((fdcull=open(filename,O_WRONLY|O_CREAT,0744))== -1)
	{
		info.error=True;
		(void)sprintf(error_string,"Error opening cull table %s.",filename);
		SaManageWarningBox(error_string);
	}
	else
	{
		info.error=False;
	/* write out file */
		hashinfo.leg=dataptr->leg;
		hashinfo.subleg=dataptr->subleg;
		hashinfo.site=dataptr->site;
		flag=BAD_CULL1 | BAD_CULL2 | BAD_TOP;
		for(i=0;i<dataptr->numholes;i++)
		{
			hashinfo.hole=dataptr->holes[i]->name;
			{
				for(j=0;j<dataptr->holes[i]->numcores;j++)
				{
					coreptr=dataptr->holes[i]->core[j];
					hashinfo.core=coreptr->num;
					for(k=0;k<coreptr->numvalues;k++)
					{
						hashinfo.coretype=coreptr->value[k]->type;
						(void)strcpy(hashinfo.section,coreptr->value[k]->section);
						hashinfo.top_int=coreptr->value[k]->top_int;
					/* write flagged values to file */
						if((hashinfo.value=coreptr->value[k]->quality) & flag)
							(void)write(fdcull,(char*)&info,sizeof(HashInfo));
					}
				}
			}
		}
		close(fdcull);
	}
	return(!info.error);
}


void WriteEquivLogDepthTable()
{
  int i, j, k, corearr, logarr;
  char did;
  FILE *fptr;

  if((fptr = fopen(eqlogdepth.filename, "w")) == NULL)
    return;

  /* write header info
  */
  (void)fprintf(fptr,"Equivalent Log Depth Table:  Leg %d  Site %d\n", eqlogdepth.leg, eqlogdepth.site);

  /* write the auto corr info
  */
  if(eqlogdepth.autocorr_status == YES) {
    (void)fprintf(fptr,"Overall  Y  MudlineOffset %.2f  Stretch/Compress %.2f\n", 
                       	eqlogdepth.autocorr_log_mudline_offset, eqlogdepth.autocorr_squish);
  }
  else {
    (void)fprintf(fptr, "Overall N\n");
  }

  /* write the affine info
  */
  if(eqlogdepth.affine_status == NOAFFINE) {
     (void)fprintf(fptr,"Affine  N\n");
  }
  else if(eqlogdepth.affine_status == YESAFFINE) {
     (void)fprintf(fptr,"Affine Y     %s\n", eqlogdepth.affine_filename);
  }
  else {
     (void)fprintf(fptr,"Affine Used: ERROR\n");
  }

  /* per core write each tie
  */
  for(i=0; i<eqlogdepth.numhole; ++i) {
    for(j=0; j<eqlogdepth.hole[i]->numcore; ++j) {

      if(eqlogdepth.affine_status == NOAFFINE) {
        (void)fprintf(fptr,"Hole\t%c\tCore\t%d\t#Ties\t%d\n",
                    eqlogdepth.hole[i]->name,
                    eqlogdepth.hole[i]->core[j]->name,
                    eqlogdepth.hole[i]->core[j]->nties);
      }
      else if(eqlogdepth.affine_status == YESAFFINE) {
        if(eqlogdepth.hole[i]->core[j]->affine_status == YESAFFINE_DID) {
          did = 'Y';
        }
        else if(eqlogdepth.hole[i]->core[j]->affine_status == YESAFFINE_DIDNT) {
          did = 'N';
        }
        (void)fprintf(fptr,"Hole\t%c\tCore\t%d\tAffineOffset\t%.2f\t%c\t#Ties\t%d\n",
                    eqlogdepth.hole[i]->name,
                    eqlogdepth.hole[i]->core[j]->name,
                    eqlogdepth.hole[i]->core[j]->affine_offset, did,
                    eqlogdepth.hole[i]->core[j]->nties);
      }
      if(eqlogdepth.hole[i]->core[j]->nties > 0) {
        for(k=0; k<eqlogdepth.hole[i]->core[j]->nties; ++k) {
          (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%6.1f\t%6.1f\t%10.2f\t%10.4f\t%10.4f\t%10.4f", 
              eqlogdepth.site,
              eqlogdepth.hole[i]->name, 
              eqlogdepth.hole[i]->core[j]->name, 
              eqlogdepth.hole[i]->core[j]->type,
              eqlogdepth.hole[i]->core[j]->tie[k]->coresection, 
              eqlogdepth.hole[i]->core[j]->tie[k]->topsect, 
              eqlogdepth.hole[i]->core[j]->tie[k]->botsect,
              eqlogdepth.hole[i]->core[j]->tie[k]->corembsf, 
              eqlogdepth.hole[i]->core[j]->tie[k]->logdep,
              eqlogdepth.hole[i]->core[j]->tie[k]->rate,
              eqlogdepth.hole[i]->core[j]->tie[k]->b);
          if(eqlogdepth.hole[i]->core[j]->tie[k]->shared == YES) {
            (void)fprintf(fptr,"\tS\n");
          }
          else {
            (void)fprintf(fptr,"\tN\n");
          }
        }
      }
    }
  }
  fclose(fptr);
}

int ReadEquivLogDepthTable()
{
  char hole, checkhole, checkchar[20], yesno, did;
  int i, j, k, h, c, core, nties, num, nline, all_inc;
  float dep[MAXPERCORE], off;
  FILE *fptr;

  /* open the file
  */
  if((fptr=fopen(eqlogdepth.filename,"r"))==NULL) {
               
    info.error=TRUE;
    if((info.message=(char*)calloc(50+(int)(int)strlen(eqlogdepth.filename),CHAR_SIZE))==NULL) {
      SaBadMemAlloc("SaFileSelectionBoxOKCB 21");
    }
    else {
      (void)sprintf(info.message,"Error opening ELD table %s!",eqlogdepth.filename);
      SaManageMessage(info);
    }
    return 1;
  }

  nline = 0;
  eqlogdep_core_alloc = max_num_cores;
  all_inc = 25;
  total_num_ties = 0;
      
  /* get the leg and site from the header
  */
  fgets(line, MAXLINE, fptr);
  num = sscanf(line, "%*s %*s %*s %*s %*s %d %*s %d", &eqlogdepth.leg, &eqlogdepth.site);
  if(num != 2) {
    (void)sprintf(buffer, "Error; ELD Table; Header record missing or incorrectly formatted.");
    SaManageWarningBox(buffer);
    return 1;
  }
  (void)strcpy(line, "");
  ++nline;

  /* get the auto corr info
  */
  fgets(line, MAXLINE, fptr);
  if(strncmp(line, "Overall", 7) != 0) {
    (void)sprintf(buffer, "Error; ELD Table; Overall adjust info record missing.");
    SaManageWarningBox(buffer);
    return 1;
  }
  num = sscanf(line, "%*s %c", &yesno);
  if(num != 1) {
    (void)sprintf(buffer, "Error; ELD Table; Overall adjust record incorrectly formatted.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(yesno == 'Y') {
    num = sscanf(line, "%*s %*s %*s %f %*s %f", 
                   &eqlogdepth.autocorr_log_mudline_offset, &eqlogdepth.autocorr_squish);
    if(num != 2) {
      (void)sprintf(buffer, "Error; ELD Table; Overall adjust record incorrectly formatted.");
      SaManageWarningBox(buffer);
      return 1;
    }
    else {
      eqlogdepth.autocorr_status = YES;
      eqlogdepth.did_log_mudline_adj = YES;
    }
  }
  else {
    eqlogdepth.autocorr_status = NO;
    eqlogdepth.did_log_mudline_adj = NO;
    eqlogdepth.autocorr_squish = 100.0;
  }

  /* get the affine info
  */
  fgets(line, MAXLINE, fptr);
  if(strncmp(line, "Affine", 6) != 0) {
    (void)sprintf(buffer, "Error; ELD Table format problem. Missing Affine Table info.");
    SaManageWarningBox(buffer);
    return 1;
  }
  num = sscanf(line, "%*s %c", &yesno);
  if(num != 1) {
    (void)sprintf(buffer, "Error; ELD Table format problem. Missing Affine Table filename.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(yesno == 'Y') {
    if((eqlogdepth.affine_filename=(char*)calloc((int)strlen(line), CHAR_SIZE)) == NULL) {
                                             SaBadMemAlloc("Sa");
    }
    num = sscanf(line, "%*s %*s %s", eqlogdepth.affine_filename);
    if(have_affine == DONT) {
      (void)sprintf(buffer, "Error; ELD Table uses the following Affine Table.\n");
      (void)strcat(buffer, eqlogdepth.affine_filename);
      (void)strcat(buffer, "\nYou have not set option to read this Affine Table.");
      SaManageWarningBox(buffer);
      free(eqlogdepth.affine_filename);
      eqlogdepth.affine_filename = NULL;
      return 1;
    }
    if((int)strlen(eqlogdepth.affine_filename) < 1) {
      (void)sprintf(buffer, "Warning; ELD Table. There is no Affine Table filename.");
      SaManageWarningBox(buffer);
    } 
    if(have_affine == DO) {
      if(strcmp(eqlogdepth.affine_filename, affinetable.filename) != 0) {
        (void)sprintf(buffer, "Warning; ELD Table. Affine Table filename you chose to read is different.\n");
        (void)strcat(buffer, "from the one recorded in the ELD Table.");
        SaManageWarningBox(buffer);
      }
    }
    eqlogdepth.affine_status = YESAFFINE;
  }
  else {
    if(have_affine == DO) {
      (void)sprintf(buffer, "Error; ELD Table; you have read an Affine Table but this.\n");
      (void)strcat(buffer, "ELD Table does not record that one was used.");
      SaManageWarningBox(buffer);
      return 1;
    }
    eqlogdepth.affine_status = NOAFFINE;
  }
  (void)strcpy(line, "");

  ++nline;

  eqlogdepth.numhole = 0;
  c = 0;

  while(fgets(line, MAXLINE, fptr) != NULL) {

    ++nline;
    (void)sscanf(line,"%s", checkchar);
    if(strcmp(checkchar, "Hole") == 0) {
      if(eqlogdepth.affine_status == NOAFFINE) {
        num = sscanf(line, "%*s %c %*s %d %*s %d", &hole, &core, &nties);
        if(num != 3) {
          (void)sprintf(buffer, "Error; ELD Table format. Core header. line %d", nline);
          SaManageWarningBox(buffer);
          return 1;
        }
      }
      else if(eqlogdepth.affine_status == YESAFFINE) {
        num = sscanf(line, "%*s %c %*s %d %*s %f %c %*s %d", &hole, &core, &off, &did, &nties);
        if(num != 5) {
          (void)sprintf(buffer, "Error; ELD Table format. Core header. line %d", nline);
          SaManageWarningBox(buffer);
          if(eqlogdepth.affine_filename) {
            free(eqlogdepth.affine_filename);
            eqlogdepth.affine_filename = NULL;
          }
          return 1;
        }
        else {
        }
      }
      if(eqlogdepth.numhole == 0) {
        eqlogdepth.numhole = 1;
        eld.nholes = eqlogdepth.numhole;
        h = 0;
        checkhole = hole;
        if((eqlogdepth.hole=(EqLogDepthHole**)calloc(1,sizeof(EqLogDepthHole*)))==NULL)
                                SaBadMemAlloc("Sa");
        if((eld.hole_eqt = (HoleELDEqt**)calloc(1, sizeof(HoleELDEqt*)))==NULL)
                                           SaBadMemAlloc("Sa");
        if((eqlogdepth.hole[0]=(EqLogDepthHole*)calloc(1,sizeof(EqLogDepthHole)))==NULL)
                                SaBadMemAlloc("Sa");
        if((eld.hole_eqt[0] = (HoleELDEqt*)calloc(1, sizeof(HoleELDEqt)))==NULL)
                                    SaBadMemAlloc("Sa");
        eqlogdepth.hole[0]->name = hole;
        eqlogdepth.hole[0]->numcore = 1;
        if((eqlogdepth.hole[0]->core=(EqLogDepthCore**)calloc(eqlogdep_core_alloc,sizeof(EqLogDepthCore*)))==NULL)
                                SaBadMemAlloc("Sa");
        for(i=0; i<eqlogdep_core_alloc; ++i) {
          if((eqlogdepth.hole[0]->core[i]=(EqLogDepthCore*)calloc(1,sizeof(EqLogDepthCore)))==NULL)
                                SaBadMemAlloc("Sa");
          eqlogdepth.hole[0]->core[i]->nties = 0;
        }
        eqlogdepth.hole[0]->core[0]->name = core;
        eqlogdepth.hole[0]->core[0]->nties = nties;
        eld.hole_eqt[0]->nties = 0;
      }
      else if(checkhole == hole) {
        ++eqlogdepth.hole[h]->numcore;
        c = eqlogdepth.hole[h]->numcore - 1;

        if(eqlogdepth.hole[h]->numcore > eqlogdep_core_alloc) {
          for(i=0; i<eqlogdepth.numhole; ++i) {
            if((charptr=(char*)realloc((char*)eqlogdepth.hole[i]->core,(eqlogdep_core_alloc+all_inc)*sizeof(EqLogDepthCore*)))==NULL) {
                                SaBadMemAlloc("");
            }
            else {
              eqlogdepth.hole[i]->core=(EqLogDepthCore**)charptr;
            }
            for(j=eqlogdep_core_alloc; j<eqlogdep_core_alloc+all_inc; ++j) {
              if((eqlogdepth.hole[i]->core[j]=(EqLogDepthCore*)calloc(1,sizeof(EqLogDepthCore)))==NULL)
                                SaBadMemAlloc("Sa");
              eqlogdepth.hole[i]->core[j]->nties = 0;
            }
          }
          eqlogdep_core_alloc = eqlogdep_core_alloc + all_inc;
        } 

        eqlogdepth.hole[h]->core[c]->name = core;
        eqlogdepth.hole[h]->core[c]->nties = nties;
      }
      else if(checkhole != hole) {
        ++eqlogdepth.numhole;
        eld.nholes = eqlogdepth.numhole;
        h = eqlogdepth.numhole -1;
        checkhole = hole;
        if((charptr=(char*)realloc((char*)eqlogdepth.hole,eqlogdepth.numhole*sizeof(EqLogDepthHole*)))==NULL) {
                                SaBadMemAlloc("");
        }
        else {
          eqlogdepth.hole=(EqLogDepthHole**)charptr;
        }
        if((eqlogdepth.hole[h]=(EqLogDepthHole*)calloc(1,sizeof(EqLogDepthHole)))==NULL)
                                SaBadMemAlloc("Sa");
        if((charptr=(char*)realloc((char*)eld.hole_eqt,eqlogdepth.numhole*sizeof(HoleELDEqt*)))==NULL) {
                                SaBadMemAlloc("Sa");
        }
        else {
          eld.hole_eqt=(HoleELDEqt**)charptr;
        }
        if((eld.hole_eqt[h]=(HoleELDEqt*)calloc(1,sizeof(HoleELDEqt)))==NULL)
                                SaBadMemAlloc("Sa");
        eqlogdepth.hole[h]->name = hole;
        eqlogdepth.hole[h]->numcore = 1;
        c = 0;
        if((eqlogdepth.hole[h]->core=(EqLogDepthCore**)calloc(eqlogdep_core_alloc,sizeof(EqLogDepthCore*)))==NULL)
                                SaBadMemAlloc("Sa");
        for(i=0; i<eqlogdep_core_alloc; ++i) {
          if((eqlogdepth.hole[h]->core[i]=(EqLogDepthCore*)calloc(1,sizeof(EqLogDepthCore)))==NULL)
                                SaBadMemAlloc("Sa");
          eqlogdepth.hole[h]->core[i]->nties = 0;
        }
        eqlogdepth.hole[h]->core[c]->name = core;
        eqlogdepth.hole[h]->core[c]->nties = nties;
        eld.hole_eqt[h]->nties = 0;
      }
      else {
        printf("did I miss something?\n");
      }
      if(eqlogdepth.affine_status == YESAFFINE) {
        eqlogdepth.hole[h]->core[c]->affine_offset = off;
        if(did == 'Y') {
          eqlogdepth.hole[h]->core[c]->affine_status = YESAFFINE_DID;
        }
        else {
          eqlogdepth.hole[h]->core[c]->affine_status = YESAFFINE_DIDNT;
        }
      }
      if(nties > 0) {
        if((eqlogdepth.hole[h]->core[c]->tie=(TieList**)calloc(nties,sizeof(TieList*)))==NULL)
                                SaBadMemAlloc("Sa");
        for(i=0; i<nties; ++i) {
          if((eqlogdepth.hole[h]->core[c]->tie[i]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                                SaBadMemAlloc("Sa");
        }
        eqlogdepth.hole[h]->core[c]->did_eld = YES;
        total_num_ties = total_num_ties + nties;
        for(i=0; i<nties; ++i) {
          if(fgets(line, MAXLINE, fptr) == NULL) {
            return 0;
          }
          else {
            num = sscanf(line,"%*d %*c %*d %c %s %f %f %f %f %f %f %s",
                    &eqlogdepth.hole[h]->core[c]->type,
                    eqlogdepth.hole[h]->core[c]->tie[i]->coresection,
                    &eqlogdepth.hole[h]->core[c]->tie[i]->topsect,
                    &eqlogdepth.hole[h]->core[c]->tie[i]->botsect,
                    &eqlogdepth.hole[h]->core[c]->tie[i]->corembsf,
                    &eqlogdepth.hole[h]->core[c]->tie[i]->logdep,
                    &eqlogdepth.hole[h]->core[c]->tie[i]->rate,
                    &eqlogdepth.hole[h]->core[c]->tie[i]->b,
                    &yesno);
            if(num != 9) {
              (void)sprintf(buffer, "Error; ELD Table; Tie record incorrectly formatted.");
              SaManageWarningBox(buffer);
              SaCleanUp(ELDTABLE);
              return 1;
            }
            if(yesno == 'S') {
              eqlogdepth.hole[h]->core[c]->tie[i]->shared = YES;
            }
            else {
              eqlogdepth.hole[h]->core[c]->tie[i]->shared = NO;
            }
            eqlogdepth.hole[h]->core[c]->tie[i]->holename = eqlogdepth.hole[h]->name;
            eqlogdepth.hole[h]->core[c]->tie[i]->corename = eqlogdepth.hole[h]->core[c]->name;
            if(eqlogdepth.affine_status == NOAFFINE) {
              eqlogdepth.hole[h]->core[c]->tie[i]->coremcd = eqlogdepth.hole[h]->core[c]->tie[i]->corembsf;
            }
            else if(eqlogdepth.affine_status == YESAFFINE) {
              eqlogdepth.hole[h]->core[c]->tie[i]->coremcd = eqlogdepth.hole[h]->core[c]->tie[i]->corembsf +
                                                             eqlogdepth.hole[h]->core[c]->affine_offset;
            }
            eqlogdepth.hole[h]->core[c]->tie[i]->equivlogdep = eqlogdepth.hole[h]->core[c]->tie[i]->logdep;
            if(eqlogdepth.autocorr_status == YES) {
              eqlogdepth.hole[h]->core[c]->tie[i]->coreautoeld = eqlogdepth.hole[h]->core[c]->tie[i]->coremcd * (eqlogdepth.autocorr_squish/100.0);
            }
            else {
              eqlogdepth.hole[h]->core[c]->tie[i]->coreautoeld = eqlogdepth.hole[h]->core[c]->tie[i]->coremcd;
            }
          }
        }
      }
      else {
        eqlogdepth.hole[h]->core[c]->did_eld = NO;
      }
    }
    else if((strlen(line)) < 1) {
    }
    (void)sprintf(checkchar, "");
    (void)sprintf(line, "");
  }

  if((charptr=(char*)calloc(50+(int)strlen(eqlogdepth.filename),CHAR_SIZE))==NULL) { 
    SaBadMemAlloc("SaFileSelectionBoxOKCB 22"); 
  } 
  else { 
    (void)sprintf(charptr,"\nEquivilent log depth table read from  %s.", eqlogdepth.filename);
    SaUpdateReport(charptr); 
    free(charptr); 
  } 
  fclose(fptr);

  return 0;
}

void ReadPreviewInput(fd, file)
FILE *fd;
char *file;
{
  int l, len, doit;
  char *text;
  XmTextPosition  pos;

  l = 0;
  len = 500;
  doit = NO;
  if((text=(char*)calloc(len * MAXPREVIEW, CHAR_SIZE))==NULL)
              SaBadMemAlloc("ReadPreviewInput 2");
  (void)sprintf(text, "");
  if((pos=XmTextGetLastPosition(textPreviewInput)) < 1) {
    doit = YES;
  }
  while((fgets(line,500,fd) != NULL) && l<MAXPREVIEW) {
    ++l;
    (void)strcat(text, line);
  }
 
  XmTextInsert(textPreviewInput,pos,text);
  free(text);

  if(doit == YES) {
    if((text=(char*)calloc(1000, CHAR_SIZE))==NULL)
              SaBadMemAlloc("ReadPreviewInput 2");
    if(l < MAXPREVIEW) {
      (void)sprintf(text, "File %s\n\n", file);
    }
    else {
      (void)sprintf(text, "First %d lines of file %s\n\n", l, file);
    }
    XmTextInsert(textPreviewInput,0,text);
    free(text);
  }

  if(l < MAXPREVIEW) {
    XtSetSensitive(pushButtonPreviewMore, False); 
  }
}

int SaReadAffineTable(filename, ds)
char    *filename;
int     ds;
{
        FILE    *fptr;
        int     i, j, k, all, alloc_cnt, alloc_inc, status, nh, nc,leg,site,corename;
        float   xvalue;
        char    did_it, holename, type;
        void CheckAffineTableOrder(),
             MapHoleToAffineTable(),
             MapCoreToAffineTable();

/* open input affine table */
        if((fptr=fopen(filename,"r"))==NULL)
        {
                info.error=True;
                info.fatal=False;
                info.default_button=XmDIALOG_OK_BUTTON;
                info.type=XmDIALOG_ERROR;
                info.caller=AFFINE;
                if((info.message=(char*)calloc((int)strlen(filename)+50,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
                (void)sprintf(info.message,"error opening affine table %s.",filename);
                SaManageMessage(info);
        }
        else
        {
               info.error=False;
        /* allocate one hole for affine table and alloc some cores for it
        */
                alloc_cnt = 50;
                alloc_inc = 25;
                if((affinetable.hole=(AffineHole**)calloc(1,sizeof(AffineHole*)))==NULL)
                        SaBadMemAlloc("Sa");
                if((affinetable.hole[0] = (AffineHole*)calloc(1,sizeof(AffineHole)))==NULL)
                        SaBadMemAlloc("Sa");
                if((affinetable.hole[0]->core=(AffineCore**)calloc(alloc_cnt,sizeof(AffineCore*)))==NULL)
                        SaBadMemAlloc("Sa");
                for(i=0; i<alloc_cnt; ++i) {
                  if((affinetable.hole[0]->core[i] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                        SaBadMemAlloc("Sa");
                }
        /* read affine table */
                for(j=0;(status=fscanf(fptr,"%d %d %c %d %c %f %c",&leg,&site,&holename,&corename,&type,&xvalue,&did_it))==7; j++) 
                {
                        holename = toupper(holename);
                        type=toupper(type);
                        did_it=toupper(did_it);
              
                        if(j == 0) {
                          nh = 0;
                          nc = 0;
                          affinetable.leg = leg;
                          affinetable.site = site;
                          affinetable.hole[0]->name = holename;
                          affinetable.numhole = 1;
                          affinetable.hole[0]->numcore = 0;
                        }
                        else if(j > 0 && affinetable.hole[nh]->name != holename) {
                          ++nh;
                          ++affinetable.numhole;
                          if((charptr=(char*)realloc((char*)affinetable.hole,affinetable.numhole*sizeof(AffineHole*)))==NULL)
                                        SaBadMemAlloc("Sa");
                          else
                                        affinetable.hole=(AffineHole**)charptr;
                          if((affinetable.hole[nh] = (AffineHole*)calloc(1,sizeof(AffineHole)))==NULL)
                                        SaBadMemAlloc("Sa");
                          if((affinetable.hole[nh]->core=(AffineCore**)calloc(alloc_cnt,sizeof(AffineCore*)))==NULL)
                                          SaBadMemAlloc("Sa");
                          for(i=0; i<alloc_cnt; ++i) {
                            if((affinetable.hole[nh]->core[i] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                                            SaBadMemAlloc("Sa");
                          }
                          affinetable.hole[nh]->name = holename;
                          nc = 0;
                          affinetable.hole[nh]->numcore = 0;
                        }
                        ++affinetable.hole[nh]->numcore;
                        if(affinetable.hole[nh]->numcore > alloc_cnt) {
                          all = alloc_cnt;
                          while(affinetable.hole[nh]->numcore > all) {
                            all = all + alloc_inc;
                          }
                          for(k=0; k<affinetable.numhole; ++k) {
                            if((charptr=(char*)realloc((char*)affinetable.hole[k]->core,(all)*sizeof(AffineCore*)))==NULL)
                                        SaBadMemAlloc("Sa");
                            else
                                        affinetable.hole[k]->core=(AffineCore**)charptr;
                            for(i=alloc_cnt; i<all; ++i) {
                              if((affinetable.hole[k]->core[i] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                                           SaBadMemAlloc("Sa");
                            }
                          }
                          alloc_cnt = all;
                        }
                        affinetable.hole[nh]->core[nc]->name = corename;
                        affinetable.hole[nh]->core[nc]->type = type;
                        affinetable.hole[nh]->core[nc]->cum_depth_offset = xvalue;
                        if(did_it=='N')
                                affinetable.hole[nh]->core[nc]->did_offset = False;
                        else
                                affinetable.hole[nh]->core[nc]->did_offset = True;
                        ++nc;
                }

                if(status!=EOF)
                {
                        info.error=True;
                        info.fatal=False;
                        info.default_button=XmDIALOG_OK_BUTTON;
                        info.caller=AFFINE;
                        info.type=XmDIALOG_ERROR;
                        if((info.message=(char*)calloc((int)strlen(filename)+100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("Sa");
                        (void)sprintf(info.message,"Error reading affine table %s line %d.",filename,j);
                        SaCleanUp(AFFINE);
                }
                else
                {
                  affinetable.core_alloc_cnt = alloc_cnt;
                  affinetable.core_alloc_inc = alloc_inc;
                  (void)CheckAffineTableOrder(&affinetable);
                  (void)MapHoleToAffineTable(&affinetable, ds);
                  (void)MapCoreToAffineTable(&affinetable, ds);
                  info.error = False;
                }

                fclose(fptr);
        }
        return (info.error);
}

void SaReadSplice(file, s)
char *file;
int  s;
{
  int i, status, n, l_cnt, hold_alloc_cnt;
  char *line;
  FILE *fd;

  if((fd = fopen(file, "r"))==NULL) {
    info.error=True;
    info.fatal=False;
    info.default_button=XmDIALOG_OK_BUTTON;
    info.type=XmDIALOG_ERROR;
    info.caller=NONE;
    if((info.message=(char*)calloc((int)strlen(file)+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("ReadSplice");
    (void)sprintf(info.message,"Error: could not open Splice input file  %s.",file);
    SaManageMessage(info);
  }
  else {

    if((line=(char*)calloc(550,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("ReadSplice");
    
    n = 0;
    l_cnt = 1;
    if(fgets(line,500,fd) != NULL) {
      if(status = sscanf(line, "%d %c %d %c %s %f %f %f %f %f %f", 
         &splice[s]->site, &splice[s]->value[n]->hole, &splice[s]->value[n]->corename, &splice[s]->value[n]->type,
         splice[s]->value[n]->section, &splice[s]->value[n]->sect_top, &splice[s]->value[n]->sect_bot,
         &splice[s]->value[n]->mbsf, &splice[s]->value[n]->data, &splice[s]->value[n]->affine_offset, 
         &splice[s]->value[n]->mcd) == 11) {
        splice[s]->value[n]->quality = GOOD;
        splice[s]->value[n]->sm_data = splice[s]->value[n]->data;
        ++n;
        ++l_cnt;
      }
      else {
        info.error=True;
        info.fatal=False;
        info.default_button=XmDIALOG_OK_BUTTON;
        info.type=XmDIALOG_ERROR;
        info.caller=NONE;
        if((info.message=(char*)calloc((int)strlen(file)+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("ReadSplice");
        (void)sprintf(info.message,"Error: Reading Splice input file  %s  format problem at line %d.",file, l_cnt);
        SaManageMessage(info);
        return;
      }
    }
    else {

    }
    while(fgets(line,500,fd) != NULL) {

      if(n == splice[s]->alloc_cnt) {
        hold_alloc_cnt = splice[s]->alloc_cnt;
        splice[s]->alloc_cnt = splice[s]->alloc_cnt + 500;
        if((charptr=(char*)realloc((char*)splice[s]->value,splice[s]->alloc_cnt*sizeof(SpliceValue*)))==NULL) {
                                SaBadMemAlloc("ReadSplice");
        }
        else {
          splice[s]->value=(SpliceValue**)charptr;
        }
        for(i=hold_alloc_cnt; i<splice[s]->alloc_cnt; ++i) {
          if((splice[s]->value[i]=(SpliceValue*)calloc(1,sizeof(SpliceValue)))==NULL)
                                SaBadMemAlloc("ReadSplice");
        }
      }
 
      if(status = sscanf(line, "%*d %c %d %c %s %f %f %f %f %f %f", 
         &splice[s]->value[n]->hole, &splice[s]->value[n]->corename, &splice[s]->value[n]->type,
         splice[s]->value[n]->section, &splice[s]->value[n]->sect_top, &splice[s]->value[n]->sect_bot,
         &splice[s]->value[n]->mbsf, &splice[s]->value[n]->data, &splice[s]->value[n]->affine_offset, 
         &splice[s]->value[n]->mcd) == 10) {
        splice[s]->value[n]->quality = GOOD;
        splice[s]->value[n]->sm_data = splice[s]->value[n]->data;
        ++n;
        ++l_cnt;
      }
      else {
        info.error=True;
        info.fatal=False;
        info.default_button=XmDIALOG_OK_BUTTON;
        info.type=XmDIALOG_ERROR;
        info.caller=NONE;
        if((info.message=(char*)calloc((int)strlen(file)+100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("ReadSplice");
        (void)sprintf(info.message,"Error: Reading Splice input file  %s  format problem at line %d.",file, l_cnt);
        SaManageMessage(info);
        return;
      }
    }

    splice[s]->numvalues = n;

    free(line);
    line = NULL;
  }
}

int     SpReadSplicerTable(fptr,splicerptr,cnt)
FILE    *fptr;
SplicerTableInfo        *splicerptr;
int     *cnt;
{
        int     status,i;

        status=fscanf(fptr,"%d %c %d %c %s %f %f %f %f %s %d %c %d %c %s %f %f %f %f",
                        &splicerptr[*cnt].site,&splicerptr[*cnt].hole,&splicerptr[*cnt].core,&splicerptr[*cnt].core_type,
                        splicerptr[*cnt].section,&splicerptr[*cnt].top_int,&splicerptr[*cnt].bot_int,
                        &splicerptr[*cnt].mbsf,&splicerptr[*cnt].mcd, splicerptr[*cnt+1].type_of_splice,
                        &splicerptr[*cnt+1].site,&splicerptr[*cnt+1].hole,&splicerptr[*cnt+1].core,&splicerptr[*cnt+1].core_type,
                        splicerptr[*cnt+1].section,&splicerptr[*cnt+1].top_int,&splicerptr[*cnt+1].bot_int,
                        &splicerptr[*cnt+1].mbsf,&splicerptr[*cnt+1].mcd);
        if(status==9 || status==19)     /* more data to read */
        {
                *cnt=*cnt+status/9;
                splicerptr[*cnt].hole=toupper(splicerptr[*cnt].hole);
                splicerptr[*cnt].core_type=toupper(splicerptr[*cnt].core_type);
                for(i=0;i<(int)strlen(splicerptr[*cnt].section);i++)
                {
                        if(isalpha(splicerptr[*cnt].section[i]))
                                splicerptr[*cnt].section[i]=toupper(splicerptr[*cnt].section[i]);
                }
                splicerptr[*cnt+1].hole=toupper(splicerptr[*cnt+1].hole);
                splicerptr[*cnt+1].core_type=toupper(splicerptr[*cnt+1].core_type);
                for(i=0;i<(int)strlen(splicerptr[*cnt+1].section);i++)
                {
                        if(isalpha(splicerptr[*cnt+1].section[i]))
                                splicerptr[*cnt+1].section[i]=toupper(splicerptr[*cnt+1].section[i]);
                }
        }
        return(status);
}

int WriteCoreData(holeptr, filename, ds)
Hole    *holeptr;
char    *filename;
int     ds;
{

  int     i,j;
  Core    *coreptr;
  Value   *valueptr;
  FILE    *fptr;

  if((fptr = fopen(filename, "w")) == NULL) {
    return 1;
  }

  for(i=0;i<holeptr->numcores;i++) {
    coreptr=holeptr->core[i];
    for(j=0;j<coreptr->numvalues;j++) {
      valueptr=coreptr->value[j];
      valueptr=coreptr->value[j];
      if(valueptr->quality==GOOD) {
        switch (data[ds]->in_format) { 

          case MST95REPORT:
          case TKREPORT:
            (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.1f\t%.2f\t%.3f",
                        data[ds]->site,holeptr->name,coreptr->num,valueptr->type,
                        valueptr->section,valueptr->top_int,
                        valueptr->bot_int, valueptr->mbsf, *valueptr->data);
            break;
          case ODPOTHER1:
            (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                          data[ds]->site,holeptr->name,coreptr->num,valueptr->type,
                          valueptr->section,valueptr->top_int,
                          valueptr->bot_int, valueptr->mbsf, *valueptr->data);
            break;
          case ODPOTHER2:
            (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                          data[ds]->site, holeptr->name,coreptr->num,valueptr->type,
                          valueptr->section,valueptr->top_int,
                          valueptr->bot_int, valueptr->mbsf, *valueptr->data);
            break;
          case ODPOTHER3:
            (void)fprintf(fptr,"%d\t%c\t%d\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                          data[ds]->site, holeptr->name,coreptr->num,
                          valueptr->section,valueptr->top_int,
                          valueptr->bot_int, valueptr->mbsf, *valueptr->data);
            break;
          case ODPOTHER4:
            (void)fprintf(fptr,"%d\t%c\t%d\t%s\t%.1f\t%.2f\t%.2f",
                          data[ds]->site, holeptr->name,coreptr->num,
                          valueptr->section,valueptr->top_int,
                          valueptr->mbsf, *valueptr->data);
            break;
          case ODPOTHER5:
            (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.2f\t%.2f",
                          data[ds]->site,holeptr->name,coreptr->num,valueptr->type,
                          valueptr->section,valueptr->top_int,
                          valueptr->mbsf, *valueptr->data);
            break;
        }
        if(smooth.method == GAUSSIAN) {
          (void)fprintf(fptr,"\t%.2f",valueptr->sm_data);
        }
        (void)fprintf(fptr,"\t%.2f\t%.2f\n", (valueptr->mbsf+coreptr->affine_offset), valueptr->eld);
      }
    }
  }
  fclose(fptr);
  return 0;
}

int WriteSpliceData(s, ds)
int s;
int ds;
{
  int i;
  FILE *fptr;

  if((fptr = fopen(save_splice_filename, "w")) == NULL)
    return 1;

  for(i=0; i<splice[s]->numvalues; ++i) {
    if(splice[s]->value[i]->quality == GOOD) {
      
      switch (data[ds]->in_format) {

        case MST95REPORT:
        case TKREPORT:
          (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                        splice[s]->site, splice[s]->value[i]->hole, splice[s]->value[i]->corename, splice[s]->value[i]->type,
                        splice[s]->value[i]->section, splice[s]->value[i]->sect_top, splice[s]->value[i]->sect_bot,
                        splice[s]->value[i]->mbsf, splice[s]->value[i]->data);
          break;
        case ODPOTHER1:
          (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                        splice[s]->site, splice[s]->value[i]->hole, splice[s]->value[i]->corename, splice[s]->value[i]->type,
                        splice[s]->value[i]->section, splice[s]->value[i]->sect_top, splice[s]->value[i]->sect_bot,
                        splice[s]->value[i]->mbsf, splice[s]->value[i]->data);
          break;
        case ODPOTHER2:
          (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                        splice[s]->site, splice[s]->value[i]->hole, splice[s]->value[i]->corename, splice[s]->value[i]->type,
                        splice[s]->value[i]->section, splice[s]->value[i]->sect_top, splice[s]->value[i]->sect_bot,
                        splice[s]->value[i]->mbsf, splice[s]->value[i]->data);
          break;
        case ODPOTHER3:
          (void)fprintf(fptr,"%d\t%c\t%d\t%s\t%.1f\t%.1f\t%.2f\t%.2f",
                        splice[s]->site, splice[s]->value[i]->hole, splice[s]->value[i]->corename,
                        splice[s]->value[i]->section, splice[s]->value[i]->sect_top, splice[s]->value[i]->sect_bot,
                        splice[s]->value[i]->mbsf, splice[s]->value[i]->data);
          break;
        case ODPOTHER4:
          (void)fprintf(fptr,"%d\t%c\t%d\t%s\t%.1f\t%.2f\t%.2f",
                        splice[s]->site, splice[s]->value[i]->hole, splice[s]->value[i]->corename,
                        splice[s]->value[i]->section, splice[s]->value[i]->sect_top,
                        splice[s]->value[i]->mbsf, splice[s]->value[i]->data);
          break;
        case ODPOTHER5:
          (void)fprintf(fptr,"%d\t%c\t%d\t%c\t%s\t%.1f\t%.2f\t%.2f",
                        splice[s]->site, splice[s]->value[i]->hole, splice[s]->value[i]->corename, splice[s]->value[i]->type,
                        splice[s]->value[i]->section, splice[s]->value[i]->sect_top,
                        splice[s]->value[i]->mbsf, splice[s]->value[i]->data);
          break;
      }
      if(splice[s]->smooth.method == GAUSSIAN) {
        (void)fprintf(fptr,"\t%.2f", splice[s]->value[i]->sm_data);
      }
      (void)fprintf(fptr,"\t%.2f\t%.2f\n", splice[s]->value[i]->mcd, splice[s]->value[i]->eld);
    }
  }
  fclose(fptr);
  return 0;
}

int     SaWriteStrat(fptr,stratptr)
FILE    *fptr;
StratData       *stratptr;
{
        char    topoffset[6],botoffset[6],
                topmcd[9],botmcd[9];

        if(stratptr->top.in_affine == NO)
        {
                (void)sprintf(topmcd,"N/A");
                (void)sprintf(topoffset,"N/A");
        }
        else
        {
                (void)sprintf(topmcd,"%.2f",stratptr->top.mbsf+stratptr->top.aff_offset);
                (void)sprintf(topoffset,"%.2f",stratptr->top.aff_offset);
        }
        if(stratptr->bot.in_affine == NO)
        {
                (void)sprintf(botmcd,"N/A");
                (void)sprintf(botoffset,"N/A");
        }
        else
        {
                (void)sprintf(botmcd,"%.2f",stratptr->bot.mbsf+stratptr->bot.aff_offset);
                (void)sprintf(botoffset,"%.2f",stratptr->bot.aff_offset);
        }
        fprintf(fptr,"%s\t%s\t%.2f\t%.2f\t%d\t%d\t%c\t%d\t%c\t%s\t%.2f\t%.2f\t%s\t%s\t%.2f\t%d\t%d\t%c\t%d\t%c\t%s\t%.2f\t%.2f\t%s\t%s\t%.2f\n",
                stratptr->name,stratptr->code,stratptr->top_age,stratptr->bot_age,stratptr->top.leg,
                stratptr->top.site,stratptr->top.hole,stratptr->top.core,
                stratptr->top.coretype,stratptr->top.section,stratptr->top.interval,
                stratptr->top.mbsf, topoffset, topmcd, stratptr->top.eld,
                stratptr->bot.leg,stratptr->bot.site,stratptr->bot.hole,stratptr->bot.core,
                stratptr->bot.coretype,stratptr->bot.section,stratptr->bot.interval,stratptr->bot.mbsf,
                botoffset,botmcd, stratptr->bot.eld);
        return(True);
}

int WriteLogCoreDataPairs(filename)
char    *filename;
{

  int     i, k;
  FILE    *fptr;

  if(datapairs_cnt < 1) {
    return 1;
  }
  if((fptr = fopen(filename, "w")) == NULL) {
    return 1;
  }

  fprintf(fptr,"Leg %d,  Site %d, Core data type %s,  Log data type %s,\n", 
          data[dset]->leg, data[dset]->site, type[data[dset]->type], logdata[lset]->info.id[0]);
  if(datapairs_how == TOLOGDEPTH) {
    fprintf(fptr,"Data repicked to log depth\n");
  }
  else if(datapairs_how == TOCOREDEPTH) {
    fprintf(fptr,"Data repicked to core depth\n");
  }
  else if(datapairs_how == CONSTANTDT) {
    fprintf(fptr,"Data repicked at constant step = %.2fm\n", datapairs_dt);
  }
  else {
    fprintf(fptr,"Data repicked unknown\n");
  } 
  fprintf(fptr,"Linear Equation;  Core = %.3fLog + %.3f\n", datapairs_a, datapairs_b);
  if(datapairs_sm == YES) {
    if(have_affine == DO) {
      fprintf(fptr,"             MBSF     MCD     ELD    SmCore  SmLog\n");
    }
    else {
      fprintf(fptr,"             MBSF     ELD    SmCore  SmLog\n");
    }
  }
  else {
    if(have_affine == DO) {
      fprintf(fptr,"             MBSF     MCD     ELD       Core     Log\n");
    }
    else {
      fprintf(fptr,"             MBSF     ELD      Core      Log\n");
    }
  }
  for(k=0; k<datapairs_cnt; ++k) {
    for(i=0; i<logcoredatapairs[k]->n; ++i) {
      if(logcoredatapairs[k]->logdat[i] != BADPT && logcoredatapairs[k]->coredat[i] != BADPT) {
        if(have_affine == DO) {
          fprintf(fptr,"%c%5d%10.2f%10.2f%10.2f%10.2f%10.2f\n", logcoredatapairs[k]->holename, logcoredatapairs[k]->corename[i],
                  logcoredatapairs[k]->mbsf[i], logcoredatapairs[k]->mcd[i], logcoredatapairs[k]->eld[i], 
                  logcoredatapairs[k]->coredat[i], logcoredatapairs[k]->logdat[i]);
        }
        else {
          fprintf(fptr,"%c%5d%10.2f%10.2f%10.2f%10.2f\n", logcoredatapairs[k]->holename, logcoredatapairs[k]->corename[i],
                  logcoredatapairs[k]->mbsf[i], logcoredatapairs[k]->eld[i],
                  logcoredatapairs[k]->coredat[i], logcoredatapairs[k]->logdat[i]);
        }
      }
    }
  }
  fclose(fptr);
  return 0; 
}

int SaReadJanus(fptr,dataptr)
FILE    *fptr;
DataFile        *dataptr;
{
        char    hole_name, munsell[2][20];
        int     core_num,status,this_hole,this_core,max_alloc;
        float   val[6];
        Value   tmp_value,*valueptr;
        Core    *coreptr;
        Hole    *holeptr;

        /*return False=more data, true=done */
/* read data line */
        dataptr->subleg=' ';
        if(fgets(buffer,1000,fptr)==NULL)
                return(True);

        if(dataptr->in_format == JANUSORIG) {
          if(strcmp(buffer, "Sql statement executed OK!\n") == 0) {
            return(True);
          }
        }

/* get variables */
        switch (dataptr->type)
        {
                case    GRAPE:
                case    PWAVE:
                case    NATURALGAMMA:
                        if((status=sscanf(buffer, "%d %d %c %d %c %s %f %f %f",
                                   &dataptr->leg, &dataptr->site, &hole_name, &core_num, &tmp_value.type,
                                   tmp_value.section, &tmp_value.top_int, &tmp_value.mbsf,
                                   &val[0]))!=9)
                        {
                                if(status==EOF)
                                        return(True);
                                else
                                        info.error=True;
                        }
                        break;
                case    SUSCEPTIBILITY:
                        if((status=sscanf(buffer, "%d %d %c %d %c %s %f %f %f %f",
                                   &dataptr->leg, &dataptr->site, &hole_name, &core_num, &tmp_value.type,
                                   tmp_value.section, &tmp_value.top_int, &tmp_value.mbsf,
                                   &val[0], &val[1]))!=10)
                        {
                                if(status==EOF)
                                        return(True);
                                else
                                        info.error=True;
                        }
                        break;
                case    REFLECTANCE:
                        if((status=sscanf(buffer, "%d %d %c %d %c %s %f %f %f %f %f %s, %s, %f %f %f",
                                   &dataptr->leg, &dataptr->site, &hole_name, &core_num, &tmp_value.type,
                                   tmp_value.section, &tmp_value.top_int, &tmp_value.mbsf,
                                   &val[0], val[1], &val[2], munsell[0], munsell[1], &val[3], &val[4], &val[5]))!=16)
                        {
                                if(status == 13) {
                                }
                                else if(status==EOF) {
                                        return(True);
                                }
                                else {
                                        info.error=True;
                                }
                        }
                        break;
                default:
                        info.fatal=False;
                        info.default_button=XmDIALOG_OK_BUTTON;
                        info.type=XmDIALOG_ERROR;
                        info.caller=READJANUS;
                        if((info.message=(char*)calloc(100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaReadJanus 1");
                        (void)sprintf(info.message,"Invalid other odp data type %d",dataptr->type);
                        SaManageMessage(info);
                        return(True);
        }
        tmp_value.bot_int=tmp_value.top_int;
        if(info.error)
        {
                info.fatal=False;
                info.default_button=XmDIALOG_OK_BUTTON;
                info.type=XmDIALOG_ERROR;
                info.caller=READJANUS;
                if((info.message=(char*)calloc(50+(int)strlen(type[dataptr->type])+
                                        (int)strlen(format[dataptr->in_format]),CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaReadJanus 2");
                (void)sprintf(info.message,"Bad %s %s line",format[dataptr->in_format],type[dataptr->type]);
                SaManageMessage(info);
                return(False);
        }
        /* check if new hole */
        hole_name=toupper(hole_name);
        if(dataptr->holes_used==NULL || strchr(dataptr->holes_used,hole_name)==NULL)
        {
                dataptr->numholes++;
                if(dataptr->numholes >1)        /* not first hole */
                {
                        if((charptr=(char*)realloc(dataptr->holes_used,(dataptr->numholes+1)*CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaReadJanus 3");
                        else
                                dataptr->holes_used = charptr;
                        if((charptr=(char*)realloc((char*)dataptr->holes,dataptr->numholes*sizeof(Hole*)))==NULL)
                                SaBadMemAlloc("SaReadJanus 4");
                        else
                                dataptr->holes=(Hole**)charptr;
                }
                else            /* first hole */
                {
                        if((dataptr->holes_used=(char*)calloc(2,sizeof(char)))==NULL)
                                SaBadMemAlloc("SaReadJanus 5");
                        dataptr->holes_used[0]='\0';
                        if((dataptr->holes=(Hole**)calloc(dataptr->numholes,sizeof(Hole*)))==NULL)
                                SaBadMemAlloc("SaReadJanus 6");
                }
                /* set hole values */
                if((dataptr->holes[dataptr->numholes-1]=(Hole*)calloc(1,sizeof(Hole)))==NULL)
                        SaBadMemAlloc("SaReadJanus 7");
                this_hole=dataptr->numholes-1;
                holeptr=dataptr->holes[this_hole];
                holeptr->name=hole_name;
                (void)sprintf(dataptr->holes_used,"%s%c",dataptr->holes_used,hole_name);
                holeptr->numcores=0;
        }
        else    /* hole already exists */
        {
                for(this_hole=0;this_hole<dataptr->numholes && dataptr->holes[this_hole]->name!=hole_name;this_hole++);
        }
        holeptr=dataptr->holes[this_hole];
        /* check if new core*/
        for(this_core=0;this_core<holeptr->numcores && core_num != holeptr->core[this_core]->num;this_core++);
        if(this_core >= holeptr->numcores)
        {
                if(holeptr->numcores++>0)       /* not first core*/
                {
                        if((charptr=(char*)realloc((char*)holeptr->core,holeptr->numcores*sizeof(Core*)))==NULL)
                                SaBadMemAlloc("SaReadJanus 8");
                        else
                                data[dset]->holes[this_hole]->core=(Core**)charptr;
                }
                else            /* firstcore */
                {
                        if((holeptr->core=(Core**)calloc(holeptr->numcores,sizeof(Core*)))==NULL)
                                SaBadMemAlloc("SaReadJanus 9");
                }
                /* set core values */
                if((holeptr->core[this_core]=(Core*)calloc(1,sizeof(Core)))==NULL)
                        SaBadMemAlloc("SaReadJanus 10");
                coreptr=holeptr->core[this_core];
                coreptr->num=core_num;
                coreptr->numvalues=0;
                coreptr->good = NULL;
                if((coreptr->value=(Value**)calloc(ALLOC_INT,sizeof(Value*)))==NULL)
                        SaBadMemAlloc("SaReadJanus 11");
                for(coreptr->allocvalues=0;coreptr->allocvalues<ALLOC_INT;coreptr->allocvalues++)
                {
                        if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
                                SaBadMemAlloc("SaReadJanus 12");
                }
        }
        coreptr=holeptr->core[this_core];
        coreptr->have_equil_log_depths=DONT;
        if(coreptr->numvalues++ >= coreptr->allocvalues)
        {
                max_alloc=coreptr->allocvalues+ALLOC_INT;
                if((charptr=(char*)realloc((char*)coreptr->value,max_alloc*sizeof(Value*)))==NULL)
                        SaBadMemAlloc("SaReadJanus 13");
                else
                        coreptr->value=(Value**) charptr;
                for(;coreptr->allocvalues<max_alloc;coreptr->allocvalues++)
                {
                        if((coreptr->value[coreptr->allocvalues]=(Value*)calloc(1,sizeof(Value)))==NULL)
                                SaBadMemAlloc("SaReadJanus 14");
                }
        }
        valueptr=coreptr->value[coreptr->numvalues-1];
/* get standard core values */
        valueptr->type=tmp_value.type;
        (void)strcpy(valueptr->section,tmp_value.section);
        valueptr->top_int=tmp_value.top_int;
        valueptr->bot_int=tmp_value.bot_int;
        valueptr->mbsf=tmp_value.mbsf;
        valueptr->eld=valueptr->mbsf;
        if(fabs(valueptr->mbsf - BAD_CODE) <= ROUNDCHECK)
                valueptr->quality=BAD_SB_DEPTH;
        else
                valueptr->quality=GOOD;
        valueptr->run_num=0;
        switch (dataptr->type)
        {
          case GRAPE:
          case PWAVE:
          case NATURALGAMMA:
            valueptr->otherodp = val[0];
            valueptr->data = &valueptr->otherodp;
            break;
          case SUSCEPTIBILITY:
            if((valueptr->susceptibility=(Susceptibility*)calloc(1,sizeof(Susceptibility)))==NULL)
                                        SaBadMemAlloc("SaReadJanus 15");
            valueptr->susceptibility->observed = val[0];
            valueptr->susceptibility->corrected = val[1];
            valueptr->data = &valueptr->susceptibility->corrected;
            break;
          case REFLECTANCE:
            if((valueptr->reflectance=(Reflectance*)calloc(1,sizeof(Reflectance)))==NULL)
                                        SaBadMemAlloc("SaReadJanus 16");
            valueptr->reflectance->L = val[0];
            valueptr->reflectance->a = val[1];
            valueptr->reflectance->b = val[2];
            strcpy(valueptr->reflectance->munsell_hue, munsell[0]);
            strcpy(valueptr->reflectance->munsell_value, munsell[1]);
            valueptr->data = &valueptr->reflectance->L;
            break;
          default:
            dataptr->readerrors++;
            info.error=True;
        }

        if(info.error==True)
        {
          info.fatal=False;
          info.default_button=XmDIALOG_OK_BUTTON;
          info.type=XmDIALOG_ERROR;
          info.caller=READJANUS; 
          if((info.message=(char*)calloc(50+(int)strlen(format[dataptr->in_format])+(int)strlen(type[dataptr->type]),CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaReadJanus 17");
          (void)sprintf(info.message,"Bad %s %s line",format[dataptr->in_format],type[dataptr->type]);
          SaManageMessage(info);
        }
        else
          valueptr->sm_data=*valueptr->data;

        return(False);
}
