/* 
     Sagan math functions

      These functions perform various 'math' related tasks

    int Correlate()
    int Correlate2()
    int CorrelationSetup()
    int RepickDepthAroundTie()
    int FindAveDepth()
    int FindMinMax()
    int FindLogMinMax()
    void SetupCalcEquivLogDepths()
    int CalcEquivLogDepths()
    int StdDev()
    int LogStdDev()
    int MakeEquivLogDepthEquation()
    int GetTieListPosition()
    int CheckTie()
    void GetTieArrayNum()
    void MakeTempties()
    void MakeTies()
    void MakeNewELDTable()
    void MapHoleToELDTable()
    void MapCoreToELDTable()
    void CheckELDTableOrder()
    void CheckELDTableMissingHole()
    void CheckELDTableMissingCore()
    int InterpolateCoreTiePtValue()
    int InterpolateLogTiePtValue()
    void AdjustToTies()
    int SaAutoCoreAdjust()
    int GetCoreToLogDepths()
    int RepickDepth()
    void MapHoleToAffineTable()
    void MapCoreToAffineTable()
    void CheckAffineTableOrder()
    void ApplyCoreAdjust()
    void ApplyCoreAdjustToOneCore()
*/
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <stdio.h>
#include <math.h>
#include <Xm/Xm.h>
#include <X11/cursorfont.h>
#include "sagan_vars.h"
#include "sagan-creation-c.h"

char    buffer[MAXBUF];

/* correlation function.
*/
int Correlate(nx,ny,offx,offy,x,y,pcoef,pn)
int nx, ny, offx, offy, *pn;
float *x, *y, *pcoef;
{
  int j;
  float sumx, sumy, sumxy, sumxsq, sumysq, stdevx, stdevy;


  *pn=0;
  sumx=0.0;
  sumy=0.0;
  sumxy=0.0;
  sumxsq=0.0;
  sumysq=0.0;

  /* sum up
  */
  for(j=0; j<ny; ++j){
    if(j+offx+offy < 0 || j+offx+offy >= nx){
    }
    else if(x[j+offx+offy] == BADPT || y[j+offy] == BADPT){
    }
    else {
      ++*pn;
      sumx=sumx+x[j+offx+offy];
      sumy=sumy+y[j+offy];
      sumxy=sumxy+x[j+offx+offy]*y[j+offy];
      sumxsq=sumxsq+x[j+offx+offy]*x[j+offx+offy];
      sumysq=sumysq+y[j+offy]*y[j+offy];
    }
  }
 
  /* calculate the standard deviation and
     the correlation coeficiant
  */
  if(*pn > 1){
    stdevx=((*pn)*sumxsq)-(sumx*sumx);
    stdevy=((*pn)*sumysq)-(sumy*sumy);
    if(stdevx <= 0 || stdevy <= 0){
      *pcoef=0.0;
      *pn=0;
    }
    else {
      stdevx=sqrt(stdevx);
      stdevy=sqrt(stdevy);
      *pcoef=(((*pn)*sumxy)-(sumx*sumy))/(stdevx*stdevy);
    }
  }
  else {
    *pcoef=0.0;
    *pn=0;
  }
  return 0;
}

int Correlate2(nx,ny,offx,offy,x,y,pcoef,pn)
int nx, ny, offx, offy, *pn;
float *x, *y, *pcoef;
{
  int j;
  float sumx, sumy, sumxy, sumxsq, sumysq, stdevx, stdevy;


  *pn=0;
  sumx=0.0;
  sumy=0.0;
  sumxy=0.0;
  sumxsq=0.0;
  sumysq=0.0;

  /* sum up
  */
  for(j=0; j<ny; ++j){
    if(j+offx < 0 || j+offx >= nx){
    }
    else if(j+offy < 0 || j+offy >= ny){
    }
    else if(x[j+offx] == BADPT || y[j+offy] == BADPT){
    }
    else {
      ++*pn;
      sumx=sumx+x[j+offx];
      sumy=sumy+y[j+offy];
      sumxy=sumxy+x[j+offx]*y[j+offy];
      sumxsq=sumxsq+x[j+offx]*x[j+offx];
      sumysq=sumysq+y[j+offy]*y[j+offy];
    }
  }

  /* calculate the standard deviation and
     the correlation coeficiant
  */
  if(*pn > 1){
    stdevx=((*pn)*sumxsq)-(sumx*sumx);
    stdevy=((*pn)*sumysq)-(sumy*sumy);
    if(stdevx <= 0 || stdevy <= 0){
      *pcoef=0.0;
      *pn=0;
    }
    else {
      stdevx=sqrt(stdevx);
      stdevy=sqrt(stdevy);
      *pcoef=(((*pn)*sumxy)-(sumx*sumy))/(stdevx*stdevy);
    }
  }
  else {
    *pcoef=0.0;
    *pn=0;
  }
  return 0;
}


int CorrelationSetup(ds)
int ds;
{
  int i, j, n, h, c, coren, logn, coreoffset, logoffset, tiecore, tielog,
      lag, nwin, sm, err, num_lead_lag, pos, offset, all_cnt, start;
  char string[50];
  float coef, range, depstep, winlen, leadlagmeters, checktop, checkbot, flip, intpt,
        *coredep, *corevar, *logdep, *logvar;
  Ties  *tempties;
  String str;
  void MakeTempties();
  int GetTieListPosition(), MakeEquivLogDepthEquation(), InterpolateTiePtValue();

  if(have_coretotie == DO) {
    h = ties->hole;
    c = ties->core;
    n = data[ds]->holes[h]->core[c]->numvalues;
    tiecore = ties->coretie;
    if(data[ds]->flip == YES) {
      flip = -1.0;
    }
    else {
      flip = 1.0;
    }
  }
  else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
    tiecore = ties->coretie;
    n = splice[sset]->numvalues;
    if(data[ds]->flip == YES) {
      flip = -1.0;
    }
    else {
      flip = 1.0;
    }
  }
  else {
    return 1;
  }

  /* get the depth step
  */
  str = XmTextGetString(textFieldDepthStep);
  depstep = atof(str);
  XtFree(str);
  if(depstep <= 0.0) {
    return 1;
  }
  /* get the window length and lead/lag
  */
  str= XmTextGetString(textFieldWindowLength);
  winlen = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldLeadLag);
  leadlagmeters = atof(str);
  XtFree(str);
  if(winlen <= 0.0 || leadlagmeters <= 0.0) {
    return 1;
  }

  if(smooth.method == GAUSSIAN) {
    sm  = YES;
  }
  else {
    sm = NO;
  }

  /* calc how much space i will need to alloc for current depth step
  */
  if(have_coretotie == DO) {
    all_cnt = (data[ds]->holes[h]->core[c]->value[n-1]->mbsf - data[ds]->holes[h]->core[c]->value[0]->mbsf)/depstep + 1;
  }
  else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
    all_cnt = (splice[sset]->value[n-1]->mcd - splice[sset]->value[0]->mcd)/depstep + 1;
  }
  if(all_cnt < n) {
    all_cnt = n;
  }

  /* alloc space for temp arrays that hold core depth and var and
     assign them their values culling out bad samples
  */
  if((coredep=(float*)calloc(all_cnt + 500,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
  if((corevar=(float*)calloc(all_cnt + 500,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
  coren = 0;
  if(have_coretotie == DO) {
    for(i=0; i<n; ++i) {
      if(data[ds]->holes[h]->core[c]->value[i]->quality & BAD_SB_DEPTH) {
      }
      else if((data[ds]->holes[h]->core[c]->value[i]->mbsf >= ties->tiecorembsf - winlen - leadlagmeters) &&
         (data[ds]->holes[h]->core[c]->value[i]->mbsf <= ties->tiecorembsf + winlen + leadlagmeters)) {
        if(data[ds]->holes[h]->core[c]->value[i]->quality == GOOD) {
          coredep[coren] = data[ds]->holes[h]->core[c]->value[i]->mbsf;
          if(have_affine == DO) {
            coredep[coren] = coredep[coren] + data[ds]->holes[h]->core[c]->affine_offset;
          }
          if(sm == YES) {
            corevar[coren] = data[ds]->holes[h]->core[c]->value[i]->sm_data * flip;
          }
          else {
            corevar[coren] = *data[ds]->holes[h]->core[c]->value[i]->data * flip;
          }
          ++coren;
        }
      }  
      else if(data[ds]->holes[h]->core[c]->value[i]->mbsf > ties->tiecoremcd + winlen + leadlagmeters) {
        break;
      }  
    }
  }  
  else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
    for(i=0; i<n; ++i) {
      if((splice[sset]->value[i]->mcd >= ties->tiecoremcd - winlen - leadlagmeters) &&
         (splice[sset]->value[i]->mcd <= ties->tiecoremcd + winlen + leadlagmeters)) {
        coredep[coren] = splice[sset]->value[i]->mcd;
        if(sm == YES) {
          corevar[coren] = splice[sset]->value[i]->sm_data * flip;
        }
        else {
          corevar[coren] = splice[sset]->value[i]->data * flip;
        }
        ++coren;
      }
      else if(splice[sset]->value[i]->mcd > ties->tiecoremcd + winlen + leadlagmeters) {
        break;
      }
    }
  }
  if(tiecore == INTERP) {
    if(have_affine == DONT && (ties->tiecorembsf < coredep[0] || ties->tiecorembsf > coredep[coren-1])) { 
    }
    else if(have_affine == DO && (ties->tiecoremcd < coredep[0] || ties->tiecoremcd > coredep[coren-1])) {
    }
    else {
      err = InterpolateTiePtValue(coren, coredep, corevar, ties->tiecoremcd, &intpt, &start);
      if(err > 0) {
      }
      else if(start < 0 || start >= coren) {
      }
      else {
        for(i=coren; i>start+1; --i) {
          coredep[i] = coredep[i-1];
          corevar[i] = corevar[i-1];
        }
        ++coren;
        if(have_affine == DO) {
          coredep[start + 1] = ties->tiecoremcd;
        }
        else {
          coredep[start + 1] = ties->tiecorembsf;
        }
        corevar[start + 1] = intpt;
        tiecore = start + 1;
      }
    }
  }
  else {
    tiecore = -1;
    for(i=0; i<coren; ++i) {
      if(have_affine == DO || have_splice == DO) {
        if(fabs(coredep[i] - ties->tiecoremcd) < ROUNDCHECK ) {
          tiecore = i;
          break;
        }
      }
      else {
        if(fabs(coredep[i] - ties->tiecorembsf) < ROUNDCHECK) {
          tiecore = i;
          break;
        }
      }
    }
  }
  if(tiecore == -1) {
    free(coredep);
    free(corevar);
    return 2;
  }

  /* copy current core's tie list to a temp tie list and add current tie to it
  */
  if((tempties=(Ties*)calloc(1,sizeof(Ties)))==NULL)
                                SaBadMemAlloc("Sa");
  if((tempties->tie=(TieList**)calloc(ties->nties+1,sizeof(TieList*)))==NULL)
                                SaBadMemAlloc("Sa");
  for(i=0; i<=ties->nties; ++i) {
    if((tempties->tie[i]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                              SaBadMemAlloc("Sa");
  }
  MakeTempties(tempties);
  err = GetTieListPosition(YES, ds, &pos, tempties);
  MakeEquivLogDepthEquation(tempties);

  /* calc the eq. log depths using tie list and current tie
  */
  err = CalcEquivLogDepths(coren, coredep, tempties);
  if(tempties) {
    if(tempties->nties > 0) {
      for(i=0; i<tempties->nties; ++i) {
        if(tempties->tie[i]) {
          free(tempties->tie[i]);
          tempties->tie[i] = NULL;
        }
      }
    }
    if(tempties->tie) {
      free(tempties->tie);
      tempties->tie = NULL;
    }
    free(tempties);
    tempties = NULL;
  }

  if(err != 0 || coren < 1 || tiecore < 0) {
    (void)sprintf(buffer, "Error trying to get temp core for correlation.");
    SaManageWarningBox(buffer);
    ClearTie(NULL, NULL, NULL);
    free(coredep);
    free(corevar);
    return 1;
  } 
  else {
  }

  /* find the lead,lag in data points
  */
  if(leadlagmeters > 0.0){
    lead_lag = leadlagmeters/depstep;
    num_lead_lag = lead_lag * 2 + 1;
    lag = -1.0 * lead_lag;
  }
  else {
    lead_lag=0;
    lag=0;
  }
  if(num_lead_lag > MAXCORR){
    (void)sprintf(buffer, "The lead/lag you have chosen exceeds the limit for number of correlations allowed.");
    SaManageWarningBox(buffer);
    free(coredep);
    free(corevar);
    return 1;
  }

  /* find the top and botton depths for the portion of the log data needed.
     calc how many pts. will need to alloc for current depth step and
     alloc space for these arrays
  */
  if(logdata[lset]->flip == YES) {
    flip = -1.0;
  }
  else {
    flip = 1.0;
  }

  checktop = ties->tielogdep - winlen - leadlagmeters;
  checkbot = ties->tielogdep + winlen + leadlagmeters;

  start = -1;
  all_cnt = 0;
  for(i=0; i<logdata[lset]->data_num; ++i) {
    if(logdata[lset]->value[i]->depth > checkbot) {
      break;
    }
    else if(logdata[lset]->value[i]->depth >= checktop) {
      ++all_cnt;
      if(start == -1) {
        start = i;
      }
    }
  }
  if(start == -1 || all_cnt < 2) {
    (void)sprintf(buffer, "Corr. Error: Could not get overlapping section of log to correlate.");
    SaManageWarningBox(buffer);
    free(coredep);
    free(corevar);
    return 1;
  }

  if(all_cnt < (checkbot - checktop)/depstep + 1) {
    all_cnt = (checkbot - checktop)/depstep + 1;
  }
  if((logdep=(float*)calloc(all_cnt + 200, sizeof(float))) == NULL)
                   SaBadMemAlloc("Sa");
  if((logvar=(float*)calloc(all_cnt + 200, sizeof(float))) == NULL)
                   SaBadMemAlloc("Sa");

  /* put the portion of log needed in a temp array
  */
  logn = 0;
  for(i=start; i<logdata[lset]->data_num; ++i) {
    if(logdata[lset]->value[i]->depth > checkbot) {
      break;
    }
    else if(logdata[lset]->value[i]->data == logdata[lset]->info.missing_value || logdata[lset]->value[i]->quality != GOOD) { 
    }
    else {
      logdep[logn] = logdata[lset]->value[i]->depth;
      logvar[logn] = logdata[lset]->value[i]->data * flip;
      ++logn;
    }
  }
  if(logn < 2) {
    (void)sprintf(buffer, "Error: Correlation; there is no possible overlap of core and log.");
    SaManageWarningBox(buffer);
    free(coredep);
    free(corevar);
    free(logdep);
    free(logvar);
    return 1;
  }

  /* find where the tie is
  */
  tielog = -1;
  if(ties->logtie == INTERP) {
    err = InterpolateTiePtValue(logn, logdep, logvar, ties->tielogdep, &intpt, &start);
    if(err > 0) {
    }
    else {
      for(i=logn; i>start+1; --i) {
        logdep[i] = logdep[i-1];
        logvar[i] = logvar[i-1];
      }
      ++logn;
      logdep[start + 1] = ties->tielogdep;
      logvar[start + 1] = intpt;
      tielog = start + 1;
    }
  }
  else if(ties->logtie >= 0) {
    err = 1;
    for(i=0; i<logn; ++i) {
      if(fabs(logdep[i] - ties->tielogdep) < ROUNDCHECK ) {
        tielog = i;
        err = 0;
        break;
      }
    }
  }
  if(err > 0 || tielog < 0 || tielog >= logn) {
    (void)sprintf(buffer, "Error: Correlation; Could not find log tie in log overlapping section.");
    SaManageWarningBox(buffer);
    free(coredep);
    free(corevar);
    free(logdep);
    free(logvar);
    return 1;
  }

  /* repick log at constant depth step
  */
  if(depstep > 0.0){
    RepickDepthAroundTie(logdep, logvar, &logn, &tielog, ties->tielogdep, depstep);
  }
  else {
    (void)sprintf(buffer, "Error: The depth step is less than or equal to 0.0");
    SaManageWarningBox(buffer);
    free(coredep);
    free(corevar);
    free(logdep);
    free(logvar);
    return 1;
  }

  /* repick core at constant depth step
  */
  if(have_affine == DO) {
    RepickDepthAroundTie(coredep, corevar, &coren, &tiecore, ties->tiecoremcd, depstep);
  }
  else {
    RepickDepthAroundTie(coredep, corevar, &coren, &tiecore, ties->tiecorembsf, depstep);
  }
    
  /* find the first value(array member) of core that is within the window.
     then count out to the last value that is in the window
  */
  if(winlen > 0.0) {
    for(i=0; i<coren; ++i){
      if(coredep[i] >= coredep[tiecore]-winlen) {
        coreoffset=i;
        break;
      }
    }
    nwin=0;
    for(j=i; j<coren; ++j){
      if(coredep[j] > coredep[tiecore]+winlen){
        break;
      }
      else {
        ++nwin;
      }
    }
  }
  else {
    (void)sprintf(buffer, "Error: The window length is less than or equal to 0.0.");
    SaManageWarningBox(buffer);
    free(coredep);
    free(corevar);
    free(logdep);
    free(logvar);
    return 1;
  }
 
  /* calc. corr. coef. for each lead,lag in given window
     note that logoffset causes log to lag and then lead core
  */
  corr.num_lead_lag = 0;
  tie_corr_num = 0;
  for(i=0;i<num_lead_lag; ++i){
    logoffset = tielog - tiecore - lag;
    Correlate(logn, nwin, logoffset, coreoffset, logvar, corevar,
                    &coef, &n);
 
    if(n < TOOFEWCORRPAIRS){
 
    }
    else {
      corr.coef[corr.num_lead_lag] = coef;
      corr.n[corr.num_lead_lag] = n;
      corr.leadlag[corr.num_lead_lag] = lag;
      corr.leadlag_in_meter[corr.num_lead_lag] = lag * depstep;
      corr.depoffset[corr.num_lead_lag] = (ties->tielogdep - ties->temp_equivlogdep[ties->coretie]) + (-1 * lag * depstep);
      if(lag == 0){
        tie_corr_num = corr.num_lead_lag;
      }
      ++corr.num_lead_lag;
    }
    ++lag;
  }
 
  /* find the highest corr. coef. and save its array member number
  */
  if(lead_lag > 0){
    checktop = -1.0;
    best_corr_num = 0;
    for(i=1; i<corr.num_lead_lag; ++i){
      if(corr.coef[i] > checktop){
        checktop = corr.coef[i];
        best_corr_num = i;
      }
    }
    have_correlate = DO;
    (void)sprintf(string, "best: corr.coef. %.2f,  at a depth offset of %.2f",
                      corr.coef[best_corr_num], corr.depoffset[best_corr_num]);
    XtVaSetValues(textFieldCorrelate, XmNvalue, string, NULL);

    if(XtIsManaged(XtParent(drawingAreaOptimizeTieCorrelate))) {
      DrawDrawingAreaOptimizeTieCorrelate(drawingAreaOptimizeTieCorrelate,
                           (XtPointer)NULL, (XtPointer)NULL);
    }
    if(XtIsManaged(XtParent(drawingAreaCorrelate))) {
      DrawDrawingAreaCorrelate(drawingAreaCorrelate, (XtPointer)NULL, (XtPointer)NULL);
    }
  }
  else {
    free(coredep);
    free(corevar);
    free(logdep);
    free(logvar);
    return 1;
  }

  free(coredep);
  free(corevar);
  free(logdep);
  free(logvar);
  return 0;
}

/* repick depth at constant depth step
   pdep and pvar point to temp array
   of one core's depth and var.
   the data should be culled of
   bad values before coming here.
   note that the repickng is done from the tie point so that its
   original value is preserved
*/
int RepickDepthAroundTie(pdep, pvar, pn, ptie, tiedep, dt)
int *pn, *ptie;
float *pdep, *pvar, tiedep, dt;
{
  int i, k, kt, n, lastk, lastn;
  float dep[MAXPERCORE], var[MAXPERCORE];
 
  /* repick from tie point backwards to first sample
  */
  n=(pdep[*ptie]-pdep[0])/dt+1;
  if(n > MAXPERCORE){
    return 1;
  } 
  dep[n-1]=pdep[*ptie];
  for(i=n-2; i>=0;--i){
    dep[i]=dep[i+1]-dt;
  } 
  k=*ptie;
  kt=n-1;
  while(kt >= 0){
    if((k-1) < 0 && dep[kt] <= pdep[0]){
      var[kt]=(pvar[1]-pvar[0])/
                (pdep[1]-pdep[0])*
                (dep[kt]-pdep[0])+pvar[0];
      --kt;
    }
    else if(dep[kt] <= pdep[k] &&
            dep[kt] >= pdep[k-1]){
      var[kt]=(pvar[k]-pvar[k-1])/
                (pdep[k]-pdep[k-1])*
                (dep[kt]-pdep[k-1])+pvar[k-1];
      --kt;
    }
    else if(dep[kt] < pdep[k] && (k-1) > 0){
      --k;
    }
    else if(dep[kt] < pdep[0]){
      var[kt]=(pvar[1]-pvar[0])/
                (pdep[1]-pdep[0])*
                (dep[kt]-pdep[0])+pvar[0];
      --kt;
    }
  }
   
  /* repick from tie point to last sample
  */
  lastk=*pn-1;
  lastn=(pdep[lastk]-pdep[*ptie])/dt+n;
  if(lastn > MAXPERCORE){
    return 1;
  } 
  for(i=n;i<lastn;++i){
    dep[i]=dep[i-1]+dt;
  } 
  k=*ptie;
  kt=n;
  while(kt < lastn){
    if(k > lastk && dep[kt] >= pdep[lastk]){
      var[kt]=(pvar[lastk]-pvar[lastk-1])/
         (pdep[lastk]-pdep[lastk-1])*
         (dep[kt]-pdep[lastk])+pvar[lastk];
      ++kt;
    }
    else if(dep[kt] >= pdep[k-1] &&
            dep[kt] <= pdep[k]){
      var[kt]=(pvar[k]-pvar[k-1])/
                (pdep[k]-pdep[k-1])*
                (dep[kt]-pdep[k-1])+pvar[k-1];
      ++kt;
    }
    else if(dep[kt] > pdep[k] && (k+1) <= lastk){
      ++k;
    }
    else if(dep[kt] > pdep[lastk]){
      var[kt]=(pvar[lastk]-pvar[lastk-1])/
         (pdep[lastk]-pdep[lastk-1])*
         (dep[kt]-pdep[lastk])+pvar[lastk];
      ++kt;
    }
  }
  *ptie=n-1;
  *pn=lastn;
  for(i=0; i<lastn; ++i){
    pdep[i]=dep[i];
    pvar[i]=var[i];
  }
   
    
  /* check that have no duplicate depths
  */
  for(i=1; i<lastn; ++i){
    if(dep[i-1] == dep[i]){
      (void)printf("Repick; found duplicate depths at %.2f\n", dep[i]);
      return 1;
    }
    else if(dep[i-1] > dep[i]){
      (void)printf("Repick; depths not in descending order at %.2f\n", dep[i]);
      return 1;
    }
  }
    
  return 0;
}


int FindAveDepth(ds, holenum, corenum, pavedt)
int ds;
float *pavedt;
{
  int i, n;
  float nave, avedt, *tempdep;
  void Round();
 
  /* calculate the average depth step
  */ 
  *pavedt = 0.0;
  nave = 0.0;
  n = 0;
  if(data[ds]->holes[holenum]->core[corenum]->numvalues > 1){

    if((tempdep=(float*)calloc(data[ds]->holes[holenum]->core[corenum]->numvalues,sizeof(float))) == NULL)
                     SaBadMemAlloc("Sa");

    for(i=0; i<data[ds]->holes[holenum]->core[corenum]->numvalues; ++i){
      if((data[ds]->holes[holenum]->core[corenum]->value[i]->quality & ~BAD_SMOOTH) == GOOD) {
        tempdep[n] = data[ds]->holes[holenum]->core[corenum]->value[i]->mbsf;
        ++n;
      }
    }  
    for(i=0; i<n-1; ++i) {
      *pavedt = *pavedt + tempdep[i+1] - tempdep[i];
    }
    free(tempdep);
    tempdep = NULL;

    nave = n - 1.0;
    if(nave >= 1.0){
      *pavedt = *pavedt/nave;
    }
    else {
      (void)sprintf(buffer, "Warning; can not calculate average depth. using 0.05m.");
      SaManageWarningBox(buffer);
      *pavedt = 0.05;
      return 1;
    }
  } 
  else {
    (void)sprintf(buffer, "Warning; core has less than 2 points!");
    SaManageWarningBox(buffer);
    return 1;
  }

  /* round ave depth up to nearest cm
  */
  Round(pavedt, 0.01);

  return 0;
}

/* find min and max of current var for entire site.
   convert these to nearest reasonable whole numbers.
   use this range to calculate the initial drawing
   scale to use.
   this would be (max-min)/VAR_AXIS_LEN
   also find the maximum depth; minimum always 0.
*/
int FindMinMax(ds)
int ds;
{
  int i, j, k, sm, err, dum, coren, nstd, nsum;
  float *phold, *coredep, *corevar, sumavedep, affoff;
  double std, mean, sumstd, summean;
  int FindAveDepth();
  void Round();
 
  if(data[ds]->numholes < 1) {
    return 1;
  } 


  if(smooth.method == NONE) {
    sm = NO;
  } 
  else if((smooth.method == GAUSSIAN || smooth.method == LOWPASS) &&
          (smooth.plot == UNSMOOTH || smooth.plot == SMOOTHBOTH)){
    sm = NO;
  }  
  else if((smooth.method == GAUSSIAN || smooth.method == LOWPASS) &&
           smooth.plot == SMOOTH){
    sm = YES;
  } 
 
  minvar = 9999.0;
  maxvar = -9999.0;
  mincoredep = 9999.0;
  maxcoredep = -9999.0;
  summean = 0.0;
  mean = 0.0;
  sumstd = 0.0;
  std = 0.0;
  nstd = 0;
  sumavedep = 0.0;
  nsum = 0;
 
  for(i=0; i<data[ds]->numholes; ++i) {
    if(data[ds]->holes[i]->numcores < 1) {
    }
    else {
      for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
        if(data[ds]->holes[i]->core[j]->numgood < 1) {
        }
        else {
          for(k=0; k<data[ds]->holes[i]->core[j]->numvalues; ++k) {
 
            if(sm == NO &&
               (data[ds]->holes[i]->core[j]->value[k]->quality & ~BAD_SMOOTH) == GOOD){
              phold = data[ds]->holes[i]->core[j]->value[k]->data;
              if(*phold > maxvar) {
                maxvar = *phold;
              }
              if(*phold < minvar) {
                minvar = *phold;
              }
            }
            else if(sm == YES &&
                    data[ds]->holes[i]->core[j]->value[k]->quality == GOOD){
              if(data[ds]->holes[i]->core[j]->value[k]->sm_data > maxvar) {
                maxvar = data[ds]->holes[i]->core[j]->value[k]->sm_data;
              }
              if(data[ds]->holes[i]->core[j]->value[k]->sm_data < minvar) {
                minvar = data[ds]->holes[i]->core[j]->value[k]->sm_data;
              }
            }
            else {
            }
          }
     
          if((coredep=(float*)calloc(data[ds]->holes[i]->core[j]->numvalues, sizeof(float))) == NULL)
                      SaBadMemAlloc("Sa");
          if((corevar=(float*)calloc(data[ds]->holes[i]->core[j]->numvalues, sizeof(float))) == NULL)
                      SaBadMemAlloc("Sa");
          dum = 0;
          err = CleanUpCore(ds, i, j, coredep, corevar, NO, &dum, DONT);
          if(err == 0) {
            coren = data[ds]->holes[i]->core[j]->numvalues;
            err = StdDev(corevar, coren, 0, coren, &std, &mean);
            if(err == 0) {
              sumstd = sumstd + std;
              summean = summean + mean;
              ++nstd;
            }
          }
          free(coredep);
          coredep = NULL;
          free(corevar);
          corevar = NULL;
        }
        if(data[ds]->holes[i]->core[j]->numgood > 1) {
          err == FindAveDepth(ds, i, j, &data[ds]->holes[i]->core[j]->avedepstep);
          if(err == 0) {
            sumavedep = sumavedep + data[ds]->holes[i]->core[j]->avedepstep;
            ++nsum;
          }
        }
      }
    }
    j = data[ds]->holes[i]->numcores - 1;
    k = data[ds]->holes[i]->core[j]->numvalues - 1;
    if(have_affine == DO) {
      affoff = data[ds]->holes[i]->core[j]->affine_offset;
    }
    else {
      affoff = 0.0;
    }
    if(data[ds]->holes[i]->core[j]->value[k]->mbsf + affoff > maxcoredep){
      maxcoredep = data[ds]->holes[i]->core[j]->value[k]->mbsf + affoff;
    }
    if(have_affine == DO) {
      affoff = data[ds]->holes[i]->core[0]->affine_offset;
    }
    else {
      affoff = 0.0;
    }
    if(data[ds]->holes[i]->core[0]->value[0]->mbsf + affoff < mincoredep){
      mincoredep = data[ds]->holes[i]->core[0]->value[0]->mbsf + affoff;
    }
  }

  if(nsum > 0) {
    data[ds]->avedepstep = sumavedep/nsum;
    /* round ave depth to nearest cm
    */
    Round(&data[ds]->avedepstep, 0.01);
  }
  else {
    (void)sprintf(buffer, "Warning; can not calculate average depth for this data set. using 0.05m.");
    SaManageWarningBox(buffer);
    data[ds]->avedepstep = 0.05;
  }

  if((have_log == DO || have_ancill == DO || have_splice == DO) && maxdep < maxcoredep){
    maxdep = maxcoredep;
  }
  else if(have_log == DONT  && have_ancill == DONT && have_splice == DONT){
    maxdep = maxcoredep;
  }

  if(nstd > 0 && minvar != 9999.0 && maxvar != -9999.0 && maxvar > minvar) {
    realminvar = minvar;
    realmaxvar = maxvar;
    std = sumstd/nstd;
    mean = summean/nstd;
    minvar = mean - (PLUSMINUSSTD * std);
    maxvar = mean + (PLUSMINUSSTD * std);
    data[dset]->min = minvar;
    data[dset]->max = maxvar;
  }
  else {
    SaManageWarningBox("Warning; Could not find the min and max values of this data set: using defaults");
    if(data[dset]->type == GRAPE) {
      minvar = data[dset]->min = GRAPEMIN;
      maxvar = data[dset]->max = GRAPEMAX;
    }
    else if(data[dset]->type == PWAVE) {
      minvar = data[dset]->min = PWAVEMIN;
      maxvar = data[dset]->max = PWAVEMAX;
    }
    else if(data[dset]->type == SUSCEPTIBILITY) {
      minvar = data[dset]->min = SUSCMIN;
      maxvar = data[dset]->max = SUSCMAX;
    }
    else if(data[dset]->type == NATURALGAMMA) {
      minvar = data[dset]->min = NGAMMAMIN;
      maxvar = data[dset]->max = NGAMMAMAX;
    }
    else if(data[dset]->type == REFLECTANCE) {
      minvar = data[dset]->min = REFLECTMIN;
      maxvar = data[dset]->max = REFLECTMAX;
    }
    else if(data[dset]->type == OTHERTYPE) {
      minvar = data[dset]->min = OTHERMIN;
      maxvar = data[dset]->max = OTHERMAX;
    }
    else {
    }
    realminvar = minvar;
    realmaxvar = maxvar;
    return 1;
  }
    
  return 0;
}

int FindLogMinMax(ptrlog)
LogData *ptrlog;
{
  int i, err, dum, sm, culldep;
  double std, mean;
  void Round();

  if(logdata[lset]->info.smooth.method == NONE) {
    sm = NO;
  }
  else if((logdata[lset]->info.smooth.method == GAUSSIAN || logdata[lset]->info.smooth.method == LOWPASS) &&
          (logdata[lset]->info.smooth.plot == UNSMOOTH || logdata[lset]->info.smooth.plot == SMOOTHBOTH)){
    sm = NO;
  }
  else if((logdata[lset]->info.smooth.method == GAUSSIAN || logdata[lset]->info.smooth.method == LOWPASS) &&
           logdata[lset]->info.smooth.plot == SMOOTH){
    sm = YES;
  }

  realminlog = 9999.0;
  realmaxlog = -9999.0;
  ptrlog->avedepthstep = 0.0;
  for(i=0; i<ptrlog->data_num; ++i) {
    if(sm == YES) {
      if(ptrlog->value[i]->data != ptrlog->info.missing_value && ptrlog->value[i]->quality == GOOD &&
         ptrlog->value[i]->sm_data > realmaxlog) {
        realmaxlog = ptrlog->value[i]->sm_data;
      }
      else if(ptrlog->value[i]->data != ptrlog->info.missing_value && ptrlog->value[i]->quality == GOOD &&
              ptrlog->value[i]->sm_data < realminlog) {
        realminlog = ptrlog->value[i]->sm_data; 
      }
    }
    else {
      if(ptrlog->value[i]->data != ptrlog->info.missing_value && ptrlog->value[i]->quality == GOOD &&
         ptrlog->value[i]->data > realmaxlog) {
        realmaxlog = ptrlog->value[i]->data;
      }
      else if(ptrlog->value[i]->data != ptrlog->info.missing_value && ptrlog->value[i]->quality == GOOD &&
              ptrlog->value[i]->data < realminlog) {
        realminlog = ptrlog->value[i]->data;
      }
    }
    if(i < ptrlog->data_num - 1) {
      if(ptrlog->value[i]->data != ptrlog->info.missing_value && ptrlog->value[i]->quality == GOOD &&
         ptrlog->value[i + 1]->data != ptrlog->info.missing_value) {
        ptrlog->avedepthstep = ptrlog->avedepthstep +
                                        ptrlog->value[i+1]->depth - ptrlog->value[i]->depth;
      }
    }
  }
  ptrlog->avedepthstep = ptrlog->avedepthstep/ptrlog->data_num;
  /* round ave depth to nearest cm
  */
  Round(&ptrlog->avedepthstep, 0.01);

  if(ptrlog->info.cull.method == CULL_PARAMETERS) {
    if(ptrlog->info.cull.depmax <= ptrlog->value[0]->depth) {
      culldep = NO;
    }
    else if(ptrlog->info.cull.depmin <= ptrlog->value[0]->depth && ptrlog->info.cull.depmax >= ptrlog->value[ptrlog->data_num-1]->depth) {
      culldep = NO;
    }
    else {
      culldep = YES;
    }
  }
  else {
    culldep = NO;
  }

  ptrlog->info.depth_min = ptrlog->value[0]->depth;
  if(culldep == YES) {
    ptrlog->info.depth_max = ptrlog->info.cull.depmax;
  }
  else {
    ptrlog->info.depth_max = ptrlog->value[ptrlog->data_num-1]->depth;
  }
  err = LogStdDev(ptrlog, ptrlog->info.depth_max, &std, &mean);
  if(err == 0) {
    minlog = mean - (PLUSMINUSSTD * std);
    maxlog = mean + (PLUSMINUSSTD * std);
  }
  else {
    return 1;
  }    
  ptrlog->axis_min = minlog;
  ptrlog->axis_max = maxlog;

  if((have_cores == DO || have_ancill == DO || have_splice == DO) && maxdep < logdata[lset]->info.depth_max) {
    maxdep = logdata[lset]->info.depth_max;
  }
  else if(have_cores == DONT && have_ancill == DONT && have_splice == DONT) {
    maxdep = logdata[lset]->info.depth_max;
  }

  return 0;
} 

void SetupCalcEquivLogDepths(d)
int d;
{
  int l, h, c, e;
  float *pdep, aff_adj, tie1 ,tie2;
  void ApplyCoreAdjustToOneCore(), MakeHoleEqt();

  /* calculate the equiv log depths for the entire data set
  */

  MakeHoleEqt();
  for(h=0; h<data[d]->numholes; ++h) {
    for(c=0; c<data[d]->holes[h]->numcores; ++c) {
      if((pdep=(float*)calloc(data[d]->holes[h]->core[c]->numvalues, sizeof(float))) == NULL)
                                    SaBadMemAlloc("Sa");
      for(l=0; l<data[d]->holes[h]->core[c]->numvalues; ++l) {
        if(data[d]->holes[h]->core[c]->value[l]->quality  & BAD_SB_DEPTH) {
          pdep[l] = BADPT;
        }
        else {
          pdep[l] = data[d]->holes[h]->core[c]->value[l]->mbsf;
          if(have_affine == DO) {
            pdep[l] = pdep[l] + data[d]->holes[h]->core[c]->affine_offset;
          }
        }
      }
      if(eqlogdepth.autocorr_status == YES) {
        ApplyCoreAdjustToOneCore(pdep, data[d]->holes[h]->core[c]->numvalues);
      }
      e = data[d]->holes[h]->map_hole_to_eqlogdepth;
      if(eld.hole_eqt[e]->nties == 0 && eqlogdepth.autocorr_status == NO) {
        data[d]->holes[h]->core[c]->have_equil_log_depths = DONT;
      }
      else {
        if(eld.hole_eqt[e]->nties > 0) {
          CalcELDInHole(d, h, c, eld.hole_eqt[e], pdep);
        }
        data[d]->holes[h]->core[c]->have_equil_log_depths = DO;
        for(l=0; l<data[d]->holes[h]->core[c]->numvalues; ++l) {
          data[d]->holes[h]->core[c]->value[l]->eld = pdep[l];
        }
      }
      free(pdep);
      pdep = NULL;
    }
  }
}

/* calculate log equivanlent depths
   note; do this for all levels irregardless
   of value quality; a bad value will return
   a depth = BADPT
*/
int CalcEquivLogDepths(ndep, pdep, ptie)
int ndep;
Ties *ptie;
float *pdep;
{
  int i, k, n;
  float *tempdep, *tiecoredep, *tielogdep, *tiea, *tieb;

  /* check that have at least two ties
     if only one tie deal with it below as tails
  */
  if(ptie->nties < 1){
    return 1;
  }

  if((tiecoredep=(float*)calloc(ptie->nties,sizeof(float))) == NULL)
                     SaBadMemAlloc("SaCalcEquivLogDepths1");
  if((tielogdep=(float*)calloc(ptie->nties,sizeof(float))) == NULL)
                     SaBadMemAlloc("SaCalcEquivLogDepths2");
  if((tiea=(float*)calloc(ptie->nties,sizeof(float))) == NULL)
                    SaBadMemAlloc("SaCalcEquivLogDepths3");
  if((tieb=(float*)calloc(ptie->nties,sizeof(float))) == NULL)
                     SaBadMemAlloc("SaCalcEquivLogDepths4");

  for(i=0; i<ptie->nties; ++i) {
    if(eqlogdepth.autocorr_status == YES) {
      tiecoredep[i] = ptie->tie[i]->coreautoeld;
    }
    else if(have_affine == DO) {
      tiecoredep[i] = ptie->tie[i]->coremcd;
    }
    else {
      tiecoredep[i] = ptie->tie[i]->corembsf;
    }
    tielogdep[i] = ptie->tie[i]->logdep;
    tiea[i] = ptie->tie[i]->rate;
    tieb[i] = ptie->tie[i]->b;
  }
 
  if((tempdep=(float*)calloc(ndep,sizeof(float))) == NULL)
                     SaBadMemAlloc("SaCalcEquivLogDepths5");

  /*get rid of bad values by putting depth in tempdep
  */
  n = 0;
  for(i=0; i< ndep; ++i) {
    if(pdep[i] == BADPT) {
    }
    else {
      tempdep[n] = pdep[i];
      ++n;
    }
  }
  if(n < 1) {
    free(tempdep);
    tempdep = NULL;
    free(tiecoredep);
    tiecoredep = NULL;
    free(tielogdep);
    tielogdep = NULL;
    free(tiea);
    tiea = NULL;
    free(tieb);
    tieb = NULL;
    return 1;
  }

  /* for each section defined by two ties
     check to see that have at least two ties; otherwise only have tails
  */
  if(ptie->nties > 1) {
    k = 0;
    while(tempdep[k] < tiecoredep[0] && k < n) {
      tempdep[k] = tempdep[k] + tielogdep[0] - tiecoredep[0];
      ++k;
    }
    for(i=0; i<ptie->nties - 1; ++i){
      while(tempdep[k] >= tiecoredep[i] && tempdep[k] <= tiecoredep[i+1] && k < n) {
        tempdep[k] = tempdep[k] * tiea[i] + tieb[i];
        ++k;
      }
      if(k == n || tempdep[k] > tiecoredep[ptie->nties-1]) {
        break;
      }
    }
  }
  else if(ptie->nties == 1) {
    k = 0;
    while(k < n) {
      tempdep[k] = tempdep[k] + tielogdep[0] - tiecoredep[0];
      ++k;
    }
  }

  if(k < n-1){
    for(k=k; k<n; ++k) {
      tempdep[k] = tempdep[k] + tielogdep[ptie->nties-1] - tiecoredep[ptie->nties-1];
    }
  }

  /* put the tempdep into the pdep array
  */
  k = 0;
  for(i=0; i<ndep; ++i) {
    if(pdep[i] == BADPT) {
    }
    else {
      pdep[i]= tempdep[k];
      ++k;
    }
    if(k>=n) {
      break;
    }
  }

  /* free the tempdep
  */
  free(tempdep);
  tempdep = NULL;
  free(tiecoredep);
  tiecoredep = NULL;
  free(tielogdep);
  tielogdep = NULL;
  free(tiea);
  tiea = NULL;
  free(tieb);
  tieb = NULL;

  return 0;
}


int StdDev(x, n, off, win, stddev, xmean)
int n, off, win;
float *x;
double *stddev, *xmean;
{
  int i;
  float sum, sumsq, nx;

  if(win < 1 || off + win > n) {
    return 1;
  } 

  sum = 0.0;
  sumsq = 0.0;

  nx = 0.0;
  for(i=off; i<off+win; ++i) {
    if(x[i] != BADPT) {
      sum = sum + x[i];
      sumsq = sumsq + x[i] * x[i];
      ++nx;
    }
  } 
  if(nx < 2.0) {
    return 1;
  }

  *xmean = sum/nx;
 
  *stddev = (nx * sumsq - sum * sum)/(nx * (nx - 1));
  if(*stddev < 0.0) {
    *stddev = 0.0;
  }
  else {
    *stddev = sqrt(*stddev);
  }
    
  return 0;
}

int LogStdDev(log, max, stddev, xmean)
LogData  log;
float    max;
double   *stddev, *xmean;
{
  int i;
  float x, sum, sumsq, nx;
 
  if(log.data_num < 2) {
    return 1;
  }
 
  sum = 0.0;
  sumsq = 0.0;
 
  nx = 0.0;
  for(i=0; i<log.data_num; ++i) {
    if(log.value[i]->depth > max) {
      break;
    }
    else if(log.value[i]->data != log.info.missing_value && log.value[i]->quality == GOOD) {
      x = log.value[i]->data;
      sum = sum + x;
      sumsq = sumsq + x * x;
      ++nx;
    }
  }
 
  if(nx < 1.0) {
    return 1;
  }
  else {
    *xmean = sum/nx;
    *stddev = (nx * sumsq - sum * sum)/(nx * (nx - 1));
    if(*stddev < 0.0) {
      *stddev = 0.0;
    }
    else {
      *stddev = sqrt(*stddev);
    }
  }  
 
  return 0;
}

int MakeEquivLogDepthEquation(pties)
Ties *pties;
{
  int i;

  if(pties->nties > 1) {
    
    if(eqlogdepth.autocorr_status == YES) {
      for(i=0; i< pties->nties-1; ++i) {
        pties->tie[i]->rate = (pties->tie[i+1]->logdep - pties->tie[i]->logdep)/(pties->tie[i+1]->coreautoeld - pties->tie[i]->coreautoeld);
        pties->tie[i]->b = pties->tie[i]->logdep - (pties->tie[i]->coreautoeld * pties->tie[i]->rate);
      }
      pties->tie[pties->nties-1]->rate = 1.0;
      pties->tie[pties->nties-1]->b = pties->tie[pties->nties-1]->logdep - pties->tie[pties->nties-1]->coreautoeld;
    }
    else if(have_affine == DO) {
      for(i=0; i< pties->nties-1; ++i) {
        pties->tie[i]->rate = (pties->tie[i+1]->logdep - pties->tie[i]->logdep)/(pties->tie[i+1]->coremcd - pties->tie[i]->coremcd);
        pties->tie[i]->b = pties->tie[i]->logdep - (pties->tie[i]->coremcd * pties->tie[i]->rate);
      }
      pties->tie[pties->nties-1]->rate = 1.0;
      pties->tie[pties->nties-1]->b = pties->tie[pties->nties-1]->logdep - pties->tie[pties->nties-1]->coremcd;
    }
    else {
      for(i=0; i< pties->nties-1; ++i) {
        pties->tie[i]->rate = (pties->tie[i+1]->logdep - pties->tie[i]->logdep)/(pties->tie[i+1]->corembsf - pties->tie[i]->corembsf);
        pties->tie[i]->b = pties->tie[i]->logdep - (pties->tie[i]->corembsf * pties->tie[i]->rate);
      }
      pties->tie[pties->nties-1]->rate = 1.0;
      pties->tie[pties->nties-1]->b = pties->tie[pties->nties-1]->logdep - pties->tie[pties->nties-1]->corembsf;
    }
  }
}
 
/* puts the current tie in the the list
*/
int GetTieListPosition(temp, ds, ppos, pties)
int temp, ds, *ppos;
Ties *pties;
{
  int i, j;
 
  /* is this the first tie? then it is the current tie
  */
  if((have_coretotie == DO && pties->nties == 0) || (have_splice == DO && pties->nties == 1)) {

    if(have_coretotie == DO) {
      *ppos = 1;
    }
    else if(have_splice == DO) {
      *ppos = 2;
    }
    ++pties->nties;
    pties->tie[*ppos-1]->array = pties->coretie;
    pties->tie[*ppos-1]->log = pties->logtie;
    pties->tie[*ppos-1]->logdep = pties->tielogdep;
    pties->tie[*ppos-1]->corembsf = pties->tiecorembsf;
    pties->tie[*ppos-1]->coremcd = pties->tiecoremcd;
    pties->tie[*ppos-1]->coreautoeld = pties->tiecoreautoeld;
    pties->tie[*ppos-1]->equivlogdep = pties->tiecoreeqlogdep;
    pties->tie[*ppos-1]->temp_eld = pties->tiecoretempeqlogdep;
    pties->tie[*ppos-1]->rate = 1.0;
    pties->tie[*ppos-1]->b = 0.0;
    if(XmToggleButtonGetState(toggleButtonAddTieToAll) == True && have_affine == DO) {
      pties->tie[*ppos-1]->shared = YES;
    }
    else {
      pties->tie[*ppos-1]->shared = NO;
    }
    if(temp == YES) {
      pties->tie[*ppos-1]->topsect = 0.0;
      pties->tie[*ppos-1]->botsect = 0.0;
      (void)sprintf(pties->tie[*ppos-1]->coresection, "NA");
    }
    else if(temp == NO) {
      pties->tie[*ppos-1]->holename = pties->tiecoreholename;
      pties->tie[*ppos-1]->corename = pties->tiecorename;
      pties->tie[*ppos-1]->coretype = pties->tiecoretype;
      pties->tie[*ppos-1]->topsect = pties->tiecoretopsect;
      pties->tie[*ppos-1]->botsect = pties->tiecorebotsect;
      (void)sprintf(pties->tie[*ppos-1]->coresection, "%s", pties->tiecoresection);
    }
    return 0;
  }

  /* need to fit new tie in list
  */
  if((have_coretotie == DO && pties->nties >= 1) || (have_splice == DO && pties->nties >= 2)) {

    *ppos = -1;

    /* check that this tie does not conflict with others
       because working in the same core can use any type depth so use mbsf
    */
    for(i=0; i<pties->nties; ++i) {
      if(pties->tiecorembsf == pties->tie[i]->corembsf || pties->tielogdep == pties->tie[i]->logdep) {
        (void)sprintf(buffer, "This tie allready is in the list.");
        SaManageWarningBox(buffer);
        ClearTie(NULL, NULL, NULL);
        return 1;
      }
    }

    if(have_splice == DO) {
      j = 1;
    }
    else {
      j = 0;
    }
    if(pties->tiecoremcd < pties->tie[j]->coremcd) {
      if(pties->tielogdep > pties->tie[j]->logdep) {
        (void)sprintf(buffer, "This tie overlaps the one below it.");
        SaManageWarningBox(buffer);
        ClearTie(NULL, NULL, NULL);
        return 1;
      }
      else {
        *ppos = j + 1;
      }
    }
    else if(pties->tiecoremcd > pties->tie[pties->nties-1]->coremcd) {
      if(pties->tielogdep < pties->tie[pties->nties-1]->logdep) {
        (void)sprintf(buffer, "This tie overlaps the one above it.");
        SaManageWarningBox(buffer);
        ClearTie(NULL, NULL, NULL);
        return 1;
      }
      else {
        *ppos = pties->nties + 1;
      }
    }
    else if(pties->nties > 1 + j) {
      for(i=j; i<pties->nties-1; ++i) {
        if(pties->tiecoremcd > pties->tie[i]->coremcd && pties->tiecoremcd < pties->tie[i+1]->coremcd) {
          if((pties->tielogdep > pties->tie[i+1]->logdep) ||
             (pties->tielogdep < pties->tie[i]->logdep)) {
            (void)sprintf(buffer, "This tie overlaps another.");
            SaManageWarningBox(buffer);
            ClearTie(NULL, NULL, NULL);
            return 1;
          }
          else {
            *ppos = i + 2;
            break;
          }
        }  
      }
    }
    else {
      (void)sprintf(buffer, "GetTieListPos() checking new tie position in list; unthought of possibility??.");
      SaManageWarningBox(buffer);
      ClearTie(NULL, NULL, NULL);
      return 1;
    }
  }
  else {
    *ppos = 1;
  }

  if(*ppos == -1) {
    (void)sprintf(buffer, "checking tie; problem finding position in list.");
    SaManageWarningBox(buffer);
    ClearTie(NULL, NULL, NULL);
    return 1;
  }
  else {
    ++pties->nties;

    for(i=pties->nties-1; i>= *ppos; --i) {
      pties->tie[i]->holename = pties->tie[i-1]->holename;
      pties->tie[i]->corename = pties->tie[i-1]->corename;
      pties->tie[i]->coretype = pties->tie[i-1]->coretype;
      pties->tie[i]->array = pties->tie[i-1]->array;
      pties->tie[i]->log = pties->tie[i-1]->log;
      pties->tie[i]->logdep = pties->tie[i-1]->logdep;
      pties->tie[i]->corembsf = pties->tie[i-1]->corembsf;
      pties->tie[i]->coremcd = pties->tie[i-1]->coremcd;
      pties->tie[i]->coreautoeld = pties->tie[i-1]->coreautoeld;
      pties->tie[i]->equivlogdep = pties->tie[i-1]->equivlogdep;
      pties->tie[i]->temp_eld = pties->tie[i-1]->temp_eld;
      pties->tie[i]->topsect = pties->tie[i-1]->topsect;
      pties->tie[i]->botsect = pties->tie[i-1]->botsect;
      (void)sprintf(pties->tie[i]->coresection, "%s", pties->tie[i-1]->coresection);
      pties->tie[i]->rate = pties->tie[i-1]->rate; 
      pties->tie[i]->b = pties->tie[i-1]->b;
      pties->tie[i]->shared = pties->tie[i-1]->shared; 
    }
    pties->tie[*ppos-1]->array = pties->coretie;
    pties->tie[*ppos-1]->log = pties->logtie;
    pties->tie[*ppos-1]->logdep = pties->tielogdep;
    pties->tie[*ppos-1]->corembsf = pties->tiecorembsf;
    pties->tie[*ppos-1]->coremcd = pties->tiecoremcd;
    pties->tie[*ppos-1]->coreautoeld = pties->tiecoreautoeld;
    pties->tie[*ppos-1]->equivlogdep = pties->tiecoreeqlogdep;
    pties->tie[*ppos-1]->temp_eld = pties->tiecoretempeqlogdep;
    pties->tie[*ppos-1]->rate = 1.0; 
    pties->tie[*ppos-1]->b = 0.0;
    if(XmToggleButtonGetState(toggleButtonAddTieToAll) == True && have_affine == DO) {
      pties->tie[*ppos-1]->shared = YES;
    }
    else {
      pties->tie[*ppos-1]->shared = NO;
    }
    if(temp == YES) {
      pties->tie[*ppos-1]->topsect = 0.0;
      pties->tie[*ppos-1]->botsect = 0.0;
      (void)sprintf(pties->tie[*ppos-1]->coresection, "NA");
    }
    else if(temp == NO) {
      pties->tie[*ppos-1]->holename = pties->tiecoreholename;
      pties->tie[*ppos-1]->corename = pties->tiecorename;
      pties->tie[*ppos-1]->coretype = pties->tiecoretype;
      pties->tie[*ppos-1]->topsect = pties->tiecoretopsect;
      pties->tie[*ppos-1]->botsect = pties->tiecorebotsect;
      (void)sprintf(pties->tie[*ppos-1]->coresection, "%s", pties->tiecoresection);
    }
  }

  return 0;
}

int CheckTie(a, l, pties)
int a, l;
Ties *pties;
{
}

void GetTieArrayNum(who, pties, set, h, c, near)
int who;
TieList *pties;
int set;
int h, c;
int *near;
{
  int i, j, k, l, array;

  /* find the matching sample in the core
  */
  l = 0;
  *near = 0;
  if(who == SPLICEDATA) {
    pties->array = MISSING;
    while(l < splice[set]->numvalues) {
      if((fabs(pties->coremcd - splice[set]->value[l]->mcd)) < ROUNDCHECK) {
        pties->array = l;
        break;
      }
      else if(l == 0 && pties->coremcd < splice[set]->value[l]->mcd) {
        break;
      }
      else if(l == splice[set]->numvalues - 1 &&
                   pties->coremcd > splice[set]->value[l]->mcd) {
        *near = l;
        break;
      }
      else if(l < splice[set]->numvalues - 1) {
        if(pties->coremcd > splice[set]->value[l]->mcd &&
           pties->coremcd < splice[set]->value[l+1]->mcd) {
          pties->array = INTERP;
          *near = l;
          break;
        }
      }
      ++l;
    }
  }
  else if(who == COREDATA) {
    pties->array = MISSING;
    while(l < data[set]->holes[h]->core[c]->numvalues) {
      if((fabs(pties->coremcd - (data[set]->holes[h]->core[c]->value[l]->mbsf + data[set]->holes[h]->core[c]->affine_offset))) < ROUNDCHECK) {
        pties->array = l;
        break;
      }
      if(l < data[set]->holes[h]->core[c]->numvalues - 1) {
        if((fabs(pties->coremcd - (data[set]->holes[h]->core[c]->value[l+1]->mbsf + data[set]->holes[h]->core[c]->affine_offset))) < ROUNDCHECK) {
          pties->array = l + 1;
          break;
        }
      }
      if(l == 0 && pties->coremcd < (data[set]->holes[h]->core[c]->value[l]->mbsf + data[set]->holes[h]->core[c]->affine_offset)) {
        *near = l;
        break;
      }
      if(l == data[set]->holes[h]->core[c]->numvalues - 1 && 
                   pties->coremcd > (data[set]->holes[h]->core[c]->value[l]->mbsf + data[set]->holes[h]->core[c]->affine_offset)) {
        *near = l;
        break;
      }
      if(l < data[set]->holes[h]->core[c]->numvalues - 1) {
        if(pties->coremcd > (data[set]->holes[h]->core[c]->value[l]->mbsf + data[set]->holes[h]->core[c]->affine_offset) &&
           pties->coremcd < (data[set]->holes[h]->core[c]->value[l+1]->mbsf + data[set]->holes[h]->core[c]->affine_offset)) {
          pties->array = INTERP;
          *near = l;
          break;
        }
      }
      ++l;
    }
  }

  else if(who == LOGDATA) {
 
    /* find the matching sample in the log
    */
    pties->log = MISSING;
    l = 0;
    while(l < logdata[set]->data_num) {
      if((fabs(pties->logdep - logdata[set]->value[l]->depth)) < ROUNDCHECK) {
        pties->log = l;
        break;
      }
      else if(l == 0 && pties->logdep < logdata[set]->value[l]->depth) {
        break;
      }
      else if(l == logdata[set]->data_num - 1 &&
                   pties->logdep > logdata[set]->value[l]->depth) {
        *near = l;
        break;
      }
      else if(l < logdata[set]->data_num - 1) {
        if(pties->logdep > logdata[set]->value[l]->depth &&
           pties->logdep < logdata[set]->value[l+1]->depth) {
          pties->log = INTERP;
          *near = l;
          break;
        }
      }
      ++l;
    }
  }
}

void MakeTempties(pties)
Ties *pties;
{
  int i;

  pties->nties = ties->nties;
  pties->hole = ties->hole;
  pties->core = ties->core;
  pties->coretie = ties->coretie;
  pties->logtie = ties->logtie;
  pties->avedepthstep = ties->avedepthstep;
  pties->tiecorembsf = ties->tiecorembsf;
  pties->tiecoremcd = ties->tiecoremcd;
  pties->tiecoreautoeld = ties->tiecoreautoeld;
  pties->tielogdep = ties->tielogdep;
  pties->tiecoreeqlogdep = ties->tiecoreeqlogdep;
  pties->alloc_cnt = ties->alloc_cnt;

  for(i=0; i<ties->nties; ++i) {
    pties->tie[i]->holename = ties->tie[i]->holename; 
    pties->tie[i]->corename = ties->tie[i]->corename;
    (void)sprintf(pties->tie[i]->coresection, "%s", ties->tie[i]->coresection);
    pties->tie[i]->array = ties->tie[i]->array;
    pties->tie[i]->log = ties->tie[i]->log;
    pties->tie[i]->topsect = ties->tie[i]->topsect;
    pties->tie[i]->botsect = ties->tie[i]->botsect;
    pties->tie[i]->rate = ties->tie[i]->rate;
    pties->tie[i]->b = ties->tie[i]->b;
    pties->tie[i]->corembsf = ties->tie[i]->corembsf;
    pties->tie[i]->coremcd = ties->tie[i]->coremcd;
    pties->tie[i]->coreautoeld = ties->tie[i]->coreautoeld;
    pties->tie[i]->equivlogdep = ties->tie[i]->equivlogdep;
    pties->tie[i]->logdep = ties->tie[i]->logdep;
    pties->tie[i]->shared = ties->tie[i]->shared;
  }

  for(i=0; i<ties->nties; ++i) {
    pties->tie[i]->selected = ties->tie[i]->selected;
  }
}

void MakeTies(pties)
{
}

void MakeNewELDTable(eldptr, ds)
EqLogDepthTable   *eldptr;
int ds;
{
  int i, j;

  if(have_affine == DO) {
    eldptr->numhole = affinetable.numhole;
    eldptr->leg = affinetable.leg;
    eldptr->site = affinetable.site;
    eldptr->read_equil_log = NO;
    eldptr->autocorr_status = NO;
    for(i=0; i<affinetable.numhole; ++i) {
      eldptr->hole[i]->name = affinetable.hole[i]->name;
      eldptr->hole[i]->numcore = affinetable.hole[i]->numcore;
      for(j=0; j<affinetable.hole[i]->numcore; ++j) {
        eldptr->hole[i]->core[j]->name = affinetable.hole[i]->core[j]->name;
        eldptr->hole[i]->core[j]->type = affinetable.hole[i]->core[j]->type;
        eldptr->hole[i]->core[j]->affine_offset = affinetable.hole[i]->core[j]->cum_depth_offset;
        if(affinetable.hole[i]->core[j]->did_offset == True) {
          eldptr->hole[i]->core[j]->affine_status = YESAFFINE_DID;
        }
        else {
          eldptr->hole[i]->core[j]->affine_status = YESAFFINE_DIDNT;
        }
        eldptr->hole[i]->core[j]->did_eld = NO;
        eldptr->hole[i]->core[j]->nties = 0;
      }
    }
  }
  else {
    eldptr->numhole = data[ds]->numholes;
    eldptr->leg = data[ds]->leg;
    eldptr->site = data[ds]->site;
    eldptr->read_equil_log = NO;
    eldptr->autocorr_status = NO;
    for(i=0; i<data[ds]->numholes; ++i) {
      eldptr->hole[i]->name = data[ds]->holes[i]->name;
      eldptr->hole[i]->numcore = data[ds]->holes[i]->numcores;
      for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
        eldptr->hole[i]->core[j]->name = data[ds]->holes[i]->core[j]->num;
        eldptr->hole[i]->core[j]->type = data[ds]->holes[i]->core[j]->value[0]->type;
        eldptr->hole[i]->core[j]->affine_offset = 0.0;
        eldptr->hole[i]->core[j]->affine_status = YESAFFINE_DIDNT;
        eldptr->hole[i]->core[j]->did_eld = NO;
        eldptr->hole[i]->core[j]->nties = 0;
      }
    }
  }
}

void MapHoleToELDTable(eldptr, ds)
EqLogDepthTable *eldptr;
int ds;
{
  int i, k;

  for(i=0; i<data[ds]->numholes; ++i) {
    data[ds]->holes[i]->map_hole_to_eqlogdepth = -1;
  }
  for(k=0; k<eldptr->numhole; ++k) {
    eldptr->hole[k]->map_hole_to_data[ds] = -1;
  }

  for(i=0; i<data[ds]->numholes; ++i) {
    for(k=0; k<eldptr->numhole; ++k) {
      if(eldptr->hole[k]->name == data[ds]->holes[i]->name) {
        data[ds]->holes[i]->map_hole_to_eqlogdepth = k;
        eldptr->hole[k]->map_hole_to_data[ds] = i;
        break;
      }
    }
  }
}

void MapCoreToELDTable(eldptr, ds)
EqLogDepthTable *eldptr;
int ds;
{
  int i, j, k, l, n, nh, start, end;

  for(i=0; i<data[ds]->numholes; ++i) {
    for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
      data[ds]->holes[i]->core[j]->map_core_to_eqlogdepth = -1;
    }
  }
  for(i=0; i<eldptr->numhole; ++i) {
    for(k=0; k<eldptr->hole[i]->numcore; ++k) {
      eldptr->hole[i]->core[k]->map_core_to_data[ds] = -1;
      eldptr->hole[i]->core[k]->mindep = 9999.0;
      eldptr->hole[i]->core[k]->maxdep = -9999.0;
    }
  }

  for(i=0; i<data[ds]->numholes; ++i) {
    nh = data[ds]->holes[i]->map_hole_to_eqlogdepth;
    if(nh >= 0) {
      for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
        for(k=0; k<eldptr->hole[nh]->numcore; ++k) {
          if(data[ds]->holes[i]->core[j]->num == eldptr->hole[nh]->core[k]->name) {
            data[ds]->holes[i]->core[j]->map_core_to_eqlogdepth = k;
            eldptr->hole[nh]->core[k]->map_core_to_data[ds] = j;
            for(l=0; l<dataset_cnt; ++l) {
              start = 0;
              while(data[ds]->holes[i]->core[j]->value[start]->quality == BAD_SB_DEPTH && start < data[ds]->holes[i]->core[j]->numvalues) {
                ++start;
              }
              end = data[ds]->holes[i]->core[j]->numvalues - 1;
              while(data[ds]->holes[i]->core[j]->value[end]->quality == BAD_SB_DEPTH && start > 0) {
                --end;
              }
              if(start < data[ds]->holes[i]->core[j]->numvalues && end > 0 && start < end) {
                if(l == 0) {
                  eldptr->hole[nh]->core[k]->mindep = data[ds]->holes[i]->core[j]->value[start]->mbsf + 
                                                      data[ds]->holes[i]->core[j]->affine_offset;
                  eldptr->hole[nh]->core[k]->maxdep = data[ds]->holes[i]->core[j]->value[end]->mbsf + 
                                                      data[ds]->holes[i]->core[j]->affine_offset;
                }
                else {
                  if(data[ds]->holes[i]->core[j]->value[start]->mbsf + data[ds]->holes[i]->core[j]->affine_offset < 
                      eldptr->hole[nh]->core[k]->mindep) {
                    eldptr->hole[nh]->core[k]->mindep = data[ds]->holes[i]->core[j]->value[start]->mbsf + 
                                                        data[ds]->holes[i]->core[j]->affine_offset;
                  }
                  if(data[ds]->holes[i]->core[j]->value[end]->mbsf + data[ds]->holes[i]->core[j]->affine_offset  >
                      eldptr->hole[nh]->core[k]->maxdep) {
                    eldptr->hole[nh]->core[k]->maxdep = data[ds]->holes[i]->core[j]->value[end]->mbsf + 
                                                        data[ds]->holes[i]->core[j]->affine_offset;
                  }
                }
              }
              else {
              }
            }
            if(eldptr->hole[nh]->core[k]->did_eld  == YES && eldptr->hole[nh]->core[k]->nties > 0) {
              data[ds]->holes[i]->core[j]->have_tie_list = DO;
            }
            else {
              data[ds]->holes[i]->core[j]->have_tie_list = DONT;
            }
            break;
          }
        }
      }
    }
    else {
      data[ds]->holes[i]->core[j]->have_tie_list = DONT;
    }
  }
}

void CheckELDTableOrder(eldptr)
EqLogDepthTable *eldptr;
{
  int i, j;

  /* is the table in order?
  */
  /* by hole
  */
  if(eldptr->numhole > 1) {
    for(i=0; i<eldptr->numhole-1; ++i) {
      if(eldptr->hole[i]->name > eldptr->hole[i+1]->name) {
      }
    }
  }

  /* by core
  */
  for(i=0; i<eldptr->numhole-1; ++i) {
    if(eldptr->hole[i]->numcore > 1) {
      for(j=0; j<eldptr->hole[i]->numcore-1; ++j) {
        if(eldptr->hole[i]->core[j]->name > eldptr->hole[i]->core[j+1]->name) {
        }
      }
    }
  }
}

void CheckELDTableMissingHole(eldptr, ds)
EqLogDepthTable *eldptr;
int ds;
{
  int i, j, k, l, foundit, hold_alloc_cnt;
  float dep;

  /* check to see if there are more cores. if so realloc existing holes
  */
  hold_alloc_cnt = eqlogdep_core_alloc;
  if(eqlogdep_core_alloc < max_num_cores) {
    eqlogdep_core_alloc = max_num_cores;
    for(i=0; i<eldptr->numhole; ++i) {
      if((charptr=(char*)realloc((char*)eldptr->hole[i]->core,eqlogdep_core_alloc*sizeof(EqLogDepthCore*)))==NULL) {
                                SaBadMemAlloc("");
      }
      else {
        eldptr->hole[i]->core=(EqLogDepthCore**)charptr;
      }
      for(l=hold_alloc_cnt; l<eqlogdep_core_alloc; ++l) {
        if((eldptr->hole[i]->core[l]=(EqLogDepthCore*)calloc(1,sizeof(EqLogDepthCore)))==NULL)
                                SaBadMemAlloc("Sa");
      }
    }
  }

  /* are there missing holes in eqlogdepth table?
  */
  for(i=0; i<data[ds]->numholes; ++i) {
    foundit = NO;
    for(j=0; j<eldptr->numhole; ++j) {
      if(eldptr->hole[j]->name == data[ds]->holes[i]->name) {
        foundit = YES;
        /* was enough space allocated for this hole?
        */
        break;
      }
    }
    if(foundit == NO) {
      j = 0;
      if(eldptr->hole[eldptr->numhole-1]->name <= data[ds]->holes[i]->name) {
        j = eldptr->numhole;
      }
      else if(eldptr->hole[0]->name > data[ds]->holes[i]->name) {
        j = 0;
      }
      else {
        while(eldptr->hole[j]->name <= data[ds]->holes[i]->name && j < eldptr->numhole-1) {
          ++j;
        }
      }

      ++eldptr->numhole;
      ++eld.nholes;
      if((charptr=(char*)realloc((char*)eldptr->hole,eldptr->numhole*sizeof(EqLogDepthHole*)))==NULL) {
                                SaBadMemAlloc("Sa");
      }
      else {
        eldptr->hole=(EqLogDepthHole**)charptr;
      }
      if((charptr=(char*)realloc((char*)eld.hole_eqt,eld.nholes*sizeof(HoleELDEqt*)))==NULL) {
                                SaBadMemAlloc("Sa");
      }
      else {
        eld.hole_eqt=(HoleELDEqt**)charptr;
      }
      if((eldptr->hole[eldptr->numhole-1]=(EqLogDepthHole*)calloc(1,sizeof(EqLogDepthHole)))==NULL)
                                SaBadMemAlloc("Sa");
      if((eld.hole_eqt[eld.nholes-1]=(HoleELDEqt*)calloc(1,sizeof(HoleELDEqt)))==NULL)
                                SaBadMemAlloc("Sa");
      if((eldptr->hole[eldptr->numhole-1]->core=(EqLogDepthCore**)calloc(eqlogdep_core_alloc,sizeof(EqLogDepthCore*)))==NULL)
                                SaBadMemAlloc("Sa");
      for(k=0; k<eqlogdep_core_alloc; ++k) {
        if((eldptr->hole[eldptr->numhole-1]->core[k]=(EqLogDepthCore*)calloc(1,sizeof(EqLogDepthCore)))==NULL)
                                SaBadMemAlloc("Sa");
      }
      for(k=eldptr->numhole-1; k > j; --k) {
        *eldptr->hole[k] = *eldptr->hole[k-1];
        eld.hole_eqt[k] = eld.hole_eqt[k-1];
      }
      eld.hole_eqt[j]->nties = 0;
      eldptr->hole[j]->numcore = data[ds]->holes[i]->numcores;
      eldptr->hole[j]->name = data[ds]->holes[i]->name;
      for(k=0; k<data[ds]->holes[i]->numcores; ++k) {
        eldptr->hole[j]->core[k]->name = data[ds]->holes[i]->core[k]->num;
        eldptr->hole[j]->core[k]->type = data[ds]->holes[i]->core[k]->value[0]->type;
        eldptr->hole[j]->core[k]->affine_offset = 0.0;
        eldptr->hole[j]->core[k]->affine_status = YESAFFINE_DIDNT;
        if(have_affine == DO) {
          (void)sprintf(buffer, "Warning: The affine table is missing hole %c ", eldptr->hole[j]->name);
          SaManageWarningBox(buffer);
        }
        eldptr->hole[j]->core[k]->did_eld = NO;
        eldptr->hole[j]->core[k]->nties = 0;
      }
    }
  }
}

void CheckELDTableMissingCore(eldptr, ds)
EqLogDepthTable *eldptr;
int ds;
{
  int i, j, k, m, nh, check;
  float dep;

  /* within a hole are there missing cores in equiv log depth table?
     note that should not have to check for enough allocated cores
     because allocated the largest possible number of cores in
     CheckELDTableMissingHole()
  */
  check = NO;
  for(i=0; i<eldptr->numhole; ++i) {
    k = 0;
    j = 0;
    nh = eldptr->hole[i]->map_hole_to_data[ds];
    if(nh >= 0 && nh < data[ds]->numholes) {
      while(k < data[ds]->holes[nh]->numcores && nh >= 0) {
        if(data[ds]->holes[nh]->core[k]->num == eldptr->hole[i]->core[j]->name) {
          if(j+1 < eldptr->hole[i]->numcore) {
            ++j;
          }
          ++k;
        }
        else if(data[ds]->holes[nh]->core[k]->num > eldptr->hole[i]->core[eldptr->hole[i]->numcore-1]->name) {
          /*add to end*/
          ++eldptr->hole[i]->numcore;
          j = eldptr->hole[i]->numcore - 1;
          eldptr->hole[i]->core[j]->name = data[ds]->holes[nh]->core[k]->num;
          eldptr->hole[i]->core[j]->type = data[ds]->holes[nh]->core[k]->value[0]->type;
          eldptr->hole[i]->core[j]->affine_offset = 0.0;
          eldptr->hole[i]->core[j]->affine_status = YESAFFINE_DIDNT;
          eldptr->hole[i]->core[j]->did_eld = NO;
          eldptr->hole[i]->core[j]->nties = 0;
          ++k;
          check = YES;
        }
        else if(data[ds]->holes[nh]->core[k]->num > eldptr->hole[i]->core[j]->name) {
          if(j+1 < eldptr->hole[i]->numcore) {
            ++j;
          }
        }
        else if(data[ds]->holes[nh]->core[k]->num < eldptr->hole[i]->core[j]->name) {
          /*insert new core into table*/
          ++eldptr->hole[i]->numcore;
          for(m=eldptr->hole[i]->numcore-1; m > j; --m) {
            eldptr->hole[i]->core[m] = eldptr->hole[i]->core[m-1];
          }
          eldptr->hole[i]->core[j]->name = data[ds]->holes[nh]->core[k]->num;
          eldptr->hole[i]->core[j]->type = data[ds]->holes[nh]->core[k]->value[0]->type;
          eldptr->hole[i]->core[j]->affine_offset = 0.0;
          eldptr->hole[i]->core[j]->affine_status = YESAFFINE_DIDNT;
          eldptr->hole[i]->core[j]->did_eld = NO;
          eldptr->hole[i]->core[j]->nties = 0;
          ++k;
          check = YES;
          if(j+1 < eldptr->hole[i]->numcore) {
            ++j;
          }
        }
      }
    }
    else {
    }
  }
  if(check == YES) {
    if(have_affine == DO) {
      (void)sprintf(buffer, "Warning: The affine table is cores");
      SaManageWarningBox(buffer);
    }
  }
}

int InterpolateCoreTiePtValue(ds, h, c, sm, dep, val, place)
int ds, h, c, sm, *place;
float dep, *val;
{
  int i, j, k, n;
  float coredepth1, corevalue1, coredepth2, corevalue2;

  /* depth type not important
  */

  n = data[ds]->holes[h]->core[c]->numgood;
  if(n < 2) {
    return 1;
  }
  *place = -1;
  if(dep < data[ds]->holes[h]->core[c]->value[data[ds]->holes[h]->core[c]->good[0]]->mbsf ||
     dep > data[ds]->holes[h]->core[c]->value[data[ds]->holes[h]->core[c]->good[n-1]]->mbsf) {
    return 1;
  }
  for(i=0; i<n-1; ++i) {
    j = data[ds]->holes[h]->core[c]->good[i];
    k = data[ds]->holes[h]->core[c]->good[i+1];
    coredepth1 = data[ds]->holes[h]->core[c]->value[j]->mbsf;
    coredepth2 = data[ds]->holes[h]->core[c]->value[k]->mbsf;
    if(dep >= coredepth1 && dep <= coredepth2) {
      if(sm == YES) {
        corevalue1 = data[ds]->holes[h]->core[c]->value[j]->sm_data;;
        corevalue2 = data[ds]->holes[h]->core[c]->value[k]->sm_data;;
      }
      else if(sm == NO) {
        corevalue1 = *data[ds]->holes[h]->core[c]->value[j]->data;
        corevalue2 = *data[ds]->holes[h]->core[c]->value[k]->data;
      }
      else {
        return 1;
      }
      *val = corevalue1 + (corevalue2 - corevalue1)/(coredepth2 - coredepth1) * (dep - coredepth1);
      *place = j;
      break;
    }
  }
  if(*place != -1) {
    return 0;
  }
  return 1;
}

int InterpolateLogTiePtValue(ls, sm, dep, val, place)
int sm;
float dep, *val;
int   *place;
{
  int i, logn;
  float dep1, dep2, dat1, dat2, *logdep, *logvar;

  if((logdep=(float*)calloc(logdata[ls]->data_num, sizeof(float))) == NULL)
             SaBadMemAlloc("Sa");
  if((logvar=(float*)calloc(logdata[ls]->data_num, sizeof(float))) == NULL)
             SaBadMemAlloc("Sa");
  logn = 0;
  for(i=0; i<logdata[ls]->data_num; ++i) {
    if(logdata[ls]->value[i]->data == logdata[ls]->info.missing_value) {
    }
    else if(logdata[ls]->value[i]->quality != GOOD) {
    }
    else {
      logdep[logn] = logdata[ls]->value[i]->depth;
      if(sm == YES) {
        logvar[logn] = logdata[ls]->value[i]->sm_data;
      }
      else {
        logvar[logn] = logdata[ls]->value[i]->data;
      }
      ++logn;
    }
  }

  if(logn < 2) {
    *place = MISSING;
  }
  else if(dep < logdep[0] || dep > logdep[logn-1]) {
    *place = MISSING;
  }
  else {
    for(i=0; i<logn-1; ++i) {
      if(dep == logdep[i]) {
        *place = i;
        *val = logvar[i];

        free(logdep);
        logdep = NULL;
        free(logvar);
        logvar = NULL;

        return 0;
      }
      else if(dep > logdep[i] && dep < logdep[i+1]) {
        dep1 = logdep[i];
        dep2 = logdep[i+1];
        dat1 = logvar[i];
        dat2 = logvar[i+1];
        *val = dat1 + (dat2 - dat1)/(dep2 - dep1) * (dep - dep1);

        free(logdep);
        logdep = NULL;
        free(logvar);
        logvar = NULL;

        *place = INTERP;
        return 0;
      }
      else if(i + 1 == logn - 1 && dep == logdep[i+1]) { 
        *place = i + 1;
        *val = logvar[i+1];

        free(logdep);
        logdep = NULL;
        free(logvar);
        logvar = NULL;

        return 0;
      }
    }
  }

  free(logdep);
  logdep = NULL;
  free(logvar);
  logvar = NULL;

  *place = MISSING;
  return 1;
}

void AdjustToTies(set)
int set;
{
  int i, n, h, c, err;
  float *pdep, dep;
  int MakeEquivLogDepthEquation(), CalcEquivLogDepths();
  void ApplyCoreAdjustToOneCore();

  if(ties->nties > 0) {

    MakeEquivLogDepthEquation(ties);

    if(have_coretotie == DO) {
      h = ties->hole;
      c = ties->core;
      n = data[set]->holes[h]->core[c]->numvalues;
      if((pdep=(float*)calloc(data[set]->holes[h]->core[c]->numvalues, sizeof(float))) == NULL)
                                    SaBadMemAlloc("SaAdjustToTies1");
      for(i=0; i<n; ++i) {
        if(data[set]->holes[h]->core[c]->value[i]->quality  & BAD_SB_DEPTH) {
          pdep[i] = BADPT;
        }
        else {
          pdep[i] = ties->coredepth[i];
        }
      }
    }
    else if(have_splice == DO && set >= 0 && set < splice_cnt) {
      if((pdep=(float*)calloc(splice[set]->numvalues, sizeof(float))) == NULL)
                                    SaBadMemAlloc("SaAdjustToTies2");
      n = splice[set]->numvalues;
      for(i=0; i<n; ++i) {
        if(splice[set]->value[i]->quality  & BAD_SB_DEPTH) {
          pdep[i] = BADPT;
        }
        else {
          pdep[i] = splice[set]->value[i]->mcd;
        }
      }
      if(eqlogdepth.autocorr_status == YES) {
        ApplyCoreAdjustToOneCore(pdep, splice[set]->numvalues);
      }
    }

    err = CalcEquivLogDepths(n, pdep, ties);
    if(err < 1) {
      for(i=0; i<n; ++i) {
        ties->temp_equivlogdep[i] = pdep[i];
      }
    }
    free(pdep);
    pdep = NULL;

    /* assign equiv log depth for the ties
    */
    for(i=0; i<ties->nties; ++i) {
      if(eqlogdepth.autocorr_status == YES) {
        dep = ties->tie[i]->coreautoeld;
      }
      else if(have_affine == DO) {
        dep = ties->tie[i]->coremcd;
      }
      else {
        dep = ties->tie[i]->corembsf;
      }
      err = CalcEquivLogDepths(1, &dep, ties);
      if(err < 1) {
        ties->tie[i]->temp_eld = dep;
      }
      else {
        ties->tie[i]->temp_eld = 0.0;
      }
    }

    have_equivlogdep = YES;
    XtSetSensitive(pushButtonSaveChanges, True);
  }

  else {
  }
}

int SaAutoCoreAdjust(ds, ls)
int ds, ls;
{
  int i, j, k, l, n, r, c, m, all_cnt, sm, flip, flip_off, coren, logn, err, nwin,
      off, coreoffset, logoffset, row, col, lastc, lastcn, start, end, count, *position, xoff;
  char *string;
  String str;
  XmString xmstring;
  float *coredep, *corevar, *corevarsm, *logdep, *logdepoff, *logvar, depstep, squish, offset, aff_offset,
        checklow, checkhigh, rate, b, rcoef, hold, startlogdep, endlogdep;
  double check, fract, ip;
  Cursor cursor;
  XEvent event;
  int AutoCoreAdjAxesAndGrid(), GetCoreToLogDepths(), Correlate2(), RepickDepth();
  void AutoCoreAdjustDrawBox(), RankCorr();
  float mark, fred, redinc, fgreen, greeninc, fblue, blueinc;
  XColor xcolor;
  int red, green, blue;
  Pixel pixel;
  int GetColorPixelValue(), MakeBoxPixel();
  void Round(), MakeRedGreenBlue();

  int sx, sy, sdx, sdy;
  float perpix;

  autocoreadj.draw_results = NO;
  if(corr_rank.n > 0) {
    free(corr_rank.r);
    corr_rank.r = NULL;
    free(corr_rank.box);
    corr_rank.box = NULL;
    corr_rank.n = 0;
  }

  if(have_cores == DONT || have_log == DONT) {
    return 1;
  }

  /* get the selected hole(s) from the list
  */
  if(!XmListGetSelectedPos(listAutoCorrHoleSelect, &position, &count)) {
    (void)sprintf(buffer, "Error: no hole(s) selected.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(count < 1) {
    (void)sprintf(buffer, "Error: no hole or splice selected.");
    SaManageWarningBox(buffer);
    XtFree(position);
    return 1;
  }
  else if(have_splice == DONT && have_cores == DO) {
    if(count > data[ds]->numholes) {
      (void)sprintf(buffer, "Error: hole selection exceeds number of holes!?.");
      SaManageWarningBox(buffer);
      XtFree(position);
      return 1;
    } 
  }

  /* clear the result widgets
  */
  XmTextSetString(textFieldAutoCorrResultOffset, "");
  XmTextSetString(textFieldAutoCorrResultSquish, "");
  xmstring = XmStringCreateSimple("");
  XtVaSetValues(labelAutoCorrBestCorr, XmNlabelString, xmstring, NULL);
  XmStringFree(xmstring);

  /* get the depth step, range and increments of squish and offset
     note that offset inc will be mult of depth step
  */
  str = XmTextGetString(textFieldAutoCorrelateDepthStep);
  depstep = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldAutoCorrSquish1);
  autocoreadj.range_squish[0] = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldAutoCorrSquish2);
  autocoreadj.range_squish[1] = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldAutoCorrSquishInterval);
  autocoreadj.squish_inc = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldAutoCorrOffset1);
  autocoreadj.range_offset[0] = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldAutoCorrOffset2);
  autocoreadj.range_offset[1] = atof(str);
  XtFree(str);
  str = XmTextGetString(textFieldAutoCorrOffsetInterval);
  autocoreadj.offset_inc = atof(str);
  XtFree(str);

  if(autocoreadj.range_squish[0] < autocoreadj.range_squish[1]) {
    if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    hold = autocoreadj.range_squish[0];
    autocoreadj.range_squish[0] = autocoreadj.range_squish[1];
    autocoreadj.range_squish[1] = hold;
    (void)sprintf(string, "%.2f", autocoreadj.range_squish[0]);
    XtVaSetValues(textFieldAutoCorrSquish1, XmNvalue, string, NULL);
    (void)sprintf(string, "%.2f", autocoreadj.range_squish[1]);
    XtVaSetValues(textFieldAutoCorrSquish2, XmNvalue, string, NULL);
    free(string);
    string = NULL;
  }
  if(autocoreadj.range_offset[0] > autocoreadj.range_offset[1]) {
    if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    hold = autocoreadj.range_offset[0];
    autocoreadj.range_offset[0] = autocoreadj.range_offset[1];
    autocoreadj.range_offset[1] = hold;
    (void)sprintf(string, "%.2f", autocoreadj.range_offset[0]);
    XtVaSetValues(textFieldAutoCorrOffset1, XmNvalue, string, NULL);
    (void)sprintf(string, "%.2f", autocoreadj.range_offset[1]);
    XtVaSetValues(textFieldAutoCorrOffset2, XmNvalue, string, NULL);
    free(string);
    string = NULL;
  }
  if(have_affine == DO && autocoreadj.range_squish[0] == autocoreadj.range_squish[1]) {
    (void)sprintf(buffer, "Error; the stretch/compress has no range.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(autocoreadj.range_squish[0] <= 0.0 || autocoreadj.range_squish[1] <= 0.0) {
    (void)sprintf(buffer, "Error; the stretch/compress has negative value(s).");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(autocoreadj.range_offset[0] == autocoreadj.range_offset[1]) {
    (void)sprintf(buffer, "Error; the depth offset has no range.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(autocoreadj.range_offset[0] == -0.0) {
    if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    autocoreadj.range_offset[0] = 0.0;
    (void)sprintf(string, "%.2f", autocoreadj.range_offset[0]);
    XtVaSetValues(textFieldAutoCorrOffset1, XmNvalue, string, NULL);
    free(string);
    string = NULL;
  }
  if(autocoreadj.range_offset[1] == -0.0) {
    if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    autocoreadj.range_offset[1] = 0.0;
    (void)sprintf(string, "%.2f", autocoreadj.range_offset[1]);
    XtVaSetValues(textFieldAutoCorrOffset2, XmNvalue, string, NULL);
    free(string);
    string = NULL;
  }
  if(have_affine == DO && autocoreadj.squish_inc <= 0.0) {
    (void)sprintf(buffer, "Error; the stretch/compress increment is <= 0.0.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(autocoreadj.offset_inc <= 0.0) {
    (void)sprintf(buffer, "Error; the depth offset increment is <= 0.0.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(autocoreadj.offset_inc < depstep) {
    (void)sprintf(buffer, "Error; the depth offset is less than the depth step.");
    SaManageWarningBox(buffer);
    return 1;
  }
  check = autocoreadj.offset_inc/depstep;
  fract = modf(check, &ip);
  if(fract == 0.0) {
  }
  else if(fract > 0.00001 || fract < 0.99999){
    (void)sprintf(buffer, "Error: the offset increment is not a multiple of the depth step.");
    SaManageWarningBox(buffer);
    return 1;
  }

  /* calc the number of rows and columns and the total number of boxes.
     if the inc does not divide evenly into the range re-calc the range
  */
  check = autocoreadj.range_offset[0]/autocoreadj.offset_inc;
  fract = modf(check, &ip);
  if(fract != 0.0){
    autocoreadj.range_offset[0] = (ip * autocoreadj.offset_inc) - autocoreadj.offset_inc;
    if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    (void)sprintf(string, "%.2f", autocoreadj.range_offset[0]);
    XtVaSetValues(textFieldAutoCorrOffset1, XmNvalue, string, NULL);
    free(string);
    string = NULL;
  }
  check = autocoreadj.range_offset[1]/autocoreadj.offset_inc;
  fract = modf(check, &ip);
  if(fract != 0.0){
    autocoreadj.range_offset[1] = (ip * autocoreadj.offset_inc) + autocoreadj.offset_inc;
    if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    (void)sprintf(string, "%.2f", autocoreadj.range_offset[1]);
    XtVaSetValues(textFieldAutoCorrOffset2, XmNvalue, string, NULL);
    free(string);
    string = NULL;
  }
  check = (autocoreadj.range_offset[1] - autocoreadj.range_offset[0])/autocoreadj.offset_inc + 1;
  fract = modf(check, &ip);
  if(fabs(fract) < 0.0001) {
    autocoreadj.noffset = check;
  }
  else if(fract > 0.9999) {
    autocoreadj.noffset = check + 1;
  }
  else {
    (void)sprintf(buffer, "Error: the offset is not divisable by inc. could not re-calc. the range.");
    SaManageWarningBox(buffer);
    return 1;
  }
  if(have_affine == DO) {
    check = (autocoreadj.range_squish[0] - autocoreadj.range_squish[1])/autocoreadj.squish_inc + 1;
    fract = modf(check, &ip);
    if(fract != 0.0){
      autocoreadj.nsquish = ip + 1;
      autocoreadj.range_squish[1] = autocoreadj.range_squish[0] - ((ip -1.0) * autocoreadj.squish_inc) - autocoreadj.squish_inc;
      if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
      (void)sprintf(string, "%.2f", autocoreadj.range_squish[1]);
      XtVaSetValues(textFieldAutoCorrSquish2, XmNvalue, string, NULL);
      free(string);
      string = NULL;
    }
    else {
      autocoreadj.nsquish = check;
    }
  }
  else {
    autocoreadj.nsquish = 1;
  }

  if(autocoreadj.nsquish <= 0 || autocoreadj.noffset <= 0) {
    (void)sprintf(buffer, "Error: could not calc number of boxes.");
    SaManageWarningBox(buffer);
    return 1;
  }
  autocoreadj.nboxes = autocoreadj.nsquish * autocoreadj.noffset;
  autocoreadj.nhole = data[ds]->numholes;

  off = 50;
  err = AutoCoreAdjAxesAndGrid(drawingAreaAutoCorrelate, XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate),
                         gc_drawingAreaAutoCorrelate, 50, 80);
  if(err > 0) {
    (void)sprintf(buffer,"Can't draw the boxes. size too small.");
    SaManageWarningBox(buffer);
    return;
  }
  XmUpdateDisplay(drawingAreaAutoCorrelate);

  xmstring = XmStringCreateSimple("Working:  setup");
  XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
  XmStringFree(xmstring);

  /* make a blank table for results
  */
  if((autocoreadj.box=(Box**)calloc(autocoreadj.nboxes, sizeof(Box*))) == NULL)
                    SaBadMemAlloc("Sa");
  for(i=0; i<autocoreadj.nboxes; ++i) {
    if((autocoreadj.box[i]=(Box*)calloc(1, sizeof(Box))) == NULL) {
                    SaBadMemAlloc("Sa");
    }
    if((autocoreadj.box[i]->r=(float*)calloc(autocoreadj.nhole, sizeof(float))) == NULL) {
             SaBadMemAlloc("Sa");
    }
  }
  autocoreadj.is_allocated = YES;

  cursor = XCreateFontCursor(XtDisplay(formAutoCorrelateParams), XC_watch);
  if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
    XDefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams), cursor);
  } 

  if(XmToggleButtonGetState(toggleButtonAutoCorrelateFlipYes) == True) {
    flip = -1.0;
    if(have_splice == DO) {
      flip_off = splice[sset]->max - splice[sset]->min;
    }
    else {
      flip_off = maxvar - minvar;
    }
  }
  else {
    flip = 1.0;
    flip_off = 0.0;
  }

  autocoreadj.boxes_so_far = 0;
  autocoreadj.in_progress = YES;
  off = 50;
  AutoCoreAdjAxesAndGrid(drawingAreaAutoCorrelate, XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate),
                         gc_drawingAreaAutoCorrelate, 50, 80);
  XmUpdateDisplay(drawingAreaAutoCorrelate);

  if(smooth.method == GAUSSIAN && (smooth.plot == SMOOTH || smooth.plot == SMOOTHBOTH)) {
    sm = YES;
  }
  else {
    sm = NO;
  }

  /* for each selected hole
  */
  for(m=0; m<count; ++m) {

    i = position[m] - 1;

    if((string=(char*)calloc(100, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    if(have_splice == DO) {
      (void)sprintf(string, "%s<->%s  Working; Preparing Log for Splice", type[data[dset]->type], logdata[lset]->info.id[0]);
    }
    else {
      (void)sprintf(string, "%s<->%s  Working; Preparing Log for hole %c", type[data[dset]->type], logdata[lset]->info.id[0],
                     data[ds]->holes[i]->name);
    }
    xmstring = XmStringCreateSimple(string);
    free(string);
    string = NULL;
    XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);

    /* check if the interupt button was pushed
    */
    XmUpdateDisplay(formAutoCorrelate);
    while(XCheckMaskEvent(XtDisplay(formAutoCorrelate), ButtonPressMask, &event)) {
      if(event.xbutton.window == XtWindow(pushButtonAutoCorrInterupt)) {
        if(cursor) {
          XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
          XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
        }
        XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
        autocoreadj.in_progress = NO;
        autocoreadj.draw_results = NO;
        log_clip_what = NONE;
        DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                log_clip_what, 0.0, NO);
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
        XtFree(position);
        return 1;
      }
    }

    k = 0;
    squish = autocoreadj.range_squish[0];
    offset = autocoreadj.range_offset[0];
    autocoreadj.boxes_so_far = 0;

    /* get the log segment for all possible overlays for this hole
       and repick at comstant depth step
    */
    if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
      checklow = splice[sset]->value[0]->mcd;
      checkhigh = splice[sset]->value[splice[sset]->numvalues-1]->mcd * (squish/100.0);
    }
    else {
      lastc = data[ds]->holes[i]->numcores;
      if(have_affine == DO) {
        while(lastc > 0 && data[ds]->holes[i]->core[lastc-1]->did_affine_offset == NO) {
          --lastc;
        }
      }
      if(lastc < 1) {
        if(cursor) {
          XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
          XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
        }
        XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
        autocoreadj.in_progress = NO;
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
        XtFree(position);
        return 1;
      }
      lastcn = data[ds]->holes[i]->core[lastc-1]->numvalues;
      checklow = data[ds]->holes[i]->core[0]->value[0]->mbsf + data[ds]->holes[i]->core[0]->affine_offset;
      checkhigh = (data[ds]->holes[i]->core[lastc-1]->value[lastcn-1]->mbsf + 
                   data[ds]->holes[i]->core[lastc-1]->affine_offset) * (squish/100.0);
    }
    n = 0;
    end = 0;
    start = 0;
    for(j=0; j<logdata[ls]->data_num; ++j) {
      if(logdata[ls]->value[j]->depth + autocoreadj.range_offset[0] > checkhigh) {
        end = j - 1;
        break;
      }
      else if(logdata[ls]->value[j]->depth + autocoreadj.range_offset[0] <= checkhigh) {
        if(n == 0) {
          start = j;
        }
        if(j == logdata[ls]->data_num -1) {
          end = j;
        }
        ++n;
      }
    }
    if(n <= 0 || end <= start || (end - start + 1) != n) {
      (void)sprintf(buffer, "Error; could not get overlapping segment of log.");
      SaManageWarningBox(buffer);
      if(cursor) {
        XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
        XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
      }
      XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
      autocoreadj.in_progress = NO;
      xmstring = XmStringCreateSimple("");
      XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
      XmStringFree(xmstring);
      XtFree(position);
      return 1;
    }

    /* check if the interupt button was pushed
    */
    XmUpdateDisplay(formAutoCorrelate);
    while(XCheckMaskEvent(XtDisplay(formAutoCorrelate), ButtonPressMask, &event)) {
      if(event.xbutton.window == XtWindow(pushButtonAutoCorrInterupt)) {
        if(cursor) {
          XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
          XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
        }
        XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
        autocoreadj.in_progress = NO;
        autocoreadj.draw_results = NO;
        log_clip_what = NONE;
        DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                log_clip_what, 0.0, NO);
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
        XtFree(position);
        return 1;
      }
    }

    all_cnt = (logdata[ls]->value[end]->depth - logdata[ls]->value[0]->depth)/depstep;
    if(n > all_cnt) {
      all_cnt = n;
    }
    if((logdep=(float*)calloc(all_cnt+100, sizeof(float))) == NULL)
             SaBadMemAlloc("Sa");
    if((logdepoff=(float*)calloc(all_cnt+100, sizeof(float))) == NULL)
             SaBadMemAlloc("Sa");
    if((logvar=(float*)calloc(all_cnt+100, sizeof(float))) == NULL)
             SaBadMemAlloc("Sa");
    logn = 0;
    for(j=start; j<end; ++j) {
      if(logdata[ls]->value[j]->data == logdata[ls]->info.missing_value) {
      }
      else if(logdata[ls]->value[j]->quality != GOOD) {
      }
      else {
        logdep[logn] = logdata[ls]->value[j]->depth;
        logvar[logn] = logdata[ls]->value[j]->data;
        ++logn;
      }
    }

    /* check if the interupt button was pushed
    */
    XmUpdateDisplay(formAutoCorrelate);
    while(XCheckMaskEvent(XtDisplay(formAutoCorrelate), ButtonPressMask, &event)) {
      if(event.xbutton.window == XtWindow(pushButtonAutoCorrInterupt)) {
        if(cursor) {
          XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
          XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
        }
        XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
        autocoreadj.in_progress = NO;
        autocoreadj.draw_results = NO;
        log_clip_what = NONE;
        DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                log_clip_what, 0.0, NO);
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
        XtFree(position);
        return 1;
      }
    }

    err = RepickDepth(all_cnt, logdep, logvar, &logn, depstep);
    if(err > 0) {
      if(err == 1) {
        (void)sprintf(buffer, "Error; repicking log depths. results would exceed alloc memory.");
        SaManageWarningBox(buffer);
      }
      free(logdep);
      logdep = NULL;
      free(logdepoff);
      logdepoff = NULL;
      free(logvar);
      logvar = NULL;
      if(cursor) {
        XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
        XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
      }
      XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
      autocoreadj.in_progress = NO;
      xmstring = XmStringCreateSimple("");
      XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
      XmStringFree(xmstring);
      XtFree(position);
      return 1;
    }

    if((string=(char*)calloc(100, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
    if(have_splice == DO) {
      (void)sprintf(string, "%s<->%s  Working", type[data[dset]->type], logdata[lset]->info.id[0]);
    }
    else {
      (void)sprintf(string, "%s<->%s  Working;   hole %c", type[data[dset]->type], logdata[lset]->info.id[0], 
                      data[ds]->holes[i]->name);
    }
    xmstring = XmStringCreateSimple(string);
    free(string);
    string = NULL;
    XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);

    /* loop thru the rows; the offsets per each squish
    */
    for(r=0; r<autocoreadj.nsquish; ++r) {

      /* check if the interupt button was pushed
      */
      XmUpdateDisplay(formAutoCorrelate);
      while(XCheckMaskEvent(XtDisplay(formAutoCorrelate), ButtonPressMask, &event)) {
        if(event.xbutton.window == XtWindow(pushButtonAutoCorrInterupt)) {
          free(logdep);
          logdep = NULL;
          free(logdepoff);
          logdepoff = NULL;
          free(logvar);
          logvar = NULL;
          if(cursor) {
            XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
            XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
          }
          XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
          autocoreadj.in_progress = NO;
          autocoreadj.draw_results = NO;
          log_clip_what = NONE;
          DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                  log_clip_what, 0.0, NO);
          xmstring = XmStringCreateSimple("");
          XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          XtFree(position);
          return 1;
        }
      }

      offset = autocoreadj.range_offset[0];
      /* add the min offset to the log depth
      */
      for(l=0; l<logn; ++l) {
        logdepoff[l] = logdep[l] + offset;
      }

      /* make a continuous series of this hole
         Only use cores that have a value of did_it in affine table
      */
      /* alloc memory for it
      */
      all_cnt = (checkhigh - checklow)/depstep + 100; 
      if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
        n = splice[sset]->numvalues;
      }
      else {
        n = 0;
        for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
          n = n + data[ds]->holes[i]->core[j]->numvalues;
        }
      }
      if(n > all_cnt) {
        all_cnt = n;
      }
      /* alloc space for temp arrays that hold core depth and var and
         assign them their values culling out bad samples
      */
      if((coredep=(float*)calloc(all_cnt+100,sizeof(float))) == NULL)
                    SaBadMemAlloc("Sa");
      if((corevar=(float*)calloc(all_cnt+100,sizeof(float))) == NULL)
                    SaBadMemAlloc("Sa");
      if((corevarsm=(float*)calloc(all_cnt+100,sizeof(float))) == NULL)
                    SaBadMemAlloc("Sa");
      /* loop through the cores creating a continuous series and
         apply the current stretch-squish to this series
      */
      coren = 0;
      start = YES;
      rate = squish/100.0;
      if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
        for(l=0; l<splice[sset]->numvalues; ++l) {
          coredep[l] = splice[sset]->value[l]->mcd * rate;
          corevar[l] = splice[sset]->value[l]->data * flip;
          if(sm == YES) {
            corevarsm[l] = corevar[l];
          }
        }
        coren = splice[sset]->numvalues;
      }
      else {
        for(j=0; j<data[ds]->holes[i]->numcores; ++j) {

          if(data[ds]->holes[i]->core[j]->did_affine_offset == YES ||
             (have_affine == NO && autocoreadj.override_aff == YES)) {

            n = data[ds]->holes[i]->core[j]->numvalues;
            for(l=0; l<n; ++l) {
              if(data[ds]->holes[i]->core[j]->value[l]->quality == GOOD) {
                if(start == NO &&
                   data[ds]->holes[i]->core[lastc]->value[lastcn-1]->mbsf + data[ds]->holes[i]->core[lastc]->affine_offset >= 
                   data[ds]->holes[i]->core[j]->value[l]->mbsf + data[ds]->holes[i]->core[j]->affine_offset) {
                }
                else {
                  coredep[coren] = data[ds]->holes[i]->core[j]->value[l]->mbsf + data[ds]->holes[i]->core[j]->affine_offset;
                  corevar[coren] = *data[ds]->holes[i]->core[j]->value[l]->data * flip;
                  if(sm == YES) {
                     corevarsm[coren] = data[ds]->holes[i]->core[j]->value[l]->sm_data * flip;
                  }
                  coredep[coren] = coredep[coren] * rate;
                  ++coren;
                }
              }
              else {
              }
            }
            start = NO;
            lastc = j;
            lastcn = data[ds]->holes[i]->core[j]->numvalues;
          }
          else {
          }
        }
      }

      /* repick core depths to log depths.
      */
      startlogdep = logdep[0] + autocoreadj.range_offset[0];
      endlogdep = logdep[logn-1] + autocoreadj.range_offset[1]; 
      if(sm == YES) {
        err = GetCoreToLogDepths(coredep, corevarsm, &coren, startlogdep, endlogdep, depstep, autocoreadj.range_offset[1]);
      }
      else {
        err = GetCoreToLogDepths(coredep, corevar, &coren, startlogdep, endlogdep, depstep, autocoreadj.range_offset[1]);
      }
      if(err > 0 || coren < 1) {
        (void)sprintf(buffer, "Error; Repicking core depths to log depths.");
        SaManageWarningBox(buffer);
        free(coredep);
        coredep = NULL;
        free(corevar);
        corevar = NULL;
        free(corevarsm);
        corevarsm = NULL;
        free(logdep);
        logdep = NULL;
        free(logdepoff);
        logdepoff = NULL;
        free(logvar);
        logvar = NULL;
        if(cursor) {
          XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
          XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
        }
        XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
        autocoreadj.in_progress = NO;
        xmstring = XmStringCreateSimple("");
        XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
        XmStringFree(xmstring);
        XtFree(position);
        return 1;
      }

      /* loop thru all log offsets for current squish
      */
      for(c=0; c<autocoreadj.noffset; ++c) {     

        /* check if the interupt button was pushed
        */
        XmUpdateDisplay(formAutoCorrelate);
        while(XCheckMaskEvent(XtDisplay(formAutoCorrelate), ButtonPressMask, &event)) {
          if(event.xbutton.window == XtWindow(pushButtonAutoCorrInterupt)) {
            free(coredep);
            coredep = NULL;
            free(corevar);
            corevar = NULL;
            free(corevarsm);
            corevarsm = NULL;
            free(logdep);
            logdep = NULL;
            free(logdepoff);
            logdepoff = NULL;
            free(logvar);
            logvar = NULL;
            if(cursor) {
              XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
              XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
            }
            XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
            autocoreadj.in_progress = NO;
            autocoreadj.draw_results = NO;
            log_clip_what = NONE;
            DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                    log_clip_what, 0.0, NO);
            xmstring = XmStringCreateSimple("");
            XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
            XtFree(position);
            return 1;
          }
        }

        autocoreadj.box[k]->offset = offset;
        autocoreadj.box[k]->squish = squish;

        DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog, 
                log_clip_what, offset, YES);
        if(have_affine == DO) {
          XSetForeground(XtDisplay(drawingAreaLog), gc_drawingAreaLog, fgPixel[CLINE_COREMCD]);
        }
        else {
          XSetForeground(XtDisplay(drawingAreaLog), gc_drawingAreaLog, fgPixel[CLINE_COREODPSBD]);
        }
        perpix = (maxvar - minvar)/120.0;
        if(smooth.method == GAUSSIAN && (smooth.plot == SMOOTH || smooth.plot == SMOOTHBOTH)) {
          sm = YES;
        }
        else {
          sm = NO;
        }
        start = 0;
        if(sm == YES) {
          while(corevarsm[start] == BADPT && start < coren-1) {
            ++start;
          }
        }
        else {
          while(corevar[start] == BADPT && start < coren-1) {
            ++start;
          }
        }
        if(start < coren-2) {
          xoff = SPACE + 2 * logaxislen;
          XSetForeground(XtDisplay(drawingAreaLog), gc_drawingAreaLog, fgPixel[CLINE_COREOVERLAYINCORR]);
          if(sm == YES) {
            sx = (corevarsm[start] - minvar + flip_off)/perpix + xoff;
          }
          else {
            sx = (corevar[start] - minvar + flip_off)/perpix + xoff;
          }
          sy = (coredep[start] - top_meter)/depperpix + top_off;
          for(l=start+1; l<coren; ++l) {
            if(sm == YES && corevarsm[l] == BADPT) {
            }
            else if(corevar[l] == BADPT) {
            }
            else {
              if(sm == YES) {
                sdx = (corevarsm[l] - minvar + flip_off)/perpix + xoff;
              }
              else {
                sdx = (corevar[l] - minvar + flip_off)/perpix + xoff;
              }
              sdy = (coredep[l] - top_meter)/depperpix+top_off;
              if(sy < top_off || sdy < top_off) {
              }
              else {
                XDrawLine(XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog, sx, sy, sdx, sdy);
              }
              sx = sdx;
              sy = sdy;
            }
          }
        }

        autocoreadj.box[k]->row = r;
        autocoreadj.box[k]->col = c;

        /* find the beginning array indicy for corelation and correlate
        */
        if(coren > 0 && logn > 0) {
          if(fabs(coredep[0] - logdepoff[0]) < ROUNDCHECK) {
            nwin = coren;
            logoffset = 0;
            coreoffset = 0;
          }
          else if(coredep[0] < logdepoff[0]) {
            l = 0;
            while(fabs(coredep[l] - logdepoff[0]) > ROUNDCHECK && l < coren) {
              ++l;
            }
            nwin = coren - l;
            logoffset = 0;
            coreoffset = l;
          }
          else if(coredep[0] > logdepoff[0]) {
            l = 0;
            while(fabs(logdepoff[l] - coredep[0]) > ROUNDCHECK && l < logn) {
              ++l;
            }
            nwin = coren;
            logoffset = l;
            coreoffset = 0;
          }

          if(sm == YES) {
            err = Correlate2(logn, coren, logoffset, coreoffset, logvar, corevarsm, &rcoef, &n);
          }
          else {
            err = Correlate2(logn, coren, logoffset, coreoffset, logvar, corevar, &rcoef, &n);
          }
          if(err > 0) {
            (void)sprintf(buffer, "Error; correlating.");
            SaManageWarningBox(buffer);
            free(coredep);
            coredep = NULL;
            free(corevar);
            corevar = NULL;
            free(corevarsm);
            corevarsm = NULL;
            free(logdep);
            logdep = NULL;
            free(logdepoff);
            logdepoff = NULL;
            free(logvar);
            logvar = NULL;
            if(cursor) {
              XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
              XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
            }
            XClearWindow(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate));
            autocoreadj.in_progress = NO;
            xmstring = XmStringCreateSimple("");
            XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
            XmStringFree(xmstring);
            XtFree(position);
            return 1;
          }
          else if(n > 0) {
            autocoreadj.box[k]->r[m] = rcoef;
            autocoreadj.box[k]->ncore = n;
          }
          else {
            autocoreadj.box[k]->r[m] = BADPT;
            autocoreadj.box[k]->ncore = 0;
          }
        }
        else {
          autocoreadj.box[k]->r[m] = BADPT;
          autocoreadj.box[k]->ncore = 0;    
        }
        if(m == 0) {
          autocoreadj.box[k]->sum_r = autocoreadj.box[k]->r[m];
        }
        else if(m > 0) {
          autocoreadj.box[k]->sum_r = 0.0;
          n = 0;
          for(l=0; l<=m; ++l) {
            if(autocoreadj.box[k]->r[l] != BADPT) {
              autocoreadj.box[k]->sum_r = autocoreadj.box[k]->sum_r + autocoreadj.box[k]->r[l];
              ++n;
            }
          }
          if(n > 0) {
            autocoreadj.box[k]->sum_r = autocoreadj.box[k]->sum_r/n;
          }
          else {
            autocoreadj.box[k]->sum_r = BADPT;
          }
        }

         /* draw the box
        */
        ++autocoreadj.boxes_so_far;
        off = 50;
        MakeRedGreenBlue(10.0, &red, &fred, &redinc, &green, &fgreen, &greeninc, &blue, &fblue, &blueinc);
        check = MakeBoxPixel(&fred, &fgreen, &fblue, autocoreadj.box[k]->sum_r);
        if(check == 0) {
          xcolor.red = red * fred;
          xcolor.blue = blue * fblue;
          xcolor.green = green * fgreen;
          err = XAllocColor(XtDisplay(drawingAreaAutoCorrelate), cmap, &xcolor);
          if(err == 0) {
            printf("autocorr; could not alloc color; red %d  green %d  blue %d\n", xcolor.red, xcolor.green, xcolor.blue);
            GetColorPixelValue(XtDisplay(drawingAreaAutoCorrelate), cmap, "white", &pixel);
          }
          else {
            pixel = xcolor.pixel;
          }
        }
        else {
          GetColorPixelValue(XtDisplay(drawingAreaAutoCorrelate), cmap, "white", &pixel);
        }
        AutoCoreAdjustDrawBox(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate),
                         gc_drawingAreaAutoCorrelate, 50, 80, r, c, pixel, k);
        XmUpdateDisplay(drawingAreaAutoCorrelate);

        ++k;
        offset = offset + autocoreadj.offset_inc;
        /* add the current offset to the log depth
        */
        for(l=0; l<logn; ++l) {
          logdepoff[l] = logdep[l] + offset;
        }
      }

      free(coredep);
      coredep = NULL;
      free(corevar);
      corevar = NULL;
      free(corevarsm);
      corevarsm = NULL;

      /* increment the squish
      */
      squish =  autocoreadj.range_squish[0] - (autocoreadj.squish_inc * (r+1));
    }

    /* free up log data for next hole
    */
    free(logdep);
    logdep = NULL;
    free(logdepoff);
    logdepoff = NULL;
    free(logvar);
    logvar = NULL;

  }

  if(cursor) {
    XUndefineCursor(XtDisplay(formAutoCorrelateParams), XtWindow(formAutoCorrelateParams));
    XFreeCursor(XtDisplay(formAutoCorrelateParams), cursor);
  }
  if((string=(char*)calloc(100, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
  (void)sprintf(string, "%s<->%s", type[data[dset]->type], logdata[lset]->info.id[0]);
  xmstring = XmStringCreateSimple(string);
  XtVaSetValues(labelAutoCorrWorking, XmNlabelString, xmstring, NULL);
  XmStringFree(xmstring);
  free(string);
  string = NULL;

  XtFree(position);
  have_autocorr = DO;

  /* find best offset and squish and put in text fields
     also rank from best to worst
  */
  if((corr_rank.r = (float*)calloc(autocoreadj.nboxes,sizeof(float)))==NULL)
                        SaBadMemAlloc("Sa");
  if((corr_rank.box = (int*)calloc(autocoreadj.nboxes,sizeof(int)))==NULL)
                        SaBadMemAlloc("Sa");
  RankCorr();
  corr_rank.use_thisone = 0;
  GetColorPixelValue(XtDisplay(drawingAreaAutoCorrelate), cmap, "black", &pixel);
  i = corr_rank.box[0];
  AutoCoreAdjustDrawBox(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate),
                         gc_drawingAreaAutoCorrelate, 50, 80, autocoreadj.box[i]->row, autocoreadj.box[i]->col, 
                         pixel, i);
  pixel = BlackPixelOfScreen(XtScreen(drawingAreaCores));
  XSetForeground(XtDisplay(drawingAreaAutoCorrelate), gc_drawingAreaAutoCorrelate, pixel);
  off = 1;
  if(autocoreadj.nboxes < 9) {
    n = autocoreadj.nboxes;
  }
  else {
    n = 9;
  }
  if((string=(char*)calloc(10, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
  for(i=0; i<n; ++i) {
    j = corr_rank.box[i];
    r = autocoreadj.box[j]->xpix_loc;
    c = autocoreadj.box[j]->ypix_loc;
    (void)sprintf(string,"%d", off);
    XDrawString(XtDisplay(drawingAreaAutoCorrelate), XtWindow(drawingAreaAutoCorrelate), 
                gc_drawingAreaAutoCorrelate, r-2, c+5, string, (int)strlen(string));
    ++off;
  }
  free(string);

  if((string=(char*)calloc(50, sizeof(char))) == NULL)
                                    SaBadMemAlloc("Sa");
  (void)sprintf(string, "%.2f", autocoreadj.box[corr_rank.box[0]]->offset);
  XmTextSetString(textFieldAutoCorrResultOffset, string);
  (void)sprintf(string, "%.2f", autocoreadj.box[corr_rank.box[0]]->squish);
  XmTextSetString(textFieldAutoCorrResultSquish, string);
  (void)sprintf(string, "r = %.3f  n = %d  (1)", autocoreadj.box[corr_rank.box[0]]->sum_r, 
                                 autocoreadj.box[corr_rank.box[0]]->ncore);
  xmstring = XmStringCreateSimple(string);
  XtVaSetValues(labelAutoCorrBestCorr, XmNlabelString, xmstring, NULL);
  free(string);
  XmStringFree(xmstring);
  log_clip_what = NONE;
  autocoreadj.draw_results = YES;
  XtVaSetValues(arrowButtonAutoCorrUp, XmNmappedWhenManaged, True, NULL);
  XtVaSetValues(arrowButtonAutoCorrDown, XmNmappedWhenManaged, True, NULL);
  DrawLog(drawingAreaLog, XtDisplay(drawingAreaLog), XtWindow(drawingAreaLog), gc_drawingAreaLog,
                log_clip_what, 0.0, NO);

  return 0;
}

/* pdep and pvar is the core series
   startldep and endldep are the min and max depths considering
   all depth offsets
*/
int GetCoreToLogDepths(pdep, pvar, pn, startldep, endldep, dt, maxoff)
int *pn;
float *pdep, *pvar, startldep, endldep, dt, maxoff;
{
  int i, j, l, c, cc, n, ln, start, end, all_cnt, nstep;
  float *tempdep, *tempvar, step, *ldep;

  /* create an array of log depths that represent all posible
     log overlays of core series
  */
  all_cnt = (endldep - startldep)/dt + 10;
  if((ldep=(float*)calloc(all_cnt, sizeof(float))) == NULL)
                   SaBadMemAlloc("Sa");
  ldep[0] = startldep;
  ln = 1;
  for(i=1; i<all_cnt; ++i) {
    ldep[i] = ldep[i-1] + dt;
    ++ln;
    if(ldep[i] >= endldep) {
      break;
    }
  }

  /* pdep and pvar are the core values for series, pn its size
  */
  all_cnt = (pdep[*pn-1] - pdep[0])/dt + 10;
  if(all_cnt < *pn) {
    all_cnt = *pn + 10;
  }

  if((tempdep=(float*)calloc(all_cnt, sizeof(float))) == NULL)
                   SaBadMemAlloc("Sa");
  if((tempvar=(float*)calloc(all_cnt, sizeof(float))) == NULL)
                   SaBadMemAlloc("Sa");

  for(i=0; i<*pn; ++i) {
    tempdep[i] = pdep[i];
    tempvar[i] = pvar[i];
  }

  /* find where core begins to overlap log
  */
  i = 0;
  while(tempdep[i] < ldep[0] && i < *pn) {
    ++i;
  }
  if(i >= *pn) {
    return 1;
  }
  start = i;

  /* find where either overlap ends or core series ends
  */
  while(tempdep[i] <= ldep[ln-1] && i < *pn) {
    ++i;
  }
  end = i;

  if(end >= *pn) {
    end = *pn-1;
  }
  if(end <= start || (end - start) < TOOFEWCORRPAIRS) {
    return 1;
  }

  l = 0;
  c = start;
  if(start > 1) {
    --c;
  }
  n = 0;

  while(l < ln && c < end) {
    if(ldep[l] >= tempdep[c] && ldep[l] <= tempdep[c+1]) {
      pdep[n] = ldep[l];
      if((tempdep[c+1] - tempdep[c]) < GAP) {
        pvar[n] = tempvar[c] + (tempvar[c+1] - tempvar[c])/(tempdep[c+1] - tempdep[c]) * (ldep[l] - tempdep[c]);
      }
      else {
        pvar[n] = BADPT;
      }
      ++n;
      ++l;
    }
    else if(ldep[l] > tempdep[c] && c < end) {
      ++c;
    }
    else if(ldep[l] < tempdep[c]) {
      ++l;
    }
    else if(ldep[l] > tempdep[end]) {
      break;
    }
  }

  *pn = n;


  free(tempdep);
  tempdep = NULL;
  free(tempvar);
  tempvar = NULL;
  free(ldep);
  ldep = NULL;

  return 0;
}

int RepickDepth(all, pdep, pvar, pn, dt)
int all, *pn;
float *pdep, *pvar, dt;
{
  int i, j, n;
  float *dep, *var;

  n = (pdep[*pn-1] - pdep[0])/dt;
  if(n > all) {
    return 1;
  }
  if((dep=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
  if((var=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
  dep[0] = pdep[0];
  for(i=1; i<n; ++i) {
    dep[i] = dep[i-1] + dt;
  }
  var[0] = pvar[0];
  for(i=1; i<n; ++i) {
    for(j=0; j<*pn; ++j) {
      if(pdep[j] <= dep[i] && pdep[j+1] >= dep[i]) {
        var[i] = pvar[j] + (pvar[j+1] - pvar[j])/(pdep[j+1] - pdep[j]) * (dep[i] - pdep[j]);
        break;
      }
    }
  } 
  for(i=0; i<n; ++i) {
    pdep[i] = dep[i];
    pvar[i] = var[i];
  }
  *pn = n;
  free(dep);
  dep = NULL;
  free(var);
  var = NULL;

  return 0;
}

void MakeNewAffineTable(affineptr)
AffineTable *affineptr;
{
  int i, j, k, alloc_cnt, alloc_inc;
  float top_dep;

  affineptr->numhole = data[dset]->numholes;

  affinetable.core_alloc_cnt = 50;
  affinetable.core_alloc_inc = 25;
  for(i=0; i<data[dset]->numholes; ++i) {
    while(data[dset]->holes[i]->numcores > affinetable.core_alloc_cnt) {
      affinetable.core_alloc_cnt = affinetable.core_alloc_cnt + affinetable.core_alloc_inc;
    }
  }

  affineptr->leg = data[dset]->leg;
  affineptr->site = data[dset]->site;

  if((affinetable.hole=(AffineHole**)calloc(data[dset]->numholes,sizeof(AffineHole*)))==NULL)
                        SaBadMemAlloc("Sa");
  for(i=0; i<data[dset]->numholes; ++i) {
    if((affinetable.hole[i] = (AffineHole*)calloc(1,sizeof(AffineHole)))==NULL)
                        SaBadMemAlloc("Sa");
    if((affinetable.hole[i]->core=(AffineCore**)calloc(affinetable.core_alloc_cnt,sizeof(AffineCore*)))==NULL)
                        SaBadMemAlloc("Sa");
    for(k=0; k<affinetable.core_alloc_cnt; ++k) {
      if((affinetable.hole[i]->core[k] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                       SaBadMemAlloc("Sa");
    }
    affineptr->hole[i]->name = data[dset]->holes[i]->name;
    affineptr->hole[i]->numcore = data[dset]->holes[i]->numcores;
    for(j=0; j<data[dset]->holes[i]->numcores; ++j) {
      affineptr->hole[i]->core[j]->name = data[dset]->holes[i]->core[j]->num;
      affineptr->hole[i]->core[j]->type = data[dset]->holes[i]->core[j]->value[0]->type;
      affineptr->hole[i]->core[j]->did_offset = NO;
      affineptr->hole[i]->core[j]->cum_depth_offset = 0.0;
      /*
        top_dep = data[dset]->holes[i]->core[j]->value[0]->top_int + 
                   ((atoi(data[dset]->holes[i]->core[j]->value[0]->section) - 1) * 1.5);
        affineptr->hole[i]->core[j]->mbsf = data[dset]->holes[i]->core[j]->value[0]->sb_depth - top_dep/100.0;
      */
    }
  }
}

void MapHoleToAffineTable(affineptr, ds)
AffineTable *affineptr;
int ds;
{
  int i, k;

  for(i=0; i<data[ds]->numholes; ++i) {
    data[ds]->holes[i]->affine_map_to_hole = -1;
  }
  for(k=0; k<affineptr->numhole; ++k) {
    affineptr->hole[k]->data_map_to_hole[ds] = -1;
  }

  for(i=0; i<data[ds]->numholes; ++i) {
    for(k=0; k<affineptr->numhole; ++k) {
      if(affineptr->hole[k]->name == data[ds]->holes[i]->name) {
        data[ds]->holes[i]->affine_map_to_hole = k;
        affineptr->hole[k]->data_map_to_hole[ds] = i;
        break;
      }
    }
  }
}

void MapCoreToAffineTable(affineptr, ds)
AffineTable *affineptr;
int ds;
{
  int i, j, k, nh;

  for(i=0; i<data[ds]->numholes; ++i) {
    for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
      data[ds]->holes[i]->core[j]->affine_map_to_core = -1;
      data[ds]->holes[i]->core[j]->affine_offset = 0.0;
      data[ds]->holes[i]->core[j]->did_affine_offset = NO;
    }
  }
  for(i=0; i<affineptr->numhole; ++i) {
    for(k=0; k<affineptr->hole[i]->numcore; ++k) {
      affineptr->hole[i]->core[k]->data_map_to_core[ds] = -1;
    }
  }

  for(i=0; i<data[ds]->numholes; ++i) {
    nh = data[ds]->holes[i]->affine_map_to_hole;
    if(nh >= 0) {
      for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
        for(k=0; k<affineptr->hole[nh]->numcore; ++k) {
          if(data[ds]->holes[i]->core[j]->num == affineptr->hole[nh]->core[k]->name) {
            data[ds]->holes[i]->core[j]->affine_map_to_core = k;
            affineptr->hole[nh]->core[k]->data_map_to_core[ds] = j;
            data[ds]->holes[i]->core[j]->affine_offset = affineptr->hole[nh]->core[k]->cum_depth_offset;
            data[ds]->holes[i]->core[j]->did_affine_offset = affineptr->hole[nh]->core[k]->did_offset;
            break;
          }
        }
      }
    }
  }
}

void CheckAffineTableOrder(affineptr)
AffineTable *affineptr;
{
/*
  int i, j, k, l, m, foundit;
  float dep;
  AffineHole tempaffinehole;
  AffineCore  tempaffinecore;

  doholeagain:
  if(affineptr->numhole > 1) {
    for(i=0; i<affineptr->numhole-1; ++i) {
      if(affineptr->hole[i]->name > affineptr->hole[i+1]->name) {
        tempaffinehole = affineptr->hole[i];
        affineptr->hole[i] = affineptr->hole[i+1];
        affineptr->hole[i+1] = tempaffinehole;
        goto doholeagain;
      }
    }
  }

  docoreagain:
  for(i=0; i<affineptr->numhole-1; ++i) {
    if(affineptr->hole[i]->numcore > 1) {
      for(j=0; j<affineptr->hole[i]->numcore-1; ++j) {
        if(affineptr->hole[i]->core[j]->name > affineptr->hole[i]->core[j+1]->name) {
          tempaffinecore = affineptr->hole[i]->core[j];
          affineptr->hole[i]->core[j] = affineptr->hole[i]->core[j+1];
          affineptr->hole[i]->core[j+1] = tempaffinecore;
          goto docoreagain;
        }
      }
    }
  }
*/
}

void CheckAffineTableMissingHole(affineptr, ds)
AffineTable *affineptr;
int ds;
{
  int i, j, k, l, foundit, all;
  float dep;


  /* are there missing holes in affinetable?
  */
  for(i=0; i<data[ds]->numholes; ++i) {
    foundit = NO;
    for(j=0; j<affineptr->numhole; ++j) {
      if(affineptr->hole[j]->name == data[ds]->holes[i]->name) {
        foundit = YES;
        break;
      }
    }
    if(foundit == NO) {
      j = 0;
      while(affineptr->hole[j]->name <= data[ds]->holes[i]->name && j < affineptr->numhole) {
        ++j;
      }
      ++affineptr->numhole;
      if((charptr=(char*)realloc((char*)affinetable.hole,affinetable.numhole*sizeof(AffineHole*)))==NULL)
                                        SaBadMemAlloc("Sa");
      else
                affinetable.hole=(AffineHole**)charptr;
      if((affinetable.hole[affineptr->numhole-1] = (AffineHole*)calloc(1,sizeof(AffineHole)))==NULL)
                                        SaBadMemAlloc("Sa");
      all = affinetable.core_alloc_cnt;
      if(data[ds]->holes[i]->numcores > all) {
        while(data[ds]->holes[i]->numcores > all) {
          all = all + affinetable.core_alloc_inc;
        }
        for(k=0; k<affineptr->numhole-1; ++k) {
          if((charptr=(char*)realloc((char*)affinetable.hole[k]->core,(all)*sizeof(AffineCore*)))==NULL)
                                        SaBadMemAlloc("Sa");
          else
                     affinetable.hole[k]->core=(AffineCore**)charptr;
          for(l=affinetable.core_alloc_cnt; l<all; ++l) {
            if((affinetable.hole[k]->core[l] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                                           SaBadMemAlloc("Sa");
          }
        }
        
      }
      affinetable.core_alloc_cnt = all;
      if((affinetable.hole[affineptr->numhole-1]->core=(AffineCore**)calloc(affinetable.core_alloc_cnt,sizeof(AffineCore*)))==NULL)
                                          SaBadMemAlloc("Sa");
      for(k=0; k<affinetable.core_alloc_cnt; ++k) {
        if((affinetable.hole[affineptr->numhole-1]->core[k] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                                            SaBadMemAlloc("Sa");
      }


      for(k=affineptr->numhole-1; k > j; --k) {
        affineptr->hole[k] = affineptr->hole[k-1];
      }
      affineptr->hole[j]->name = data[ds]->holes[i]->name;
      affineptr->hole[j]->numcore = data[ds]->holes[i]->numcores;
      for(k=0; k<data[ds]->holes[i]->numcores; ++k) {
        affineptr->hole[j]->core[k]->name = data[ds]->holes[i]->core[k]->num;
        affineptr->hole[j]->core[k]->type = data[ds]->holes[i]->core[k]->value[0]->type;
        /*
          dep = data[ds]->holes[i]->core[k]->value[0]->top_int + ((atoi(data[ds]->holes[i]->core[k]->value[0]->section) - 1) * 1.5);
          affineptr->hole[j]->core[k]->mbsf = data[ds]->holes[i]->core[k]->value[0]->sb_depth - dep/100.0;
        */
        affineptr->hole[j]->core[k]->cum_depth_offset = 0.0;
        affineptr->hole[j]->core[k]->did_offset = NO;
      }
    }
  }
}

void CheckAffineTableMissingCore(affineptr, ds)
AffineTable *affineptr;
int ds;
{
  int i, j, k, l, m, nh, all;
  float dep;

  /* within a hole are there missing cores in affinetable?
  */
  for(i=0; i<affineptr->numhole; ++i) {
    k = 0;
    j = 0;
    nh = affineptr->hole[i]->data_map_to_hole[ds];
    if(nh >= 0 && nh < data[ds]->numholes) {
      while(k < data[ds]->holes[nh]->numcores && nh >= 0) {
        if(data[ds]->holes[nh]->core[k]->num == affineptr->hole[i]->core[j]->name) {
          if(j+1 < affineptr->hole[i]->numcore) {
            ++j;
          }
          ++k;
        }
        else if(data[ds]->holes[nh]->core[k]->num > affineptr->hole[i]->core[affineptr->hole[i]->numcore-1]->name) {
          /*add to end*/
          ++affineptr->hole[i]->numcore;
          j = affineptr->hole[i]->numcore - 1;
          affineptr->hole[i]->core[j]->name = data[ds]->holes[nh]->core[k]->num;
          affineptr->hole[i]->core[j]->type = data[ds]->holes[nh]->core[k]->value[0]->type;
          /*
            dep = data[ds]->holes[nh]->core[k]->value[0]->top_int + ((atoi(data[ds]->holes[nh]->core[k]->value[0]->section) - 1) * 1.5);
            affineptr->hole[i]->core[j]->mbsf = data[ds]->holes[nh]->core[k]->value[0]->sb_depth - dep/100.0;
          */
          if(j > 0) {
            affineptr->hole[i]->core[j]->cum_depth_offset = affineptr->hole[i]->core[j-1]->cum_depth_offset;
          }
          else {
            affineptr->hole[i]->core[j]->cum_depth_offset = 0.0;
          }
          affineptr->hole[i]->core[j]->did_offset = NO;
          ++k;
        }
        else if(data[ds]->holes[nh]->core[k]->num > affineptr->hole[i]->core[j]->name) {
          if(j+1 < affineptr->hole[i]->numcore) {
            ++j;
          }
        }
        else if(data[ds]->holes[nh]->core[k]->num < affineptr->hole[i]->core[j]->name) {
          /*insert new core into table; check if exceed allocated memory*/
          ++affineptr->hole[i]->numcore;
          if(affineptr->hole[i]->numcore > affineptr->core_alloc_cnt) {
            while(affineptr->hole[i]->numcore > all) {
              all = all + affinetable.core_alloc_inc;
            }
            for(m=0; m<affineptr->numhole-1; ++m) {
              if((charptr=(char*)realloc((char*)affinetable.hole[m]->core,(all)*sizeof(AffineCore*)))==NULL)
                                        SaBadMemAlloc("Sa");
              else
                     affinetable.hole[m]->core=(AffineCore**)charptr;
              for(l=affinetable.core_alloc_cnt; l<all; ++l) {
                if((affinetable.hole[m]->core[l] = (AffineCore*)calloc(1,sizeof(AffineCore)))==NULL)
                                           SaBadMemAlloc("Sa");
              }
            }

          }
          affinetable.core_alloc_cnt = all;

          for(m=affineptr->hole[i]->numcore-1; m > j; --m) {
            affineptr->hole[i]->core[m] = affineptr->hole[i]->core[m-1];
          }
          affineptr->hole[i]->core[j]->name = data[ds]->holes[nh]->core[k]->num;
          affineptr->hole[i]->core[j]->type = data[ds]->holes[nh]->core[k]->value[0]->type;
          /*
            dep = data[ds]->holes[nh]->core[k]->value[0]->top_int + 
                        ((atoi(data[ds]->holes[nh]->core[k]->value[0]->section) - 1) * 1.5);
            affineptr->hole[i]->core[j]->mbsf = data[ds]->holes[nh]->core[k]->value[0]->sb_depth - dep/100.0;
          */
          if(j > 0) {
            affineptr->hole[i]->core[j]->cum_depth_offset = affineptr->hole[i]->core[j-1]->cum_depth_offset;
          }
          else {
            affineptr->hole[i]->core[j]->cum_depth_offset = 0.0;
          }
          affineptr->hole[i]->core[j]->did_offset = NO;
          ++k;
          if(j+1 < affineptr->hole[i]->numcore) {
            ++j;
          }
        }
      }
    }
    else {
    }
  }
}

void ApplyCoreAdjust(ds)
int ds;
{
  int i, j, k, n;
  float  rate, affoff;

  rate = eqlogdepth.autocorr_squish/100.0;

  /* apply them
  */
  n = 0;
  for(i=0; i<data[ds]->numholes; ++i) {
    for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
      if(have_affine == DO) {
        affoff = data[ds]->holes[i]->core[j]->affine_offset;
      }
      else {
        affoff = 0.0;
      }
      for(k=0; k<data[ds]->holes[i]->core[j]->numvalues; ++k) {
        if(data[ds]->holes[i]->core[j]->value[k]->quality & BAD_SB_DEPTH) {
          data[ds]->holes[i]->core[j]->value[k]->eld = BADPT;
        }
        else {
          data[ds]->holes[i]->core[j]->value[k]->eld = (data[ds]->holes[i]->core[j]->value[k]->mbsf + affoff) * rate; 
        }
      }
      data[ds]->holes[i]->core[j]->have_equil_log_depths = DO;
    }
  }
  eqlogdepth.autocorr_status == YES;
}

void ApplyCoreAdjustToOneCore(ptrdep, ndep)
int ndep;
float *ptrdep;
{
  int i;
  float  rate;

  rate = eqlogdepth.autocorr_squish/100.0;

  /* apply them
  */
  for(i=0; i<ndep; ++i) {
    ptrdep[i] = ptrdep[i] * rate;
  }
}

int FindAncMinMax(ptranc)
AncillData *ptranc;
{
  int i, j, err, n, off;
  float min, max;
  double std, mean;

  realminanc[1] = 9999.0;
  realmaxanc[1] = -9999.0;
  realminanc[1] = 9999.0;
  realmaxanc[1] = -9999.0;
  realminanc[2] = 9999.0;
  realmaxanc[2] = -9999.0;
  for(i=0; i<ptranc->data_num; ++i) {
    for(j=0; j<ptranc->data_cnt; ++j) {
      if(ptranc->value[i]->data[j] != ptranc->missing_value &&
         ptranc->value[i]->data[j] > realmaxanc[j]) {
        realmaxanc[j] = ptranc->value[i]->data[j];
      }
      else if(ptranc->value[i]->data[j] != ptranc->missing_value &&
            ptranc->value[i]->data[j] < realminanc[j]) {
        realminanc[j] = ptranc->value[i]->data[j];
      }
    }
  }

  ptranc->depth_min = ptranc->value[0]->depth;
  ptranc->depth_max = ptranc->value[ptranc->data_num-1]->depth;
  for(i=0; i<ptranc->data_cnt; ++i) {
    err = AncillStdDev(ptranc, i, ptranc->depth_max, &std, &mean);
    if(err == 0) {
      min = mean - (PLUSMINUSSTD * std);
      max = mean + (PLUSMINUSSTD * std);
    }
    else {
      min = realminanc[i];
      max = realmaxanc[i];
    } 
    ptranc->axis_min[i] = min;
    ptranc->axis_max[i] = max;
  }

  if((have_cores == DO || have_log == DO) && maxdep < ptranc->depth_max) {
    maxdep = ptranc->depth_max;
  }
  else if(have_cores == DONT && have_log == DONT) {
    maxdep = ptranc->depth_max;
  }

  return 0;
}

int AncillStdDev(anc, who, max, stddev, xmean)
AncillData  anc;
int         who;
float    max;
double   *stddev, *xmean;
{
  int i;
  float x, sum, sumsq, nx;

  if(anc.data_num < 2) {
    return 1;
  }

  sum = 0.0;
  sumsq = 0.0;

  nx = 0.0;
  for(i=0; i<anc.data_num; ++i) {
    if(anc.value[i]->depth > max) {
      break;
    }
    else if(anc.value[i]->data[who] != anc.missing_value) {
      x = anc.value[i]->data[who];
      sum = sum + x;
      sumsq = sumsq + x * x;
      ++nx;
    }
  }

  *xmean = sum/nx;

  if(nx < 2.0) {
    return 1;
  }
  else {
    *stddev = (nx * sumsq - sum * sum)/(nx * (nx - 1));
    if(*stddev < 0.0) {
      *stddev = 0.0;
    }
    else {
      *stddev = sqrt(*stddev);
    }
  } 

  return 0;
}

void ApplyCoreAdjustToStrat()
{
  int i, j;
  float rate;

  if(eqlogdepth.autocorr_status == YES) {
    rate = eqlogdepth.autocorr_squish/100.0;
  }
  else {
    rate = 1.0;
  }

  for( i=0; i<strat_file_cnt; ++i) {
    for(j=0; j<strat[i]->data_cnt; ++j) {
      strat[i]->data[j].top.eld = (strat[i]->data[j].top.mbsf + strat[i]->data[j].top.aff_offset) * rate;
      strat[i]->data[j].bot.eld = (strat[i]->data[j].bot.mbsf + strat[i]->data[j].bot.aff_offset) * rate;
    }
    strat[i]->autocorr_status = YES;
  }
}

void RankCorr()
{
  int i, j, b;

  for(i=0; i<autocoreadj.nboxes; ++i) {
    corr_rank.box[i] = -1;
  }
  corr_rank.n = autocoreadj.nboxes;

  for(b=0; b<autocoreadj.nboxes; ++b) {

    if(b == 0) {
      corr_rank.r[b] = autocoreadj.box[b]->sum_r;
      corr_rank.box[b] = b;
    }
    else {
      for(i=0; i<b; ++i) {
        if(i == b-1 && autocoreadj.box[b]->sum_r < corr_rank.r[i]) {
          corr_rank.r[b] = autocoreadj.box[b]->sum_r;
          corr_rank.box[b] = b;
        }
        else if(autocoreadj.box[b]->sum_r > corr_rank.r[i]) {
          for(j=b; j>i; --j) {
            corr_rank.r[j] = corr_rank.r[j-1];
            corr_rank.box[j] = corr_rank.box[j-1];
          }
          corr_rank.r[i] = autocoreadj.box[b]->sum_r;
          corr_rank.box[i] = b;
          break;
        }
      }
    }
  }

  for(i=0; i<corr_rank.n; ++i) {
    b = corr_rank.box[i];
    if(b < 0 || b >= autocoreadj.nboxes) {
    }
    else {
      autocoreadj.box[b]->rank = i;
    }
  }
}

void CalcStratELD()
{
  int i, j, h;
  float strateld;
  void ApplyCoreAdjustToStrat();
  float CalcELD();

  ApplyCoreAdjustToStrat();

  for(i=0; i<strat_file_cnt; ++i) {
    for(j=0; j<strat[i]->data_cnt; ++j) {
      h = strat[i]->data[j].top.eld_map_hole;
      if(h >= 0 && h < eqlogdepth.numhole) {
        strateld = CalcELD(eld.hole_eqt[h], strat[i]->data[j].top.eld);
        if(strateld != BADPT) {
          strat[i]->data[j].top.eld = strateld;
          strat[i]->data[j].top.did_eld = YES;
        }
        else if(eqlogdepth.autocorr_status == YES) {
          strat[i]->data[j].top.did_eld = YES;
        }
        else {
          strat[i]->data[j].top.did_eld = NO;
        }
      }
      h = strat[i]->data[j].bot.eld_map_hole;
      if(h >= 0 && h < eqlogdepth.numhole) {
        strateld = CalcELD(eld.hole_eqt[h], strat[i]->data[j].bot.eld);
        if(strateld != BADPT) {
          strat[i]->data[j].bot.eld = strateld;
          strat[i]->data[j].bot.did_eld = YES;
        }
        else if(eqlogdepth.autocorr_status == YES) {
          strat[i]->data[j].bot.did_eld = YES;
        }
        else {
          strat[i]->data[j].bot.did_eld = NO;
        }
      }
    }
  } 
}

float CalcMbsfMcdRatio(ds)
int ds;
{
  int i, j, n, nr;
  float sum_ratio;

  if(have_affine == DONT || have_cores == DONT) {
    return BADPT;
  }

  nr = 0;
  sum_ratio = 0.0;
  for(i=0; i<data[ds]->numholes; ++i) {
    for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
      if(data[ds]->holes[i]->core[j]->did_affine_offset  == YES) {
        n = data[ds]->holes[i]->core[j]->numvalues - 1;
        while(data[ds]->holes[i]->core[j]->value[n]->quality & BAD_SB_DEPTH) {
          --n;
        }
        if(n >= 0) {
          sum_ratio = sum_ratio + (data[ds]->holes[i]->core[j]->value[n]->mbsf/
                     (data[ds]->holes[i]->core[j]->value[n]->mbsf + data[ds]->holes[i]->core[j]->affine_offset));
          ++nr;
        }
      }
    }
  }
  if(nr > 0) {
    sum_ratio = sum_ratio/nr;
  }
  else {
    sum_ratio = BADPT;
  }
  
  return sum_ratio;
}

FindSpliceMinMax(s)
int s;
{
  int i, n, err, last, start;
  float *var, sumdepstep;
  double std, mean;
  void Round();

  splice[s]->minreal = 9999.0;
  splice[s]->maxreal = -9999.0;
  mean = 0.0;
  std = 0.0;
  n = 0;
  sumdepstep = 0.0;

  if((var=(float*)calloc(splice[s]->numvalues, sizeof(float))) == NULL)
                 SaBadMemAlloc("Sa");

  start = YES;
  for(i=0; i<splice[s]->numvalues; ++i) {
    if(splice[s]->value[i]->quality == GOOD) {
      if(start == YES) {
        start = NO;
        last = i;
      }
      else {
        sumdepstep = sumdepstep + splice[s]->value[i]->mcd - splice[s]->value[last]->mcd;
        last = i;
      }
      if(splice[s]->value[i]->data > splice[s]->maxreal) {
        splice[s]->maxreal = splice[s]->value[i]->data;
      }
      if(splice[s]->value[i]->data < splice[s]->minreal) {
        splice[s]->minreal = splice[s]->value[i]->data;
      }
      var[n] = splice[s]->value[i]->data;
      ++n;
    }
  }

  if(n - 1 > 0) {
    splice[s]->avedepstep = sumdepstep/(n-1);
  }
  else {
    splice[s]->avedepstep = 0.05;
  }
  Round(&splice[s]->avedepstep, 0.01);

  err = StdDev(var, n, 0, n, &std, &mean);

  free(var);
  var = NULL;

  splice[s]->depth_min = splice[s]->value[0]->mcd;
  splice[s]->depth_max = splice[s]->value[splice[s]->numvalues-1]->mcd;
  if((have_cores == DO || have_log == DO || have_ancill == DO) && maxdep < splice[s]->depth_max) {
    maxdep = splice[s]->depth_max;
  }
  else if(have_cores == DONT && have_log == DONT && have_ancill == DONT) {
    maxdep = splice[s]->depth_max;
  }
  
  if(err == 0) {
    splice[s]->min = mean - (PLUSMINUSSTD * std);
    splice[s]->max = mean + (PLUSMINUSSTD * std);
    return 0;
  }
  else {
    return 1;
  }
}

int CheckDepthInDescendingOrder(ds)
int ds;
{
  int i, j, k, ndep, out, alloc, *map, howmany;
  char *text, *line;
  float    *pdep;
  XmString xmstring;
  XmTextPosition  pos;

  alloc = TOOMANYOUTOFORDER + 5;
  if((text=(char*)calloc(300*alloc, CHAR_SIZE))==NULL)
                SaBadMemAlloc("CheckDepthInDescendingOrder");
  if((line=(char*)calloc(300, CHAR_SIZE))==NULL)
                SaBadMemAlloc("CheckDepthInDescendingOrder");

  out = NO;
  howmany = 0;
  for(i=0; i<data[ds]->numholes; ++i) {
    for(j=0; j<data[ds]->holes[i]->numcores; ++j) {
      if(data[ds]->holes[i]->core[j]->numvalues < 2) {
      }
      else if(data[ds]->holes[i]->core[j]->numvalues == 2) {

      }
      else {
        if((pdep=(float*)calloc(data[ds]->holes[i]->core[j]->numvalues,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
        if((map=(int*)calloc(data[ds]->holes[i]->core[j]->numvalues,sizeof(int))) == NULL)
                        SaBadMemAlloc("Sa");
        ndep = 0;
        for(k=0; k<data[ds]->holes[i]->core[j]->numvalues; ++k) {
          if(data[ds]->holes[i]->core[j]->value[k]->quality & BAD_SB_DEPTH) {
          }
          else {
            pdep[ndep] = data[ds]->holes[i]->core[j]->value[k]->mbsf;
            map[ndep] = k;
            ++ndep;
          }
        }

        if(ndep >= 3) {
          for(k=1; k<ndep-1; ++k) {
            if(pdep[k] <= pdep[k+1] && pdep[k] >= pdep[k-1]) {
            }
            else {
              ++howmany;
              if(out == NO) {
                out = YES;
                (void)sprintf(text,"");
                (void)sprintf(text,"\n");
                (void)strcat(text, "The following depths are out of order\n\n");
              }
              if(howmany >= TOOMANYOUTOFORDER) {
                (void)sprintf(line, "There may be more; stopped checking after %d were found.\n", 
                              TOOMANYOUTOFORDER);
                (void)strcat(text, line);
                (void)sprintf(line, "Could there be a data type or format mismatch?\n");
                (void)strcat(text, line);
                break;
              }
              (void)sprintf(line,"site %d  hole %c  core %d  sect. %s top sect. depth  %.2f(cm)   %.2f(mbsf)\n", 
                                  data[ds]->site, data[ds]->holes[i]->name, data[ds]->holes[i]->core[j]->num,
                                  data[ds]->holes[i]->core[j]->value[map[k]]->section, 
                                  data[ds]->holes[i]->core[j]->value[map[k]]->top_int,
                                  data[ds]->holes[i]->core[j]->value[map[k]]->mbsf);
              (void)strncat(text, line, (int)strlen(line));
            }
          }
        }
        free(pdep);
        pdep = NULL;
        free(map);
        map = NULL;
        if(howmany >= TOOMANYOUTOFORDER) {
          break;
        }
      }
    }   
    if(howmany >= TOOMANYOUTOFORDER) {
      break;
    }
  }

  if(out == YES) {
    pos=XmTextGetLastPosition(textGenericText);
    XmTextInsert(textGenericText,pos,text);
    XmTextShowPosition(textGenericText,0);
    XtManageChild(formGenericText);
    XtPopup(XtParent(formGenericText),XtGrabNone);
    xmstring=XmCvtCTToXmString("Out Of Order Core Depths");
    XtVaSetValues(formGenericText,XmNdialogTitle,xmstring,NULL);
    XmStringFree(xmstring);
  }

  free(text);
  text = NULL;
  free(line);
  line = NULL;

  return out;
}

void Round(val, place)
float *val, place;
{
  double check, fract, ip;

  check = *val/place;
  fract = modf(check, &ip);
  if(fract < 0.5) {
    *val = ip * place;
  }
  else if(fract >= 0.5) {
    *val = (ip + 1) * place;
  }
}

int InterpolateSpliceTiePtValue(s, sm, dep, val, place)
int s;
int sm;
float dep, *val;
int *place;
{
  int i, err;

  err = 1;
  if(splice[s]->numvalues < 2) {
  }
  else if(dep < splice[s]->value[0]->mcd || dep > splice[s]->value[splice[s]->numvalues-1]->mcd) {
  }
  else {
    if(sm == YES) {
      for(i=0; i<splice[s]->numvalues-1; ++i) {
        if(dep >= splice[s]->value[i]->mcd && dep <= splice[s]->value[i+1]->mcd) {
          *val = splice[s]->value[i]->sm_data + (splice[s]->value[i+1]->sm_data - splice[s]->value[i]->sm_data)/
                     (splice[s]->value[i+1]->mcd - splice[s]->value[i]->mcd) * (dep - splice[s]->value[i]->mcd);
          err = 0;
          *place = i;
          break;
        }
      }
    }
    else if(sm == NO) {
      for(i=0; i<splice[s]->numvalues-1; ++i) {
        if(dep >= splice[s]->value[i]->mcd && dep <= splice[s]->value[i+1]->mcd) {
          *val = splice[s]->value[i]->data + (splice[s]->value[i+1]->data - splice[s]->value[i]->data)/
                     (splice[s]->value[i+1]->mcd - splice[s]->value[i]->mcd) * (dep - splice[s]->value[i]->mcd);
          err = 0;
          *place = i;
          break;
        }
      }
    }
  }

  return err;
}

int CountTies()
{
  int  i, j, n;

  n = 0;

  for(i=0; i<eqlogdepth.numhole; ++i) {
    for(j=0; j<eqlogdepth.hole[i]->numcore; ++j) {
      n = n + eqlogdepth.hole[i]->core[j]->nties;
    }
  }

  return n;
}

int TestRepickDepthAroundTie(pdep, pvar, pn, ptie, tiedep, dt)
int *pn, *ptie;
float *pdep, *pvar, tiedep, dt;
{
  int i, j, n;
  float dep[MAXPERCORE], var[MAXPERCORE], startdep, enddep;

  n = 1;
  startdep = tiedep;
  while(startdep - dt >= pdep[0]) {
    startdep = startdep - dt;
    ++n;
  }
  *ptie = n - 1;
  enddep = tiedep;
  while(enddep + dt <= pdep[*pn-1]) {
    enddep = enddep + dt;
    ++n;
  }
  dep[0] = startdep;
  for(i=0; i<n; ++i) {
    dep[i] = dep[i] + dt;
  }
  if(n < 2) {
    return;
  }

  for(i=0; i<n; ++i) {
    for(j=0; j<*pn-1; ++j) { 
      if(dep[i] >= pdep[j] && dep[i] <= pdep[j+1]) { 
        var[i] = pvar[j] + (pvar[j+1] - pvar[j])/(pdep[j+1] - pdep[j]) * (dep[i] - pdep[j]);
        break;
      }
    }
  }

  *pn = n;
  for(i=0; i<n; ++i){
    pdep[i]=dep[i];
    pvar[i]=var[i];
  }
}

int InterpolateTiePtValue(n, depth, value, targetdep, intval, place)
int n, *place;
float *depth, *value, targetdep, *intval;
{
  int i;

  /* depth type not important
  */

  if(targetdep < depth[0] || targetdep > depth[n-1]) { 
  }
  else {
    *place = -1;
    if(n > 1) {
      for(i=0; i<n-1; ++i) {
        if(targetdep >= depth[i] && targetdep <= depth[i+1]) {
          *intval = value[i] + (value[i+1] - value[i])/(depth[i+1] - depth[i]) * (targetdep - depth[i]);
          *place = i;
          break;
        }
      }
    }
    if(*place != -1) {
      return 0;
    }
  }
  return 1;
}


void MakeHoleEqt()
{
  int i, j, n, k;


  for(i=0; i<eld.nholes; ++i) {

    if(eld.hole_eqt[i]->nties > 0) {
      for(j=0; j<eld.hole_eqt[i]->nties; ++j) {
        free(eld.hole_eqt[i]->tie[j]);
        eld.hole_eqt[i]->tie[j] = NULL;
      }
      free(eld.hole_eqt[i]->tie);
      eld.hole_eqt[i]->tie = NULL;
      eld.hole_eqt[i]->nties = 0;
    }
    n = 0;
    for(j=0; j<eqlogdepth.hole[i]->numcore; ++j) {
      n = n + eqlogdepth.hole[i]->core[j]->nties;
    }
    if(n > 0) {
      eld.hole_eqt[i]->nties = n + 1;
      if((eld.hole_eqt[i]->tie = (HoleTie**)calloc(eld.hole_eqt[i]->nties, sizeof(HoleTie*)))==NULL) {
                    SaBadMemAlloc("MakeHoleEqt");
      }
      for(j=0; j<eld.hole_eqt[i]->nties; ++j) {
        if((eld.hole_eqt[i]->tie[j] = (HoleTie*)calloc(1, sizeof(HoleTie)))==NULL) {
                    SaBadMemAlloc("MakeHoleEqt");
        } 
      }
      eld.hole_eqt[i]->tie[0]->mbsf = 0.0;
      eld.hole_eqt[i]->tie[0]->mcd = 0.0;
      eld.hole_eqt[i]->tie[0]->autoeld = 0.0;
      eld.hole_eqt[i]->tie[0]->log = 0.0;
      n = 1;
      for(j=0; j<eqlogdepth.hole[i]->numcore; ++j) {
        if(eqlogdepth.hole[i]->core[j]->nties > 0) {
          for(k=0; k<eqlogdepth.hole[i]->core[j]->nties; ++k) {
            eld.hole_eqt[i]->tie[n]->mbsf = eqlogdepth.hole[i]->core[j]->tie[k]->corembsf;
            eld.hole_eqt[i]->tie[n]->mcd = eqlogdepth.hole[i]->core[j]->tie[k]->coremcd;
            eld.hole_eqt[i]->tie[n]->autoeld = eqlogdepth.hole[i]->core[j]->tie[k]->coreautoeld;
            eld.hole_eqt[i]->tie[n]->log = eqlogdepth.hole[i]->core[j]->tie[k]->logdep;
            ++n;
          }
        }
      }
      if(eqlogdepth.autocorr_status == YES) {
        for(k=0; k< eld.hole_eqt[i]->nties-1; ++k) {
          eld.hole_eqt[i]->tie[k]->a = 
             (eld.hole_eqt[i]->tie[k+1]->log - eld.hole_eqt[i]->tie[k]->log)/(eld.hole_eqt[i]->tie[k+1]->autoeld - eld.hole_eqt[i]->tie[k]->autoeld);
          eld.hole_eqt[i]->tie[k]->b = eld.hole_eqt[i]->tie[k]->log - (eld.hole_eqt[i]->tie[k]->autoeld * eld.hole_eqt[i]->tie[k]->a);
        }
        eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->a = 1.0;
        eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->b = 
             eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->log - eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->autoeld;
      }
      else if(have_affine == DO) {
        for(k=0; k< eld.hole_eqt[i]->nties-1; ++k) {
          eld.hole_eqt[i]->tie[k]->a = 
             (eld.hole_eqt[i]->tie[k+1]->log - eld.hole_eqt[i]->tie[k]->log)/(eld.hole_eqt[i]->tie[k+1]->mcd - eld.hole_eqt[i]->tie[k]->mcd);
          eld.hole_eqt[i]->tie[k]->b = eld.hole_eqt[i]->tie[k]->log - (eld.hole_eqt[i]->tie[k]->mcd * eld.hole_eqt[i]->tie[k]->a);
        }
        eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->a = 1.0;
        eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->b = 
             eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->log - eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->mcd;
      }
      else {
        for(k=0; k< eld.hole_eqt[i]->nties-1; ++k) {
          eld.hole_eqt[i]->tie[k]->a = 
             (eld.hole_eqt[i]->tie[k+1]->log - eld.hole_eqt[i]->tie[k]->log)/(eld.hole_eqt[i]->tie[k+1]->mbsf - eld.hole_eqt[i]->tie[k]->mbsf);
          eld.hole_eqt[i]->tie[k]->b = eld.hole_eqt[i]->tie[k]->log - (eld.hole_eqt[i]->tie[k]->mbsf * eld.hole_eqt[i]->tie[k]->a);
        }
        eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->a = 1.0;
        eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->b = 
             eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->log - eld.hole_eqt[i]->tie[eld.hole_eqt[i]->nties-1]->mbsf;
      }
    }
  }
}

int CalcELDInHole(ds, hcore, ccore, eqt, tempdep)
int ds;
int hcore, ccore;
HoleELDEqt *eqt;
float *tempdep;
{
  int i, k, n;
  float  *tiecoredep;

  /* check that have at least two ties
     if only one tie deal with it below as tails
  */
  if(eqt->nties < 1){
    return 1;
  }

  if((tiecoredep=(float*)calloc(eqt->nties,sizeof(float))) == NULL)
                     SaBadMemAlloc("SaCalcEquivLogDepths1");

  for(i=0; i<eqt->nties; ++i) {
    if(eqlogdepth.autocorr_status == YES) {
      tiecoredep[i] = eqt->tie[i]->autoeld;
    }
    else if(have_affine == DO) {
      tiecoredep[i] = eqt->tie[i]->mcd;
    }
    else {
      tiecoredep[i] = eqt->tie[i]->mbsf;
    }
  }

  /* for each section defined by two ties
     check to see that have at least two ties; otherwise only have tails
  */
  if(eqt->nties > 1) {
    k = 0;
    while(tempdep[k] < tiecoredep[0] && k < data[ds]->holes[hcore]->core[ccore]->numvalues) {
      tempdep[k] = tempdep[k] + eqt->tie[0]->log - tiecoredep[0];
      ++k;
    }
    for(i=0; i<eqt->nties - 1; ++i){
      while(tempdep[k] >= tiecoredep[i] && tempdep[k] <= tiecoredep[i+1] && k < data[ds]->holes[hcore]->core[ccore]->numvalues) {
        tempdep[k] = tempdep[k] * eqt->tie[i]->a + eqt->tie[i]->b;
        ++k;
      }
      if(k == data[ds]->holes[hcore]->core[ccore]->numvalues || tempdep[k] > tiecoredep[eqt->nties-1]) {
        break;
      }
    }
  }

  else if(eqt->nties == 1) {
    for(k=0; k<data[ds]->holes[hcore]->core[ccore]->numvalues; ++k) {
      tempdep[k] = tempdep[k] + eqt->tie[0]->log - tiecoredep[0];
    }
  }

  if(k < data[ds]->holes[hcore]->core[ccore]->numvalues-1){
    for(k=k; k<data[ds]->holes[hcore]->core[ccore]->numvalues; ++k) {
      tempdep[k] = tempdep[k] + eqt->tie[eqt->nties-1]->log - tiecoredep[eqt->nties-1];
    }
  }

  free(tiecoredep);
  tiecoredep = NULL;

  return 0;
}

int DoEvalCoreLogData(d, l, pairs, how, dt, top, bot, sm)
int d, l;
LogCoreDataPairs *pairs;
int how;
float dt;
float top, bot;
int   sm;
{
  int i, j, k, h, npair, n, *cname;
  float *mbsf, *mcd, *eld, *var, *dum;

  /* create matched pairs of core and log data
  */
  h = pairs->hole;
  if(how == TOLOGDEPTH) {
    if((pairs->eld=(float*)calloc(logdata[l]->data_num + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->mbsf=(float*)calloc(logdata[l]->data_num + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->mcd=(float*)calloc(logdata[l]->data_num + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->logdat=(float*)calloc(logdata[l]->data_num + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->coredat=(float*)calloc(logdata[l]->data_num + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->corename=(int*)calloc(logdata[l]->data_num + 20,sizeof(int))) == NULL)
                        SaBadMemAlloc("Sa");
    npair = 0;
    for(i=1; i<logdata[l]->data_num; ++i) {
      if(logdata[l]->value[i]->depth >= top && logdata[l]->value[i]->depth <= bot) {
        pairs->eld[npair] = logdata[l]->value[i]->depth;
        pairs->logdat[npair] = logdata[l]->value[i]->data;
        ++npair;
      }
      else if(logdata[l]->value[i]->depth > bot) {
        break;
      }
    }
    pairs->n = npair;
    if(pairs->n < 1) {
      return 1;
    }
    n = 0;
    if(pairs->holename == 'S') {
      n = splice[sset]->numvalues;
    }
    else {
      for(i=0; i<data[d]->holes[h]->numcores; ++i) {
        n = n + data[d]->holes[h]->core[i]->numgood + 1;
      }
    }
    if((eld=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((mbsf=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((mcd=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((var=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((cname=(int*)calloc(n,sizeof(int))) == NULL)
                        SaBadMemAlloc("Sa");
    n = 0;
    if(pairs->holename == 'S') {
      for(i=0; i<splice[sset]->numvalues; ++i) {
        eld[n] = splice[sset]->value[i]->eld;
        mbsf[n] = splice[sset]->value[i]->mbsf;
        mcd[n] = splice[sset]->value[i]->mcd;
        cname[n] = splice[sset]->value[i]->corename;
        if(sm == YES) {
          var[n] = splice[sset]->value[i]->sm_data;
        }
        else {
          var[n] = splice[sset]->value[i]->data;
        }
        ++n;
      }
    }
    else {
      for(i=0; i<data[d]->holes[h]->numcores; ++i) {
        for(j=0; j<data[d]->holes[h]->core[i]->numgood; ++j) {
          k = data[d]->holes[h]->core[i]->good[j];
          eld[n] = data[d]->holes[h]->core[i]->value[k]->eld;
          mbsf[n] = data[d]->holes[h]->core[i]->value[k]->mbsf;
          mcd[n] = data[d]->holes[h]->core[i]->value[k]->mbsf + data[d]->holes[h]->core[i]->affine_offset;
          cname[n] = data[d]->holes[h]->core[i]->num;
          if(sm == YES) {
            var[n] = data[d]->holes[h]->core[i]->value[k]->sm_data;
          }
          else {
            var[n] = *data[d]->holes[h]->core[i]->value[k]->data;
          }
          ++n;
        }
        var[n] = BADPT;
        ++n;
      }
    }
    RepickDepth2(pairs->eld, pairs->mbsf, pairs->mcd, pairs->corename, pairs->coredat, &npair, eld, mbsf, mcd, cname, var, n, YES);
    free(eld);
    eld = NULL;
    free(mbsf);
    mbsf = NULL;
    free(mcd);
    mcd = NULL;
    free(cname);
    cname = NULL;
    free(var);
    var = NULL;
  }
  else if(how == TOCOREDEPTH) {
    n = 0;
    if(pairs->holename == 'S') {
      n = splice[sset]->numvalues;
    }
    else {
      for(i=0; i<data[d]->holes[h]->numcores; ++i) {
        n = n + data[d]->holes[h]->core[i]->numgood;
      }
    }
    if((pairs->eld=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->mbsf=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->mcd=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->logdat=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->coredat=(float*)calloc(n + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->corename=(int*)calloc(n + 20,sizeof(int))) == NULL)
                        SaBadMemAlloc("Sa");
    npair = 0;
    if(pairs->holename == 'S') {
      for(i=0; i<splice[sset]->numvalues; ++i) {
        if(splice[sset]->value[i]->eld >= top && splice[sset]->value[i]->eld <= bot) {
          pairs->eld[npair] = splice[sset]->value[i]->eld;
          pairs->mbsf[npair] = splice[sset]->value[i]->mbsf;
          pairs->mcd[npair] = splice[sset]->value[i]->mcd;
          pairs->corename[npair] = splice[sset]->value[i]->corename;
          if(sm == YES) {
            pairs->coredat[npair] = splice[sset]->value[i]->sm_data;
          }
          else {
            pairs->coredat[npair] = splice[sset]->value[i]->data;
          }
          ++npair;
        }
        else if(splice[sset]->value[i]->eld > bot) {
          break;
        }
      }
    }
    else {
      for(i=0; i<data[d]->holes[h]->numcores; ++i) {
        for(j=0; j<data[d]->holes[h]->core[i]->numgood; ++j) {
          k = data[d]->holes[h]->core[i]->good[j];
          if(data[d]->holes[h]->core[i]->value[k]->eld >= top && data[d]->holes[h]->core[i]->value[k]->eld <= bot) {
            pairs->eld[npair] = data[d]->holes[h]->core[i]->value[k]->eld;
            pairs->mbsf[npair] = data[d]->holes[h]->core[i]->value[k]->mbsf;
            pairs->mcd[npair] = data[d]->holes[h]->core[i]->value[k]->mbsf + data[d]->holes[h]->core[i]->affine_offset;
            pairs->corename[npair] = data[d]->holes[h]->core[i]->num;
            if(sm == YES) {
              pairs->coredat[npair] = data[d]->holes[h]->core[i]->value[k]->sm_data;
            }
            else {
              pairs->coredat[npair] = *data[d]->holes[h]->core[i]->value[k]->data;
            }
            ++npair;
          }
          else if(data[d]->holes[h]->core[i]->value[k]->eld > bot) {
            break;
          }
        }
      }
    }
    pairs->n = npair;
    if(pairs->n < 1) {
      return 1;
    }
    if((eld=(float*)calloc(logdata[l]->data_num,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((var=(float*)calloc(logdata[l]->data_num,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    for(i=0; i<logdata[l]->data_num; ++i) {
      eld[i] = logdata[l]->value[i]->depth;
      var[i] = logdata[l]->value[i]->data;
    }
    RepickDepth2(pairs->eld, dum, dum, dum, pairs->logdat, &npair, eld, dum, dum, dum, var, logdata[l]->data_num, NO);
    free(eld);
    eld = NULL;
    free(var);
    var = NULL;
  }
  else if(how == CONSTANTDT) {
    npair = (bot - top)/dt;
    pairs->n = npair;
    if((pairs->eld=(float*)calloc(npair + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->mbsf=(float*)calloc(npair + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->mcd=(float*)calloc(npair + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->logdat=(float*)calloc(npair + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->coredat=(float*)calloc(npair + 20,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((pairs->corename=(int*)calloc(npair + 20,sizeof(int))) == NULL)
                        SaBadMemAlloc("Sa");
    pairs->eld[0] = top;
    for(i=1; i<npair; ++i) {
      pairs->eld[i] = pairs->eld[i-1] + dt;
    }
    if((eld=(float*)calloc(logdata[l]->data_num,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((var=(float*)calloc(logdata[l]->data_num,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    for(i=0; i<logdata[l]->data_num; ++i) {
      eld[i] = logdata[l]->value[i]->depth;
      var[i] = logdata[l]->value[i]->data;
    }
    RepickDepth2(pairs->eld, dum, dum, dum, pairs->logdat, &npair, eld, dum, dum, dum, var, logdata[l]->data_num, NO);
    free(eld);
    eld = NULL;
    free(var);
    var = NULL;
    n = 0;
    if(pairs->holename == 'S') {
      n = splice[sset]->numvalues;
    }
    else {
      for(i=0; i<data[d]->holes[h]->numcores; ++i) {
        n = n + data[d]->holes[h]->core[i]->numgood + 1;
      }
    }
    if((eld=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((mbsf=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((mcd=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((var=(float*)calloc(n,sizeof(float))) == NULL)
                        SaBadMemAlloc("Sa");
    if((cname=(int*)calloc(n,sizeof(int))) == NULL)
                        SaBadMemAlloc("Sa");
    n = 0;
    if(pairs->holename == 'S') {
      for(i=0; i<splice[sset]->numvalues; ++i) {
        eld[n] = splice[sset]->value[i]->eld;
        mbsf[n] = splice[sset]->value[i]->mbsf;
        mcd[n] = splice[sset]->value[i]->mcd;
        cname[n] = splice[sset]->value[i]->corename;
        if(sm == YES) {
          var[n] = splice[sset]->value[i]->sm_data;
        }
        else {
          var[n] = splice[sset]->value[i]->data;
        }
        ++n;
      }
    }
    else {
      for(i=0; i<data[d]->holes[h]->numcores; ++i) {
        for(j=0; j<data[d]->holes[h]->core[i]->numgood; ++j) {
          k = data[d]->holes[h]->core[i]->good[j];
          eld[n] = data[d]->holes[h]->core[i]->value[k]->eld;
          mbsf[n] = data[d]->holes[h]->core[i]->value[k]->mbsf;
          mcd[n] = data[d]->holes[h]->core[i]->value[k]->mbsf + data[d]->holes[h]->core[i]->affine_offset;
          cname[n] = data[d]->holes[h]->core[i]->num;
          if(sm == YES) {
            var[n] = data[d]->holes[h]->core[i]->value[k]->sm_data;
          }
          else {
            var[n] = *data[d]->holes[h]->core[i]->value[k]->data;
          }
          ++n;
        }
        var[n] = BADPT;
        ++n;
      }
    }
    RepickDepth2(pairs->eld, pairs->mbsf, pairs->mcd, pairs->corename, pairs->coredat, &npair, eld, mbsf, mcd, cname, var, n, YES);
    free(eld);
    eld = NULL;
    free(mbsf);
    mbsf = NULL;
    free(mcd);
    mcd = NULL;
    free(cname);
    cname = NULL;
    free(var);
    var = NULL;
  }

  return 0;
}

int RepickDepth2(ptargeld, ptargmbsf, ptargmcd, ptargcname, ptargvar, pnpair, prealeld, prealmbsf, prealmcd, prealcname, prealvar, n, doit)
int *pnpair, n, doit, *ptargcname, *prealcname;
float *ptargeld, *ptargmbsf, *ptargmcd, *ptargvar, *prealeld, *prealmbsf, *prealmcd, *prealvar;
{
  int i, j, count;

  count = 0;
  for(i=0; i<*pnpair; ++i) {
    ptargvar[i] = BADPT;
    for(j=count; j<n-1; ++j) {
      if(prealvar[j] == BADPT || prealvar[j+1] == BADPT) {

      }
      else if(ptargeld[i] < prealeld[j]) {
        break;
      }
      else if(ptargeld[i] >= prealeld[j] && ptargeld[i] <= prealeld[j+1]) {
        ptargvar[i] = prealvar[j] + (prealvar[j+1] - prealvar[j])/(prealeld[j+1] - prealeld[j]) * (ptargeld[i] - prealeld[j]);
        if(doit == YES) {
          ptargcname[i] = prealcname[j];
          if(have_affine == YES) {
            ptargmcd[i] = prealmcd[j] + (prealmcd[j+1] - prealmcd[j])/(prealeld[j+1] - prealeld[j]) * (ptargeld[i] - prealeld[j]);
            ptargmbsf[i] = prealmbsf[j] + (prealmbsf[j+1] - prealmbsf[j])/(prealeld[j+1] - prealeld[j]) * (ptargeld[i] - prealeld[j]);
          }
          else {
            ptargmbsf[i] = prealmbsf[j] + (prealmbsf[j+1] - prealmbsf[j])/(prealeld[j+1] - prealeld[j]) * (ptargeld[i] - prealeld[j]); 
          }
        }
        count = j;
        break;
      }
    }
  }

  return 0;
}

float CalcELD(eqt, dep)
HoleELDEqt *eqt;
float      dep;
{
  int i, k, n;
  float  *tiecoredep, eld;

  /* check that have at least two ties
     if only one tie deal with it below as tails
  */
  if(eqt->nties < 1){
    return BADPT;
  }

  if((tiecoredep=(float*)calloc(eqt->nties,sizeof(float))) == NULL)
                     SaBadMemAlloc("SaCalcEquivLogDepths1");

  for(i=0; i<eqt->nties; ++i) {
    if(eqlogdepth.autocorr_status == YES) {
      tiecoredep[i] = eqt->tie[i]->autoeld;
    }
    else if(have_affine == DO) {
      tiecoredep[i] = eqt->tie[i]->mcd;
    }
    else {
      tiecoredep[i] = eqt->tie[i]->mbsf;
    }
  }

  /* for each section defined by two ties
     check to see that have at least two ties; otherwise only have tails
  */
  if(dep < tiecoredep[0]) {
    eld = dep + eqt->tie[0]->log - tiecoredep[0];
    free(tiecoredep);
    tiecoredep = NULL;
    return eld;
  }
  else if(dep > tiecoredep[eqt->nties-1]) {
    eld = dep + eqt->tie[eqt->nties-1]->log - tiecoredep[eqt->nties-1];
    free(tiecoredep);
    tiecoredep = NULL;
    return eld;
  }
  else if(eqt->nties > 1) {
    for(i=0; i<eqt->nties - 1; ++i){
      if(dep >= tiecoredep[i] && dep <= tiecoredep[i+1]) {
        eld = dep * eqt->tie[i]->a + eqt->tie[i]->b;
        free(tiecoredep);
        tiecoredep = NULL;
        return eld;
      }
    }
  }

  free(tiecoredep);
  tiecoredep = NULL;

  return BADPT;
}
