/*
sagan_functions.c

      These functions perform various general tasks

     float   getfloat()
     int     getword()
     void    gauss_weights()
     void    SaBadMemAlloc()
     void    SaCleanUp()
     void    SaOpenCoreInfo()
     void    SaExit()
     int     SaGetEquationValues()
     int     SaGetSign()
     void    SaInitialize()
     void    SaManageMessage()
     void    SaManageWarningBox()
     void    SaSetEquationValues()
     void    SaSetSign()
     void    SaUpdateOpen()
     void    SaUpdateReport()
     void    SaUpdateFileSelection()
     int     VerifyFileReadable()
     void    MapStratToData()
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <malloc.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/FileSB.h>
#include <X11/cursorfont.h>
#include "sagan-creation-c.h"
#include "sagan_vars.h"
#include "EquationMenu.h"
#include <errno.h>
int errno;

char    buffer[MAXBUF];

float	getfloat(string,col)
char	*string;
int	col;
{
	float	outval;
	int	this_col,found=0,len=0,i;

	len=(int)strlen(string);
	this_col=0;
	i=0;
	while(i<len && !found)
	{
		for(;i<len && isspace(string[i]);i++);
		if(this_col++==col)
		{
			found=1;
			(void)sscanf(&string[i],"%f",&outval);
			return(outval);
		}
		else
			for(;i<len && !isspace(string[i]);i++);
	}
}	
int getword(string,col,word)
char	*string,*word;
int	col;
{
	int	this_col,found=0,len=0,i;

	len=(int)strlen(string);
	this_col=0;
	i=0;
	while(i<len && !found)
	{
		for(;i<len && isspace(string[i]);i++);
		if(this_col++==col)
		{
			found=1;
			(void)sscanf(&string[i],"%s",word);
			return(True);
		}
		else
			for(;i<len && !isspace(string[i]);i++);
	}
	return(False);
}	

void gauss_weights(width,weight)
int	width;
float	*weight;
{
	double	exp (), foldwidth, halfwidth, power, sqrt (), sum = 0.0;
	int	ia;

	foldwidth = ((double) width - 1.0) / 3.0;
	halfwidth = ((double) width - 1.0) / 2.0;

	/*  compute unscaled weights  */
	for (ia = 1;  ia <= width;  ia++)
	{
		power = (halfwidth - (double) ia + 1.0) / foldwidth;
		power = -1.0 * power * power;
		weight[ia] = exp (power);
		sum += weight[ia];

		/*  set pointers  */
	}

	/*  scale weights to add to 1.0  */
	for (ia = 1;  ia <= width;  ia++)
		weight[ia] /= sum;

}  /*  end gauss_weights  */

void    SaBadMemAlloc(string)
char    *string;
{
        (void)fprintf(stderr,"ERROR: Memory allocation failed at %s! Exit & restart program!!\n",string);
        info.fatal=True;
        info.error=True;
        info.caller=BADMEMALLOC;
        info.type=XmDIALOG_ERROR;
        info.default_button=XmDIALOG_OK_BUTTON;
        info.message=(char*)calloc(200+(int)strlen(string),CHAR_SIZE);
        (void)sprintf(info.message,"ERROR: Memory allocation failed at %s! Exit & restart program!!",string);
        SaManageMessage(info);

  perror("SaBadMemAlloc");
  (void)fprintf(stderr, "%d  %d\n", ENOMEM, EAGAIN);
}

void SaCleanUp(cflag)
int     cflag;
{
        int     i,j,k;
        char    *string;
        XmString xmstring;
        Value   *valueptr;
        extern  Widget  pushButtonOpenStrat;
        time_t  tp;
        void    AutoPair(), DrawDrawingAreaCores(), DrawDrawingAreaLog();
 
        if(cflag==EXIT)
        {
                if(defaults_filename!=NULL) free(defaults_filename);
                if(current_path!=NULL) free(current_path);
                if(report.ptr!=NULL) fclose(report.ptr);
        }
        if(cflag == COREDATA || cflag == LOGDATA) 
        {
                /* turn off the auto pair option
                */
                autopair = NO;
                AutoPair(autopair);
                XtSetSensitive(pushButtonPairCoreLog, False);
                XtSetSensitive(labelAutoPair, False);
        }
        if(cflag==ALL || cflag==COREDATA)
        {
                done_reading=done_cull=done_decimate=False;
                datafileopen=False;
                for(i=0; i<dataset_cnt; ++i) {
                  SaFreeData(data[i]);
                  if(data[i])
                  {
                    free(data[i]);
                    data[i]=NULL;
                  }
                }
                if(data)
                {
                  free(data);
                  data=NULL;
                }
                dset = 0;
                dataset_cnt = 0;
                if(have_log == DO) {
                  for(i=0; i<logset_cnt; ++i) {
                    logdata[i]->info.maptodata = -1;
                  }
                }
                XtUnmanageChild(optionMenuDataSet);
                XtUnmanageChild(pushButtonDataSet1);
                XtUnmanageChild(pushButtonDataSet2);
                XtUnmanageChild(pushButtonDataSet3);
                XtUnmanageChild(pushButtonDataSet4);
                XtUnmanageChild(pushButtonDataSet5);
                XtUnmanageChild(pushButtonDataSet6);
                XtUnmanageChild(pushButtonDataSet7);
                XtUnmanageChild(pushButtonDataSet8);
                XtUnmanageChild(pushButtonDataSet9);

                have_cores=DONT;
		have_coretotie=DONT;
		have_tie1=have_tie2=have_ties=DONT;
		have_correlate=DONT;
                eqlogdepth.autocorr_status = NO;
                max_num_cores = 0;
                XtSetSensitive(pushButtonOpenCore,True);
                XtSetSensitive(pushButtonClearCores, False);
                XtSetSensitive(pushButtonOpenSplicer, False);
                XtSetSensitive(pushButtonClearSplice, False);
                XtSetSensitive(cascadeButtonProcessData, False);
                XtSetSensitive(pushButtonAutoCorrelate, False);
                XtSetSensitive(pushButtonOpenStrat, False);
                XtSetSensitive(pushButtonOpenSpliceTable, False);
                XtSetSensitive(pushButtonOpenEquivLogDepth, True);
                XmListDeleteAllItems(listOpenSelected);
                XtSetSensitive(pushButtonOpenAffine, True);
                XtSetSensitive(pushButtonEvalCoreLogData, False);
            
                if(report.saveflag)
                {
                        report.saveflag=False;
                        fclose(report.ptr);
                }
                if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCleanUp 1");
                (void)time(&tp);
                (void)sprintf(charptr,"\n%s  Core data has been cleared.",ctime(&tp));
                XmTextReplace(textReport,0,XmTextGetLastPosition(textReport),charptr);
                free(charptr);

        }
        if(cflag==ALL || cflag==LOGDATA)
        {
          have_log = DONT;
          have_coretotie = DONT;
          have_tie1=have_tie2=have_ties=DONT;
          have_correlate=DONT;
          for(i=0; i<logset_cnt; ++i) {
            if(logdata[i]->value) {
              for(j=0; j<logdata[i]->data_alloc; ++j) {
                free(logdata[i]->value[j]);
                logdata[i]->value[j] = NULL;
              }
              free(logdata[i]->value);
              logdata[i]->value = NULL;
            }
            logdata[i]->data_alloc = 0;
            logdata[i]->info.id_alloc = 0;
            free(logdata[i]->info.filename);
            logdata[i]->info.filename = NULL;
            free(logdata[i]->info.dirname);
            logdata[i]->info.dirname = NULL;
            free(logdata[i]->info.id[0]);
            logdata[i]->info.id[0] = NULL;
            free(logdata[i]->info.id);
            logdata[i]->info.id = NULL;
            free(logdata[i]->good);
            logdata[i]->good = NULL;
          }
          logset_cnt = 0;
          lset = 0;
          if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCleanUp 1");
          (void)time(&tp);
          (void)sprintf(charptr,"\n%s  Log data has been cleared.",ctime(&tp));
          XmTextReplace(textReport,0,XmTextGetLastPosition(textReport),charptr);
          free(charptr);
          if(have_cores == DO) {
            for(i=0; i< dataset_cnt; ++i) {
              data[i]->maptolog = -1;
            }
          }
          XtSetSensitive(pushButtonOpenLog, True);
          XtSetSensitive(pushButtonClearLog, False);
          XtSetSensitive(cascadeButtonProcessLog, False);
          XtSetSensitive(pushButtonAutoCorrelate, False);
          XtSetSensitive(pushButtonEvalCoreLogData, False);
          XtUnmanageChild(optionMenuLogSet);
          XtUnmanageChild(pushButtonLogSet1);
          XtUnmanageChild(pushButtonLogSet2);
          XtUnmanageChild(pushButtonLogSet3);
          XtUnmanageChild(pushButtonLogSet4);
          XtUnmanageChild(pushButtonLogSet5);
          XtUnmanageChild(pushButtonLogSet6);
          XtUnmanageChild(pushButtonLogSet7);
          XtUnmanageChild(pushButtonLogSet8);
          XtUnmanageChild(pushButtonLogSet9);
        }
        if(cflag==ALL || cflag==AFFINE || (cflag == COREDATA && have_affine == DO))
        {
          have_affine = DONT;

          for(i=0;i<affinetable.numhole;i++)
          {
            for(j=0; j<affinetable.core_alloc_cnt; ++j) 
            {
              if(affinetable.hole[i]->core[j])
              {
                free(affinetable.hole[i]->core[j]);
                affinetable.hole[i]->core[j]=NULL;
              }
            }
            if(affinetable.hole[i]->core)
            {
              free(affinetable.hole[i]->core);
              affinetable.hole[i]->core = NULL;
            }
            if(affinetable.hole[i])
            {
              free(affinetable.hole[i]);
              affinetable.hole[i]=NULL;
            }
          }
          if(affinetable.hole)
          {
            free(affinetable.hole);
            affinetable.hole=NULL;
          }
          affinetable.numhole = 0;
        }
        if(((cflag==ALL || cflag==STRATDATA) && have_strat == DO) || (cflag == COREDATA && have_strat == DO))
        {
          XtSetSensitive(pushButtonClearStrat, False);
          for(i=0; i<strat_file_cnt; ++i) {
            free(strat[i]->data);
            strat[i]->data = NULL;
            free(strat[i]->filename);
            strat[i]->filename = NULL;
            free(strat[i]->dirname);
            strat[i]->dirname = NULL;
            free(strat[i]);
            strat[i] = NULL;
          }
          free(strat);
          strat = NULL;
          have_strat = DONT;
          strat_file_cnt = 0;
          XtSetSensitive(pushButtonClearStrat, False);
        }
        if(cflag==ALL || cflag==SPLICEDATA || (have_splice_table == DO && cflag == COREDATA))
        {
          for(i=0; i<splice_cnt; ++i) {
            if(splice[i]->value) {
              for(j=0; j<splice[i]->alloc_cnt; ++j) {
                free(splice[i]->value[j]);
                splice[i]->value[j] = NULL;
              }
              free(splice[i]->value);
              splice[i]->value = NULL;
            }
            free(splice[i]);
            splice[i] = NULL;
          }
          free(splice);
          splice = NULL;
          
          have_splice = DONT;
          splice_cnt = 0;
          sset = 0;
          if(have_cores == DO && have_affine == DO) {
/*
            XtSetSensitive(pushButtonOpenSplicer, True);
*/
            XtSetSensitive(pushButtonOpenSpliceTable, True);
          }
          XtSetSensitive(pushButtonClearSplice, False);
        }
        if(cflag == ALL || cflag == SPLICETABLE || cflag == COREDATA)
        {
          if(splicertable.data)
          {
            free(splicertable.data);
            splicertable.data=NULL;
          }
          splicertable.data_cnt = 0;
          splicertable.alloc_cnt = 0;
          have_splice_table = DONT;
          if(have_cores == DO && have_affine == DO) {
/*
            XtSetSensitive(pushButtonOpenSplicer, True);
*/
            XtSetSensitive(pushButtonOpenSpliceTable, True);
          }
        }
        if(cflag==ALL || cflag==ANCDATA)
        {
          have_ancill = DONT;
          if(ancill.id) {
            for(i=0; i<ancill.id_alloc; ++i) {
              free(ancill.id[i]);
              ancill.id[i] = NULL;
            }
            free(ancill.id);
            ancill.id = NULL;
          }
          if(ancill.value) {
            for(i=0; i<ancill.data_alloc; ++i) {
              free(ancill.value[i]->data);
              ancill.value[i]->data = NULL;
              free(ancill.value[i]);
              ancill.value[i] = NULL;
            }
            free(ancill.value);
            ancill.value = NULL;
          }
          ancill.data_alloc = 0;
          ancill.id_alloc = 0;
          if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCleanUp 1");
          (void)time(&tp);
          (void)sprintf(charptr,"\n%s  Ancillary data has been cleared.",ctime(&tp));
          XmTextReplace(textReport,0,XmTextGetLastPosition(textReport),charptr);
          free(charptr);
          XtSetSensitive(pushButtonOpenAnc, True);
          XtSetSensitive(pushButtonClearAncillary, False);
          xmstring = XmStringCreateSimple("1");
          XtVaSetValues(labelColorAncillaryLog1, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          xmstring = XmStringCreateSimple("2");
          XtVaSetValues(labelColorAncillaryLog2, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          xmstring = XmStringCreateSimple("3");
          XtVaSetValues(labelColorAncillaryLog3, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
        }

        if(cflag==ALL || cflag==LOGDATA || cflag == COREDATA)
        {
          /* set the auto corr offset and squish params to defualts
          */
          if((string=(char*)calloc(50,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("Sa");
          (void)sprintf(string, "%.2f", STRETCH);
          XtVaSetValues(textFieldAutoCorrSquish1, XmNvalue, string, NULL);
          (void)sprintf(string, "%.2f", SQUISH);
          XtVaSetValues(textFieldAutoCorrSquish2, XmNvalue, string, NULL);
          (void)sprintf(string, "%.2f", INC_SQUISH);
          XtVaSetValues(textFieldAutoCorrSquishInterval, XmNvalue, string, NULL);
          (void)sprintf(string, "%.2f", NEG_OFFSET);
          XtVaSetValues(textFieldAutoCorrOffset1, XmNvalue, string, NULL);
          (void)sprintf(string, "%.2f", POS_OFFSET);
          XtVaSetValues(textFieldAutoCorrOffset2, XmNvalue, string, NULL);
          (void)sprintf(string, "%.2f", INC_OFFSET);
          XtVaSetValues(textFieldAutoCorrOffsetInterval, XmNvalue, string, NULL);
          free(string);

          /* turn off the auto pair option
          */
          autopair = NO;
          xmstring = XmStringCreateSimple("Auto Pair Off");
          XtVaSetValues(labelAutoPair, XmNlabelString, xmstring, NULL);
          XtVaSetValues(pushButtonPairCoreLog, XmNlabelString, xmstring, NULL);
          XmStringFree(xmstring);
          XtSetSensitive(pushButtonPairCoreLog, False);
          XtSetSensitive(labelAutoPair, False);

          XtSetSensitive(pushButtonSelectCore, False);
          XtSetSensitive(pushButtonCancelCore, False);
          XtSetSensitive(pushButtonClearTie, False);
          XtSetSensitive(pushButtonClearLastTieSaved, False);
          XtSetSensitive(pushButtonShowHideTieList, False);
          XtSetSensitive(pushButtonAddTieToList, False);
          XtSetSensitive(toggleButtonAddTieToCore, False);
          XtSetSensitive(toggleButtonAddTieToAll, False);
          XtSetSensitive(pushButtonOptimizeTie, False);
          XtSetSensitive(pushButtonAdjustToTiesParameters, False);
          XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
          XtSetSensitive(pushButtonSaveChanges, False);
          XtSetSensitive(pushButtonUndoSaveChange, False);
        }

        if(cflag==ALL || (cflag==LOGDATA && have_splice == DONT) || cflag == COREDATA || cflag == SPLICEDATA)
        {
          /* if there are any ties in ties free them but don't free ties
          */
          if(ties) {
            if(ties->tie) {
              for(i=0; i<ties->alloc_cnt; ++i) {
                if(ties->tie[i]) {
                  free(ties->tie[i]);
                  ties->tie[i] = NULL;
                }
              }
              free(ties->tie);
              ties->tie = NULL;
            }
            if(ties->temp_equivlogdep) {
              free(ties->temp_equivlogdep);
              ties->temp_equivlogdep = NULL;
            }
          }
          ties->nties = 0;
          ties->alloc_cnt = 0;
          ties->core_alloc = 0;
          XmListDeleteAllItems(listTieList);
        }

        if(((cflag==ALL || cflag == COREDATA) && eqlogdepth.read_equil_log == YES) || cflag == ELDTABLE)
        {
          for(i=0; i<eqlogdepth.numhole; ++i) {
            for(j=0; j<eqlogdep_core_alloc; ++j) {
              if(eqlogdepth.hole[i]->core[j]->nties > 0) {
                for(k=0; k<eqlogdepth.hole[i]->core[j]->nties; ++k) {
                  free(eqlogdepth.hole[i]->core[j]->tie[k]);
                  eqlogdepth.hole[i]->core[j]->tie[k] = NULL;
                }
                free(eqlogdepth.hole[i]->core[j]->tie);
                eqlogdepth.hole[i]->core[j]->tie = NULL;
              }
              free(eqlogdepth.hole[i]->core[j]);
              eqlogdepth.hole[i]->core[j] = NULL;
            }
            if(eld.hole_eqt[i]) {
              if(eld.hole_eqt[i]->nties > 0) {
                for(j=0; j<eld.hole_eqt[i]->nties; ++j) {
                  free(eld.hole_eqt[i]->tie[j]);
                  eld.hole_eqt[i]->tie[j] = NULL;
                }
                free(eld.hole_eqt[i]->tie);
                eld.hole_eqt[i]->tie = NULL;
              }
              free(eld.hole_eqt[i]);
              eld.hole_eqt[i] = NULL;
            }
            free(eqlogdepth.hole[i]->core);
            eqlogdepth.hole[i]->core = NULL;
            free(eqlogdepth.hole[i]);
            eqlogdepth.hole[i] = NULL;
          }
          free(eqlogdepth.hole);
          eqlogdepth.hole = NULL;
          if(eld.hole_eqt) {
            free(eld.hole_eqt);
            eld.hole_eqt = NULL;
          }
        }

        if(cflag == ALL) {
          XtSetSensitive(pushButtonClearAll, False);
        }
        else if(have_cores + have_log + have_ancill + have_strat + have_splice < 2) {
          XtSetSensitive(pushButtonClearAll, False);
        }
        if(cflag == COREDATA &&  have_log == DO) {
          if(eqlogdepth.did_log_mudline_adj == YES && eqlogdepth.autocorr_log_mudline_offset != 0.0) {
            for(i=0; i<logset_cnt; ++i) {
              for(j=0; j<logdata[i]->data_num; ++j) {
                logdata[i]->value[j]->depth = logdata[i]->value[j]->depth - eqlogdepth.autocorr_log_mudline_offset; 
              }
            }
          }
          eqlogdepth.did_log_mudline_adj = NO;
          eqlogdepth.autocorr_log_mudline_offset = 0.0;
        }

        if(cflag == ALL) {
          XtVaSetValues(scaleDrawingAreas, XmNmaximum, 100, NULL);
          XtVaSetValues(scaleDrawingAreas, XmNvalue, 0, NULL);
        }
        else {
        }

        if(have_log == DO && have_cores == DO && have_splice == DONT) {
            XtSetSensitive(textFieldSelectHole, True);
            XtSetSensitive(textFieldSelectCore, True);
            XtSetSensitive(pushButtonSelectCore, True);
            XtSetSensitive(pushButtonCancelCore, True);
        }
        else {
            XtSetSensitive(textFieldSelectHole, False);
            XtSetSensitive(textFieldSelectCore, False);
            XtSetSensitive(pushButtonSelectCore, False);
            XtSetSensitive(pushButtonCancelCore, False);
        }

        if(XtIsManaged(XtParent(drawingAreaCores))) {
          DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
        }

        if(XtIsManaged(XtParent(drawingAreaLog))) {
          DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
        }
}


void	SaOpenCoreInfo()
{
        XmString        *items;
        int     string_len,i,tmp_len,len;
        char    *string,*charptr,*tmp_buf, *filename, *openfilter;
        String  data_col;
        Widget  tmp_widget;

        /*SUPPRESS 594*/
/* get file names */
        XtVaGetValues(listOpenSelected, XmNitemCount,&openfileinfo.num_in_files, XmNitems,&items, NULL);
/* no data files selected */
        if(openfileinfo.num_in_files<=0)
        {
                info.error=False;
                info.fatal=False;
		info.default_button=XmDIALOG_OK_BUTTON;
		info.type=XmDIALOG_ERROR;
		info.helpfile=NULL;
                if((info.message=(char*)calloc(50,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 1");
                (void)sprintf(info.message,"You must select at least one data file!");
                SaManageMessage(info);
                return;
        }
        if((openfileinfo.in_file_names=(char**)calloc(openfileinfo.num_in_files,sizeof(char*)))==NULL)
                SaBadMemAlloc("SaCoreOpenInfo 2");
        for(i=0;i<openfileinfo.num_in_files;i++)
	{
  		if(!XmStringGetLtoR(items[i],XmFONTLIST_DEFAULT_TAG,&filename))
  		{
        		info.error=True;
        		info.fatal=False;
        		if((info.message=(char*)calloc(200,CHAR_SIZE))==NULL)
                		SaBadMemAlloc("SaCoreOpenInfo 1b");
        		else
                		(void)sprintf(info.message,"Unable to convert selected items.");
        		SaManageMessage(info);
  		}
                else
                {
                  openfilter = XmTextGetString(textFieldOpenFilter);
                  string_len = (int)strlen(openfilter);
                  if(strcmp(&openfilter[string_len-1], "*") == 0) {
                    *strrchr(openfilter,'*')= '\0';
                  }
                  string_len = (int)strlen(openfilter);
                  if(strcmp(&openfilter[string_len-1], "/") == 0) {
                    *strrchr(openfilter,'/')= '\0';
                  }
                  if((openfileinfo.in_file_names[i]=(char*)calloc((int)strlen(openfilter)+(int)strlen(filename)+5,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaOpenCoreInfo 1c");

                  (void)sprintf(openfileinfo.in_file_names[i],"%s/%s",openfilter,filename);
                  XtFree(openfilter);
                  XtFree(filename);
                }
	}

/* get format & type */
        XtVaGetValues(optionMenuOpenFormat,XmNmenuHistory,&tmp_widget,NULL);
        XtVaGetValues(tmp_widget,XmNuserData,&openfileinfo.in_data_format,NULL);
        if(openfileinfo.in_data_format == ODPOTHER) {
          if(XmToggleButtonGetState(toggleButtonODPOtherFormat1) == True) {
            openfileinfo.in_data_format = ODPOTHER1;
          }
          else if(XmToggleButtonGetState(toggleButtonODPOtherFormat2) == True) {
            openfileinfo.in_data_format = ODPOTHER2;
          }
          else if(XmToggleButtonGetState(toggleButtonODPOtherFormat3) == True) {
            openfileinfo.in_data_format = ODPOTHER3;
          }
          else if(XmToggleButtonGetState(toggleButtonODPOtherFormat4) == True) {
            openfileinfo.in_data_format = ODPOTHER4;
          }
          else if(XmToggleButtonGetState(toggleButtonODPOtherFormat5) == True) {
            openfileinfo.in_data_format = ODPOTHER5;
          }
          else {
            openfileinfo.in_data_format = ODPOTHER1;
          }
          data_col = XmTextGetString(textFieldODPOtherFormatValueCol);
          openfileinfo.other_format_value_col = atoi(data_col);
          XtFree(data_col);
        }
        if(openfileinfo.in_data_format == JANUS) {
          if(XmToggleButtonGetState(toggleButtonJanusClean) == True) {
            openfileinfo.in_data_format = JANUSCLEAN;
          }
          else if(XmToggleButtonGetState(toggleButtonJanusOrig) == True) {
            openfileinfo.in_data_format = JANUSORIG;
          }
        }

        XtVaGetValues(optionMenuOpenType,XmNmenuHistory,&tmp_widget,NULL);
        XtVaGetValues(tmp_widget,XmNuserData,&openfileinfo.in_data_type,NULL);
 
/* make tmp buffer */
        tmp_len=1000;
        if((tmp_buf=(char*)calloc(tmp_len,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaCoreOpenInfo 5");
/* create output text */
        string_len=2000;
        if((string=(char*)calloc(string_len,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SPOpenOKCB 6");
        (void)sprintf(string,"Proceed as follows:\n\nSagan Input Files:\n");
        for(i=0;i<openfileinfo.num_in_files;i++)
        {
                if((int)strlen(string)+(int)strlen(openfileinfo.in_file_names[i])+2 >= string_len)
                {
                        string_len+=500;
                        if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaCoreOpenInfo 7");
                        else
                                string=charptr;
                }
                (void)strcat(string,openfileinfo.in_file_names[i]);
                (void)strcat(string,"\n");
        }
        if((len=50+(int)strlen(type[openfileinfo.in_data_type]))>tmp_len)
        {
                tmp_len=len;
                if((charptr=(char*)realloc(tmp_buf,tmp_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 8");
                else
                        tmp_buf=charptr;
        }
        (void)sprintf(tmp_buf,"\nData Type: %s\n",type[openfileinfo.in_data_type]);
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 9");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        if((len=50+(int)strlen(format[openfileinfo.in_data_format]))>tmp_len)
        {
                tmp_len=len;
                if((charptr=(char*)realloc(tmp_buf,tmp_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 10");
                else
                        tmp_buf=charptr;
        }
        (void)sprintf(tmp_buf,"\nData File Format: %s\n",format[openfileinfo.in_data_format]);
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 11");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        if((len=50+(int)strlen(smooth_units[smooth.width_units])+(int)strlen(plot_display[smooth.plot]))>tmp_len)
        { 
                tmp_len=len;
                if((charptr=(char*)realloc(tmp_buf,tmp_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
                else
                        tmp_buf=charptr;
        }
        if(smooth.method==GAUSSIAN)
                (void)sprintf(tmp_buf,"\nGaussian Smoothing: %d %s\nPlot Display: %s\n",
                                        smooth.width,smooth_units[smooth.width_units],
                                        plot_display[smooth.plot]);
        else
                (void)sprintf(tmp_buf,"\nNo data smoothing.\n");
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("a");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        if((len=300+(int)strlen(cull.filename))>tmp_len)
        {
                tmp_len=len;
                if((charptr=(char*)realloc(tmp_buf,tmp_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 16");
                else
                        tmp_buf=charptr;
        }
        if(cull.method == CULL_PARAMETER_AND_TABLE) {
          (void)sprintf(tmp_buf,"\nCull Table: %s\n",cull.filename);
          (void)strcat(tmp_buf, "Culling will be done with user set parameters.\n");
        }
        else if(cull.method==CULL_TABLE)
                (void)sprintf(tmp_buf,"\nCull Table: %s\n",cull.filename);
        else if(cull.method==CULL_PARAMETERS)
                (void)sprintf(tmp_buf,"\nCulling will be done with user set parameters.\n");
        else
                (void)sprintf(tmp_buf,"\nNo culling by data values.\n");
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 17");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        (void)sprintf(tmp_buf,"Top %.1f cm will be removed from core.\n",cull.top);
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 18");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        if(cull.by_core_num)
                (void)sprintf(tmp_buf,"Only cores <= %d will be used.\n",cull.max_core);
        else
                (void)sprintf(tmp_buf,"All cores will be used.\n");
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)

        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 19");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        if(decimate_value>1)
                (void)sprintf(tmp_buf,"\nDecimation: every %d point will be used.\n",decimate_value);
        else
                (void)sprintf(tmp_buf,"\nEvery data point will be used.\n");
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 20");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        (void)sprintf(tmp_buf,"\nNote: multiple data values will be averaged.\n");
        if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
        {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 21");
                else
                        string=charptr;
        }
        (void)strcat(string,tmp_buf);
        if(affinetable.read_affine == YES) {
          (void)sprintf(tmp_buf,"\nAffine Table: %s.\n", affinetable.filename);
          if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
          {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 21");
                else
                        string=charptr;
          }
          (void)strcat(string,tmp_buf);
        }
        if(eqlogdepth.read_equil_log == YES) {
          (void)sprintf(tmp_buf,"\nEquivalent Log Depth Table: %s.\n", eqlogdepth.filename);
          if((int)strlen(string)+(int)strlen(tmp_buf)+2 >=string_len)
          {
                string_len+=500;
                if((charptr=(char*)realloc(string,string_len*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaCoreOpenInfo 21");
                else
                        string=charptr;
          }
          (void)strcat(string,tmp_buf);
        }
 
/* display stringrmation window */
        XmTextSetString(textOpenInfo,string);
        XtManageChild(formOpenInfo);
        free(string);
        free(tmp_buf);
}


void SaExit()
{
        FILE    *defaults_ptr;

/* write defaults file */
        if((defaults_ptr=fopen(defaults_filename,"w"))==NULL)
                (void)fprintf(stderr,"sagan: Warning: cannot open defaults file %s for writing.\n", defaults_filename);
        else
        {
                (void)fprintf(stderr,"sagan: set defaults written to %s.\n",defaults_filename);
                Sawrite_defaults(defaults_ptr); /* write defaults file */
                (void)fclose(defaults_ptr);
        }
  
	fprintf(stderr,"Goodbye!\n");
	exit(0);
}

int SaGetEquationValues(equ)
Equation        *equ;
{
        int     err, sign1, sign2, join;
        char    *string, *string2, s1[3], s2[3];
        float   value1, value2;
        void SaSetEquationValues();
        Widget  *tmp_widget;

/* get text values */
        charptr=XmTextGetString(textFieldCullValue1);
        value1=(float)atof(charptr);
        XtFree(charptr);
        charptr=XmTextGetString(textFieldCullValue2);
        value2=(float)atof(charptr);
        XtFree(charptr);
/* get signs */
        sign1=SaGetSign(equationMenuCull1);
        sign2=SaGetSign(equationMenuCull2);
/* get join */
        XtVaGetValues(optionMenuCullJoin,XmNmenuHistory,&tmp_widget,NULL);
        if(*tmp_widget==pushButtonCullOnly)
                join=ONLY;
        else
                join=OR;

  /*check that values make sense 
  */
  err = 0;
  if(join == OR) {
    if((sign1 == LESS_THAN || sign1 == LESS_THAN_OR_EQUAL_TO) &&
       (sign2 == LESS_THAN || sign2 == LESS_THAN_OR_EQUAL_TO)) {
      err = 1;
    }
    else if((sign1 == GREATER_THAN || sign1 == GREATER_THAN_OR_EQUAL_TO) &&
            (sign2 == GREATER_THAN || sign2 == GREATER_THAN_OR_EQUAL_TO)) {
      err = 1;
    }
    else if((sign1 == LESS_THAN || sign1 == LESS_THAN_OR_EQUAL_TO) &&
            (sign2 == GREATER_THAN || sign2 == GREATER_THAN_OR_EQUAL_TO)) {
      if(value1 >= value2) {
        err = 1;
      }
    }
    else if((sign1 == GREATER_THAN || sign1 == GREATER_THAN_OR_EQUAL_TO) &&
            (sign2 == LESS_THAN || sign2 == LESS_THAN_OR_EQUAL_TO)) {
      if(value1 <= value2) {
        err = 1;
      }
    }
    else if((sign1 == LESS_THAN || sign1 == LESS_THAN_OR_EQUAL_TO) &&
            sign2 == EQUAL_TO) {
      if(value2 <= value1) {
        err = 1;
      }
    }
    else if((sign1 == GREATER_THAN || sign1 == GREATER_THAN_OR_EQUAL_TO) &&
            sign2 == EQUAL_TO) {
      if(value2 >= value1) {
        err = 1;
      }
    }
    else if((sign2 == LESS_THAN || sign2 == LESS_THAN_OR_EQUAL_TO) &&
            sign1 == EQUAL_TO) {
      if(value1 <= value2) {
        err = 1;
      }
    }
    else if((sign2 == GREATER_THAN || sign2 == GREATER_THAN_OR_EQUAL_TO) &&
            sign1 == EQUAL_TO) {
      if(value1 >= value2) {
        err = 1;
      }
    }
  }
  
  if(err == 0) {
    equ->value1 = value1;
    equ->value2 = value2;
    equ->sign1 = sign1;
    equ->sign2 = sign2;
    equ->join = join;
  }
  else {
    SaSetEquationValues(equ);
    if((string = (char *)calloc(500,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sa");
    (void)sprintf(string, "The Cull Parameter Equation does not make sense.\n");
    if(join == OR) {
      if(sign1 == LESS_THAN) {
        (void)sprintf(s1, "<");
      }
      else if(sign1 == LESS_THAN_OR_EQUAL_TO) {
        (void)sprintf(s1, "<=");
      }
      else if(sign1 == GREATER_THAN) {
        (void)sprintf(s1, ">");
      }
      else if(sign1 == GREATER_THAN_OR_EQUAL_TO) {
        (void)sprintf(s1, ">=");
      }
      else if(sign1 == EQUAL_TO) {
        (void)sprintf(s1, "=");
      }
      else {
        (void)sprintf(s1, "");
      }
      if(sign2 == LESS_THAN) {
        (void)sprintf(s2, "<");
      }
      else if(sign2 == LESS_THAN_OR_EQUAL_TO) {
        (void)sprintf(s2, "<=");
      }
      else if(sign2 == GREATER_THAN) {
        (void)sprintf(s2, ">");
      }
      else if(sign2 == GREATER_THAN_OR_EQUAL_TO) {
        (void)sprintf(s2, ">=");
      }
      else if(sign2 == EQUAL_TO) {
        (void)sprintf(s2, "=");
      }
      else {
        (void)sprintf(s2, "");
      }
      if((string2 = (char *)calloc(200,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sa");
      (void)sprintf(string2, "\nCull core value %s %.3f  OR  core value %s %.3f\n", s1, value1, s2, value2);
      (void)strcat(string, string2);
      free(string2);
      string2 = NULL;
    }
    (void)strcat(string, "\nReplacing with previous settings.");
    SaManageWarningBox(string);
    free(string);
    string = NULL;
  }
  return err;
}

int     SaGetSign(equ_widget)
EquationMenuData        equ_widget;
{
        Widget  tmp_widget;
        int     value;

        XtVaGetValues(equ_widget.pulldownMenuEquation,XmNmenuHistory,&tmp_widget,NULL);
        if(tmp_widget==equ_widget.pushButtonLessThan)
                value=LESS_THAN;
        else if(tmp_widget==equ_widget.pushButtonLessThanEqual)
                value=LESS_THAN_OR_EQUAL_TO;
        else if(tmp_widget==equ_widget.pushButtonGreaterThan)
                value=GREATER_THAN;
        else if(tmp_widget==equ_widget.pushButtonGreaterThanEqual)
                value=GREATER_THAN_OR_EQUAL_TO;
        else
                value=EQUAL_TO;
        return(value);
}

int SaGetEquationValuesLog(equ)
Equation        *equ;
{
        int     err, sign1, sign2, join;
        char    *string, *string2, s1[3], s2[3];   
        float   value1, value2;
        Widget  *tmp_widget;
        void    SaSetEquationValuesLog();

/* get text values */
        charptr=XmTextGetString(textFieldCullLogValue1);
        value1=(float)atof(charptr);
        XtFree(charptr);
        charptr=XmTextGetString(textFieldCullLogValue2);
        value2=(float)atof(charptr);
        XtFree(charptr);
/* get signs */
        sign1=SaGetSignLog(equationMenuLogCull1);
        sign2=SaGetSignLog(equationMenuCullLog2);
/* get join */
        XtVaGetValues(optionMenuCullLogJoin,XmNmenuHistory,&tmp_widget,NULL);
        if(*tmp_widget==pushButtonCullLogOnly)
                join=ONLY;
        else
                join=OR;

  /*check that values make sense
  */
  err = 0;
  if(join == OR) {
    if((sign1 == LESS_THAN || sign1 == LESS_THAN_OR_EQUAL_TO) &&
       (sign2 == LESS_THAN || sign2 == LESS_THAN_OR_EQUAL_TO)) {
      err = 1;
    }
    else if((sign1 == GREATER_THAN || sign1 == GREATER_THAN_OR_EQUAL_TO) &&
            (sign2 == GREATER_THAN || sign2 == GREATER_THAN_OR_EQUAL_TO)) {
      err = 1;
    }
    else if((sign1 == LESS_THAN || sign1 == LESS_THAN_OR_EQUAL_TO) &&
            (sign2 == GREATER_THAN || sign2 == GREATER_THAN_OR_EQUAL_TO)) {
      if(value1 >= value2) {
        err = 1;
      }
    }
    else if((sign1 == GREATER_THAN || sign1 == GREATER_THAN_OR_EQUAL_TO) &&
            (sign2 == LESS_THAN || sign2 == LESS_THAN_OR_EQUAL_TO)) {
      if(value1 <= value2) {
        err = 1;
      }
    }
    else if((sign1 == LESS_THAN || sign1 == LESS_THAN_OR_EQUAL_TO) &&
            sign2 == EQUAL_TO) {
      if(value2 <= value1) {
        err = 1;
      }
    }
    else if((sign1 == GREATER_THAN || sign1 == GREATER_THAN_OR_EQUAL_TO) &&
            sign2 == EQUAL_TO) {
      if(value2 >= value1) {
        err = 1;
      }
    }
    else if((sign2 == LESS_THAN || sign2 == LESS_THAN_OR_EQUAL_TO) &&
            sign1 == EQUAL_TO) {
      if(value1 <= value2) {
        err = 1;
      }
    }
    else if((sign2 == GREATER_THAN || sign2 == GREATER_THAN_OR_EQUAL_TO) &&
            sign1 == EQUAL_TO) {
      if(value1 >= value2) {
        err = 1;
      }
    }
  }

  if(err == 0) {
    equ->value1 = value1;
    equ->value2 = value2;
    equ->sign1 = sign1;
    equ->sign2 = sign2;
    equ->join = join;
  }
  else {
    SaSetEquationValuesLog(equ);
    if((string = (char *)calloc(500,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sa");
    (void)sprintf(string, "The Cull Parameter Equation does not make sense.\n");
    if(join == OR) {
      if(sign1 == LESS_THAN) {
        (void)sprintf(s1, "<");
      }
      else if(sign1 == LESS_THAN_OR_EQUAL_TO) {
        (void)sprintf(s1, "<=");
      }
      else if(sign1 == GREATER_THAN) {
        (void)sprintf(s1, ">");
      }
      else if(sign1 == GREATER_THAN_OR_EQUAL_TO) {
        (void)sprintf(s1, ">=");
      }
      else if(sign1 == EQUAL_TO) {
        (void)sprintf(s1, "=");
      }
      else {
        (void)sprintf(s1, "");
      }
      if(sign2 == LESS_THAN) {
        (void)sprintf(s2, "<");
      }
      else if(sign2 == LESS_THAN_OR_EQUAL_TO) {
        (void)sprintf(s2, "<=");
      }
      else if(sign2 == GREATER_THAN) {
        (void)sprintf(s2, ">");
      }
      else if(sign2 == GREATER_THAN_OR_EQUAL_TO) {
        (void)sprintf(s2, ">=");
      }
      else if(sign2 == EQUAL_TO) {
        (void)sprintf(s2, "=");
      }
      else {
        (void)sprintf(s2, "");
      }
      if((string2 = (char *)calloc(200,CHAR_SIZE))==NULL)
                SaBadMemAlloc("Sa");
      (void)sprintf(string2, "\nCull log value %s %.3f  OR  log value %s %.3f\n", s1, value1, s2, value2);
      (void)strcat(string, string2);
      free(string2);
      string2 = NULL;
    }
    (void)strcat(string, "\nReplacing with previous settings.");
    SaManageWarningBox(string);
    free(string);
    string = NULL;
  }
  return err;
}

int     SaGetSignLog(equ_widget)
EquationMenuData        equ_widget;
{
        Widget  tmp_widget;
        int     value;

        XtVaGetValues(equ_widget.pulldownMenuEquation,XmNmenuHistory,&tmp_widget,NULL);
        if(tmp_widget==equ_widget.pushButtonLessThan)
                value=LESS_THAN;
        else if(tmp_widget==equ_widget.pushButtonLessThanEqual)
                value=LESS_THAN_OR_EQUAL_TO;
        else if(tmp_widget==equ_widget.pushButtonGreaterThan)
                value=GREATER_THAN;
        else if(tmp_widget==equ_widget.pushButtonGreaterThanEqual)
                value=GREATER_THAN_OR_EQUAL_TO;
        else
                value=EQUAL_TO;
        return(value);
}

void SaInitialize(argc,argv)
int     argc;
char    **argv;
{
        char    *tmpfile;
        FILE    *fptmp,*defaults_ptr;
        int     error=FALSE,i,path_len;

/* initialize values */
        report.lineno=2;
        if((defaults_filename= (char *)calloc(25,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaInitialize 1");
        (void)strcpy(defaults_filename,".sagandefaults");
/* get current path */
        tmpfile=NULL;
        tmpfile=tmpnam(tmpfile);
        if((charptr=(char*)calloc((int)strlen(tmpfile)*2+50,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaInitialize 2");
        (void)sprintf(charptr,"pwd|wc -c>%s;pwd>>%s",tmpfile,tmpfile);
        (void)system(charptr);
        free(charptr);
        fptmp=fopen(tmpfile,"r");
        (void)fscanf(fptmp,"%d",&path_len);
        if((current_path = (char*)calloc(path_len+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaInitialize 3");
        (void)fscanf(fptmp,"%s",current_path);
        fclose(fptmp);
        (void)remove(tmpfile);
/* make preview input path large, it is going to be used in such a way
   that calloc/recalloc will be impractable and its size could vary greatly
*/
        if((preview_input_path = (char*)calloc(1000,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaInitialize 3a");
        (void)sprintf(preview_input_path, "%s", current_path);

/* loop through arguments */
        if(argc>1)
        {
                for(i=1;i<argc && !error;i++)
                {
                        if(!(int)strcmp(argv[i],"-f"))       /* defaults file */
                        {
                                if((charptr=(char*)realloc((char*)defaults_filename,((int)strlen(argv[++i])+1)*CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaInitialize 4");
                                else
                                        defaults_filename=charptr;
                                (void)strcpy(defaults_filename,argv[i]);
                        }
                        else
                                error=True;
                
                }
        }
/* read defaults file */
        if(!Saread_defaults(defaults_filename))
        {
                (void)fprintf(stderr,"sagan: Warning: cannot read defaults file %s.\n", defaults_filename);
                (void)fprintf(stderr,"\t\tUsing program defaults.\n");
        }
        else
                (void)fprintf(stderr,"sagan: Using defaults file %s.\n",defaults_filename);
}

void SaManageMessage(tmp)
Information	tmp;
{
        XmString        xmstring;
        XmFontList      fontlist;

	XtUnmanageChild(messageBoxInfo);
	XtManageChild(XmMessageBoxGetChild(messageBoxInfo,XmDIALOG_HELP_BUTTON));
	XtManageChild(XmMessageBoxGetChild(messageBoxInfo,XmDIALOG_OK_BUTTON));
	XtManageChild(XmMessageBoxGetChild(messageBoxInfo,XmDIALOG_CANCEL_BUTTON));

        xmstring=XmCvtCTToXmString(tmp.message);
        XtVaGetValues(messageBoxInfo,XmNlabelFontList,&fontlist,NULL);
	XtVaSetValues(messageBoxInfo,XmNmessageString,xmstring,
			XmNdialogType,tmp.type,
			XmNwidth,XmStringWidth(fontlist,xmstring)+150,
                        XmNx, 200, XmNy, 200,
			NULL);
/* this keeps crashing ???
	XtVaSetValues(messageBoxInfo,XmNdefaultButton,tmp.default_button,NULL);
*/
        if(tmp.type==XmDIALOG_WARNING || tmp.type==XmDIALOG_ERROR|| tmp.type==XmDIALOG_INFORMATION)
                XtUnmanageChild(XmMessageBoxGetChild(messageBoxInfo,XmDIALOG_CANCEL_BUTTON));
	if(tmp.helpfile==NULL ||(int)strlen(tmp.helpfile)<=0)
                XtUnmanageChild(XmMessageBoxGetChild(messageBoxInfo,XmDIALOG_HELP_BUTTON));
        XmStringFree(xmstring);
        XtManageChild(messageBoxInfo);
        XtPopup(XtParent(messageBoxInfo),XtGrabNone);
	free(tmp.message);
}

void SaManageWarningBox(string)
char    *string;
{
        info.fatal=False;
        info.error=True;
        info.caller=MANAGEWARNINGBOX;
        info.type=XmDIALOG_WARNING;
        info.default_button=XmDIALOG_OK_BUTTON;
        if((info.message=(char*)calloc((int)strlen(string)+2,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaManageWarningBox 1");
        (void)strcat(info.message,string);
        SaManageMessage(info);
}


void SaSetEquationValues(equ)
Equation        equ;
{
        Widget  *tmp_widget;

/* set text values */
        if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaSetEquationValues 1");
        (void)sprintf(charptr,"%.1f",equ.value1);
        XmTextSetString(textFieldCullValue1,charptr);
        (void)sprintf(charptr,"%.2f",equ.value2);
        XmTextSetString(textFieldCullValue2,charptr);
        free(charptr);

/* set signs */
        SaSetSign(equationMenuCull1,equ.sign1);
        SaSetSign(equationMenuCull2,equ.sign2);
/* set join */
        if(equ.join==ONLY)
        {
                tmp_widget=&pushButtonCullOnly;
                XtSetSensitive(equationMenuCull2.EquationMenu,False);
                XtSetSensitive(textFieldCullValue2,False);
        }
        else
        {
                tmp_widget=&pushButtonCullOr;
                XtSetSensitive(equationMenuCull2.EquationMenu,True);
                XtSetSensitive(textFieldCullValue2,True);
        }
        XtVaSetValues(optionMenuCullJoin,XmNmenuHistory,*tmp_widget,NULL);
}

void SaSetEquationValuesLog(equ)
Equation        equ;
{
        Widget  *tmp_widget;

/* set text values */
        if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                SaBadMemAlloc("SaSetEquationValues 1");
        (void)sprintf(charptr,"%.1f",equ.value1);
        XmTextSetString(textFieldCullLogValue1,charptr);
        (void)sprintf(charptr,"%.2f",equ.value2);
        XmTextSetString(textFieldCullLogValue2,charptr);
        free(charptr);

/* set signs */
        SaSetSign(equationMenuLogCull1,equ.sign1);
        SaSetSign(equationMenuCullLog2,equ.sign2);
/* set join */
        if(equ.join==ONLY)
        {
                tmp_widget=&pushButtonCullLogOnly;
                XtSetSensitive(equationMenuCullLog2.EquationMenu,False);
                XtSetSensitive(textFieldCullLogValue2,False);
        }
        else
        {
                tmp_widget=&pushButtonCullLogOr;
                XtSetSensitive(equationMenuCullLog2.EquationMenu,True);
                XtSetSensitive(textFieldCullLogValue2,True);
        }
        XtVaSetValues(optionMenuCullLogJoin,XmNmenuHistory,*tmp_widget,NULL);
}

void SaSetSign(equ_widget,sign)
EquationMenuData        equ_widget;
int     sign;
{
        Widget  *tmp_widget;
        int     value;

        if(sign==LESS_THAN)
                tmp_widget=&equ_widget.pushButtonLessThan;
        else if(sign==LESS_THAN_OR_EQUAL_TO)
                tmp_widget=&equ_widget.pushButtonLessThanEqual;
        else if(sign==GREATER_THAN)
                tmp_widget=&equ_widget.pushButtonGreaterThan;
        else if(sign==GREATER_THAN_OR_EQUAL_TO)
                tmp_widget=&equ_widget.pushButtonGreaterThanEqual;
        else
                tmp_widget=&equ_widget.pushButtonEqual;
        XtVaSetValues(equ_widget.EquationMenu,XmNmenuHistory,*tmp_widget,NULL);
}


#define filesearch_string "ls -lAd1 %s|awk '{ if($1 ~ /^-/) { num=$NF; type=\"FILE\" }else if($1 ~ /^l/) { num=$(NF-2); type=\"LINK\" }else if($1 ~ /^d/) { num=$NF; type=\"DIR\" } nw=split(num,word,\"/\"); print type,word[nw] }'>%s\n"
#define dirsearch_string "ls -lA1 %s|awk '{ if($1 ~ /^-/) { num=$NF; type=\"FILE\" }else if($1 ~ /^l/) { num=$(NF-2); type=\"LINK\" }else if($1 ~ /^d/) { num=$NF; type=\"DIR\" } nw=split(num,word,\"/\"); print type,word[nw] }'>%s\n"

void SaUpdateOpen(widget_type)
int	widget_type;
{
        char            *tmpfile=NULL, filetype[13], *filename;
        FILE            *fptr;
        int             numfiles=0,numdirs=0,alloc_file,alloc_dir, len;
        XmString        *filelist,*dirlist,*selected_dir;
        char            *textstring;

        tmpfile=tmpnam(tmpfile);
        if(widget_type==FILE_FILTER) /* file filter was updated */
        {
                textstring = XmTextGetString(textFieldOpenFilter);
                if((int)strlen(textstring)==0)       /* no filename */
                {
                        if((filename=(char*)calloc((int)strlen(current_path)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 1");
                        (void)sprintf(filename,"%s/*",current_path);
                }
                else if(textstring[0]=='/') /* full path used */
                {
                        if((filename=(char*)calloc((int)strlen(textstring)+2,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 2");
                        (void)strcpy(filename,textstring);
                        *strrchr(textstring,'/')= '\0';
                        if((charptr=(char*)realloc(current_path,((int)strlen(textstring)+2)*CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 3");
                        else
                                current_path=charptr;
                        (void)strcpy(current_path,textstring);
                        /* set current path */
                }
                else if (!strchr(textstring,'/'))       /* only filename */
                {
                        if((filename=(char*)calloc((int)strlen(current_path)+(int)strlen(textstring)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 4");
                        (void)sprintf(filename,"%s/%s",current_path,textstring);
                }
                else                            /* relative path and filename */
                {
                        if((filename=(char*)calloc((int)strlen(current_path)+(int)strlen(textstring)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 5");
                        (void)sprintf(filename,"%s/%s",current_path,textstring);
                        *strrchr(textstring,'/')= '\0';
                        if((charptr=(char*)realloc(current_path,((int)strlen(textstring)+2)*CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 6");
                        else
                                current_path=charptr;
                        (void)strcpy(current_path,textstring);
                }

                len = (int)strlen(filesearch_string) + (int)strlen(filename) + (int)strlen(tmpfile) + 100;
                if((charptr=(char*)calloc(len, CHAR_SIZE))==NULL)
                {
                        SaBadMemAlloc("SaUpdateOpen 7");
                }
                (void)sprintf(charptr,filesearch_string,filename,tmpfile);
                XtFree(textstring);
        }
        else /* directories was updates */
        {
                XtVaGetValues(listOpenDirectory,XmNselectedItems,&selected_dir,XmNselectedItemCount,&numdirs,NULL);
                if(numdirs == 1)
                {
  			if(!XmStringGetLtoR(selected_dir[0],XmFONTLIST_DEFAULT_TAG,&textstring))
  			{
        			info.error=True;
        			info.fatal=False;
        			if((info.message=(char*)calloc(200,CHAR_SIZE))==NULL)
                			SaBadMemAlloc("SaCoreOpenInfo 1b");
        			else
                			(void)sprintf(info.message,"Unable to convert selected items.");
        			SaManageMessage(info);
  			}
                        if(!(int)strcmp(textstring,".."))    /* move back a directory */
                                *strrchr(current_path,'/')='\0';
                        else  if((int)strcmp(textstring,"."))        /* new directory */
                        {
                                if((charptr=(char*)realloc(current_path,((int)strlen(current_path)+(int)strlen(textstring)+2)*CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaUpdateOpen 8");
                                else
                                        current_path=charptr;
                                (void)sprintf(current_path,"%s/%s",current_path,textstring);
                        }
			XtFree(textstring);
                        if((charptr=(char*)calloc((int)strlen(dirsearch_string)+(int)strlen(current_path)+(int)strlen(tmpfile)+100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 9");
                        (void)sprintf(charptr,dirsearch_string,current_path,tmpfile);
                        /* update filter field */
                        if((filename=(char*)calloc((int)strlen(current_path)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateOpen 10");
                        (void)sprintf(filename,"%s/*",current_path);
                        XmTextSetString(textFieldOpenFilter,filename);
                        XmListDeselectAllItems(listOpenDirectory);
                }
                else
                        SaManageWarningBox("Error opening new directory!");
        }
        system(charptr);
        free(charptr);
 
        /* get file and directory names */
        numdirs=0;
        if((filelist=(XmString*)calloc((alloc_file=1000),sizeof(XmString**)))==NULL)
                        SaBadMemAlloc("SaUpdateOpen 10b");
        if((dirlist=(XmString*)calloc((alloc_dir=1000),sizeof(XmString**)))==NULL)
                        SaBadMemAlloc("SaUpdateOpen 10c");
        dirlist[numdirs++]=XmStringCreateLocalized(".");
        dirlist[numdirs++]=XmStringCreateLocalized("..");
        if((fptr=fopen(tmpfile,"r"))!=NULL)
        {
                if((charptr=(char*)realloc(filename,1000*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaUpdateOpen 11");
                else
                        filename=charptr;
                while((int)fscanf(fptr,"%s %s",filetype,filename)!=EOF)
                {
                        if(!(int)strcmp(filetype,"DIR"))             /* directory names */
                        {
                                if(numdirs>=alloc_dir)
                                {
                                        if((charptr=(char*)realloc((char*)dirlist,(alloc_dir+=1000)*sizeof(XmString*)))==NULL)
                                                        SaBadMemAlloc("SaUpdateOpen 11b");
                                        else
                                                dirlist=(XmString*)charptr;
                                }
                                dirlist[numdirs++]=XmStringCreateLocalized(filename);
                        }
                        else if(!(int)strcmp(filetype,"FILE"))       /* file names */
                        {
                                if(numfiles>=alloc_file)
                                {
                                        if((charptr=(char*)realloc((char*)filelist,(alloc_file+=1000)*sizeof(XmString*)))==NULL)
                                                        SaBadMemAlloc("SaUpdateOpen 11c");
                                        else
                                                filelist=(XmString*)charptr;
                                }
                                filelist[numfiles++]=XmStringCreateLocalized(filename);
                        }
                        else if(!(int)strcmp(filetype,"LINK"))       /*  symbolic link name */
                        {
                                if((charptr=(char*)realloc((char*)filename,((int)strlen(filename)+25)*sizeof(XmString*)))==NULL)
                                        SaBadMemAlloc("SaUpdateOpen 11d");
                                else
                                        filename=charptr;
                                (void)strcat(filename," (sym-link)");
                                if(numfiles>=alloc_file)
                                {
                                        if((charptr=(char*)realloc((char*)filelist,(alloc_file+=1000)*sizeof(XmString*)))==NULL)
                                                        SaBadMemAlloc("SaUpdateOpen 11e");
                                        else
                                                filelist=(XmString*)charptr;
                                }
                                filelist[numfiles++]=XmStringCreateLocalized(filename);
                        }
                }
                free(filename);
        }
 
        /* invalid directory */
        if(numfiles <=0 && numdirs<=2)
        {
                if((charptr=(char*)calloc((int)strlen(filename)+50,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaUpdateOpen 12");
                (void)sprintf(charptr,"Invalid or empty filter: %s !",filename);
                SaManageWarningBox(charptr);
                free(charptr);
        }
        /* if files found - put them in file list */
        else
        {
                if(numfiles>0)
			XtVaSetValues(listOpenFiles,XmNitemCount,numfiles,XmNvisibleItemCount,numfiles,XmNitems,filelist,NULL);
                else    /* no files found  - clear list */
                        XmListDeleteAllItems(listOpenFiles);
 
                /* put directories in dir list */
                XtVaSetValues(listOpenDirectory,XmNitemCount,numdirs,XmNvisibleItemCount,numdirs,XmNitems,dirlist,NULL);
                 /* clean up memory used and remove temporary file */
                while(numdirs>0) XmStringFree(dirlist[--numdirs]);
                while(numfiles>0) XmStringFree(filelist[--numfiles]);
        }
        free(dirlist);
        free(filelist);
        (void)remove(tmpfile);

}

void SaUpdateReport(text_mes)
char	*text_mes;
{
	XmTextPosition  last_pos;

	last_pos=XmTextGetLastPosition(textReport);
	XmTextInsert(textReport,last_pos,text_mes);
	last_pos=last_pos+(int)strlen(text_mes);
	XmTextShowPosition(textReport,last_pos);
	if(report.saveflag)
	{
		fputs("\n",report.ptr);
		fputs(text_mes,report.ptr);
	}
}

void SaUpdateFileSelection(how, who, ptrpath)
int     how;
int     who;
char    *ptrpath;
{
        char            *tmpfile=NULL, *textstring, filetype[13], *filename;
        int             numfiles=0,numdirs=0,alloc_file,alloc_dir,i;
        XmString        *filelist, *dirlist, *selected_dir;
        FILE            *fptr;
        Cursor cursor;
        Widget widget;
 
        if(how != FILE_FILTER && how != DIRECTORIES) {
          return;
        }

        if(who == PREVIEWINPUT) {
          widget = formFileSelectionPreviewInput;
        }
        else if(who == SAVECORETOELD) {
          widget = formSaveCoreData;
        }
        else {
          widget = formFileSelection;
        }
        cursor = XCreateFontCursor(XtDisplay(widget), XC_watch);
        if(cursor != BadAlloc && cursor != BadFont && cursor != BadValue) {
          XDefineCursor(XtDisplay(widget), XtWindow(widget), cursor);
          XmUpdateDisplay(widget);
        }

        tmpfile=tmpnam(tmpfile);
        if(how==FILE_FILTER) /* file filter was updated */
        {
                if(who == PREVIEWINPUT) {
                  textstring = XmTextGetString(textFieldFileSelectionPreviewInputFilter);
                }
                else if(who == SAVECORETOELD) {
                  textstring = XmTextGetString(textFieldSaveCoreDataFilter);
                }
                else {
                  textstring = XmTextGetString(textFieldFileSelectionFilter);
                }
                if((int)strlen(textstring)==0)  /* no filename */
                {
                        if((filename=(char*)calloc((int)strlen(ptrpath)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateFileSelction 1");
                        (void)sprintf(filename,"%s/*",ptrpath);
                }
                else if(textstring[0]=='/') /* full path used */
                {
                        if((filename=(char*)calloc((int)strlen(textstring)+2,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateFileSelction 2");
                        (void)strcpy(filename,textstring);
                        *strrchr(textstring,'/')= '\0';
                        (void)strcpy(ptrpath,textstring);
                        /* set current path */
                }
                else if (!strchr(textstring,'/'))       /* only filename */
                {
                        if((filename=(char*)calloc((int)strlen(ptrpath)+(int)strlen(textstring)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateFileSelction 4");
                        (void)sprintf(filename,"%s/%s",ptrpath,textstring);
                }
                else                            /* relative path and filename */
                {
                        if((filename=(char*)calloc((int)strlen(ptrpath)+(int)strlen(textstring)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateFileSelction 5");
                        (void)sprintf(filename,"%s/%s",ptrpath,textstring);
                        *strrchr(textstring,'/')= '\0';
                        (void)strcpy(ptrpath,textstring);
                }
                if(who == PREVIEWINPUT) {
                  XtVaSetValues(textFieldFileSelectionPreviewInputFilter, XmNvalue, filename, NULL);
                }
                else if(who == SAVECORETOELD) {
                  XtVaSetValues(textFieldSaveCoreDataFilter, XmNvalue, filename, NULL);
                }
                else {
                  XtVaSetValues(textFieldFileSelectionFilter, XmNvalue, filename, NULL);
                }
                i=(int)strlen(filesearch_string)+(int)strlen(filename)+(int)strlen(tmpfile)+100;
                if((charptr=(char*)calloc(i,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaUpdateFileSelction 7");
                else
                        (void)sprintf(charptr,filesearch_string,filename,tmpfile);

                XtFree(textstring);
        }
        else if(how == DIRECTORIES) /* directories were updated */
        {
                if(who == PREVIEWINPUT) {
                  XtVaGetValues(listFileSelectionPreviewInputDirectories,XmNselectedItems,&selected_dir,XmNselectedItemCount,&numdirs,NULL);
                }
                else if(who == SAVECORETOELD) {
                  XtVaGetValues(listSaveCoreDataDir,XmNselectedItems,&selected_dir,XmNselectedItemCount,&numdirs,NULL);
                }
                else {
                  XtVaGetValues(listFileSelectionDirectories,XmNselectedItems,&selected_dir,XmNselectedItemCount,&numdirs,NULL);
                }
                if(numdirs == 1)
                {
                        if(!XmStringGetLtoR(selected_dir[0],XmFONTLIST_DEFAULT_TAG,&textstring))
                        {
                                info.error=True;
                                info.fatal=False;
                                info.caller=NONE;
                                info.type=XmDIALOG_WARNING;
                                info.default_button=XmDIALOG_OK_BUTTON;
                                if((info.message=(char*)calloc(200,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("SaUpdateFileSelction 7a");
                                else
                                        (void)sprintf(info.message,"Unable to convert selected_dir[0]");
                                SaManageMessage(info);
                        }
                        if(strcmp(textstring,"..") == 0 && strcmp(ptrpath, "/") == 0)
                        {
                        }
                        else if(!strcmp(textstring,".."))       /* move back a directory */
                        {
                                *strrchr(ptrpath,'/')='\0';
                                if(strcmp(ptrpath, "") == 0)
                                {
                                  (void)sprintf(ptrpath, "/");
                                }
                        }
                        else  if(strcmp(textstring,"."))        /* new directory */
                        {
                                if(strcmp(ptrpath, "/") == 0)
                                {
                                  (void)sprintf(ptrpath,"/%s",textstring);
                                }
                                else
                                {
                                  (void)sprintf(ptrpath,"%s/%s",ptrpath, textstring);
                                }
                        }
                        XtFree(textstring);
                        if((charptr=(char*)calloc((int)strlen(dirsearch_string)+(int)strlen(ptrpath)+(int)strlen(tmpfile)+100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateFileSelction 9");
                        (void)sprintf(charptr,dirsearch_string,ptrpath, tmpfile);

                        /* update filter field */
                        if((filename=(char*)calloc((int)strlen(ptrpath)+5,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("SaUpdateFileSelction 10");
                        if(strcmp(ptrpath, "/") == 0)
                        {
                          (void)sprintf(filename,"/*");
                        }
                        else
                        {
                          (void)sprintf(filename,"%s/*",ptrpath);
                        }
                        if(who == PREVIEWINPUT) {
                          XmTextSetString(textFieldFileSelectionPreviewInputFilter,filename);
                          XmListDeselectAllItems(listFileSelectionPreviewInputDirectories);
                        }
                        else if(who == SAVECORETOELD) {
                          XmTextSetString(textFieldSaveCoreDataFilter,filename);
                          XmListDeselectAllItems(listSaveCoreDataDir);
                        }
                        else {
                          XmTextSetString(textFieldFileSelectionFilter,filename);
                          XmListDeselectAllItems(listFileSelectionDirectories);
                        }
                }
                else
                {
                        SaManageWarningBox("Error opening new directory!");
                }
        }
        system(charptr);
        free(charptr);

        /* get file and directory names for filter path */
        numdirs=0;
        if((filelist=(XmString*)calloc((alloc_file=1000),sizeof(XmString**)))==NULL)
                        SaBadMemAlloc("SaUpdateFileSelction 10b");
        if((dirlist=(XmString*)calloc((alloc_dir=1000),sizeof(XmString**)))==NULL)
                        SaBadMemAlloc("SaUpdateFileSelction 10c");
        dirlist[numdirs++]=XmStringCreateLocalized(".");
        dirlist[numdirs++]=XmStringCreateLocalized("..");
        if((fptr=fopen(tmpfile,"r"))!=NULL)
        {
                if((charptr=(char*)realloc(filename,1000*CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaUpdateFileSelction 11");
                else
                        filename=charptr;
                while(fscanf(fptr,"%s %s",filetype,filename)!=EOF)
                {
                        if(!strcmp(filetype,"DIR"))             /* directory names */
                        {
                                if(numdirs>=alloc_dir)
                                {
                                        if((charptr=(char*)realloc((char*)dirlist,(alloc_dir+=1000)*sizeof(XmString*)))==NULL)
                                                        SaBadMemAlloc("SaUpdateFileSelction 11b");
                                        else
                                                dirlist=(XmString*)charptr;
                                }
                                dirlist[numdirs++]=XmStringCreateLocalized(filename);
                        }
                        else if(!strcmp(filetype,"FILE"))       /* file names */
                        {
                                if(numfiles>=alloc_file)
                                {
                                        if((charptr=(char*)realloc((char*)filelist,(alloc_file+=1000)*sizeof(XmString*)))==NULL)
                                                        SaBadMemAlloc("SaUpdateFileSelction 11c");
                                        else
                                                filelist=(XmString*)charptr;
                                }
                                filelist[numfiles++]=XmStringCreateLocalized(filename);
                        }
                        else if(!strcmp(filetype,"LINK"))       /*  symbolic link name */
                        {
                                if((charptr=(char*)realloc((char*)filename,((int)strlen(filename)+25)*sizeof(XmString*)))==NULL)
                                        SaBadMemAlloc("SaUpdateFileSelction 11d");
                                else
                                        filename=charptr;
                                (void)strcat(filename," (sym-link)");
                                if(numfiles>=alloc_file)
                                {
                                        if((charptr=(char*)realloc((char*)filelist,(alloc_file+=1000)*sizeof(XmString*)))==NULL)
                                                        SaBadMemAlloc("SaUpdateFileSelction 11e");
                                        else
                                                filelist=(XmString*)charptr;
                                }
                                filelist[numfiles++]=XmStringCreateLocalized(filename);
                        }
                }
                free(filename);
        }

        /* invalid directory */
        if(numfiles <=0 && numdirs<=2)
        {
                if((charptr=(char*)calloc((int)strlen(filename)+50,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("SaUpdateFileSelction 12");
                (void)sprintf(charptr,"Invalid or empty filter: %s !",filename);
                SaManageWarningBox(charptr);
                free(charptr);
        }

        /* if files found - put them in file list */
        if(who == PREVIEWINPUT) {
          if(numfiles>0)
            XtVaSetValues(listFileSelectionPreviewInputFiles,XmNitemCount,numfiles,XmNvisibleItemCount,numfiles
                                            ,XmNitems,filelist,NULL);
          else    /* no files found  - clear list */
            XmListDeleteAllItems(listFileSelectionPreviewInputFiles);

          /* put directories in dir list */
          XtVaSetValues(listFileSelectionPreviewInputDirectories,XmNitemCount,numdirs,XmNvisibleItemCount,numdirs
                                        ,XmNitems,dirlist,NULL);
        }
        else if(who == SAVECORETOELD) {

          /* put directories in dir list */
          XtVaSetValues(listSaveCoreDataDir,XmNitemCount,numdirs,XmNvisibleItemCount,numdirs
                                        ,XmNitems,dirlist,NULL);
        }
        else {
          if(numfiles>0)
            XtVaSetValues(listFileSelectionFiles,XmNitemCount,numfiles,XmNvisibleItemCount,numfiles
                                            ,XmNitems,filelist,NULL);
          else    /* no files found  - clear list */
            XmListDeleteAllItems(listFileSelectionFiles);

          /* put directories in dir list */
          XtVaSetValues(listFileSelectionDirectories,XmNitemCount,numdirs,XmNvisibleItemCount,numdirs
                                        ,XmNitems,dirlist,NULL);
        }

        /* clean up memory used and remove temporary file */
        while(numdirs>0) XmStringFree(dirlist[--numdirs]);
        while(numfiles>0) XmStringFree(filelist[--numfiles]);

        free(dirlist);
        free(filelist);
        (void)remove(tmpfile);

        if(cursor) {
          XUndefineCursor(XtDisplay(widget), XtWindow(widget));
          XFreeCursor(XtDisplay(widget), cursor);
        }
}

int VerifyFileReadable(file, kind, who)
char *file;
int kind;
int who;
{
  int len;
  char *checkfile, *tmpfile=NULL, *tmpinfo;
  FILE *fptr;

  /* check that file can be openned
  */
  if(who != SAVECORETOELD) {
    if(kind == OPEN) {
      if((fptr=fopen(file, "r")) == NULL) {
        return NO;
      }
      fclose(fptr);
    }
    else if(kind == SAVE) {
      if((fptr=fopen(file, "w")) == NULL) {
        return NO;
      }
      fclose(fptr);
    }
    else {
      return NO;
    }
  }

  /* check that file is of a readable type
  */
  tmpfile=tmpnam(tmpfile);
  if((tmpinfo=(char*)calloc((int)strlen(file)+(int)strlen(tmpfile)+50,CHAR_SIZE))==NULL) {
                        SaBadMemAlloc("VerifyFileReadable1");
  }
  else {
    (void)sprintf(tmpinfo,"ls -dF %s > %s", file, tmpfile);
  }
  system(tmpinfo);
  XtFree(tmpinfo);
  if(tmpinfo) {
    tmpinfo = NULL;
  }

  if((fptr=fopen(tmpfile,"r"))!=NULL) {
    if((checkfile=(char*)calloc((int)strlen(file)+100,CHAR_SIZE))==NULL) {
                        SaBadMemAlloc("VerifyFileReadable2");
    }
    (void)fscanf(fptr, "%s", checkfile);
    fclose(fptr);
    (void)remove(tmpfile);
    len=strlen(checkfile);

    if(len<1) {
      XtFree(checkfile);
      if(checkfile) {
        checkfile = NULL;
      }
      return NO;
    }
    /* '/' indicates the file is a directory, '*' an executable and '@ a symbolic link
    */
    else if(who == OPENCULLTABLEDATA &&
            (strcmp(&checkfile[len-1], "/") == 0 || strcmp(&checkfile[len-1], "@") == 0)) {
      XtFree(checkfile);
      if(checkfile) {
        checkfile = NULL;
      }
      return NO;
    }
    else if(who != OPENCULLTABLEDATA && who != SAVECORETOELD &&
            (strcmp(&checkfile[len-1], "/") == 0 || strcmp(&checkfile[len-1], "*") == 0 || strcmp(&checkfile[len-1], "@") == 0)) {
      XtFree(checkfile);
      if(checkfile) {
        checkfile = NULL;
      }
      return NO;
    }
    else if(who == SAVECORETOELD && (strcmp(&checkfile[len-1], "/") != 0)) {
      XtFree(checkfile);
      if(checkfile) {
        checkfile = NULL;
      }
      return NO;
    }
    else {
      XtFree(checkfile);
      if(checkfile) {
        checkfile = NULL;
      }
      return YES;
    }
  }
  else {
    (void)remove(tmpfile);
    return NO;
  }
}

/*Function*/
int VerifyFileWritable(file, kind, who)
char *file;
int  kind;
int who;
{
  int  err;
  FILE *fptr;

  if((fptr=fopen(file, "r")) == NULL) {
    err = OK;
  }
  else {
    err = VerifyFileReadable(file, kind, who);
    if(err == OK) {
      err = FILEXISTS;
    }
    fclose(fptr);
  }
  return err;
}

void MapStratToData(ds)
int ds;
{
  int i, j, k, n;

  for(n=0; n<strat_file_cnt; ++n) {
    for(i=0; i<strat[n]->data_cnt; ++i) {
      strat[n]->data[i].top.maptohole[ds] = -1;
      strat[n]->data[i].bot.maptohole[ds] = -1;
      strat[n]->data[i].top.maptocore[ds] = -1;
      strat[n]->data[i].bot.maptocore[ds] = -1;
      for(j=0; j<data[ds]->numholes; ++j) {
        if(strat[n]->data[i].top.hole == data[ds]->holes[j]->name && strat[n]->data[i].bot.hole == data[ds]->holes[j]->name) {
          strat[n]->data[i].top.maptohole[ds] = j;
          strat[n]->data[i].bot.maptohole[ds] = j;
          for(k=0; k<data[ds]->holes[j]->numcores; ++k) {
            if(strat[n]->data[i].top.core == data[ds]->holes[j]->core[k]->num) {
              strat[n]->data[i].top.maptocore[ds] = k;
            }
            if(strat[n]->data[i].bot.core == data[ds]->holes[j]->core[k]->num) {
              strat[n]->data[i].bot.maptocore[ds] = k;
            }
            if(strat[n]->data[i].top.maptocore[ds] != -1 && strat[n]->data[i].bot.maptocore[ds] != -1) {
              break;
            }
          }
        }
      }
    }
  }
}

void MapStratToAffine()
{
  int i, j, k, l, m;

  /* find if the datum is in the affine table
  */
  for( i=0; i<strat_file_cnt; ++i) {
    for(j=0; j<strat[i]->data_cnt; ++j) {
      strat[i]->data[j].hole_is_in_dataset = NO;
      strat[i]->data[j].top.in_affine = NO;
      strat[i]->data[j].bot.in_affine = NO;
      strat[i]->data[j].top.aff_map_hole = -1;
      strat[i]->data[j].top.aff_map_core = -1;
      strat[i]->data[j].bot.aff_map_hole = -1;
      strat[i]->data[j].bot.aff_map_core = -1;
      for(k=0; k<affinetable.numhole; ++k) {
        if(strat[i]->data[j].top.hole == affinetable.hole[k]->name) {
          strat[i]->data[j].top.aff_map_hole = k;
          strat[i]->data[j].bot.aff_map_hole = k;
          for(m=0; m<data[dset]->numholes; ++m) {
            if(strat[i]->data[j].top.hole == data[dset]->holes[m]->name) {
              strat[i]->data[j].hole_is_in_dataset = YES;
            }
          }
          for(l=0; l<affinetable.hole[k]->numcore; ++l) {
            if(strat[i]->data[j].top.core == affinetable.hole[k]->core[l]->name) {
              strat[i]->data[j].top.in_affine = YES;
              strat[i]->data[j].top.aff_map_core = l;
            }
            if(strat[i]->data[j].bot.core == affinetable.hole[k]->core[l]->name) {
              strat[i]->data[j].bot.in_affine = YES;
              strat[i]->data[j].bot.aff_map_core = l;
            }
          }
        }
      }
    }
  }
}

void MapStratToeld()
{
  int i, j, k, l, m;

  for(i=0; i<strat_file_cnt; ++i) {
    for(j=0; j<strat[i]->data_cnt; ++j) {
      strat[i]->data[j].top.eld_map_hole = -1;
      strat[i]->data[j].top.eld_map_core = -1;
      strat[i]->data[j].bot.eld_map_hole = -1;
      strat[i]->data[j].bot.eld_map_core = -1;
      for(k=0; k<eqlogdepth.numhole; ++k) {
        if(eqlogdepth.hole[k]->name == strat[i]->data[j].top.hole) {
          strat[i]->data[j].top.eld_map_hole = k;
          strat[i]->data[j].bot.eld_map_hole = k;
          for(l=0; l<eqlogdepth.hole[k]->numcore; ++l) {
            if(strat[i]->data[j].top.core == eqlogdepth.hole[k]->core[l]->name) {
              strat[i]->data[j].top.eld_map_core = l;
            }
            if(strat[i]->data[j].bot.core == eqlogdepth.hole[k]->core[l]->name) {
              strat[i]->data[j].bot.eld_map_core = l;
            }
            if(strat[i]->data[j].top.eld_map_core != -1 && strat[i]->data[j].bot.eld_map_core != -1) {
              break;
            }
          }
        }
      }
    }
  }
}


void FindStratDepthOffset()
{
  int i, j, tophole, bothole, topcore, botcore;

  /* set the strat cum depth offset

     use the following rules for whether to use strat:

     if the affine table does not contain the hole of datum;
           dont'-can't use datum
     if the affine table does not have the core(s) for both the top and bot of datum;
           ?
     if the affine table has the core for the top of datum but not the bot;
           use the top offset for bot;
     if the affine table has the core for the bot of datum but not the top;
           use the bot offset for top;
  */

  for( i=0; i<strat_file_cnt; ++i) {
    for(j=0; j<strat[i]->data_cnt; ++j) {
      tophole = strat[i]->data[j].top.aff_map_hole;
      topcore = strat[i]->data[j].top.aff_map_core;
      bothole = strat[i]->data[j].bot.aff_map_hole;
      botcore = strat[i]->data[j].bot.aff_map_core;
      if(strat[i]->data[j].top.in_affine == YES && strat[i]->data[j].bot.in_affine == YES) {
        strat[i]->data[j].top.aff_offset = affinetable.hole[tophole]->core[topcore]->cum_depth_offset;
        strat[i]->data[j].bot.aff_offset = affinetable.hole[bothole]->core[botcore]->cum_depth_offset; 
      }
      else if(strat[i]->data[j].top.in_affine == YES && strat[i]->data[j].bot.in_affine == NO) {
        strat[i]->data[j].top.aff_offset = affinetable.hole[tophole]->core[topcore]->cum_depth_offset; 
        strat[i]->data[j].bot.aff_offset = strat[i]->data[j].top.aff_offset;
      }
      else if(strat[i]->data[j].top.in_affine == NO && strat[i]->data[j].bot.in_affine == YES) {
        strat[i]->data[j].bot.aff_offset = affinetable.hole[bothole]->core[botcore]->cum_depth_offset; 
        strat[i]->data[j].top.aff_offset = strat[i]->data[j].bot.aff_offset;
      }
      else {
        strat[i]->data[j].top.aff_offset = 0.0;
        strat[i]->data[j].bot.aff_offset = 0.0;
      }

      strat[i]->data[j].aveage = (strat[i]->data[j].bot_age + strat[i]->data[j].top_age)/2.0;
      strat[i]->data[j].avembsf = (strat[i]->data[j].bot.mbsf + strat[i]->data[j].top.mbsf)/2.0;
      strat[i]->data[j].avemcd = (strat[i]->data[j].bot.mbsf + strat[i]->data[j].bot.aff_offset +
                                  strat[i]->data[j].top.mbsf + strat[i]->data[j].top.aff_offset)/2.0;
    }
  }
}

void SetupMakingSpliceSeries()
{
  int i, j, k, l, h, c, start, err, alloc, warning;
  char *string, *logstring;
  float dum;
  XmString xmstring;
  void    SaGaussianSplice(), AdjustToTies();
  int     MakeSpliceSeries();

  if(splice_cnt == 0) {
    start = 0;
    splice_cnt = dataset_cnt;
    ties->core_alloc = 0;
    if((splice = (Splice**)calloc(splice_cnt, sizeof(Splice*)))==NULL) {
                    SaBadMemAlloc("Sa");
    }
  }
  else {
    ++splice_cnt;
    start = dataset_cnt - 1;
    if((charptr= (char*)realloc((char*)splice,(dataset_cnt*sizeof(Splice*))))==NULL) {
                                SaBadMemAlloc("Sa");
    }
    else {
      splice=(Splice**)charptr;
    }
  }
  for(i=start; i<dataset_cnt; ++i) {
    if((splice[i] = (Splice*)calloc(1, sizeof(Splice)))==NULL) {
                SaBadMemAlloc("SaInputData");
    }
    splice[i]->alloc_cnt = SPLICE_ALLOC;
    if((splice[i]->value = (SpliceValue**)calloc(splice[i]->alloc_cnt, sizeof(SpliceValue*)))==NULL) {
                  SaBadMemAlloc("Sa");
    }
    for(j=0; j<splice[i]->alloc_cnt; ++j) {
      if((splice[i]->value[j] = (SpliceValue*)calloc(1, sizeof(SpliceValue)))==NULL) {
                  SaBadMemAlloc("Sa");
      }
    }
    err = MakeSpliceSeries(i, &warning);
    if(err > 0) {
      have_splice = DONT;
      SaCleanUp(SPLICEDATA);
      return;
    }
    else {
      if(warning == YES) {
        if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
        (void)sprintf(charptr, "\nWarning; One or more cores missing in splice series.");
        SaManageWarningBox(charptr);
        free(charptr);
      }
      have_splice = DO;
      splice[i]->maptodata = i;
      err = FindSpliceMinMax(i);
      if(start == 0 && ties->core_alloc < splice[i]->numvalues) {
        ties->core_alloc = splice[i]->numvalues;
      }

      if(smooth.method == NONE) {
        splice[i]->smooth.method = NONE;
      }
      else if(smooth.method == GAUSSIAN) {
        splice[i]->smooth.method = GAUSSIAN;
        splice[i]->smooth.width = smooth.width;
        splice[i]->smooth.width_units = smooth.width_units;
      }
      else {
        splice[i]->smooth.method = NONE;
      }
      if(smooth.plot == UNSMOOTH) {
        splice[i]->smooth.plot = UNSMOOTH;
      }
      else if(smooth.plot == SMOOTH) {
        splice[i]->smooth.plot = SMOOTH;
      }
      else if(smooth.plot == SMOOTHBOTH) {
        splice[i]->smooth.plot = SMOOTHBOTH;
      }
      else {
        splice[i]->smooth.plot = UNSMOOTH;
      }
      if(smooth.method == GAUSSIAN) {
        SaGaussianSplice(i);
      }

      /* alloc some space for tie list
      */
      if(splice_cnt == i + 1 && start == 0) {
        ties->alloc_cnt = MAXTIES;
        if((ties->tie=(TieList**)calloc(ties->alloc_cnt,sizeof(TieList*)))==NULL)
                              SaBadMemAlloc("Sa");
        for(j=0; j<ties->alloc_cnt; ++j) {
          if((ties->tie[j]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                            SaBadMemAlloc("Sa");
        }
        if((ties->temp_equivlogdep=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
        if((ties->coredepth=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
        /* give the tie list a tie that fixes 0mcd eq 0 log
        */
        if((string=(char*)calloc(200,CHAR_SIZE))==NULL)
                 SaBadMemAlloc("Sa");
        if((logstring=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("Sa");
        ties->nties = 1;
        ties->tie[0]->holename = splicertable.data[0].hole; 
        ties->tie[0]->corename = splicertable.data[0].core;
        ties->tie[0]->coretype = splicertable.data[0].core_type;
        ties->tie[0]->array = -1;
        ties->tie[0]->log = -1;
        ties->tie[0]->logdep = 0.0;
        ties->tie[0]->corembsf = 0.0;
        ties->tie[0]->coremcd = 0.0;
        ties->tie[0]->coreautoeld = 0.0;
        ties->tie[0]->equivlogdep = 0.0;
        ties->tie[0]->temp_eld = 0.0;
        ties->tie[0]->rate = 1.0;
        ties->tie[0]->b = 0.0;
        ties->tie[0]->shared = NO;
        ties->tie[0]->topsect = 0.0;
        ties->tie[0]->botsect = 0.0;
        (void)sprintf(ties->tie[0]->coresection, " 1");
        (void)sprintf(string,"Core:  %4d %4d %c  %3d %c %2s %5.1f %5.1f  %8.2fmbsf   %8.2fmcd",
                splice[i]->leg,
                splicertable.data[0].site,
                splicertable.data[0].hole,
                splicertable.data[0].core,
                splicertable.data[0].core_type,
                ties->tie[0]->coresection,
                ties->tie[0]->topsect,
                ties->tie[0]->botsect,
                ties->tie[0]->corembsf,
                ties->tie[0]->coremcd);
        if(eqlogdepth.autocorr_status == YES) {
          (void)sprintf(logstring, "  %8.2feld(auto)", ties->tie[0]->coreautoeld);
          (void)strncat(string, logstring, (int)strlen(logstring));
        }
        (void)sprintf(logstring, "  Tied to    Log:  %8.3fm", ties->tie[0]->logdep);
        (void)strncat(string, logstring, (int)strlen(logstring));
        xmstring = XmStringCreateSimple(string);
        XmListAddItemUnselected(listTieList, xmstring, 0);
        XmStringFree(xmstring);
        free(string);
        string = NULL;
        free(logstring);
        logstring = NULL;

        /* if have read an eld table find any ties that are in the splice
           since have the splice table use it
        */
        j = 0;
        while(j<splicertable.data_cnt) {
          for(h=0; h<eqlogdepth.numhole; ++h) {
            if(eqlogdepth.hole[h]->name == splicertable.data[j].hole) {
              for(c=0; c<eqlogdepth.hole[h]->numcore; ++c) {
                if(eqlogdepth.hole[h]->core[c]->name == splicertable.data[j].core) {
                  if(eqlogdepth.hole[h]->core[c]->nties > 0) {

                    /* is there enough space alloc
                    */
                    if(eqlogdepth.hole[h]->core[c]->nties  + ties->nties > ties->alloc_cnt) {
                      alloc = ties->alloc_cnt;
                      while(alloc < eqlogdepth.hole[h]->core[c]->nties  + ties->nties) {
                        alloc = alloc + MAXTIES;
                      }
                      if((charptr=(char*)realloc((char*)ties->tie,alloc*sizeof(TieList*)))==NULL) {
                                SaBadMemAlloc("");
                      }
                      else {
                        ties->tie = (TieList**)charptr;
                      }
                      for(k=ties->alloc_cnt; k<alloc; ++k) {
                        if((ties->tie[k]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                                SaBadMemAlloc("Sa");
                      }
                      ties->alloc_cnt = alloc;
                    }

                    for(k=0; k<eqlogdepth.hole[h]->core[c]->nties; ++k) {
                      if((j == 0 && eqlogdepth.hole[h]->core[c]->tie[k]->coremcd >= splicertable.data[j].mcd &&
                          eqlogdepth.hole[h]->core[c]->tie[k]->coremcd <= splicertable.data[j+1].mcd) ||
                         (j > 0 && eqlogdepth.hole[h]->core[c]->tie[k]->coremcd > splicertable.data[j].mcd &&
                          eqlogdepth.hole[h]->core[c]->tie[k]->coremcd <= splicertable.data[j+1].mcd)) {

                        ties->tie[ties->nties]->selected = NO;
                        ties->tie[ties->nties]->holename = eqlogdepth.hole[h]->name;
                        ties->tie[ties->nties]->corename = eqlogdepth.hole[h]->core[c]->name;
                        ties->tie[ties->nties]->coretype = eqlogdepth.hole[h]->core[c]->type;
                        (void)sprintf(ties->tie[ties->nties]->coresection, "%s", eqlogdepth.hole[h]->core[c]->tie[k]->coresection);
                        ties->tie[ties->nties]->topsect = eqlogdepth.hole[h]->core[c]->tie[k]->topsect;
                        ties->tie[ties->nties]->botsect = eqlogdepth.hole[h]->core[c]->tie[k]->botsect;
                        ties->tie[ties->nties]->rate = eqlogdepth.hole[h]->core[c]->tie[k]->rate;
                        ties->tie[ties->nties]->b = eqlogdepth.hole[h]->core[c]->tie[k]->b;
                        ties->tie[ties->nties]->coremcd = eqlogdepth.hole[h]->core[c]->tie[k]->coremcd;
                        ties->tie[ties->nties]->corembsf = eqlogdepth.hole[h]->core[c]->tie[k]->corembsf;
                        ties->tie[ties->nties]->coreautoeld = eqlogdepth.hole[h]->core[c]->tie[k]->coreautoeld;
                        ties->tie[ties->nties]->equivlogdep = eqlogdepth.hole[h]->core[c]->tie[k]->equivlogdep;
                        ties->tie[ties->nties]->temp_eld = eqlogdepth.hole[h]->core[c]->tie[k]->equivlogdep;
                        ties->tie[ties->nties]->logdep = eqlogdepth.hole[h]->core[c]->tie[k]->logdep;
                        ties->tie[ties->nties]->shared = eqlogdepth.hole[h]->core[c]->tie[k]->shared;
                        ++ties->nties;
                      }
                    }
                  }
                  break;
                }
              }
            }
          }
          j = j + 2;
        }
        if(ties->nties > 1) {
          if((string=(char*)calloc(200,CHAR_SIZE))==NULL)
                 SaBadMemAlloc("Sa");
          if((logstring=(char*)calloc(100,CHAR_SIZE))==NULL)
                   SaBadMemAlloc("Sa");
          for(k=1; k<ties->nties; ++k) {
            (void)sprintf(string,"Core:  %4d %4d %c  %3d %c %2s %5.1f %5.1f  %8.2fmbsf  %8.2fmcd",
                       eqlogdepth.leg,
                       eqlogdepth.site,
                       ties->tie[k]->holename,
                       ties->tie[k]->corename,
                       ties->tie[k]->coretype,
                       ties->tie[k]->coresection,
                       ties->tie[k]->topsect,
                       ties->tie[k]->botsect,
                       ties->tie[k]->corembsf,
                       ties->tie[k]->coremcd);
          
            if(eqlogdepth.autocorr_status == YES) {
              (void)sprintf(logstring, "  %8.2feld(auto)", ties->tie[k]->coreautoeld);
              (void)strncat(string, logstring, (int)strlen(logstring));
            }
            (void)sprintf(logstring, "  Tied to    Log:  %8.3fm", ties->tie[k]->logdep);
            (void)strncat(string, logstring, (int)strlen(logstring));
            if(ties->tie[k]->shared == YES) {
              (void)strcat(string, "  S");
            }
            else {
              (void)strcat(string, "  NS");
            }

            xmstring = XmStringCreateSimple(string);
            XmListAddItemUnselected(listTieList, xmstring, 0);
            XmStringFree(xmstring);
          }
          free(string);
          string = NULL;
          free(logstring);
          logstring = NULL;
        }
      }
    }
  }
  
  sset = dset;
  ties->avedepthstep = splice[sset]->avedepstep;

  /* assign temp_equivlogdep
  */
  if(ties->core_alloc < splice[sset]->numvalues) {
    ties->core_alloc = splice[sset]->numvalues;
    if((charptr=(char*)realloc(ties->temp_equivlogdep, ties->core_alloc*sizeof(float))) == NULL) {
                                SaBadMemAlloc("Sa");
    }
    else {
      ties->temp_equivlogdep=(float*)charptr;
    }
    if((charptr=(char*)realloc(ties->coredepth, ties->core_alloc*sizeof(float))) == NULL) {
                             SaBadMemAlloc("Sa");
    }
    else {
      ties->coredepth=(float*)charptr;
    }
  }
  if(ties->nties > 1) {
    AdjustToTies(dset);
  }
  else {
    for(i=0; i<splice[sset]->numvalues; ++i) {
      if(splice[sset]->value[i]->quality  & BAD_SB_DEPTH) {
        ties->temp_equivlogdep[i] = BADPT;
        ties->coredepth[i] = BADPT;
      }
      else {
        ties->coredepth[i] = splice[sset]->value[i]->mcd;
        if(eqlogdepth.autocorr_status == YES) {
          ties->coredepth[i] = ties->coredepth[i] * (eqlogdepth.autocorr_squish/100.0);
        }
        ties->temp_equivlogdep[i]= ties->coredepth[i];
      }
    }
  }
  for(i=0; i<splice[sset]->numvalues; ++i) {
    splice[sset]->value[i]->eld = ties->temp_equivlogdep[i];
  }
}

int MakeSpliceSeries(ds, warn)
int ds;
int *warn;
{
  int i, j, k, t, ncore, nsplice, core, hole, hold_alloc_cnt;
  float depth;

  *warn = NO;
  ncore = splicertable.data_cnt/2.0;
  if(ncore < 1) {
    return 1;
  }

  splice[ds]->site = data[ds]->site;
  splice[ds]->leg = data[ds]->leg;
  splice[ds]->data_type = data[ds]->type;
  splice[ds]->smooth.method = smooth.method;
  splice[ds]->smooth.plot = smooth.plot;

  /* let t = table indicy; it will increment in two's
  */
  t = 0;
  nsplice = 0;

  for(i=0; i<ncore; ++i) {
    hole = -1;
    core = -1;
    for(j=0; j<data[ds]->numholes; ++j) {
      if(splicertable.data[t].hole == data[ds]->holes[j]->name) {
        hole = j;	
        for(k=0; k<data[ds]->holes[j]->numcores; ++k) {
          if(splicertable.data[t].core == data[ds]->holes[j]->core[k]->num) {
            core = k;
            break;
          }
        }
      }
    }
    if(hole != -1 && core != -1) {
      for(j=0; j<data[ds]->holes[hole]->core[core]->numvalues; ++j) {
        depth = data[ds]->holes[hole]->core[core]->value[j]->mbsf + data[ds]->holes[hole]->core[core]->affine_offset;
        if(i == 0 && depth >= splicertable.data[t].mcd && depth <= splicertable.data[t+1].mcd) {
          if(nsplice + 1 >= splice[ds]->alloc_cnt) {
            hold_alloc_cnt = splice[ds]->alloc_cnt;
            splice[ds]->alloc_cnt = splice[ds]->alloc_cnt + 500;

            if((charptr=(char*)realloc((char*)splice[ds]->value,splice[ds]->alloc_cnt*sizeof(SpliceValue*)))==NULL) {
                              SaBadMemAlloc("ReadSplice");
            }
            else {
              splice[ds]->value=(SpliceValue**)charptr;
            }

            for(k=hold_alloc_cnt; k<splice[ds]->alloc_cnt; ++k) {
              if((splice[ds]->value[k]=(SpliceValue*)calloc(1,sizeof(SpliceValue)))==NULL) {
                              SaBadMemAlloc("ReadSplice");
              }
            }
          }
          if(data[ds]->holes[hole]->core[core]->value[j]->quality == GOOD) {
            (void)strcpy(splice[ds]->value[nsplice]->section, data[ds]->holes[hole]->core[core]->value[j]->section);
            splice[ds]->value[nsplice]->hole = data[ds]->holes[hole]->name;
            splice[ds]->value[nsplice]->type = data[ds]->holes[hole]->core[core]->value[j]->type;
            splice[ds]->value[nsplice]->corename = data[ds]->holes[hole]->core[core]->num;
            splice[ds]->value[nsplice]->quality = data[ds]->holes[hole]->core[core]->value[j]->quality;
            splice[ds]->value[nsplice]->sect_top = data[ds]->holes[hole]->core[core]->value[j]->top_int;
            splice[ds]->value[nsplice]->sect_bot = data[ds]->holes[hole]->core[core]->value[j]->bot_int;
            splice[ds]->value[nsplice]->mbsf = data[ds]->holes[hole]->core[core]->value[j]->mbsf;
            splice[ds]->value[nsplice]->affine_offset = data[ds]->holes[hole]->core[core]->affine_offset;
            splice[ds]->value[nsplice]->mcd = splice[ds]->value[nsplice]->mbsf + splice[ds]->value[nsplice]->affine_offset;
            splice[ds]->value[nsplice]->eld = data[ds]->holes[hole]->core[core]->value[j]->eld;
            splice[ds]->value[nsplice]->data = *data[ds]->holes[hole]->core[core]->value[j]->data;
            splice[ds]->value[nsplice]->sm_data = data[ds]->holes[hole]->core[core]->value[j]->sm_data;
            ++nsplice;
          }
        }
        else if(depth > splicertable.data[t-1].mcd && depth <= splicertable.data[t+1].mcd) {
          if(nsplice + 1 >= splice[ds]->alloc_cnt) {
            hold_alloc_cnt = splice[ds]->alloc_cnt;
            splice[ds]->alloc_cnt = splice[ds]->alloc_cnt + 500;

            if((charptr=(char*)realloc((char*)splice[ds]->value,splice[ds]->alloc_cnt*sizeof(SpliceValue*)))==NULL) {
                              SaBadMemAlloc("ReadSplice");
            }
            else {
              splice[ds]->value=(SpliceValue**)charptr;
            }

            for(k=hold_alloc_cnt; k<splice[ds]->alloc_cnt; ++k) {
              if((splice[ds]->value[k]=(SpliceValue*)calloc(1,sizeof(SpliceValue)))==NULL) {
                              SaBadMemAlloc("ReadSplice");
              }
            }

          }
          if(data[ds]->holes[hole]->core[core]->value[j]->quality == GOOD) {
            (void)strcpy(splice[ds]->value[nsplice]->section, data[ds]->holes[hole]->core[core]->value[j]->section);
            splice[ds]->value[nsplice]->hole = data[ds]->holes[hole]->name;
            splice[ds]->value[nsplice]->type = data[ds]->holes[hole]->core[core]->value[j]->type;
            splice[ds]->value[nsplice]->corename = data[ds]->holes[hole]->core[core]->num;
            splice[ds]->value[nsplice]->quality = data[ds]->holes[hole]->core[core]->value[j]->quality;
            splice[ds]->value[nsplice]->sect_top = data[ds]->holes[hole]->core[core]->value[j]->top_int;
            splice[ds]->value[nsplice]->sect_bot = data[ds]->holes[hole]->core[core]->value[j]->bot_int;
            splice[ds]->value[nsplice]->mbsf = data[ds]->holes[hole]->core[core]->value[j]->mbsf;
            splice[ds]->value[nsplice]->affine_offset = data[ds]->holes[hole]->core[core]->affine_offset;
            splice[ds]->value[nsplice]->mcd = splice[ds]->value[nsplice]->mbsf + splice[ds]->value[nsplice]->affine_offset;
            splice[ds]->value[nsplice]->eld = data[ds]->holes[hole]->core[core]->value[j]->eld;
            splice[ds]->value[nsplice]->data = *data[ds]->holes[hole]->core[core]->value[j]->data;
            splice[ds]->value[nsplice]->sm_data = data[ds]->holes[hole]->core[core]->value[j]->sm_data;
            ++nsplice;
          }
        }
      } 
    }
    else {
      *warn = YES;
    }
    t = t + 2;
    if(t >= splicertable.data_cnt) {
      break;
    }
  }

  splice[ds]->numvalues = nsplice;
  splice[ds]->numgood = splice[ds]->numvalues;
  if((splice[ds]->good=(int*)calloc(splice[ds]->numgood,sizeof(int))) == NULL)
                                                SaBadMemAlloc("MakeSplice");
  for(i=0; i<splice[ds]->numgood; ++i) {
    splice[ds]->good[i] = i;
  }

  return 0;
}

void CheckSpliceTableOffsets()
{

}

void SearchELDTableForSpliceTies(s)
int s;
{
}

void SaveCoreTies(ds)
int ds;
{
  int a, i, j, k, l, m, o, p, new, hold, h, c, hh, cc, d, eh, ec, hnew, cnew, dck, n, s, hole, core, err,
      sofar, all_cnt, all_temp, array, foundit, warn, nt;
  float *pdep, checktop, checkbot, affoff;
  Ties   *tempties;
  int CountTies(), MakeEquivLogDepthEquation(), CalcEquivLogDepths(), CalcELDInHole();
  void ApplyCoreAdjustToStrat(), DrawDrawingAreaCores(), DrawDrawingAreaLog(),
       CalcStratELD(), GetTieArrayNum(), FindMissingSectDep(), ApplyCoreAdjustToOneCore(), MakeHoleEqt();

  warn = NO;

  h = data[ds]->holes[ties->hole]->map_hole_to_eqlogdepth;
  c = data[ds]->holes[ties->hole]->core[ties->core]->map_core_to_eqlogdepth;

  if(have_coretotie == DO && ties->nties > 0) {

    /* put the tie list in the eqlogdepth table
       free the eld tie list and alloc again with new size.
    */
    if(eqlogdepth.hole[h]->core[c]->nties > 0) {
      for(i=0; i<eqlogdepth.hole[h]->core[c]->nties; ++i) {
        free(eqlogdepth.hole[h]->core[c]->tie[i]);
        eqlogdepth.hole[h]->core[c]->tie[i] = NULL;
      }
      free(eqlogdepth.hole[h]->core[c]->tie);
    }
    if((eqlogdepth.hole[h]->core[c]->tie=(TieList**)calloc(ties->nties,sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("Sa");
    }
    for(i=0; i<ties->nties; ++i) {
      if((eqlogdepth.hole[h]->core[c]->tie[i]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                              SaBadMemAlloc("Sa");
    }
    eqlogdepth.hole[h]->core[c]->nties = ties->nties;
    eqlogdepth.hole[h]->core[c]->did_eld = YES;
    eqlogdepth.hole[h]->core[c]->type = data[ds]->holes[ties->hole]->core[ties->core]->value[0]->type;
    for(i=0; i<ties->nties; ++i) {
      eqlogdepth.hole[h]->core[c]->tie[i]->holename = ties->tie[i]->holename;
      eqlogdepth.hole[h]->core[c]->tie[i]->corename = ties->tie[i]->corename;
      (void)sprintf(eqlogdepth.hole[h]->core[c]->tie[i]->coresection, "%s", ties->tie[i]->coresection);
      eqlogdepth.hole[h]->core[c]->tie[i]->array = ties->tie[i]->array;
      eqlogdepth.hole[h]->core[c]->tie[i]->log = ties->tie[i]->log;
      eqlogdepth.hole[h]->core[c]->tie[i]->selected = ties->tie[i]->selected;
      eqlogdepth.hole[h]->core[c]->tie[i]->topsect = ties->tie[i]->topsect;
      eqlogdepth.hole[h]->core[c]->tie[i]->botsect = ties->tie[i]->botsect;
      eqlogdepth.hole[h]->core[c]->tie[i]->rate = ties->tie[i]->rate;
      eqlogdepth.hole[h]->core[c]->tie[i]->b = ties->tie[i]->b;
      eqlogdepth.hole[h]->core[c]->tie[i]->corembsf = ties->tie[i]->corembsf;
      eqlogdepth.hole[h]->core[c]->tie[i]->coremcd = ties->tie[i]->coremcd;
      eqlogdepth.hole[h]->core[c]->tie[i]->coreautoeld = ties->tie[i]->coreautoeld;
      eqlogdepth.hole[h]->core[c]->tie[i]->equivlogdep = ties->tie[i]->logdep;
      eqlogdepth.hole[h]->core[c]->tie[i]->logdep = ties->tie[i]->logdep;
      eqlogdepth.hole[h]->core[c]->tie[i]->shared = ties->tie[i]->shared;
    }
  }
  /* if the the tie list was empty. then If have ties in eqlogdepth table then they
     were removed so take the ties away from this hole-core
  */
  else if(have_coretotie == DO && eqlogdepth.hole[h]->core[c]->nties > 0) {
    for(i=0; i<eqlogdepth.hole[h]->core[c]->nties; ++i) {
      free(eqlogdepth.hole[h]->core[c]->tie[i]);
      eqlogdepth.hole[h]->core[c]->tie[i] = NULL;
    }
    free(eqlogdepth.hole[h]->core[c]->tie);
    eqlogdepth.hole[h]->core[c]->tie = NULL;
    eqlogdepth.hole[h]->core[c]->nties = 0;
  }

  /* if there are other holes in this data set or other data sets
     see if any of these saved ties are in them
     only for cores that have an affine table(can never share using mbsf)
     also check toggle option for share-don't share
     note; check any core that overlaps coretotie; may have deleted
     all ties in coretotie
  */
  if(have_coretotie == DO && data[ds]->holes[ties->hole]->core[ties->core]->did_affine_offset == YES) {
    checktop = data[ds]->holes[ties->hole]->core[ties->core]->value[0]->mbsf;
    if(have_affine == DO) {
      checktop = checktop + data[ds]->holes[ties->hole]->core[ties->core]->affine_offset;
    }
    n = data[ds]->holes[ties->hole]->core[ties->core]->numvalues;
    checkbot = data[ds]->holes[ties->hole]->core[ties->core]->value[n-1]->mbsf;
    if(have_affine == DO) {
      checkbot = checkbot + data[ds]->holes[ties->hole]->core[ties->core]->affine_offset;
    }
    for(dck=0; dck<dataset_cnt; ++dck) {

      for(i=0; i<data[dck]->numholes; ++i) {
        if(data[dck]->holes[i]->name == data[ds]->holes[ties->hole]->name) {

        }
        else {
          for(j=0; j<data[dck]->holes[i]->numcores; ++j) {
            n = data[dck]->holes[i]->core[j]->numvalues;
            /*does this core overlap selected core?
              give any overlap ties to it
            */
            if(have_affine == DO) {
              affoff = data[dck]->holes[i]->core[j]->affine_offset;
            }
            else {
              affoff = 0.0;
            }
            if(checkbot < data[dck]->holes[i]->core[j]->value[0]->mbsf + affoff) {
              break;
            }
            else if(checkbot >= data[dck]->holes[i]->core[j]->value[0]->mbsf + affoff &&
                    checktop <= data[dck]->holes[i]->core[j]->value[n-1]->mbsf + affoff &&
                    data[dck]->holes[i]->core[j]->did_affine_offset == YES) {
              hnew = data[dck]->holes[i]->map_hole_to_eqlogdepth;
              cnew = data[dck]->holes[i]->core[j]->map_core_to_eqlogdepth;
              
              if(have_affine == DO) {

                all_temp = eqlogdepth.hole[hnew]->core[cnew]->nties;
                if((tempties=(Ties*)calloc(1,sizeof(Ties)))==NULL)
                                SaBadMemAlloc("Sa");
                if((tempties->tie=(TieList**)calloc(all_temp,sizeof(TieList*)))==NULL)
                            SaBadMemAlloc("Sa");
                for(l=0; l<all_temp; ++l) {
                  if((tempties->tie[l]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                            SaBadMemAlloc("Sa");
                }
                tempties->nties = 0;
                if(eqlogdepth.hole[hnew]->core[cnew]->nties > 0) {
                  for(k=0; k<eqlogdepth.hole[hnew]->core[cnew]->nties; ++k) {
                    if((eqlogdepth.hole[hnew]->core[cnew]->tie[k]->shared == NO) ||
                       (eqlogdepth.hole[hnew]->core[cnew]->tie[k]->coremcd < checktop ||
                        eqlogdepth.hole[hnew]->core[cnew]->tie[k]->coremcd > checkbot)) {
                      *tempties->tie[tempties->nties] = *eqlogdepth.hole[hnew]->core[cnew]->tie[k];
                      ++tempties->nties;
                    }
                  }
                  for(k=0; k<eqlogdepth.hole[hnew]->core[cnew]->nties; ++k) {
                    free(eqlogdepth.hole[hnew]->core[cnew]->tie[k]);
                    eqlogdepth.hole[hnew]->core[cnew]->tie[k] = NULL;
                  }
                  free(eqlogdepth.hole[hnew]->core[cnew]->tie);
                  eqlogdepth.hole[hnew]->core[cnew]->nties = 0;
                }

                all_cnt = 0;
                for(k=0; k<eqlogdepth.hole[h]->core[c]->nties; ++k) {
                  if(eqlogdepth.hole[h]->core[c]->tie[k]->shared == YES) {
                    if(eqlogdepth.hole[h]->core[c]->tie[k]->coremcd >= data[dck]->holes[i]->core[j]->value[0]->mbsf + affoff &&
                       eqlogdepth.hole[h]->core[c]->tie[k]->coremcd <= data[dck]->holes[i]->core[j]->value[n-1]->mbsf + affoff) {
                      ++all_cnt;
                    }
                  }
                }

                if(all_cnt + tempties->nties > 0) {
                  data[dck]->holes[i]->core[j]->have_tie_list = DO;

                  /* allocate space for new tie list
                  */
                  if((eqlogdepth.hole[hnew]->core[cnew]->tie=(TieList**)calloc(all_cnt+tempties->nties,sizeof(TieList*)))==NULL) {
                                SaBadMemAlloc("Sa");
                  }
                  for(l=0; l<all_cnt+tempties->nties; ++l) {
                    if((eqlogdepth.hole[hnew]->core[cnew]->tie[l]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                                SaBadMemAlloc("Sa");
                  }
                  eqlogdepth.hole[hnew]->core[cnew]->nties = all_cnt + tempties->nties;
                  eqlogdepth.hole[hnew]->core[cnew]->did_eld = YES;
                  eqlogdepth.hole[hnew]->core[cnew]->type = data[dck]->holes[i]->core[j]->value[0]->type;
                  if(eqlogdepth.hole[hnew]->core[cnew]->type == ' ') {
                    SaManageWarningBox("core type blank!");              
                  }

                  l = 0;
                  for(k=0; k<eqlogdepth.hole[h]->core[c]->nties; ++k) {

                    /* is the current tie within this cores depth range
                    */
                    if(eqlogdepth.hole[h]->core[c]->tie[k]->shared == YES && 
                       eqlogdepth.hole[h]->core[c]->tie[k]->coremcd >= data[dck]->holes[i]->core[j]->value[0]->mbsf + affoff &&
                       eqlogdepth.hole[h]->core[c]->tie[k]->coremcd <= data[dck]->holes[i]->core[j]->value[n-1]->mbsf + affoff) {

                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->holename = data[dck]->holes[i]->name;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->corename = data[dck]->holes[i]->core[j]->num;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->log = eqlogdepth.hole[h]->core[c]->tie[k]->log;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->selected = NO;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->rate = eqlogdepth.hole[h]->core[c]->tie[k]->rate;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->b = eqlogdepth.hole[h]->core[c]->tie[k]->b;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coremcd = eqlogdepth.hole[h]->core[c]->tie[k]->coremcd;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->corembsf = eqlogdepth.hole[h]->core[c]->tie[k]->coremcd - affoff;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coreautoeld = eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coremcd;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->equivlogdep = eqlogdepth.hole[h]->core[c]->tie[k]->equivlogdep;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->logdep = eqlogdepth.hole[h]->core[c]->tie[k]->logdep;
                      eqlogdepth.hole[hnew]->core[cnew]->tie[l]->shared = YES;

                      if(eqlogdepth.autocorr_status == YES) {
                         eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coreautoeld =
                             eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coreautoeld * (eqlogdepth.autocorr_squish/100.0);
                      }
                      GetTieArrayNum(COREDATA, eqlogdepth.hole[hnew]->core[cnew]->tie[l], dck, i, j, &err);
                      if(eqlogdepth.hole[hnew]->core[cnew]->tie[l]->array != -1) {
                        array = eqlogdepth.hole[hnew]->core[cnew]->tie[l]->array;
                        (void)sprintf(eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coresection, "%s",
                               data[dck]->holes[i]->core[j]->value[array]->section);
                        eqlogdepth.hole[hnew]->core[cnew]->tie[l]->topsect = data[dck]->holes[i]->core[j]->value[array]->top_int;
                        eqlogdepth.hole[hnew]->core[cnew]->tie[l]->botsect = data[dck]->holes[i]->core[j]->value[array]->bot_int;
                      }
                      else if(err > -1) {
                        FindMissingSectDep(err, dck, i, j, hnew, cnew, eqlogdepth.hole[hnew]->core[cnew]->tie[l]->corembsf, 
                                           &eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coresection, 
                                           &eqlogdepth.hole[hnew]->core[cnew]->tie[l]->topsect, 
                                           &eqlogdepth.hole[hnew]->core[cnew]->tie[l]->botsect);
                      }
                      else {
                        warn = YES;
                        (void)sprintf(eqlogdepth.hole[hnew]->core[cnew]->tie[l]->coresection, "X");
                        eqlogdepth.hole[hnew]->core[cnew]->tie[l]->topsect = 0.0;
                        eqlogdepth.hole[hnew]->core[cnew]->tie[l]->botsect = 0.0;
                      }
                      ++l;
                    }
                  }
                  /* sort in any ties that were held in tempties
                  */
                  if(tempties->nties > 0) {
                    for(m=0; m<tempties->nties; ++m) {
                      foundit = NO;
                      if(l > 0) {
                        for(o=0; o<l; ++o) {
                          if(o == 0 && tempties->tie[m]->coremcd < eqlogdepth.hole[hnew]->core[cnew]->tie[o]->coremcd) {
                            for(p=l; p>0; --p) {
                              *eqlogdepth.hole[hnew]->core[cnew]->tie[p] = *eqlogdepth.hole[hnew]->core[cnew]->tie[p-1];
                            }
                            *eqlogdepth.hole[hnew]->core[cnew]->tie[o] = *tempties->tie[m];
                            ++l;
                            foundit = YES;
                            break;
                          }
                          else if(o == l - 1 && tempties->tie[m]->coremcd > eqlogdepth.hole[hnew]->core[cnew]->tie[o]->coremcd) {
                            *eqlogdepth.hole[hnew]->core[cnew]->tie[l] = *tempties->tie[m];
                            ++l;
                            foundit = YES;
                            break;
                          }
                          else if(o < l - 1) {
                            if(tempties->tie[m]->coremcd > eqlogdepth.hole[hnew]->core[cnew]->tie[o]->coremcd &&
                               tempties->tie[m]->coremcd < eqlogdepth.hole[hnew]->core[cnew]->tie[o+1]->coremcd) {
                              for(p=l; p>o+1; --p) {
                                *eqlogdepth.hole[hnew]->core[cnew]->tie[p] = *eqlogdepth.hole[hnew]->core[cnew]->tie[p-1];
                              }
                              *eqlogdepth.hole[hnew]->core[cnew]->tie[o+1] = *tempties->tie[m];
                              ++l;
                              foundit = YES;
                              break;
                            }
                          }
                        }
                      }
                      else {
                        *eqlogdepth.hole[hnew]->core[cnew]->tie[m] = *tempties->tie[m];
                        foundit = YES;
                      }
                      if(foundit == NO) {
                        if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                                             SaBadMemAlloc("Sa");
                        (void)sprintf(charptr, "\nWarning; Updating ELD Table. Could not sort in existing tie hole %c, core %d.",
                                      data[dck]->holes[i]->name, data[dck]->holes[i]->core[j]->num);
                        SaManageWarningBox(charptr);
                        free(charptr);
                      }
                    }
                  }
                  if(all_temp > 0) {
                    for(l=0; l<all_temp; ++l) {
                      free(tempties->tie[l]);
                      tempties->tie[l] = NULL;
                    }
                    free(tempties->tie);
                    tempties->tie = NULL;
                    free(tempties);
                    tempties = NULL;
                  }

                }
                else {
                }
              }
            }
          }
        }
      }
    }
  }

  /* calculate the elds
  */
  MakeHoleEqt();
  for(d=0; d<dataset_cnt; ++d) {
    for(h=0; h<data[d]->numholes; ++h) {
      for(c=0; c<data[d]->holes[h]->numcores; ++c) {

        if((pdep=(float*)calloc(data[d]->holes[h]->core[c]->numvalues, sizeof(float))) == NULL)
                                    SaBadMemAlloc("Sa");
        for(l=0; l<data[d]->holes[h]->core[c]->numvalues; ++l) {
          if(data[d]->holes[h]->core[c]->value[l]->quality  & BAD_SB_DEPTH) {
            pdep[l] = BADPT;
          }
          else {
            pdep[l] = data[d]->holes[h]->core[c]->value[l]->mbsf;
            if(have_affine == DO) {
              pdep[l] = pdep[l] + data[d]->holes[h]->core[c]->affine_offset;
            }
          }
        }
        if(eqlogdepth.autocorr_status == YES) {
          ApplyCoreAdjustToOneCore(pdep, data[d]->holes[h]->core[c]->numvalues);
        }
        l = data[d]->holes[h]->map_hole_to_eqlogdepth;
        if(eld.hole_eqt[l]->nties == 0 && eqlogdepth.autocorr_status == NO) {
          data[d]->holes[h]->core[c]->have_equil_log_depths = DONT;
        }
        else {
          if(eld.hole_eqt[l]->nties > 0) {
            CalcELDInHole(d, h, c, eld.hole_eqt[l], pdep);
          }
          data[d]->holes[h]->core[c]->have_equil_log_depths = DO;
          for(l=0; l<data[d]->holes[h]->core[c]->numvalues; ++l) {
            data[d]->holes[h]->core[c]->value[l]->eld = pdep[l];
          }
        }
        free(pdep);
        pdep = NULL;
      }
    }
  }

  if(have_strat == DO) {
    CalcStratELD();
  }

  if(warn == YES) {
    if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
    (void)sprintf(charptr, "\nWarning; Updating ELD Table. one or more shared ties could not find sect. and sect. depth.");
    SaManageWarningBox(charptr);
    free(charptr);
  }

  have_coretotie = DONT;
  have_tie1 = DONT;
  have_tie2 = DONT;
  have_ties = DONT;
  XtSetSensitive(pushButtonSelectCore, True);
  XtSetSensitive(pushButtonCancelCore, False);
  XtSetSensitive(pushButtonClearTie, False);
  XtSetSensitive(pushButtonClearLastTieSaved, False);
  XtSetSensitive(pushButtonShowHideTieList, False);
  XtSetSensitive(pushButtonAddTieToList, False);
  XtSetSensitive(toggleButtonAddTieToCore, False);
  XtSetSensitive(toggleButtonAddTieToAll, False);
  XtSetSensitive(pushButtonOptimizeTie, False);
  XtSetSensitive(pushButtonAdjustToTiesParameters, False);
  XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
  XtSetSensitive(pushButtonSaveChanges, False);
  XtSetSensitive(pushButtonUndoSaveChange, False);
  XtSetSensitive(pushButtonOpenSplicer, False);
  XtSetSensitive(pushButtonOpenSpliceTable, True);
  if(have_cores == DO && have_log == DO) {
    XtSetSensitive(pushButtonEvalCoreLogData, True);
  }
  if((nt = CountTies()) > 0 || eqlogdepth.autocorr_status == YES) {
    XtSetSensitive(pushButtonSaveEquvilentLogDepthTable, True);
    XtSetSensitive(pushButtonSaveCoreData, True);
    if(have_strat == DO) {
      XtSetSensitive(pushButtonSaveStrat, True);
    }
  }
  else {
    XtSetSensitive(pushButtonSaveEquvilentLogDepthTable, False);
    XtSetSensitive(pushButtonSaveCoreData, False);
    XtSetSensitive(pushButtonSaveStrat, False);
  }
  if(nt > 0) {
    XtSetSensitive(pushButtonAutoCorrelate, False);
  }
  else {
    XtSetSensitive(pushButtonAutoCorrelate, True);
  }

  if(XtIsManaged(XtParent(drawingAreaCores))) {
    core_clip_what = CORESAVED;
    DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
  }
  if(XtIsManaged(XtParent(drawingAreaLog))) {
    log_clip_what = CORESAVED;
    DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
  }

  XmListDeleteAllItems(listTieList);

  have_equivlogdep = DONT;

  /* free all alloc values in ties but don't free ties
  */
  if(have_coretotie == DO) {
    if(ties) {
      if(ties->tie) {
        for(i=0; i<ties->alloc_cnt; ++i) {
          if(ties->tie[i]) {
            free(ties->tie[i]);
            ties->tie[i] = NULL;
          }
        }
        free(ties->tie);
        ties->tie = NULL;
      }
      if(ties->temp_equivlogdep) {
        free(ties->temp_equivlogdep);
        ties->temp_equivlogdep = NULL;
      }
    }
    ties->nties = 0;
    ties->alloc_cnt = 0;
    ties->core_alloc = 0;
  }
}

void SaveSpliceTies(s, ds)
int s;
int ds;
{
  int i, j, k, l, m, d, eh, ec, h, c, hh, cc, dck, dhole, dcore, array, err, foundit, warn, nt;
  float *pdep;
  Ties   *tempties;
  int  CountTies(), MakeEquivLogDepthEquation(), CalcEquivLogDepths();
  void ApplyCoreAdjustToStrat(), DrawDrawingAreaCores(), DrawDrawingAreaLog(),
       CalcStratELD(), GetTieArrayNum(), FindMissingSectDep(), ApplyCoreAdjustToOneCore(), MakeHoleEqt();

  warn = NO;

  if(have_splice == DO && s >=0 && s < splice_cnt) {

    /* will have to deal with the situation of having previously
       saved ties and then deleted them. and now are saving again. 
       maybe the easiest way would be to remove all ties from
       the eld table (except unshared ones?)  Try this
       and what if got some ties from cores? they could be missing
       from splice?
    */
    for(h=0; h<eqlogdepth.numhole; ++h) {
      for(c=0; c<eqlogdepth.hole[h]->numcore; ++c) {
        if(eqlogdepth.hole[h]->core[c]->nties > 0) {
          for(i=0; i<eqlogdepth.hole[h]->core[c]->nties; ++i) {
            free(eqlogdepth.hole[h]->core[c]->tie[i]);
            eqlogdepth.hole[h]->core[c]->tie[i] = NULL;
          }
          free(eqlogdepth.hole[h]->core[c]->tie);
          eqlogdepth.hole[h]->core[c]->tie = NULL;
          eqlogdepth.hole[h]->core[c]->nties = 0;
        }
      }
    }

    /* search the ELD Table for each tie in splice. are searching
       for the particular hole-core of the tie
       skip the first tie; it's the one that pins the top of the splice at mudline
    */
    for(i=1; i<ties->nties; ++i) {
      foundit = NO;
      for(h=0; h<eqlogdepth.numhole; ++h) {
        for(c=0; c<eqlogdepth.hole[h]->numcore; ++c) {
          if(eqlogdepth.hole[h]->name == ties->tie[i]->holename &&
             eqlogdepth.hole[h]->core[c]->name == ties->tie[i]->corename) {

            if(eqlogdepth.hole[h]->core[c]->nties < 1) {
              if((eqlogdepth.hole[h]->core[c]->tie=(TieList**)calloc(1,sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("Sa");
              }
              if((eqlogdepth.hole[h]->core[c]->tie[0]=(TieList*)calloc(1,sizeof(TieList)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies1");
              }
              eqlogdepth.hole[h]->core[c]->nties = 1;
              eqlogdepth.hole[h]->core[c]->did_eld = YES;
              eqlogdepth.hole[h]->core[c]->tie[0]->holename = ties->tie[i]->holename;
              eqlogdepth.hole[h]->core[c]->tie[0]->corename = ties->tie[i]->corename;
              eqlogdepth.hole[h]->core[c]->type = ties->tie[i]->coretype;
              (void)sprintf(eqlogdepth.hole[h]->core[c]->tie[0]->coresection, "%s", ties->tie[i]->coresection);
              eqlogdepth.hole[h]->core[c]->tie[0]->array = 0;
              eqlogdepth.hole[h]->core[c]->tie[0]->log = ties->tie[i]->log;
              eqlogdepth.hole[h]->core[c]->tie[0]->selected = ties->tie[i]->selected;
              eqlogdepth.hole[h]->core[c]->tie[0]->topsect = ties->tie[i]->topsect;
              eqlogdepth.hole[h]->core[c]->tie[0]->botsect = ties->tie[i]->botsect;
              eqlogdepth.hole[h]->core[c]->tie[0]->rate = ties->tie[i]->rate;
              eqlogdepth.hole[h]->core[c]->tie[0]->b = ties->tie[i]->b;
              eqlogdepth.hole[h]->core[c]->tie[0]->corembsf = ties->tie[i]->corembsf;
              eqlogdepth.hole[h]->core[c]->tie[0]->coremcd = ties->tie[i]->coremcd;
              eqlogdepth.hole[h]->core[c]->tie[0]->coreautoeld = ties->tie[i]->coreautoeld;
              eqlogdepth.hole[h]->core[c]->tie[0]->equivlogdep = ties->tie[i]->logdep;
              eqlogdepth.hole[h]->core[c]->tie[0]->logdep = ties->tie[i]->logdep;
              eqlogdepth.hole[h]->core[c]->tie[0]->shared = ties->tie[i]->shared;
              foundit = YES;
            }
            else if(eqlogdepth.hole[h]->core[c]->nties >= 1) {

              /* sort tie into core tie list
              */
              l = -1;
              for(k=0; k<eqlogdepth.hole[h]->core[c]->nties; ++k) {
                if(fabs(eqlogdepth.hole[h]->core[c]->tie[k]->coremcd - ties->tie[i]->coremcd) <
                                     ROUNDCHECK) {
                  break;
                }
                else if(eqlogdepth.hole[h]->core[c]->tie[k]->coremcd > ties->tie[i]->coremcd) {
                  if((charptr=(char*)realloc((char*)eqlogdepth.hole[h]->core[c]->tie,(eqlogdepth.hole[h]->core[c]->nties+1)*sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies2");
                  }
                  else {
                    eqlogdepth.hole[h]->core[c]->tie = (TieList**)charptr;
                  }
                  if((eqlogdepth.hole[h]->core[c]->tie[eqlogdepth.hole[h]->core[c]->nties]=(TieList*)calloc(1,sizeof(TieList)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies3");
                  }
                  for(j=eqlogdepth.hole[h]->core[c]->nties; j>k; --j) {
                    *eqlogdepth.hole[h]->core[c]->tie[j] = *eqlogdepth.hole[h]->core[c]->tie[j-1];
                  }
                  l = k;
                  break;
                }
                else if(k == eqlogdepth.hole[h]->core[c]->nties - 1 && 
                        eqlogdepth.hole[h]->core[c]->tie[k]->coremcd < ties->tie[i]->coremcd) {
                  if((charptr=(char*)realloc((char*)eqlogdepth.hole[h]->core[c]->tie,(eqlogdepth.hole[h]->core[c]->nties+1)*sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies4");
                  }
                  else {
                    eqlogdepth.hole[h]->core[c]->tie = (TieList**)charptr;
                  }
                  if((eqlogdepth.hole[h]->core[c]->tie[eqlogdepth.hole[h]->core[c]->nties]=(TieList*)calloc(1,sizeof(TieList)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies5");
                  }
                  l = eqlogdepth.hole[h]->core[c]->nties;
                  break;
                }
              }
              if(l != -1) {
                ++eqlogdepth.hole[h]->core[c]->nties;
                eqlogdepth.hole[h]->core[c]->tie[l]->holename = ties->tie[i]->holename;
                eqlogdepth.hole[h]->core[c]->tie[l]->corename = ties->tie[i]->corename;
                (void)sprintf(eqlogdepth.hole[h]->core[c]->tie[l]->coresection, "%s", ties->tie[i]->coresection);
                eqlogdepth.hole[h]->core[c]->tie[l]->array = 0;
                eqlogdepth.hole[h]->core[c]->tie[l]->log = ties->tie[i]->log;
                eqlogdepth.hole[h]->core[c]->tie[l]->selected = ties->tie[i]->selected;
                eqlogdepth.hole[h]->core[c]->tie[l]->topsect = ties->tie[i]->topsect;
                eqlogdepth.hole[h]->core[c]->tie[l]->botsect = ties->tie[i]->botsect;
                eqlogdepth.hole[h]->core[c]->tie[l]->rate = ties->tie[i]->rate;
                eqlogdepth.hole[h]->core[c]->tie[l]->b = ties->tie[i]->b;
                eqlogdepth.hole[h]->core[c]->tie[l]->corembsf = ties->tie[i]->corembsf;
                eqlogdepth.hole[h]->core[c]->tie[l]->coremcd = ties->tie[i]->coremcd;
                eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld = ties->tie[i]->coremcd;
                eqlogdepth.hole[h]->core[c]->tie[l]->equivlogdep = ties->tie[i]->logdep;
                eqlogdepth.hole[h]->core[c]->tie[l]->logdep = ties->tie[i]->logdep;
                eqlogdepth.hole[h]->core[c]->tie[l]->shared = ties->tie[i]->shared;

                if(eqlogdepth.autocorr_status == YES) {
                  eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld =
                                     eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld * (eqlogdepth.autocorr_squish/100.0);
                }
                foundit = YES;
              }
            }
          }
          if(foundit == YES) {
            break;
          }
        }
        if(foundit == YES) {
          break;
        }
      }
    }

    /* share ties with other holes
       skip the first tie; it's the one that pins the top of the splice at mudline
    */
    for(i=1; i<ties->nties; ++i) {
      for(h=0; h<eqlogdepth.numhole; ++h) {
        if(eqlogdepth.hole[h]->name != ties->tie[i]->holename && ties->tie[i]->shared == YES) {
          foundit = NO;
          hh = eqlogdepth.hole[h]->map_hole_to_data[ds];
          for(c=0; c<eqlogdepth.hole[h]->numcore; ++c) {
            cc = eqlogdepth.hole[h]->core[c]->map_core_to_data[ds];
            if(ties->tie[i]->coremcd >= eqlogdepth.hole[h]->core[c]->mindep && ties->tie[i]->coremcd <= eqlogdepth.hole[h]->core[c]->maxdep &&
               data[ds]->holes[hh]->core[cc]->did_affine_offset == YES) {
              if(eqlogdepth.hole[h]->core[c]->nties < 1) {
                if((eqlogdepth.hole[h]->core[c]->tie=(TieList**)calloc(1,sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies6");
                }
                if((eqlogdepth.hole[h]->core[c]->tie[0]=(TieList*)calloc(1,sizeof(TieList)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies7");
                }
                l = 0;
                eqlogdepth.hole[h]->core[c]->nties = 1;
                eqlogdepth.hole[h]->core[c]->did_eld = YES;
                eqlogdepth.hole[h]->core[c]->tie[l]->holename = eqlogdepth.hole[h]->name;
                eqlogdepth.hole[h]->core[c]->tie[l]->corename = eqlogdepth.hole[h]->core[c]->name;
                eqlogdepth.hole[h]->core[c]->tie[l]->array = 0;
                eqlogdepth.hole[h]->core[c]->tie[l]->log = ties->tie[i]->log;
                eqlogdepth.hole[h]->core[c]->tie[l]->selected = ties->tie[i]->selected;
                eqlogdepth.hole[h]->core[c]->tie[l]->rate = ties->tie[i]->rate;
                eqlogdepth.hole[h]->core[c]->tie[l]->b = ties->tie[i]->b;
                eqlogdepth.hole[h]->core[c]->tie[l]->coremcd = ties->tie[i]->coremcd;
                eqlogdepth.hole[h]->core[c]->tie[l]->corembsf = ties->tie[i]->coremcd - eqlogdepth.hole[h]->core[c]->affine_offset;
                eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld = ties->tie[i]->coreautoeld;
                eqlogdepth.hole[h]->core[c]->tie[l]->equivlogdep = ties->tie[i]->logdep;
                eqlogdepth.hole[h]->core[c]->tie[l]->logdep = ties->tie[i]->logdep;
                eqlogdepth.hole[h]->core[c]->tie[l]->shared = ties->tie[i]->shared;
                foundit = YES;
              }
              else if(eqlogdepth.hole[h]->core[c]->nties >= 1) {

                /* sort tie into core tie list
                */
                l = -1;
                for(k=0; k<eqlogdepth.hole[h]->core[c]->nties; ++k) {
                  if(fabs(eqlogdepth.hole[h]->core[c]->tie[k]->coremcd - ties->tie[i]->coremcd) <
                                     ROUNDCHECK) {
                    break;
                  }
                  else if(eqlogdepth.hole[h]->core[c]->tie[k]->coremcd > ties->tie[i]->coremcd) {
                    if((charptr=(char*)realloc((char*)eqlogdepth.hole[h]->core[c]->tie,(eqlogdepth.hole[h]->core[c]->nties+1)*sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies8");
                    }
                    else {
                      eqlogdepth.hole[h]->core[c]->tie = (TieList**)charptr;
                    }
                    if((eqlogdepth.hole[h]->core[c]->tie[eqlogdepth.hole[h]->core[c]->nties]=(TieList*)calloc(1,sizeof(TieList)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies9");
                    }
                    for(j=eqlogdepth.hole[h]->core[c]->nties; j>k; --j) {
                      *eqlogdepth.hole[h]->core[c]->tie[j] = *eqlogdepth.hole[h]->core[c]->tie[j-1];
                    }
                    l = k;
                    break;
                  }
                  else if(k == eqlogdepth.hole[h]->core[c]->nties - 1 &&
                          eqlogdepth.hole[h]->core[c]->tie[k]->coremcd < ties->tie[i]->coremcd) {
                    if((charptr=(char*)realloc((char*)eqlogdepth.hole[h]->core[c]->tie,(eqlogdepth.hole[h]->core[c]->nties+1)*sizeof(TieList*)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies10");
                    }
                    else {
                      eqlogdepth.hole[h]->core[c]->tie = (TieList**)charptr;
                    }
                    if((eqlogdepth.hole[h]->core[c]->tie[eqlogdepth.hole[h]->core[c]->nties]=(TieList*)calloc(1,sizeof(TieList)))==NULL) {
                              SaBadMemAlloc("SaSaveSpliceTies11");
                    }
                    l = eqlogdepth.hole[h]->core[c]->nties;
                    break;
                  }
                }
                if(l != -1) {
                  ++eqlogdepth.hole[h]->core[c]->nties;
                  eqlogdepth.hole[h]->core[c]->tie[l]->holename = eqlogdepth.hole[h]->name;
                  eqlogdepth.hole[h]->core[c]->tie[l]->corename = eqlogdepth.hole[h]->core[c]->name;
                  eqlogdepth.hole[h]->core[c]->tie[l]->array = 0;
                  eqlogdepth.hole[h]->core[c]->tie[l]->log = ties->tie[i]->log;
                  eqlogdepth.hole[h]->core[c]->tie[l]->selected = ties->tie[i]->selected;
                  eqlogdepth.hole[h]->core[c]->tie[l]->rate = ties->tie[i]->rate;
                  eqlogdepth.hole[h]->core[c]->tie[l]->b = ties->tie[i]->b;
                  eqlogdepth.hole[h]->core[c]->tie[l]->coremcd = ties->tie[i]->coremcd;
                  eqlogdepth.hole[h]->core[c]->tie[l]->corembsf = ties->tie[i]->coremcd - eqlogdepth.hole[h]->core[c]->affine_offset;;
                  eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld = ties->tie[i]->coremcd;
                  eqlogdepth.hole[h]->core[c]->tie[l]->equivlogdep = ties->tie[i]->logdep;
                  eqlogdepth.hole[h]->core[c]->tie[l]->logdep = ties->tie[i]->logdep;
                  eqlogdepth.hole[h]->core[c]->tie[l]->shared = ties->tie[i]->shared;

                  if(eqlogdepth.autocorr_status == YES) {
                    eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld =
                                     eqlogdepth.hole[h]->core[c]->tie[l]->coreautoeld * (eqlogdepth.autocorr_squish/100.0);
                  }
                  foundit = YES;
                }
              }
              if(foundit == YES) {
                dck = -1;
                if(eqlogdepth.hole[h]->core[c]->map_core_to_data[ds] == -1) {
                  for(m=0; m<dataset_cnt; ++m) {
                    if(eqlogdepth.hole[h]->core[c]->map_core_to_data[m] != -1) {
                      dck = m;
                      break;
                    }
                  }
                }
                else {
                  dck = ds;
                }
                if(dck != -1) {
                  dhole = eqlogdepth.hole[h]->map_hole_to_data[dck];
                  dcore = eqlogdepth.hole[h]->core[c]->map_core_to_data[dck];
                  GetTieArrayNum(COREDATA, eqlogdepth.hole[h]->core[c]->tie[l], dck, dhole, dcore, &err);
                  if(eqlogdepth.hole[h]->core[c]->tie[l]->array != -1) {
                    array = eqlogdepth.hole[h]->core[c]->tie[l]->array;
                    (void)sprintf(eqlogdepth.hole[h]->core[c]->tie[l]->coresection, "%s",
                                  data[dck]->holes[dhole]->core[dcore]->value[array]->section);
                    eqlogdepth.hole[h]->core[c]->tie[l]->topsect = data[dck]->holes[dhole]->core[dcore]->value[array]->top_int;
                    eqlogdepth.hole[h]->core[c]->tie[l]->botsect = data[dck]->holes[dhole]->core[dcore]->value[array]->bot_int;
                  }
                  else if(err > -1) {
                    FindMissingSectDep(err, dck, dhole, dcore, h, c, eqlogdepth.hole[h]->core[c]->tie[l]->corembsf,
                                       &eqlogdepth.hole[h]->core[c]->tie[l]->coresection,
                                       &eqlogdepth.hole[h]->core[c]->tie[l]->topsect,
                                       &eqlogdepth.hole[h]->core[c]->tie[l]->botsect);
                  }
                  else {
                    warn = YES;
                    (void)sprintf(eqlogdepth.hole[h]->core[c]->tie[l]->coresection, "X");
                              eqlogdepth.hole[h]->core[c]->tie[l]->topsect = 0.0;
                              eqlogdepth.hole[h]->core[c]->tie[l]->botsect = 0.0;
                  }
                }
                else {
                  (void)sprintf(eqlogdepth.hole[h]->core[c]->tie[l]->coresection, "X");
                  eqlogdepth.hole[h]->core[c]->tie[l]->topsect = 0.0;
                  eqlogdepth.hole[h]->core[c]->tie[l]->botsect = 0.0;
                }
              }
            }
            if(foundit == YES) {
              break;
            }
          }
        }
      }
    }

    if(warn == YES) {
      if((charptr=(char*)calloc(200,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
      (void)sprintf(charptr, "\nWarning; Updating ELD Table. one or more shared ties could not find sect. and sect. depth.");
      SaManageWarningBox(charptr);
      free(charptr);
    }

    MakeHoleEqt();
    for(d=0; d<dataset_cnt; ++d) {
      for(h=0; h<data[d]->numholes; ++h) {
        for(c=0; c<data[d]->holes[h]->numcores; ++c) {

          if((pdep=(float*)calloc(data[d]->holes[h]->core[c]->numvalues, sizeof(float))) == NULL)
                                    SaBadMemAlloc("Sa");
          for(l=0; l<data[d]->holes[h]->core[c]->numvalues; ++l) {
            if(data[d]->holes[h]->core[c]->value[l]->quality  & BAD_SB_DEPTH) {
              pdep[l] = BADPT;
            }
            else {
              pdep[l] = data[d]->holes[h]->core[c]->value[l]->mbsf;
              if(have_affine == DO) {
                pdep[l] = pdep[l] + data[d]->holes[h]->core[c]->affine_offset;
              }
            }
          }
          if(eqlogdepth.autocorr_status == YES) {
            ApplyCoreAdjustToOneCore(pdep, data[d]->holes[h]->core[c]->numvalues);
          }
          l = data[d]->holes[h]->map_hole_to_eqlogdepth;
          if(eld.hole_eqt[l]->nties == 0 && eqlogdepth.autocorr_status == NO) {
            data[d]->holes[h]->core[c]->have_equil_log_depths = DONT;
          }
          else {
            if(eld.hole_eqt[l]->nties > 0) {
              CalcELDInHole(d, h, c, eld.hole_eqt[l], pdep);
            }
            data[d]->holes[h]->core[c]->have_equil_log_depths = DO;
            for(l=0; l<data[d]->holes[h]->core[c]->numvalues; ++l) {
              data[d]->holes[h]->core[c]->value[l]->eld = pdep[l];
            }
          }
          free(pdep);
          pdep = NULL;
        }
      }
    }

    /* assign splice eld's
    */
    for(i=0; i<splice[s]->numvalues; ++i) {
      splice[s]->value[i]->eld = ties->temp_equivlogdep[i];
    }

    if(have_strat == DO) {
      CalcStratELD();
    }

    have_tie1 = DONT;
    have_tie2 = DONT;
    have_ties = DONT;
    XtSetSensitive(pushButtonSelectCore, False);
    XtSetSensitive(pushButtonCancelCore, False);
    XtSetSensitive(pushButtonClearTie, False);
    XtSetSensitive(pushButtonClearLastTieSaved, False);
    XtSetSensitive(pushButtonShowHideTieList, True);
    XtSetSensitive(pushButtonAddTieToList, False);
    XtSetSensitive(toggleButtonAddTieToCore, False);
    XtSetSensitive(toggleButtonAddTieToAll, True);
    XtSetSensitive(pushButtonOptimizeTie, True);
    XtSetSensitive(pushButtonAdjustToTiesParameters, False);
    XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
    XtSetSensitive(pushButtonSaveChanges, False);
    XtSetSensitive(pushButtonUndoSaveChange, False);
    
    if((nt = CountTies()) > 0 || eqlogdepth.autocorr_status == YES) {
      XtSetSensitive(pushButtonSaveEquvilentLogDepthTable, True);
      XtSetSensitive(pushButtonSaveCoreData, True);
      XtSetSensitive(pushButtonSaveSplice, True);
      if(have_strat == DO) {
        XtSetSensitive(pushButtonSaveStrat, True);
      }
    }
    else {
      XtSetSensitive(pushButtonSaveEquvilentLogDepthTable, False);
      XtSetSensitive(pushButtonSaveCoreData, False);
      XtSetSensitive(pushButtonSaveSplice, False);
      XtSetSensitive(pushButtonSaveStrat, False);
    }
    if(nt > 0) {
      XtSetSensitive(pushButtonAutoCorrelate, False);
    }
    else {
      XtSetSensitive(pushButtonAutoCorrelate, True);
    }

    if(XtIsManaged(XtParent(drawingAreaCores))) {
      core_clip_what = CORESAVED;
      DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
    }
    if(XtIsManaged(XtParent(drawingAreaLog))) {
      log_clip_what = CORESAVED;
      DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
    }
  }
}

void FindMissingSectDep(near, ds, dh, dc, eh, ec, tiedep, sect, topdep, botdep)
int near;
int ds;
int dh, dc, eh, ec;
float tiedep;
char *sect;
float *topdep;
float *botdep;
{
  int   i, foundit;
  float diff;
 
  if(near < 0 || near > data[ds]->holes[dh]->core[dc]->numvalues) {
    return;
  }

  diff = fabs((tiedep - data[ds]->holes[dh]->core[dc]->value[near]->mbsf)*100.0);

  if(tiedep < data[ds]->holes[dh]->core[dc]->value[near]->mbsf) {
    /* is in the same section
    */
    if(data[ds]->holes[dh]->core[dc]->value[near]->top_int - diff >= 0.0) {
      (void)sprintf(sect, "%s", data[ds]->holes[dh]->core[dc]->value[near]->section); 
      *topdep = data[ds]->holes[dh]->core[dc]->value[near]->top_int - diff;
      *botdep = *topdep;
    }
    else {
      /* go up a section
      */
      i = near;
      diff = diff - data[ds]->holes[dh]->core[dc]->value[near]->top_int;
      foundit = NO;
      if(strcmp(data[ds]->holes[dh]->core[dc]->value[near]->section, "1") == 0) {
      }
      else {
        while(foundit == NO) {
          while(i > 0 && (strcmp(data[ds]->holes[dh]->core[dc]->value[near]->section, 
                                   data[ds]->holes[dh]->core[dc]->value[i]->section) == 0)) {
            --i;
          }
          if((150.0 - diff) < 0.0) {
            diff = diff - 150.0;
          }
          else {
            foundit = YES;
            (void)sprintf(sect, "%s", data[ds]->holes[dh]->core[dc]->value[i]->section);
            *topdep = 150.0 - diff;
            *botdep = *topdep;
          }
        }
      }
    }
  }
  else if(tiedep > data[ds]->holes[dh]->core[dc]->value[near]->mbsf) {
    /* is in the same section
    */
    if(data[ds]->holes[dh]->core[dc]->value[near]->top_int + diff <= 150.0) {
      (void)sprintf(sect, "%s", data[ds]->holes[dh]->core[dc]->value[near]->section);
      *topdep = data[ds]->holes[dh]->core[dc]->value[near]->top_int + diff;
      *botdep = *topdep;
    }
    else {
      /* go down a section
      */
      i = near;
      diff = diff + data[ds]->holes[dh]->core[dc]->value[near]->top_int - 150.0;
      foundit = NO;
      while(foundit == NO) {
        while(i < data[ds]->holes[dh]->core[dc]->numvalues-1 && (strcmp(data[ds]->holes[dh]->core[dc]->value[near]->section, 
                                   data[ds]->holes[dh]->core[dc]->value[i]->section) == 0)) {
          ++i;
        }
        if((150.0 - diff) < 0.0) {
          diff = diff - 150.0;
        }
        else {
          foundit = YES;
          (void)sprintf(sect, "%s", data[ds]->holes[dh]->core[dc]->value[i]->section);
          *topdep = diff;
          *botdep = *topdep;
        }
      }
    }
  }
  else {

  }
}

int HaltWork(who)
int who;
{
  XEvent  event;

  while(XCheckMaskEvent(XtDisplay(messageBoxWorking), ButtonPressMask, &event)) {
    if(event.xany.window == XtWindow(messageBoxWorking)) {
      XtDispatchEvent(&event);
      info.error = True;
      info.fatal=False;
      info.default_button=XmDIALOG_OK_BUTTON;
      info.type=XmDIALOG_WARNING;
      info.caller=NONE;
      if((info.message=(char*)calloc(500,CHAR_SIZE))==NULL)
                                        SaBadMemAlloc("Sa");
      if(who == COREDATA) {
        (void)sprintf(info.message,"Data input has been stopped.\n");
        (void)strcat(info.message, "An attempt to clear has been made but program may be unstable.");
      }
      else if(who == LOGDATA) {

      }
      else if(who == ANCDATA) {

      }
      else if(who == SPLICEDATA) {

      }
      else if(who == STRATDATA) {

      }
      return True;
    }
    else {
    }
  }
  return False;
}

int FindWhereInCore(coreptr, dep, place)
Core  *coreptr;
float dep;
int *place;
{
  int i, j, k;

  if(coreptr->numgood < 2) {
    return 1;
  }
  for(i=0; i<coreptr->numgood-1; ++i) {
    j = coreptr->good[i];
    k = coreptr->good[i+1];
    if(dep >= coreptr->value[j]->mbsf && dep < coreptr->value[k]->mbsf) {
      *place = j;
      return 0;
    }
  }

  return 1;
}

int FindWhereInSplice(s, dep, place)
int   s;
float dep;
int   *place;
{
  int i, j, k;

  if(splice[s]->numgood < 2) {
    return 1;
  }
  for(i=0; i<splice[s]->numgood-1; ++i) {
    j = splice[s]->good[i];
    k = splice[s]->good[i+1];
    if(dep >= splice[s]->value[j]->mcd && dep < splice[s]->value[k]->mcd) {
      *place = j;
      return 0;
    }
  }

  return 1;
}

int FindWhereInLog(logptr, dep, place)
LogData  *logptr;
float dep;
int *place;
{
  int i, j, k;

  if(logptr->numgood < 2) {
    return 1;
  }
  for(i=0; i<logptr->numgood-1; ++i) {
    j = logptr->good[i];
    k = logptr->good[i+1];
    if(dep >= logptr->value[j]->depth && dep < logptr->value[k]->depth) {
      *place = j;
      return 0;
    }
  }

  return 1;
}

void SetUpCalcStrat()
{
  int h, c;

/*
  for(h=0; h<eqlogdepth.numhole; ++h) {

    ec = 0;
    while(ec < eqlogdepth.hole[h]->numcore && eqlogdepth.hole[h]->core[ec]->nties == 0) {
        ++ec;
    }
    if(ec < eqlogdepth.hole[eh]->numcore) {
      if((tempties=(Ties*)calloc(1,sizeof(Ties)))==NULL)
                                SaBadMemAlloc("Sa");
      tempties->nties = 2;
      if((tempties->tie=(TieList**)calloc(tempties->nties,sizeof(TieList*)))==NULL)
                              SaBadMemAlloc("Sa");
      for(l=0; l<tempties->nties; ++l) {
        if((tempties->tie[l]=(TieList*)calloc(1,sizeof(TieList)))==NULL)
                                SaBadMemAlloc("Sa");
      }
      tempties->tie[0]->corembsf = 0.0;
      tempties->tie[0]->coremcd = 0.0;
      tempties->tie[0]->coreautoeld = 0.0;
      tempties->tie[0]->logdep = 0.0;
      tempties->tie[1]->corembsf = eqlogdepth.hole[eh]->core[ec]->tie[0]->corembsf;
      tempties->tie[1]->coremcd = eqlogdepth.hole[eh]->core[ec]->tie[0]->coremcd;
      tempties->tie[1]->coreautoeld = eqlogdepth.hole[eh]->core[ec]->tie[0]->coreautoeld;
      tempties->tie[1]->logdep = eqlogdepth.hole[eh]->core[ec]->tie[0]->logdep;
      MakeEquivLogDepthEquation(tempties);

      for(j=ec; j<eqlogdepth.hole[h]->numcore; ++j) {
        CalcStrat(i, j);
      }
    }
    else {

    }
  }
*/
}

void FindCurrentTieArray(who, set)
int who;
int set;
{
  int i, err, derr, dum;
  char *string, *varstring;
  float val, valsm, affoff, autooff;
  String str;
  XmString xmstring;
  int InterpolateCoreTiePtValue(), InterpolateSpliceTiePtValue(), InterpolateLogTiePtValue();
 
  if(who == COREDATA) {
    ties->coretie = -1;
    if(have_affine == DO) {
      affoff = data[set]->holes[ties->hole]->core[ties->core]->affine_offset;
    }
    else {
      affoff = 0.0;
    }
    for(i=0; i<data[set]->holes[ties->hole]->core[ties->core]->numvalues; ++i) {
      if(fabs(ties->tiecoremcd - (data[set]->holes[ties->hole]->core[ties->core]->value[i]->mbsf + affoff)) < ROUNDCHECK) {
        ties->coretie = i;
        break;
      }
    }
    if(ties->coretie != -1) {
      val = *data[set]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->data;
      valsm = data[set]->holes[ties->hole]->core[ties->core]->value[ties->coretie]->sm_data;
      err = 0;
    }
    else {
      err = InterpolateCoreTiePtValue(set, ties->hole, ties->core, NO, ties->tiecorembsf, &val, &dum);
      if(err == 0) {
        ties->coretie = INTERP;
      }
      else {
        ties->coretie = MISSING;
      }
      if(err == 0 && smooth.method != NONE) {
        derr = InterpolateCoreTiePtValue(set, ties->hole, ties->core, YES, ties->tiecorembsf, &valsm, &dum);
      }
      else if(err == 0) {
        valsm = val;
      }
    }
    if(err != 0) {
      ties->tiecoreval = 0.0;
      ties->tiecoresmval = 0.0;
    }
    else {
      ties->tiecoreval = val;
      ties->tiecoresmval = valsm;
    }

    if(err == 0) {
      if((string=(char*)calloc(1000,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
      if((varstring=(char*)calloc(100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
      (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       data[set]->leg,
                       data[set]->site,
                       data[set]->holes[ties->hole]->name,
                       data[set]->holes[ties->hole]->core[ties->core]->num,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
      if(smooth.method != NONE) {
        (void)sprintf(varstring, " %.2f", valsm);
        (void)strncat(string, varstring, (int)strlen(varstring));
      }
      if(have_affine == DO && data[set]->holes[ties->hole]->core[ties->core]->did_affine_offset ==  YES) {
        (void)sprintf(varstring, "    %.2f(mcd, Y)", ties->tiecoremcd);
        (void)strncat(string, varstring, (int)strlen(varstring));
      }
      else if(have_affine == DO && data[set]->holes[ties->hole]->core[ties->core]->did_affine_offset == NO) {
        (void)sprintf(varstring, "    %.2f(mcd, N)", ties->tiecoremcd);
        (void)strncat(string, varstring, (int)strlen(varstring));
      }
      (void)sprintf(varstring, "   %.2f(eld)", ties->tiecoretempeqlogdep);
      (void)strncat(string, varstring, (int)strlen(varstring));
      if(ties->coretie == -1) {
        (void)strcat(string, "  IntTie");
      }
      else {
        (void)strcat(string, "  Tie");
      }
      xmstring = XmStringCreateSimple(string);
      free(string);
      string = NULL;
      free(varstring);
      varstring = NULL;
    }
    else {
      xmstring = XmStringCreateSimple("Tie not within range of this core");
    }
    XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }
  else if(who == SPLICEDATA) {
    ties->coretie = -1;
    for(i=0; i<splice[set]->numvalues; ++i) {
      if(fabs(ties->tiecoremcd - splice[set]->value[i]->mcd) < ROUNDCHECK) {
        ties->coretie = i;
      }
    }
    if(ties->coretie != -1) {
      val = splice[set]->value[ties->coretie]->data;
      valsm = splice[set]->value[ties->coretie]->sm_data;
      err = 0;
    }
    else {
      err = InterpolateSpliceTiePtValue(set, NO, ties->tiecoremcd, &val, &dum);
      if(err == 0 && splice[set]->smooth.method != NONE) {
        derr = InterpolateSpliceTiePtValue(set, YES, ties->tiecoremcd, &valsm, &dum);
      }
      if(err == 0) {
        ties->coretie = INTERP;
      }
      else {
        ties->coretie = MISSING;
      }
    }
    if(err != 0) {
      ties->tiecoreval = 0.0;
      ties->tiecoresmval = 0.0;
    }
    else {
      ties->tiecoreval = val;
      ties->tiecoresmval = valsm;
    }
    if(err == 0) {
      if((string=(char*)calloc(1000,CHAR_SIZE))==NULL)
                    SaBadMemAlloc("Sa");
      if((varstring=(char*)calloc(100,CHAR_SIZE))==NULL)
                    SaBadMemAlloc("Sa");
      (void)sprintf(string,"%d %d %c  %d %c %s %.1f %.1f  %.2f(mbsf) %.2f",
                       splice[set]->leg,
                       splice[set]->site,
                       ties->tiecoreholename,
                       ties->tiecorename,
                       ties->tiecoretype,
                       ties->tiecoresection,
                       ties->tiecoretopsect,
                       ties->tiecorebotsect,
                       ties->tiecorembsf,
                       val);
      if(splice[set]->smooth.method != NONE) {
        (void)sprintf(varstring, "   %.2f", valsm);
        (void)strncat(string, varstring, (int)strlen(varstring));
      }
      if(eqlog == TEMP) {
        (void)sprintf(varstring, "   %.2f(mcd)  %.2f(eld)", ties->tiecoremcd,
                                                ties->tiecoretempeqlogdep);
        (void)strncat(string, varstring, (int)strlen(varstring));
      }
      else {
        (void)sprintf(varstring, "   %.2f(mcd)", ties->tiecoremcd);
        (void)strncat(string, varstring, (int)strlen(varstring));
      }
      if(ties->coretie == -1) {
        (void)strcat(string, "  IntTie");
      }
      else {
        (void)strcat(string, "  Tie");
      }
      xmstring = XmStringCreateSimple(string);
      free(string);
      string = NULL;
      free(varstring);
      varstring = NULL;
    }
    else {
      xmstring = XmStringCreateSimple("The tie is not within the range of this splice");
    }
    XtVaSetValues(labelCursorLocation, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }
  else if(who == LOGDATA) {
    err = InterpolateLogTiePtValue(set, NO, ties->tielogdep, &val, &ties->logtie);
    if(err == 0 && logdata[set]->info.smooth.method != NONE) {
      err = InterpolateLogTiePtValue(set, YES, ties->tielogdep, &valsm, &dum);
    }
    else if(err == 0) {
      valsm = val;
    }

    if((string=(char*)calloc(100,CHAR_SIZE))==NULL)
                                SaBadMemAlloc("Sa");
    if(err != 0 || ties->logtie == MISSING) {
      ties->tielogval = 0.0;
      ties->tielogsmval = 0.0;
      xmstring = XmStringCreateSimple("The tie is not within the range of this Log");
    }
    else {
      ties->tielogval = val;
      ties->tielogsmval = valsm;

      if(eqlogdepth.did_log_mudline_adj == YES) {
        (void)sprintf(string,"%.3f   %.3f(mud line adj.)   %.3f", (ties->tielogdep - eqlogdepth.autocorr_log_mudline_offset),
                      ties->tielogdep, ties->tielogval);
      }
      else if(autocoreadj.draw_results == YES) {
        str = XmTextGetString(textFieldAutoCorrResultOffset);
        autooff = atof(str);
        XtFree(str);
        (void)sprintf(string,"%.3f   %.3f(test mud line adj.)   %.3f", ties->tielogdep, ties->tielogdep + autooff, val);
      }
      else {
        (void)sprintf(string,"%.3f   %.3f", ties->tielogdep, val);
      }
      if(logdata[set]->info.smooth.method != NONE) {
        if((varstring=(char*)calloc(20,CHAR_SIZE))==NULL)
                                  SaBadMemAlloc("Sa");
        (void)sprintf(varstring, " %.2f", valsm);
        (void)strncat(string, varstring, (int)strlen(varstring));
        free(varstring);
        varstring = NULL;
      }
      if(ties->logtie == -1) {
        (void)strcat(string, "  IntTie");
      }
      else {
        (void)strcat(string, "  Tie");
      }
    }
    xmstring = XmStringCreateSimple(string);
    free(string);
    string = NULL;
    XtVaSetValues(labelLogTiePoint, XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);
  }
}

void ChangeLogSet(ls)
int ls;
{
  int i, err, dum, sm;
  char *string, *varstring;
  float val, smval, autooff;
  String str;
  XmString xmstring;
  void GetTieArrayNum(), SetLogMinMax(), FindCurrentTieArray();
  int  FindLogMinMax(), InterpolateLogTiePtValue(), CorrelationSetup();

  SetLogMinMax(logdata[ls]);
  GetDisplayValues(dset, ls);
  if((string = (char*)calloc(100, CHAR_SIZE)) == NULL)
               SaBadMemAlloc("Sa");
  (void)sprintf(string, "%s", logdata[ls]->info.id[0]);
  xmstring = XmStringCreateSimple(string);
  XtVaSetValues(pushButtonDisplayLog, XmNlabelString, xmstring, NULL);
  XmStringFree(xmstring);
  XtVaSetValues(optionMenuDisplayLogType, XmNmenuHistory, pushButtonDisplayLog, NULL);
  free(string);
  if(logdata[ls]->flip == YES) {
    XmToggleButtonSetState(toggleButtonLogAxisFlipNo, False, False);
    XmToggleButtonSetState(toggleButtonLogAxisFlipYes, True, False);
  }
  else if(logdata[ls]->flip == NO) {
    XmToggleButtonSetState(toggleButtonLogAxisFlipNo, True, False);
    XmToggleButtonSetState(toggleButtonLogAxisFlipYes, False, False);
  }

  if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
    if(ties->nties > 1) {
      ties->tie[0]->log = -1;
      ties->tie[0]->array = -1;
      for(i=1; i<ties->nties; ++i) {
        GetTieArrayNum(LOGDATA, ties->tie[i], lset, 0, 0, &err);
      }
    }
  }
  else if(have_cores == DO && have_coretotie == DO) {
    if(ties->nties > 0) {
      for(i=0; i<ties->nties; ++i) {
        GetTieArrayNum(LOGDATA, ties->tie[i], lset, 0, 0, &err);
      }
    }
  }

  if(have_tie2 == DO) {
    FindCurrentTieArray(LOGDATA, ls);
  }
  if(have_tie1 == DO && have_tie2 == DO && ties->coretie != MISSING && ties->logtie != MISSING) {
    if(err = CorrelationSetup(dset) != 0) {
      have_correlate = DONT;
      if(err == 2) {
        (void)sprintf(buffer, "Error; correlation, new log set is missing tie pt, could not interpolate.\n");
      }
      else {
        (void)sprintf(buffer, "Error doing correlation.\n");
      }
        SaManageWarningBox(buffer);
      }
    else {
      have_correlate = DO;
    }
  }
}

void ChangeDataSet(ds)
int ds;
{
  int i, j, k, hole, core, derr, err, dum;
  char *string, *varstring;
  float *pdep, dep, affoff, val, valsm;
  XmString xmstring;
  int  FindAveDepth(), CalcEquivLogDepths(), GetDisplayValues();
  void SetMinMax(), DrawDrawingAreaAutoCorrelate(),
       GetTieArrayNum(), ApplyCoreAdjustToOneCore(), AdjustToTies(), FindCurrentTieArray();

  /* if have log and have core to tie and there are ties
     ( in tie list or current pick)
  */
  if(have_log == DO && have_coretotie == DO) {

    /* find the hole and core
    */
    hole = -1;
    core = -1;
    for(i=0; i<data[ds]->numholes; ++i) {
      if(ties->tiecoreholename == data[ds]->holes[i]->name) {
        hole = i;
        break;
      }
    }
    if(hole != -1) {
      for(j=0; j<data[ds]->holes[hole]->numcores; ++j) {
        if(ties->tiecorename == data[ds]->holes[hole]->core[j]->num) {
          core = j;
          break;
        }
      }
    }

    if(hole != -1 && core != -1) {
      ties->hole = hole;
      ties->core = core;
      missing_coretotie = NO;
      free(ties->temp_equivlogdep);
      ties->temp_equivlogdep = NULL;
      free(ties->coredepth);
      ties->coredepth = NULL;
      ties->core_alloc = data[ds]->holes[ties->hole]->core[ties->core]->numvalues;
      if((ties->temp_equivlogdep=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
      if((ties->coredepth=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");

      /* if have a tie in core see if the tie exists in the current data set
         if not interp pt.
      */
      if(have_tie1 == DO) {
        FindCurrentTieArray(COREDATA, ds);
      }

      if(ties->nties > 0) {
        if((pdep=(float*)calloc(data[ds]->holes[ties->hole]->core[ties->core]->numvalues, sizeof(float))) == NULL) {
                                    SaBadMemAlloc("Sa");
        }
        for(i=0; i<data[ds]->holes[ties->hole]->core[ties->core]->numvalues; ++i) {
          if(data[ds]->holes[ties->hole]->core[ties->core]->value[i]->quality  & BAD_SB_DEPTH) {
            pdep[i] = BADPT;
          }
          else {
            pdep[i] = data[ds]->holes[ties->hole]->core[ties->core]->value[i]->mbsf;
            if(have_affine == DO) {
              pdep[i] = pdep[i] + data[ds]->holes[ties->hole]->core[ties->core]->affine_offset;
            }
          }
        }

        if(eqlogdepth.autocorr_status == YES) {
          ApplyCoreAdjustToOneCore(pdep, data[ds]->holes[ties->hole]->core[ties->core]->numvalues);
        }
        for(i=0; i<data[ds]->holes[ties->hole]->core[ties->core]->numvalues; ++i) {
          ties->coredepth[i] = pdep[i];
        }

        derr = CalcEquivLogDepths(data[ds]->holes[ties->hole]->core[ties->core]->numvalues, pdep, ties);
        if(derr < 1) {
          for(i=0; i<data[ds]->holes[ties->hole]->core[ties->core]->numvalues; ++i) {
            ties->temp_equivlogdep[i] = pdep[i];
          }
        }
        free(pdep);
        pdep = NULL;

        for(i=0; i<ties->nties; ++i) {
          GetTieArrayNum(COREDATA, ties->tie[i], ds, ties->hole, ties->core, &err);
        }
        XtSetSensitive(pushButtonSaveChanges, True);
      }
      else if(ties->nties < 1) {
        for(j=0; j<data[ds]->holes[ties->hole]->core[ties->core]->numvalues; ++j) {
          if(data[ds]->holes[ties->hole]->core[ties->core]->value[j]->quality  & BAD_SB_DEPTH) {
            ties->coredepth[j] = BADPT;
            ties->temp_equivlogdep[j] = BADPT;
          }
          else {
            ties->coredepth[j] = data[ds]->holes[ties->hole]->core[ties->core]->value[j]->mbsf;
            if(have_affine == DO) {
              ties->coredepth[j] = ties->coredepth[j] + data[ds]->holes[ties->hole]->core[ties->core]->affine_offset;
            }
            if(eqlogdepth.autocorr_status == YES) {
              ties->coredepth[j] = ties->coredepth[j] * (eqlogdepth.autocorr_squish/100.0);
            }
            if(data[ds]->holes[ties->hole]->core[ties->core]->have_equil_log_depths == DO) {
              ties->temp_equivlogdep[j] = data[ds]->holes[ties->hole]->core[ties->core]->value[j]->eld;
            }
            else {
              ties->temp_equivlogdep[j]= ties->coredepth[j];
            }
          }
        }
        XtSetSensitive(pushButtonSaveChanges, False);
      }
      FindAveDepth(ds, ties->hole, ties->core, &ties->avedepthstep);
      if((string=(char*)calloc(100,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
      if(ties->avedepthstep >= logdata[lset]->avedepthstep) {
        (void)sprintf(string, "%.2f", ties->avedepthstep);
      }
      else {
        (void)sprintf(string, "%.2f", logdata[lset]->avedepthstep);
      }
      XtVaSetValues(textFieldDepthStep, XmNvalue, string, NULL);
      free(string);
      string = NULL;

      /* if have a tie re-correlate
      */
      if(have_tie1 == DO && have_tie2 == DO && ties->coretie != MISSING && ties->logtie != MISSING) {
        if(derr = CorrelationSetup(ds) != 0) {
          have_correlate = DONT;
          if(ties->coretie == -1) {

          }
          else if(derr == 2) {
            (void)sprintf(buffer, "Error; correlation, the data set switched to is missing pt, could not interpolate.\n");
          }
          else {
            (void)sprintf(buffer, "Error doing correlation.\n");
          }
          SaManageWarningBox(buffer);
        }
        else {
          have_correlate = DO;
        }
      }
    }
    else {
      missing_coretotie = YES;
      XtSetSensitive(pushButtonSaveChanges, False);
    }
  }
  else if(have_splice == DO) {
    sset = -1;
    for(i=0; i<splice_cnt; ++i) {
      if(ds == splice[i]->maptodata) {
        sset = i;
      }
    }
    if(sset >= 0 && sset < splice_cnt) {
      free(ties->temp_equivlogdep);
      ties->temp_equivlogdep = NULL;
      free(ties->coredepth);
      ties->coredepth = NULL;
      ties->core_alloc = splice[sset]->numvalues;
      if((ties->temp_equivlogdep=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
      if((ties->coredepth=(float*)calloc(ties->core_alloc,sizeof(float)))==NULL)
                      SaBadMemAlloc("Sa");
      if(ties->nties > 1) {
        ties->tie[0]->log = -1;
        ties->tie[0]->array = -1;
        for(i=1; i<ties->nties; ++i) {
          GetTieArrayNum(SPLICEDATA, ties->tie[i], sset, 0, 0, &err);
        }
        AdjustToTies(sset);
      }
      else {
        for(i=0; i<splice[sset]->numvalues; ++i) {
          if(splice[sset]->value[i]->quality  & BAD_SB_DEPTH) {
            ties->temp_equivlogdep[i] = BADPT;
            ties->coredepth[i] = BADPT;
          }
          else {
            ties->coredepth[i] = splice[sset]->value[i]->mcd;
            if(eqlogdepth.autocorr_status == YES) {
              ties->coredepth[i] = ties->coredepth[i] * (eqlogdepth.autocorr_squish/100.0);
            }
            ties->temp_equivlogdep[i]= ties->coredepth[i];
          }
        }
      }
      for(i=0; i<splice[sset]->numvalues; ++i) {
        splice[sset]->value[i]->eld = ties->temp_equivlogdep[i];
      }

      if(have_tie1 == DO) {
        FindCurrentTieArray(SPLICEDATA, sset);
      }

      /* if have a tie re-correlate
      */
      if(have_tie1 == DO && have_tie2 == DO && ties->coretie != MISSING && ties->logtie != MISSING) {
        if(derr = CorrelationSetup(sset) != 0) {
          have_correlate = DONT;
          if(ties->coretie == -1) {

          }
          else if(derr == 2) {
            (void)sprintf(buffer, "Error; correlation, the data set switched to is missing pt, could not interpolate.\n");
          }
          else {
            (void)sprintf(buffer, "Error doing correlation.\n");
          }
          SaManageWarningBox(buffer);
        }
        else {
          have_correlate = DO;
        }
      }
    }
    else {

    }
  }
  else {
  }

  SetMinMax(ds);
  err = GetDisplayValues(ds, lset);

  if(have_strat == DO) {
    MapStratToData(ds);
  }

  if(data[ds]->flip == YES) {
    XmToggleButtonSetState(toggleButtonVarAxisFlipNo, False, False);
    XmToggleButtonSetState(toggleButtonVarAxisFlipYes, True, False);
    XmToggleButtonSetState(toggleButtonAutoCorrelateFlipYes, True, False);
    XmToggleButtonSetState(toggleButtonAutoCorrelateFlipNo, False, False);
  }
  else {
    XmToggleButtonSetState(toggleButtonVarAxisFlipNo, True, False);
    XmToggleButtonSetState(toggleButtonVarAxisFlipYes, False, False);
    XmToggleButtonSetState(toggleButtonAutoCorrelateFlipYes, False, False);
    XmToggleButtonSetState(toggleButtonAutoCorrelateFlipNo, True, False);
  }

  /* put the hole names in the autocorr list
  */
  if((string=(char*)calloc(10,CHAR_SIZE))==NULL)
                        SaBadMemAlloc("Sa");
  XmListDeleteAllItems(listAutoCorrHoleSelect);
  if(have_splice == DO) {
    xmstring = XmStringCreateSimple("Splice");
    XmListAddItemUnselected(listAutoCorrHoleSelect, xmstring, 0);
    XmStringFree(xmstring);
  }
  else {
    for(i=0; i<data[ds]->numholes; ++i) {
      (void)sprintf(string, "Hole %c",data[ds]->holes[i]->name);
      xmstring = XmStringCreateSimple(string);
      XmListAddItemUnselected(listAutoCorrHoleSelect, xmstring, 0);
      XmStringFree(xmstring);
    }
  }
  XmListSelectPos(listAutoCorrHoleSelect, 1, NULL);
  free(string);
  string = NULL;
}

void SetDataSetOptionMenu(ds)
int ds;
{
  XmString xmstring;

  if(data[ds]->type == GRAPE) {
    xmstring=XmCvtCTToXmString("Grape");
  }
  else if(data[ds]->type == PWAVE) {
    xmstring=XmCvtCTToXmString("Pwave");
  }
  else if(data[ds]->type == SUSCEPTIBILITY) {
    xmstring=XmCvtCTToXmString("Susceptibility");
  }
  else if(data[ds]->type == NATURALGAMMA) {
    xmstring=XmCvtCTToXmString("Natural Gamma");
  }
  else if(data[ds]->type == REFLECTANCE) {
    xmstring=XmCvtCTToXmString("Reflectance");
  }
  else if(data[ds]->type == OTHERTYPE) {
    xmstring=XmCvtCTToXmString("Other");
  }
  else {
    xmstring=XmCvtCTToXmString("Other");
  }

  if(ds + 1 == 1) {
    XtManageChild(optionMenuDataSet);
    XtManageChild(pushButtonDataSet1);
    XtVaSetValues(pushButtonDataSet1,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet1, NULL);
  }
  else if(ds + 1 == 2) {
    XtManageChild(pushButtonDataSet2);
    XtVaSetValues(pushButtonDataSet2,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet2, NULL);
  }
  else if(ds + 1 == 3) {
    XtManageChild(pushButtonDataSet3);
    XtVaSetValues(pushButtonDataSet3,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet3, NULL);
  }
  else if(ds + 1 == 4) {
    XtManageChild(pushButtonDataSet4);
    XtVaSetValues(pushButtonDataSet4,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet4, NULL);
  }
  else if(ds + 1 == 5) {
    XtManageChild(pushButtonDataSet5);
    XtVaSetValues(pushButtonDataSet5,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet5, NULL);
  }
  else if(ds + 1 == 6) {
    XtManageChild(pushButtonDataSet6);
    XtVaSetValues(pushButtonDataSet6,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet6, NULL);
  }
  else if(ds + 1 == 7) {
    XtManageChild(pushButtonDataSet7);
    XtVaSetValues(pushButtonDataSet7,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet7, NULL);
  }
  else if(ds + 1 == 8) {
    XtManageChild(pushButtonDataSet8);
    XtVaSetValues(pushButtonDataSet8,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet8, NULL);
  }
  else if(ds + 1 == 9) {
    XtManageChild(pushButtonDataSet9);
    XtVaSetValues(pushButtonDataSet9,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuDataSet, XmNmenuHistory, pushButtonDataSet9, NULL);
  }
  else {
  }
  XmStringFree(xmstring);
}

void SetLogSetOptionMenu(ls)
int ls;
{
  XmString xmstring;

  xmstring=XmCvtCTToXmString(logdata[ls]->info.id[0]);
  if(ls + 1 == 1) {
    XtManageChild(optionMenuLogSet);
    XtManageChild(pushButtonLogSet1);
    XtVaSetValues(pushButtonLogSet1,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet1, NULL);
  }
  else if(ls + 1 == 2) {
    XtManageChild(pushButtonLogSet2);
    XtVaSetValues(pushButtonLogSet2,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet2, NULL);
  }
  else if(ls + 1 == 3) {
    XtManageChild(pushButtonLogSet3);
    XtVaSetValues(pushButtonLogSet3,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet3, NULL);
  }
  else if(ls + 1 == 4) {
    XtManageChild(pushButtonLogSet4);
    XtVaSetValues(pushButtonLogSet4,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet4, NULL);
  }
  else if(ls + 1 == 5) {
    XtManageChild(pushButtonLogSet5);
    XtVaSetValues(pushButtonLogSet5,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet5, NULL);
  }
  else if(ls + 1 == 6) {
    XtManageChild(pushButtonLogSet6);
    XtVaSetValues(pushButtonLogSet6,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet6, NULL);
  }
  else if(ls + 1 == 7) {
    XtManageChild(pushButtonLogSet7);
    XtVaSetValues(pushButtonLogSet7,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet7, NULL);
  }
  else if(ls + 1 == 8) {
    XtManageChild(pushButtonLogSet8);
    XtVaSetValues(pushButtonLogSet8,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet8, NULL);
  }
  else if(ls + 1 == 9) {
    XtManageChild(pushButtonLogSet9);
    XtVaSetValues(pushButtonLogSet9,XmNlabelString,xmstring,NULL);
    XtVaSetValues(optionMenuLogSet, XmNmenuHistory, pushButtonLogSet9, NULL);
  }
  else {
  }
  XmStringFree(xmstring);
}

void AutoPair(what)
int what;
{
  int      i, have_pairing;
  XmString xmstring;
  void ChangeDataSet(), SetDataSetOptionMenu(), DrawDrawingAreaCores(), DrawDrawingAreaLog();

  if(what == YES) {
    /* are there any pairs?
    */
    have_pairing = NO;
    for(i=0; i<logset_cnt; ++i) {
      if(logdata[i]->info.maptodata > -1) {
        have_pairing = YES;
        break;
      }
    }

    if(have_pairing == NO) {
      what = NO;
    }
    else {
      /* turn off any data or log set that does not have a pair
      */
      for(i=0; i<dataset_cnt; ++i) {
        if(i == 0 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet1, False);
        }
        else if(i == 0 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet1, True);
        }
        if(i == 1 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet2, False);
        }
        else if(i == 1 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet2, True);
        }
        if(i == 2 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet3, False);
        }
        else if(i == 2 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet3, True);
        }
        if(i == 3 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet4, False);
        }
        else if(i == 3 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet4, True);
        }
        if(i == 4 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet5, False);
        }
        else if(i == 4 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet5, True);
        }
        if(i == 5 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet6, False);
        }
        else if(i == 5 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet6, True);
        }
        if(i == 6 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet7, False);
        }
        else if(i == 6 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet7, True);
        }
        if(i == 7 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet8, False);
        }
        else if(i == 7 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet8, True);
        }
        if(i == 8 && data[i]->maptolog == -1) {
          XtSetSensitive(pushButtonDataSet9, False);
        }
        else if(i == 8 && data[i]->maptolog != -1) {
          XtSetSensitive(pushButtonDataSet9, True);
        }
      }
      for(i=0; i<logset_cnt; ++i) {
        if(i == 0 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet1, False);
        }
        else if(i == 0 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet1, True);
        }
        if(i == 1 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet2, False);
        }
        else if(i == 1 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet2, True);
        }
        if(i == 2 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet3, False);
        }
        else if(i == 2 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet3, True);
        }
        if(i == 3 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet4, False);
        }
        else if(i == 3 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet4, True);
        }
        if(i == 4 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet5, False);
        }
        else if(i == 4 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet5, True);
        }
        if(i == 5 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet6, False);
        }
        else if(i == 5 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet6, True);
        }
        if(i == 6 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet7, False);
        }
        else if(i == 6 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet7, True);
        }
        if(i == 7 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet8, False);
        }
        else if(i == 7 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet8, True);
        }
        if(i == 8 && logdata[i]->info.maptodata == -1) {
          XtSetSensitive(pushButtonLogSet9, False);
        }
        else if(i == 8 && logdata[i]->info.maptodata != -1) {
          XtSetSensitive(pushButtonLogSet9, True);
        }
      }

      xmstring = XmStringCreateSimple("Auto Pair On");
      if(logdata[lset]->info.maptodata != dset) {
        if(logdata[lset]->info.maptodata >= 0 && logdata[lset]->info.maptodata < dataset_cnt) {
          dset = logdata[lset]->info.maptodata;
          ChangeDataSet(dset);
          SetDataSetOptionMenu(dset);
          if(XtIsManaged(XtParent(drawingAreaCores))) {
            core_clip_what = NONE;
            DrawDrawingAreaCores(drawingAreaCores, (XtPointer)NULL, (XtPointer)NULL);
          }
          if(have_coretotie == DO || have_splice == DO) {
            if(XtIsManaged(XtParent(drawingAreaLog))) {
              DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
            }
          }
        }
        else {
          autopair = NO;
          xmstring = XmStringCreateSimple("Auto Pair Off");
        }
      }
    }
  }
  if(what == NO) {
    xmstring = XmStringCreateSimple("Auto Pair Off");
    for(i=0; i<dataset_cnt; ++i) {
      if(i == 0) {
        XtSetSensitive(pushButtonDataSet1, True);
      }
      else if(i == 1) {
        XtSetSensitive(pushButtonDataSet2, True);
      }
      else if(i == 2) {
        XtSetSensitive(pushButtonDataSet3, True);
      }
      else if(i == 3) {
        XtSetSensitive(pushButtonDataSet4, True);
      }
      else if(i == 4) {
        XtSetSensitive(pushButtonDataSet5, True);
      }
      else if(i == 5) {
        XtSetSensitive(pushButtonDataSet6, True);
      }
      else if(i == 6) {
        XtSetSensitive(pushButtonDataSet7, True);
      }
      else if(i == 7) {
        XtSetSensitive(pushButtonDataSet8, True);
      }
      else if(i == 8) {
        XtSetSensitive(pushButtonDataSet9, True);
      }
    }
    for(i=0; i<logset_cnt; ++i) {
      if(i == 0) {
        XtSetSensitive(pushButtonLogSet1, True);
      }
      else if(i == 1) {
        XtSetSensitive(pushButtonLogSet2, True);
      }
      else if(i == 2) {
        XtSetSensitive(pushButtonLogSet3, True);
      }
      else if(i == 3) {
        XtSetSensitive(pushButtonLogSet4, True);
      }
      else if(i == 4) {
        XtSetSensitive(pushButtonLogSet5, True);
      }
      else if(i == 5) {
        XtSetSensitive(pushButtonLogSet6, True);
      }
      else if(i == 6) {
        XtSetSensitive(pushButtonLogSet7, True);
      }
      else if(i == 7) {
        XtSetSensitive(pushButtonLogSet8, True);
      }
      else if(i == 8) {
        XtSetSensitive(pushButtonLogSet9, True);
      }
    }
  }
  XtVaSetValues(labelAutoPair, XmNlabelString, xmstring, NULL);
  XtVaSetValues(pushButtonPairCoreLog, XmNlabelString, xmstring, NULL);
  XmStringFree(xmstring);
}

void RemoveTie(n)
int n;
{
  int  i, j, nt, h, c;
  int  CountTies();
  void AdjustToTies(), DrawDrawingAreaLog(), DrawDrawingAreaAdjustToTiesInfo(), ApplyCoreAdjustToOneCore();

  if(ties->nties == 1) {
    XmListDeleteAllItems(listTieList);
    ties->nties = 0;
    ties->have_new_ties = DONT;
    have_equivlogdep = DONT;
    have_ties = DONT;
    if(have_coretotie == DO) {
      for(i=0; i<data[dset]->holes[ties->hole]->core[ties->core]->numvalues; ++i) {
        if(data[dset]->holes[ties->hole]->core[ties->core]->value[i]->quality  & BAD_SB_DEPTH) {
          ties->temp_equivlogdep[i] = BADPT;
        }
        else {
          ties->temp_equivlogdep[i] = data[dset]->holes[ties->hole]->core[ties->core]->value[i]->mbsf;
          if(have_affine == DO) {
            ties->temp_equivlogdep[i] = ties->temp_equivlogdep[i] + data[dset]->holes[ties->hole]->core[ties->core]->affine_offset;
          }
        }
      }
      if(eqlogdepth.autocorr_status == YES) {
        ApplyCoreAdjustToOneCore(ties->temp_equivlogdep, data[dset]->holes[ties->hole]->core[ties->core]->numvalues);
      }

      h = data[dset]->holes[ties->hole]->map_hole_to_eqlogdepth;
      c = data[dset]->holes[ties->hole]->core[ties->core]->map_core_to_eqlogdepth;
      if(eqlogdepth.hole[h]->core[c]->nties > 0) {
        XtSetSensitive(pushButtonSaveChanges, True);
      }
      else {
        XtSetSensitive(pushButtonSaveChanges, False);
      }
    }
    else if(have_splice == DO && sset >= 0 && sset < splice_cnt) {
      for(i=0; i<splice[sset]->numvalues; ++i) {
        ties->temp_equivlogdep[i] = splice[sset]->value[i]->mcd;
      }
      if(eqlogdepth.autocorr_status == YES) {
        ApplyCoreAdjustToOneCore(ties->temp_equivlogdep, splice[sset]->numvalues);
      }
    }

    XtSetSensitive(pushButtonDisplayHideAdjustToTiesInfo, False);
  }
  else if(ties->nties > 1) {
    XmListDeletePos(listTieList, n+1);
    for(j=n; j<ties->nties-1; ++j) {
      ties->tie[j]->holename = ties->tie[j+1]->holename;
      ties->tie[j]->corename = ties->tie[j+1]->corename;
      ties->tie[j]->log = ties->tie[j+1]->log;
      ties->tie[j]->array = ties->tie[j+1]->array;
      ties->tie[j]->corembsf = ties->tie[j+1]->corembsf;
      ties->tie[j]->coremcd = ties->tie[j+1]->coremcd;
      ties->tie[j]->equivlogdep = ties->tie[j+1]->equivlogdep;
      ties->tie[j]->temp_eld = ties->tie[j+1]->temp_eld;
      ties->tie[j]->coreautoeld = ties->tie[j+1]->coreautoeld;
      ties->tie[j]->logdep = ties->tie[j+1]->logdep;
      ties->tie[j]->rate = ties->tie[j+1]->rate;
      ties->tie[j]->b = ties->tie[j+1]->b;
      ties->tie[j]->topsect = ties->tie[j+1]->topsect;
      ties->tie[j]->botsect = ties->tie[j+1]->botsect;
      (void)sprintf(ties->tie[j]->coresection, "%s", ties->tie[j+1]->coresection);
    }
    --ties->nties;
    AdjustToTies(dset);
  }

  ties->lasttiesaved = -1;
  XtSetSensitive(pushButtonClearLastTieSaved, False);
  if(have_splice == DO && ties->nties <= 1) {
    if((nt = CountTies()) == 0) {
      XtSetSensitive(pushButtonAutoCorrelate, True);
    }
  }

  if(XtIsManaged(XtParent(drawingAreaLog))) {
    log_clip_what = NONE;
    DrawDrawingAreaLog(drawingAreaLog, (XtPointer)NULL, (XtPointer)NULL);
  }
  if(XtIsManaged(XtParent(drawingAreaAdjustToTiesInfo))) {
    DrawDrawingAreaAdjustToTiesInfo(drawingAreaAdjustToTiesInfo, (XtPointer)NULL, (XtPointer)NULL);
  }
}
